//=============================================================================
// OldVersionHUD
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// 
//=============================================================================


class OldVersionHUD extends TournamentHUD;


//=============================================================================
// Localization
//=============================================================================

var localized array<string> UpdateNotice;


//=============================================================================
// PostBeginPlay
//
// Start the disconnect countdown.
//=============================================================================

function PostBeginPlay()
{
  local int i;
  
  for (i = 0; i < ArrayCount(FontArrayNames); i++) {
    FontArrayNames[i] = "UT2003Fonts.FontMono";
    default.FontArrayFonts[i] = None;
  }
  SetTimer(20, True);
}


//=============================================================================
// Timer
//
// Count down and disconnect.
//=============================================================================

function Timer()
{
  ConsoleCommand("DISCONNECT");
}


//=============================================================================
// PostRender
//
// Display typing prompt and the update notice.
//=============================================================================

function PostRender(Canvas C)
{
  Super(HUD).LinkActors();
  DisplayUpdateNotice(C);
  if ( PlayerConsole != None && PlayerConsole.bTyping )
    DrawTypingPrompt(C, PlayerConsole.TypedStr);
}


//=============================================================================
// DisplayUpdateNotice
//
// Display the update notice.
//=============================================================================

simulated function DisplayUpdateNotice(Canvas C)
{
  local float FontDX, FontDY, DY;
  local float X, Y;
  local int i;
  
  C.Font = LoadProgressFont();
  C.Style = ERenderStyle.STY_Alpha;
  C.TextSize("A", FontDX, FontDY);
  
  X = 0.5 * C.SizeX;
  Y = 0.5 * C.SizeY - FontDY * 0.5 * float(UpdateNotice.Length);
  DY = FontDY;
  
  C.DrawColor = MOTDColor;
  for(i = 0; i < UpdateNotice.Length; i++) {
    if ( UpdateNotice[i] != "" ) {
      C.TextSize(UpdateNotice[i], FontDX, FontDY);
      C.SetPos(X - FontDX * 0.5, Y);
      C.DrawText(UpdateNotice[i]);
    }
    Y += DY;
  }
}


//=============================================================================
// DrawTypingPrompt
//
// Draws a typing prompt with a blinking cursor.
//=============================================================================

function DrawTypingPrompt(Canvas C, string Text)
{
  local float XPos, YPos;
  local float XL, YL;
  
  C.Font = GetConsoleFont(C);
  C.Style = ERenderStyle.STY_Alpha;
  C.DrawColor = ConsoleColor;
  
  C.TextSize("A", XL, YL);
  
  XPos = ConsoleMessagePosX * C.SizeX;
  YPos = ConsoleMessagePosY * C.SizeY - YL;
  
  C.SetPos(XPos, YPos);
  if ( int((60.0 * Level.Minute + Level.Second + 0.001 * Level.MilliSecond) * 2 * CursorBlinkingRate % 2) == 0 )
    Text = "(>" @ Text;
  else
    Text = "(>" @ Text $ "_";
  C.DrawTextClipped(Text, false);
}


//=============================================================================
// Disabled exec functions
//=============================================================================

exec function Menu();
exec function DemoRec();
exec function bool Exec();  // needs to have a return type because otherwise the compiler complains
exec function Set();
exec function Say();
exec function TeamSay();
exec function Mutate();
exec function Admin();
exec function AdminLogin();


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  UpdateNotice(0)="This server is running Tournament."
  UpdateNotice(2)="Tournament requires at least UT2003 v2225,"
  UpdateNotice(3)="but you still have an older version installed."
  UpdateNotice(4)="Please download the latest UT2003 patch at"
  UpdateNotice(5)="http://www.unrealtournament.com/"
  UpdateNotice(8)="You are being disconnected..."
}