//=============================================================================
// TGUIInstantActionPanel
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Base class for Tournament gametype config panels.
//=============================================================================


class TGUIInstantActionPanel extends Tab_InstantActionBaseRules
    abstract;


//=============================================================================
// Configuration
//=============================================================================

var config int LastSpawnProtTime;
var config bool LastAutoAdjustSkill;


//=============================================================================
// Localization
//=============================================================================

var localized string GoalScoreCaption;


//=============================================================================
// Variables
//=============================================================================

var TGUILabeledSlider GameSpeed;
var TGUILabeledSlider FriendlyFire;
var TGUISliderEdit    GoalScore;
var TGUISliderEdit    TimeLimit;
var TGUISliderEdit    MaxLives;
var TGUISliderEdit    SpawnProtTime;
var moCheckbox        MyAutoAdjustSkill;


//=============================================================================
// InitComponent
//
// Initializes the panel and its components.
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super(GUITabPanel).Initcomponent(MyController, MyOwner);
  
  GameSpeed    = TGUILabeledSlider(Controls[4]);
  FriendlyFire = TGUILabeledSlider(Controls[5]);
  
  MyWeaponStay      = moCheckBox(Controls[6]);
  MyWeaponThrowing  = moCheckBox(Controls[7]);
  MyTranslocator    = moCheckBox(Controls[8]);
  MyAutoAdjustSkill = moCheckBox(Controls[13]);
  
  GoalScore      = TGUISliderEdit(Controls[9]);
  TimeLimit      = TGUISliderEdit(Controls[10]);
  MaxLives       = TGUISliderEdit(Controls[11]);
  SpawnProtTime  = TGUISliderEdit(Controls[12]);
  
  GameSpeed.OnDrawCaption    = InternalSliderPercentDraw;
  FriendlyFire.OnDrawCaption = InternalSliderPercentDraw;
  
  GameSpeed.SetValue(LastGameSpeed * 100);
  FriendlyFire.SetValue(LastFriendlyFire * 100);
  
  MyWeaponStay.Checked(LastWeaponStay);
  MyTranslocator.Checked(LastTranslocator);
  MyWeaponThrowing.Checked(bLastWeaponThrowing);
  MyAutoAdjustSkill.Checked(LastAutoAdjustSkill);
  
  if ( GoalScoreCaption != "" )
    GoalScore.MyLabel.Caption = GoalScoreCaption;
  GoalScore.SetValue(LastGoalScore);
  TimeLimit.SetValue(LastTimeLimit);
  MaxLives.SetValue(LastMaxLives);
  SpawnProtTime.SetValue(LastSpawnProtTime);
}


//=============================================================================
// Play
//
// Returns a part of the URL for starting the game.
//=============================================================================

function string Play()
{
  local string url;

  LastGameSpeed = GameSpeed.GetValue() / 100;
  LastFriendlyFire = FriendlyFire.GetValue()/100;
  
  LastWeaponStay = MyWeaponStay.IsChecked();
  LastTranslocator = MyTranslocator.IsChecked();
  bLastWeaponThrowing = MyWeaponThrowing.IsChecked();
  LastAutoAdjustSkill = MyAutoAdjustSkill.IsChecked();
  
  LastGoalScore     = GoalScore.GetValue();
  LastTimeLimit     = TimeLimit.GetValue();
  LastMaxLives      = MaxLives.GetValue();
  LastSpawnProtTime = SpawnProtTime.GetValue();
  
  SaveConfig();
  
  url = "?GameSpeed=" $ LastGameSpeed $ "?WeaponStay=" $ LastWeaponStay $ "?Translocator=" $ LastTranslocator
      $ "?FriendlyFireScale=" $ LastFriendlyFire $ "?GoalScore=" $ LastGoalScore $ "?TimeLimit=" $ LastTimeLimit
      $ "?MaxLives=" $ LastMaxLives $ "?SpawnProt=" $ LastSpawnProtTime $ "?AllowThrowing=" $ bLastWeaponThrowing
      $ "?AutoAdjust=" $ LastAutoAdjustSkill;
  
  return url;
}


//=============================================================================
// InternalSliderPercentDraw
//
// Returns the displayed value of the slider.
//=============================================================================

function string InternalSliderPercentDraw(TGUILabeledSlider Sender)
{
  return "(" $ int(Sender.GetValue()) $ PercentText $ ")";
}


//=============================================================================
// TournamentConfigClicked
//
// Returns the displayed value of the slider.
//=============================================================================

function bool TournamentConfigClicked(GUIComponent Sender)
{
  Controller.OpenMenu("Tournament.TGUISettingsMenu");
  
  return true;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object class=GUIImage Name=PanelBackground1
    WinWidth=0.96
    WinHeight=0.155
    WinLeft=0.02
    WinTop=0.025
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(0)=GUIImage'PanelBackground1'
  
  Begin Object class=GUIImage Name=PanelBackground2
    WinWidth=0.47
    WinHeight=0.49
    WinLeft=0.51
    WinTop=0.24
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(1)=GUIImage'PanelBackground2'
  
  Begin Object class=GUIImage Name=PanelBackground3
    WinWidth=0.47
    WinHeight=0.49
    WinLeft=0.02
    WinTop=0.24
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(2)=GUIImage'PanelBackground3'
  
  Begin Object class=GUIImage Name=PanelBackground4
    WinWidth=0.96
    WinHeight=0.155
    WinLeft=0.02
    WinTop=0.78
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(3)=GUIImage'PanelBackground4'
  
  Begin Object class=TGUILabeledSlider Name=GameSpeedSlider
    WinWidth=0.35
    WinLeft=0.1
    WinTop=0.04
    WinHeight=0.12
    MinValue=50
    MaxValue=200
    Value=100
    bIntegerOnly=True
    Caption="Game Speed"
    Hint="This option controls how fast the game will be played."
  End Object
  Controls(4)=TGUILabeledSlider'GameSpeedSlider'
  
  Begin Object class=TGUILabeledSlider Name=FriendlyFireSlider
    WinWidth=0.35
    WinLeft=0.55
    WinTop=0.04
    WinHeight=0.12
    MinValue=0
    MaxValue=200
    bIntegerOnly=True
    Caption="FriendlyFire"
    Hint="This option controls how much damage you do to teammates."
  End Object
  Controls(5)=TGUILabeledSlider'FriendlyFireSlider'
  
  Begin Object class=moCheckBox Name=WeaponStayCheck
    WinWidth=0.4
    WinHeight=0.04
    WinLeft=0.05
    WinTop=0.32
    Caption="Weapon Stay"
    Hint="When enabled, weapons will always be available for pickup."
    CaptionWidth=0.9
    bSquare=true
    ComponentJustification=TXTA_Left
  End Object
  Controls(6)=moCheckBox'WeaponStayCheck'
  
  Begin Object class=moCheckBox Name=AllowWeaponThrowCheck
    WinWidth=0.4
    WinHeight=0.04
    WinLeft=0.05
    WinTop=0.37
    Caption="Allow Weapon Throwing"
    Hint="When selected, a player will have the ability to throw out his current weapon."
    bSquare=true
    ComponentJustification=TXTA_Left
    CaptionWidth=0.9
  End Object
  Controls(7)=moCheckBox'AllowWeaponThrowCheck'
  
  Begin Object class=moCheckBox Name=TranslocatorCheck
    WinWidth=0.4
    WinHeight=0.04
    WinLeft=0.05
    WinTop=0.42
    Caption="Include Translocator"
    CaptionWidth=0.9
    bSquare=true
    ComponentJustification=TXTA_Left
    Hint="Enable this option to allow Translocators."
  End Object
  Controls(8)=moCheckBox'TranslocatorCheck'
  
  Begin Object class=TGUISliderEdit Name=GoalScoreEdit
    WinWidth=0.4
    WinLeft=0.55
    WinTop=0.3
    Caption="Goal Score"
    CaptionWidth=0.45
    EditBoxWidth=0.2
    MinValue=0
    MaxValue=100
    bIntegerOnly=True
    Hint="The game will end when this threshold is met."
  End Object
  Controls(9)=TGUISliderEdit'GoalScoreEdit'
  
  Begin Object class=TGUISliderEdit Name=TimeLimitEdit
    WinWidth=0.4
    WinLeft=0.55
    WinTop=0.4
    Caption="Time Limit"
    CaptionWidth=0.45
    EditBoxWidth=0.2
    MinValue=0
    MaxValue=60
    bIntegerOnly=True
    Hint="The game will end after this many minutes of play."
  End Object
  Controls(10)=TGUISliderEdit'TimeLimitEdit'
  
  Begin Object class=TGUISliderEdit Name=MaxLivesEdit
    WinWidth=0.4
    WinLeft=0.55
    WinTop=0.5
    Caption="Max Lives"
    CaptionWidth=0.45
    EditBoxWidth=0.2
    MinValue=0
    MaxValue=255
    bIntegerOnly=True
    Hint="If this value is not 0, you will only respawn this many times."
  End Object
  Controls(11)=TGUISliderEdit'MaxLivesEdit'
  
  Begin Object class=TGUISliderEdit Name=SpawnProtTimeEdit
    WinWidth=0.4
    WinLeft=0.55
    WinTop=0.6
    Caption="Spawn Protection"
    CaptionWidth=0.45
    EditBoxWidth=0.2
    MinValue=0
    MaxValue=30
    bIntegerOnly=True
    Hint="Players will be protected after respawn until firing their weapon."
  End Object
  Controls(12)=TGUISliderEdit'SpawnProtTimeEdit'
  
  Begin Object class=moCheckBox Name=AutoAdjustSkillCheck
    WinWidth=0.25
    WinHeight=0.156016
    WinLeft=0.375
    WinTop=0.858295
    Caption="Auto-Adjust Skill"
    Hint="When enabled, bots will adjust their skill to match yours."
    CaptionWidth=0.9
    bSquare=true
    ComponentJustification=TXTA_Left
  End Object
  Controls(13)=moCheckBox'AutoAdjustSkillCheck'
  
  Begin Object Class=GUIButton Name=TournamentConfigButton
    Caption="More Options"
    Hint="Provides additional configuration options."
    WinWidth=0.25
    WinHeight=0.06
    WinLeft=0.13
    WinTop=0.65
    OnClick=TournamentConfigClicked
  End Object
  Controls(14)=GUIButton'TournamentConfigButton'
  
  WinTop=0.15
  WinLeft=0
  WinWidth=1
  WinHeight=0.77
  bAcceptsInput=false
  
  LastGameSpeed=1.0
  LastFriendlyFire=0.0
  LastWeaponStay=True
  LastTranslocator=False
  bLastWeaponThrowing=True
  LastAutoAdjustSkill=False
  LastGoalScore=0
  LastTimeLimit=15
  LastMaxLives=0
  LastSpawnProtTime=2
  PercentText="%"
}