//=============================================================================
// TGUIMidGameMenu
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Mid-game menu for Tournament game types.
//=============================================================================


class TGUIMidGameMenu extends UT2MidGameMenu;


//=============================================================================
// Localization
//=============================================================================

var localized string SettingsCaption;
var localized string PauseCaption;
var localized string UnpauseCaption;
var localized string SetReadyCaption;
var localized string SetNotReadyCaption;


//=============================================================================
// InitComponent
//
// Set up "Tournament Settings" and "UT2003 Settings" buttons.
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super.InitComponent(MyController, MyOwner);
  
  GUIButton(Controls[4]).Caption = SettingsCaption;
  
  Controls[5].WinTop    = Controls[7].WinTop;
  
  Controls[8].WinTop    = Controls[2].WinTop;
  Controls[8].WinLeft   = Controls[4].WinLeft;
  Controls[8].WinWidth  = ButtonWidth;
  Controls[8].WinHeight = ButtonHeight;
  
  Controls[9].WinTop    = Controls[7].WinTop;
  Controls[9].WinLeft   = Controls[2].WinLeft;
  Controls[9].WinWidth  = ButtonWidth;
  Controls[9].WinHeight = ButtonHeight;
  if ( PlayerOwner().Level.NetMode == NM_Standalone )
    Controls[9].bVisible = False;
  else {
    Timer();
    SetTimer(0.1, True);
  }
}


//=============================================================================
// Timer
//
// Update the Pause button.
//=============================================================================

function Timer()
{
  if ( PlayerOwner().PlayerReplicationInfo != None && PlayerOwner().PlayerReplicationInfo.bAdmin ) {
    if ( PlayerOwner().Level.Pauser == None )
      GUIButton(Controls[9]).Caption = PauseCaption;
    else
      GUIButton(Controls[9]).Caption = UnpauseCaption;
  }
  else if ( PlayerOwner().PlayerReplicationInfo != None && !PlayerOwner().PlayerReplicationInfo.bOnlySpectator
      && !PlayerOwner().GameReplicationInfo.bMatchHasBegun ) {
    if ( PlayerOwner().PlayerReplicationInfo.bReadyToPlay )
      GUIButton(Controls[9]).Caption = SetNotReadyCaption;
    else
      GUIButton(Controls[9]).Caption = SetReadyCaption;
  }
  else {
    Controls[9].bVisible = False;
    return;
  }
  Controls[9].bVisible = True;
}


//=============================================================================
// InternalOnClick
//
// Open the Tournament settings for the current gametype.
//=============================================================================

function bool InternalOnClick(GUIComponent Sender)
{
  local string SettingsMenuClass;
  
  if ( Sender == Controls[8] ) { // Tournament Settings
    if ( TournamentHUD(PlayerOwner().MyHud) != None )
      SettingsMenuClass = TournamentHUD(PlayerOwner().MyHud).ConfigMenuClass;
    if ( SettingsMenuClass == "" )
      Controller.OpenMenu("Tournament.TGUISettingsMenu");
    else
      Controller.OpenMenu(SettingsMenuClass);
  }
  else if ( Sender == Controls[9] ) { // pause game as admin or toggle ready status
    if ( PlayerOwner().PlayerReplicationInfo != None && PlayerOwner().PlayerReplicationInfo.bAdmin )
      PlayerOwner().SetPause(PlayerOwner().Level.Pauser == None);
    else {
      if ( PlayerOwner().PlayerReplicationInfo != None && PlayerOwner().PlayerReplicationInfo.bReadyToPlay ) {
        PlayerOwner().ConsoleCommand("READY");
      }
      else {
        PlayerOwner().ConsoleCommand("READY");
        Controller.CloseMenu();
      }
    }
  }
  else
    return Super.InternalOnClick(Sender);
  
  return true;
}


//=============================================================================
// InternalOnClick
//
// Open the Tournament settings for the current gametype.
//=============================================================================

function InternalOnClose(optional Bool bCanceled)
{
  local PlayerController pc;
  
  pc = PlayerOwner();
  
  // Turn pause off if currently paused
  if ( pc != None && pc.Level.NetMode == NM_StandAlone && pc.Level.Pauser != None )
    pc.SetPause(false);
  
  Super(GUIPage).OnClose(bCanceled);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUIButton Name=TournamentButton
    Caption="TOURNAMENT SETTINGS"
    StyleName="MidGameButton"
    OnClick=InternalOnClick
  End Object
  Controls(8)=GUIButton'TournamentButton'
  
  Begin Object Class=GUIButton Name=PauseButton
    StyleName="MidGameButton"
    OnClick=InternalOnClick
  End Object
  Controls(9)=GUIButton'PauseButton'
  
  SettingsCaption="UT2003 SETTINGS"
  PauseCaption="PAUSE GAME"
  UnpauseCaption="UNPAUSE GAME"
  SetReadyCaption="SET READY"
  SetNotReadyCaption="SET NOT READY"
  
  CloseSound=sound'MenuSounds.SelectK'
}