//=============================================================================
// TGUISettingsIngameMenu
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Ingame config menu for Tournament games.
//=============================================================================


class TGUISettingsIngameMenu extends GUIPage;


//=============================================================================
// Localization
//=============================================================================

var localized string GamePanelCaption;
var localized string HUDPanelCaption;
var localized string CrosshairPanelCaption;
var localized string MessagesPanelCaption;
var localized string SpeechPanelCaption;
var localized string AdminPanelCaption;

var localized string GamePanelHint;
var localized string HUDPanelHint;
var localized string CrosshairPanelHint;
var localized string MessagesPanelHint;
var localized string SpeechPanelHint;
var localized string AdminPanelHint;


//=============================================================================
// Variables
//=============================================================================

var TGUISettingsPanel GamePanel;
var TGUISettingsPanel HUDPanel;
var TGUISettingsPanel CrosshairPanel;
var TGUISettingsPanel MessagesPanel;
var TGUISettingsPanel SpeechPanel;
var TGUISettingsPanel AdminLoginPanel;
var TGUISettingsPanel AdminPanel;

var bool bPaused;


//=============================================================================
// InitComponent
//
// Add tab panels.
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super(GUIPage).InitComponent(MyController, MyOwner);
  GUITabControl(Controls[4]).AddTab(GamePanelCaption,      "", GamePanel,       GamePanelHint);
  GUITabControl(Controls[4]).AddTab(HUDPanelCaption,       "", HUDPanel,        HUDPanelHint);
  GUITabControl(Controls[4]).AddTab(CrosshairPanelCaption, "", CrosshairPanel,  CrosshairPanelHint);
  GUITabControl(Controls[4]).AddTab(MessagesPanelCaption,  "", MessagesPanel,   MessagesPanelHint);
  GUITabControl(Controls[4]).AddTab(SpeechPanelCaption,    "", SpeechPanel,     SpeechPanelHint);
  if ( PlayerOwner().PlayerReplicationInfo.bAdmin || PlayerOwner().Level.NetMode == NM_StandAlone )
    GUITabControl(Controls[4]).AddTab(AdminPanelCaption,   "", AdminPanel,      AdminPanelHint);
  else
    GUITabControl(Controls[4]).AddTab(AdminPanelCaption,   "", AdminLoginPanel, AdminPanelHint);
  
  if ( PlayerOwner().MyHud != None ) {
    PlayerOwner().MyHud.bShowScoreBoard = False;
    PlayerOwner().MyHud.bShowLocalStats = False;
  }
}


//=============================================================================
// HandleParameters
//
// The Menu console command tells this menu whether it should unpause the game
// on close.
//=============================================================================

event HandleParameters(string Param1, string Param2)
{
  local array<string> Params;
  
  Split(Param1, " ", Params);
  if ( Params.Length > 0 && Params[0] ~= "SetPause" ) {
    bPaused = True;
    Params.Remove(0, 1);
  }
  if ( Param2 ~= "Admin" )
    GUITabControl(Controls[4]).ActivateTabByName(AdminPanelCaption, True);
}


//=============================================================================
// ReplaceAdminLoginPanel
//
// Replaces the admin login panel after the click events have been processed.
//=============================================================================

function ReplaceAdminLoginPanel()
{
  if ( AdminLoginPanel.MyButton != None )
    GUITabControl(Controls[4]).ReplaceTab(AdminLoginPanel.MyButton, AdminPanelCaption, "", AdminPanel,
        AdminPanelHint, True);
  AdminLoginPanel.bVisible = False;
}


//=============================================================================
// ReplaceAdminPanel
//
// Replaces the admin panel after the click events have been processed.
//=============================================================================

function ReplaceAdminPanel()
{
  if ( AdminPanel.MyButton != None )
    GUITabControl(Controls[4]).ReplaceTab(AdminPanel.MyButton, AdminPanelCaption, "", AdminLoginPanel,
        AdminPanelHint, True);
  AdminPanel.bVisible = False;
}


//=============================================================================
// CloseButtonClicked
//
// Closes the config dialog window.
//=============================================================================

function bool CloseButtonClicked(GUIComponent Sender)
{
  Controller.CloseMenu(false);
  
  return True;
}


//=============================================================================
// InternalOnClose
//
// Unpauses stand-alone games.
//=============================================================================

function InternalOnClose(optional bool bCanceled)
{
  local int i;
  local GUITabControl TabC;
  
  TabC = GUITabControl(Controls[4]);
  for (i = 0; i < TabC.TabStack.Length; i++)
    TGUISettingsPanel(TabC.TabStack[i].MyPanel).OnCloseDialog(bCanceled);
  
  if ( bPaused )
    PlayerOwner().SetPause(False);
  
  Super.OnClose(bCanceled);
}


//=============================================================================
// ChangedTab
//
// Update the title bar caption.
//=============================================================================

function ChangedTab(GUIComponent Sender)
{
  if ( GUITabButton(Sender) != None )
    GUILabel(Controls[1]).Caption = GUILabel(default.Controls[1]).Caption @ "|" @ GUITabButton(Sender).Caption;
}


//=============================================================================
// ChangeHint
//
// Change the displayed hint.
//=============================================================================

event ChangeHint(string NewHint)
{
  GUILabel(Controls[2]).Caption = NewHint;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUIImage name=DialogBackground
    bAcceptsInput=false
    bNeverFocus=true
    Image=Material'InterfaceContent.Menu.EditBox'
    ImageStyle=ISTY_Stretched
    WinWidth=1
    WinLeft=0
    WinHeight=1
    WinTop=0
    bBoundToParent=true
    bScaleToParent=true
  End Object
  Controls(0)=GUIImage'DialogBackground'
  
  Begin Object Class=GUILabel name=DialogCaption
    Caption="Tournament Options"
    TextFont="UT2HeaderFont"
    WinWidth=0.96
    WinHeight=0.1
    WinTop=0.01
    WinLeft=0.02
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(1)=GUILabel'DialogCaption'
  
  Begin Object Class=GUILabel name=HintLabel
    WinWidth=0.8
    WinHeight=0.1
    WinTop=0.9
    WinLeft=0.01
    TextAlign=TXTA_Center
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(2)=GUILabel'HintLabel'
  
  Begin Object Class=GUIButton name=PanelBackground
    WinWidth=0.98
    WinHeight=0.8
    WinTop=0.11
    WinLeft=0.01
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(3)=GUIButton'PanelBackground'
  
  Begin Object Class=GUITabControl name=SettingsTabs
    WinWidth=0.95
    WinHeight=0.1
    WinTop=0.995
    WinLeft=0.025
    TabHeight=0.04
    bDockPanels=False
    bFillSpace=True
    OnChange=ChangedTab
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(4)=GUITabControl'SettingsTabs'
  
  Begin Object Class=GUIButton name=CloseButton
    Caption="Close"
    Hint="Close the Tournament config menu."
    WinWidth=0.18
    WinHeight=0.05
    WinTop=0.93
    WinLeft=0.81
    OnClick=CloseButtonClicked
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(5)=GUIButton'CloseButton'
  
  Begin Object Class=TGUISettingsPanelGame name=TabPanelGame
  End Object
  GamePanel=TabPanelGame
  
  Begin Object Class=TGUISettingsPanelHUD name=TabPanelHUD
  End Object
  HUDPanel=TabPanelHUD
  
  Begin Object Class=TGUISettingsPanelCrosshair name=TabPanelCrosshair
  End Object
  CrosshairPanel=TabPanelCrosshair
  
  Begin Object Class=TGUISettingsPanelMessages name=TabPanelMessages
  End Object
  MessagesPanel=TabPanelMessages
  
  Begin Object Class=TGUISettingsPanelSpeech name=TabPanelSpeech
  End Object
  SpeechPanel=TabPanelSpeech
  
  Begin Object Class=TGUISettingsPanelAdmin name=TabPanelAdmin
    OnAdminLoggedOut=ReplaceAdminPanel
  End Object
  AdminPanel=TabPanelAdmin
  
  Begin Object Class=TGUISettingsPanelAdminLogin name=TabPanelAdminLogin
    OnAdminLoggedIn=ReplaceAdminLoginPanel
  End Object
  AdminLoginPanel=TabPanelAdminLogin
  
  GamePanelCaption="Game"
  HUDPanelCaption="HUD"
  CrosshairPanelCaption="Crosshair"
  MessagesPanelCaption="Messages"
  SpeechPanelCaption="Speech"
  AdminPanelCaption="Admin"
  
  GamePanelHint="Change Game Preferences"
  HUDPanelHint="Change HUD Preferences"
  CrosshairPanelHint="Change Crosshair Preferences"
  MessagesPanelHint="Change HUD Message Preferences"
  SpeechPanelHint="Key binder for voice commands and taunts."
  AdminPanelHint="Change Admin Settings"
  
  OpenSound=sound'MenuSounds.SelectDshort'
  CloseSound=sound'MenuSounds.SelectK'
  
  WinLeft=0.1
  WinTop=0.15
  WinWidth=0.8
  WinHeight=0.65
  OnClose=InternalOnClose
  bAllowedAsLast=True
}