//=============================================================================
// TGUISettingsMenu
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Config menu for Tournament client and server options.
//=============================================================================


class TGUISettingsMenu extends GUIPage;


//=============================================================================
// Localization
//=============================================================================

var localized string GamePanelCaption;
var localized string HUDPanelCaption;
var localized string CrosshairPanelCaption;
var localized string MessagesPanelCaption;
var localized string ServerPanelCaption;

var localized string GamePanelHint;
var localized string HUDPanelHint;
var localized string CrosshairPanelHint;
var localized string MessagesPanelHint;
var localized string ServerPanelHint;


//=============================================================================
// Variables
//=============================================================================

var TGUISettingsPanel GamePanel;
var TGUISettingsPanel HUDPanel;
var TGUISettingsPanel CrosshairPanel;
var TGUISettingsPanel MessagesPanel;
var TGUISettingsPanel ServerPanel;


//=============================================================================
// InitComponent
//
// Add tab panels.
//=============================================================================

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
  Super.InitComponent(MyController, MyOwner);
  GUITabControl(Controls[4]).AddTab(GamePanelCaption,      "", GamePanel,      GamePanelHint);
  GUITabControl(Controls[4]).AddTab(HUDPanelCaption,       "", HUDPanel,       HUDPanelHint);
  GUITabControl(Controls[4]).AddTab(CrosshairPanelCaption, "", CrosshairPanel, CrosshairPanelHint);
  GUITabControl(Controls[4]).AddTab(MessagesPanelCaption,  "", MessagesPanel,  MessagesPanelHint);
  GUITabControl(Controls[4]).AddTab(ServerPanelCaption,    "", ServerPanel,    ServerPanelHint);
}


//=============================================================================
// CloseButtonClicked
//
// Closes the config dialog window.
//=============================================================================

function bool CloseButtonClicked(GUIComponent Sender)
{
  Controller.CloseMenu(false);
  
  return True;
}


//=============================================================================
// InternalOnClose
//
// Unpauses stand-alone games.
//=============================================================================

function InternalOnClose(optional bool bCanceled)
{
  local int i;
  local GUITabControl TabC;
  
  TabC = GUITabControl(Controls[4]);
  for (i = 0; i < TabC.TabStack.Length; i++)
    TGUISettingsPanel(TabC.TabStack[i].MyPanel).OnCloseDialog(bCanceled);
  
  Super.OnClose(bCanceled);
}


//=============================================================================
// ChangedTab
//
// Update the title bar caption.
//=============================================================================

function ChangedTab(GUIComponent Sender)
{
  if ( GUITabButton(Sender) != None )
    GUILabel(Controls[1]).Caption = GUILabel(default.Controls[1]).Caption @ "|" @ GUITabButton(Sender).Caption;
}


//=============================================================================
// ChangeHint
//
// Change the displayed hint.
//=============================================================================

event ChangeHint(string NewHint)
{
  GUILabel(Controls[2]).Caption = NewHint;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUIImage name=DialogBackground
    bAcceptsInput=false
    bNeverFocus=true
    Image=Material'InterfaceContent.Menu.EditBox'
    ImageStyle=ISTY_Stretched
    WinWidth=1
    WinLeft=0
    WinHeight=1
    WinTop=0
    bBoundToParent=true
    bScaleToParent=true
  End Object
  Controls(0)=GUIImage'DialogBackground'
  
  Begin Object Class=GUILabel name=DialogCaption
    Caption="Tournament Options"
    TextFont="UT2HeaderFont"
    WinWidth=0.96
    WinHeight=0.1
    WinTop=0.01
    WinLeft=0.02
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(1)=GUILabel'DialogCaption'
  
  Begin Object Class=GUILabel name=HintLabel
    WinWidth=0.8
    WinHeight=0.1
    WinTop=0.9
    WinLeft=0.01
    TextAlign=TXTA_Center
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(2)=GUILabel'HintLabel'
  
  Begin Object Class=GUIButton name=PanelBackground
    WinWidth=0.98
    WinHeight=0.8
    WinTop=0.11
    WinLeft=0.01
    bAcceptsInput=False
    bNeverFocus=True
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(3)=GUIButton'PanelBackground'
  
  Begin Object Class=GUITabControl name=SettingsTabs
    WinWidth=0.95
    WinHeight=0.1
    WinTop=0.995
    WinLeft=0.025
    TabHeight=0.04
    bDockPanels=False
    bFillSpace=True
    OnChange=ChangedTab
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(4)=GUITabControl'SettingsTabs'
  
  Begin Object Class=GUIButton name=CloseButton
    Caption="Close"
    Hint="Close the Tournament config menu."
    WinWidth=0.18
    WinHeight=0.05
    WinTop=0.93
    WinLeft=0.81
    OnClick=CloseButtonClicked
    bBoundToParent=True
    bScaleToParent=True
  End Object
  Controls(5)=GUIButton'CloseButton'
  
  Begin Object Class=TGUISettingsPanelGame name=TabPanelGame
  End Object
  GamePanel=TabPanelGame
  
  Begin Object Class=TGUISettingsPanelHUD name=TabPanelHUD
  End Object
  HUDPanel=TabPanelHUD
  
  Begin Object Class=TGUISettingsPanelCrosshair name=TabPanelCrosshair
  End Object
  CrosshairPanel=TabPanelCrosshair
  
  Begin Object Class=TGUISettingsPanelMessages name=TabPanelMessages
  End Object
  MessagesPanel=TabPanelMessages
  
  Begin Object Class=TGUISettingsPanelServer name=TabPanelServer
  End Object
  ServerPanel=TabPanelServer
  
  GamePanelCaption="Game"
  HUDPanelCaption="HUD"
  CrosshairPanelCaption="Crosshair"
  MessagesPanelCaption="Messages"
  ServerPanelCaption="Server"
  
  GamePanelHint="Change Game Preferences"
  HUDPanelHint="Change HUD Preferences"
  CrosshairPanelHint="Change Crosshair Preferences"
  MessagesPanelHint="Change HUD Message Preferences"
  ServerPanelHint="Change Serverside Settings."
  
  WinLeft=0.1
  WinTop=0.15
  WinWidth=0.8
  WinHeight=0.65
  OnClose=InternalOnClose
  bAllowedAsLast=True
}