//=============================================================================
// TGUISettingsPanelAdmin
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for admin options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelAdmin extends TGUISettingsPanelServerBase;


//=============================================================================
// Variables
//=============================================================================

var TournamentPlayer TournamentOwner;
var() TournamentUtil.TServerSettings ReceivedServerSettings;


//=============================================================================
// InitPanel
//
// Assign delegates and start requesting server settings.
//=============================================================================

function InitPanel()
{
  local int i;
  
  Super.InitPanel();
  
  for (i = 1; i < Controls.Length; i++)
    Controls[i].MenuStateChange(MSAT_Disabled);
  
  TournamentOwner = TournamentPlayer(PlayerOwner());
  if ( TournamentOwner != None ) {
    TournamentOwner.OnClientReceivedSettings = ReceivedSettings;
    TournamentOwner.ServerRequestServerSettings();
  }
  
  Controls[24].OnClick = AdminButtonClicked;
  Controls[25].OnClick = ApplyButtonClicked;
  Controls[26].bVisible = PlayerOwner().Level.NetMode != NM_StandAlone;
  Controls[26].OnClick = LogoutButtonClicked;
}


//=============================================================================
// ShowPanel
//
// Start or kill the timer that checks whether the player has logged out.
//=============================================================================

function ShowPanel(bool bShow)
{
  Super.ShowPanel(bShow);
  
  if ( bShow && PlayerOwner().Level.NetMode != NM_StandAlone )
    SetTimer(0.1, True);
  else
    KillTimer();
}


//=============================================================================
// delegate OnAdminLoggedOut
//
// Called when an admin logged out.
//=============================================================================

delegate OnAdminLoggedOut();


//=============================================================================
// Timer
//
// Try to log in with the specified data.
// Calls OnAdminLoggedIn delegate when successful.
//=============================================================================

function Timer()
{
  if ( !PlayerOwner().PlayerReplicationInfo.bAdmin ) {
    OnAdminLoggedOut();
    KillTimer();
  }
}


//=============================================================================
// ApplyButtonClicked
//
// Sends the new settings to the server.
//=============================================================================

function bool ApplyButtonClicked(GUIComponent Sender)
{
  local int i;
  local TournamentUtil.TServerSettings NewSettings;
  
  for (i = 1; i < Controls.Length; i++)
    Controls[i].MenuStateChange(MSAT_Disabled);
  
  NewSettings = ReceivedServerSettings;
  NewSettings.SpawnProtectionTime   = TGUILabeledSlider(Controls[0]).GetValue();
  NewSettings.WarmupTime            = TGUILabeledSlider(Controls[1]).GetValue();
  NewSettings.MinPlayerNameLength   = TGUILabeledSlider(Controls[8]).GetValue();
  NewSettings.SkinGlow              = TGUILabeledSlider(Controls[9]).GetValue();
  class'TournamentUtil'.static.CompressBools(NewSettings.WeaponConfigBools, 3,
      moCheckbox(Controls[10]).IsChecked(),
      moCheckbox(Controls[11]).IsChecked(),
      moCheckbox(Controls[12]).IsChecked()
  );
  class'TournamentUtil'.static.CompressBools(NewSettings.TournamentConfigBools, 7,
      moCheckbox(Controls[7]).IsChecked(),
      moCheckbox(Controls[13]).IsChecked(),
      moCheckbox(Controls[14]).IsChecked(),
      moCheckbox(Controls[15]).IsChecked(),
      moCheckbox(Controls[16]).IsChecked(),
      moCheckbox(Controls[17]).IsChecked(),
      moCheckbox(Controls[18]).IsChecked()
  );
  class'TournamentUtil'.static.CompressBools(NewSettings.AdrenalineWarmupBools, 8,
      moCheckbox(Controls[3]).IsChecked(),
      moCheckbox(Controls[4]).IsChecked(),
      moCheckbox(Controls[5]).IsChecked(),
      moCheckbox(Controls[6]).IsChecked(),
      moCheckbox(Controls[20]).IsChecked(),
      moCheckbox(Controls[21]).IsChecked(),
      moCheckbox(Controls[22]).IsChecked(),
      moCheckbox(Controls[23]).IsChecked()
  );
  
  TournamentOwner.ServerSetServerSettings(NewSettings);
  return True;
}


//=============================================================================
// AdminButtonClicked
//
// Sends the new settings to the server.
//=============================================================================

function bool AdminButtonClicked(GUIComponent Sender)
{
  //Controller.OpenMenu("Tournament.TGUIAdminMenu");
  Controller.OpenMenu("xInterface.UT2AdminMenu");
  
  return True;
}


//=============================================================================
// OnCloseDialog
//
// Called when the config dialog is closed. Unsets the received delegate.
//=============================================================================

function OnCloseDialog(bool bCanceled)
{
  if ( TournamentOwner != None )
    TournamentOwner.OnClientReceivedSettings = None;
}


//=============================================================================
// ReceivedSettings
//
// Called when a server property value is received.
//=============================================================================

function ReceivedSettings(TournamentUtil.TServerSettings Settings)
{
  local int i;
  
  ReceivedServerSettings = Settings;
  TGUILabeledSlider(Controls[0]).SetValue(Settings.SpawnProtectionTime);
  TGUILabeledSlider(Controls[1]).SetValue(Settings.WarmupTime);
  moCheckbox(Controls[3]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 0));
  moCheckbox(Controls[4]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 1));
  moCheckbox(Controls[5]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 2));
  moCheckbox(Controls[6]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 3));
  
  moCheckbox(Controls[7]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 0));
  TGUILabeledSlider(Controls[8]).SetValue(Settings.MinPlayerNameLength);
  
  TGUILabeledSlider(Controls[9]).SetValue(Settings.SkinGlow);
  
  moCheckbox(Controls[10]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 0));
  moCheckbox(Controls[11]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 1));
  moCheckbox(Controls[12]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.WeaponConfigBools, 2));
  
  moCheckbox(Controls[13]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 1));
  moCheckbox(Controls[14]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 2));
  moCheckbox(Controls[15]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 3));
  moCheckbox(Controls[16]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 4));
  moCheckbox(Controls[17]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 5));
  moCheckbox(Controls[18]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.TournamentConfigBools, 6));
  
  moCheckbox(Controls[20]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 4));
  moCheckbox(Controls[21]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 5));
  moCheckbox(Controls[22]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 6));
  moCheckbox(Controls[23]).Checked(class'TournamentUtil'.static.GetCompressedBool(Settings.AdrenalineWarmupBools, 7));
  
  for (i = 1; i < Controls.Length; i++)
    if ( i != 15 )
      Controls[i].MenuStateChange(MSAT_Blurry);
}


//=============================================================================
// LogoutButtonClicked
//
// Logout button was clicked.
//=============================================================================

function bool LogoutButtonClicked(GUIComponent Sender)
{
  PlayerOwner().ConsoleCommand("ADMINLOGOUT");
  return True;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUIButton Name=AdminButton
    WinTop=0.9
    WinLeft=0.645
    WinHeight=0.1
    WinWidth=0.175
    Caption="Advanced..."
    Hint="Opens the advanced admin menu."
  End Object
  Controls(24)=GUIButton'AdminButton'
  
  Begin Object Class=GUIButton Name=ApplyButton
    WinTop=0.9
    WinLeft=0.825
    WinHeight=0.1
    WinWidth=0.175
    Caption="Apply"
    Hint="Apply the settings. (requires map restart)"
  End Object
  Controls(25)=GUIButton'ApplyButton'
  
  Begin Object Class=GUIButton Name=AdminLogoutButton
    WinTop=0.9
    WinLeft=0.465
    WinHeight=0.1
    WinWidth=0.175
    Caption="Logout"
    Hint="Give up admin privileges."
  End Object
  Controls(26)=GUIButton'AdminButton'
}