//=============================================================================
// TGUISettingsPanelAdminLogin
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for admin login in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelAdminLogin extends TGUISettingsPanel;


//=============================================================================
// InitPanel
//
// Try to log in with the specified data.
// Calls OnAdminLoggedIn delegate when successful.
//=============================================================================

function InitPanel()
{
  Super.InitPanel();
  
  TGUITextEdit(Controls[1]).OnEnterPressed = InternalOnEnterPressed;
  TGUITextEdit(Controls[2]).OnEnterPressed = InternalOnEnterPressed;
  Controls[3].OnClick = LoginButtonClicked;
}


//=============================================================================
// ShowPanel
//
// Start or kill the timer that checks whether the player is logged in.
//=============================================================================

function ShowPanel(bool bShow)
{
  Super.ShowPanel(bShow);
  
  if ( bShow )
    SetTimer(0.1, True);
  else
    KillTimer();
}


//=============================================================================
// delegate OnAdminLoggedIn
//
// Called when an admin successfully logged in.
//=============================================================================

delegate OnAdminLoggedIn();


//=============================================================================
// Timer
//
// Try to log in with the specified data.
// Calls OnAdminLoggedIn delegate when successful.
//=============================================================================

function Timer()
{
  if ( PlayerOwner().PlayerReplicationInfo.bAdmin ) {
    OnAdminLoggedIn();
    KillTimer();
  }
}


//=============================================================================
// TryLogin
//
// Try to log in with the specified data.
// Calls OnAdminLoggedIn delegate when successful.
//=============================================================================

function TryLogin(string LoginString)
{
  PlayerOwner().AdminLogin(LoginString);
}


//=============================================================================
// InternalOnEnterPressed
//
// Called when the user pressed Enter in one of the textboxes.
//=============================================================================

function InternalOnEnterPressed(GUIComponent Sender)
{
  if ( Sender == Controls[1] )
    NextControl(Controls[1]);
  else if ( Sender == Controls[2] ) {
    PlayerOwner().PlayOwnedSound(Controller.ClickSound, SLOT_Interface, 1.0);
    LoginButtonClicked(Sender);
  }
}


//=============================================================================
// LoginButtonClicked
//
// Login button was clicked.
//=============================================================================

function bool LoginButtonClicked(GUIComponent Sender)
{
  if ( TGUITextEdit(Controls[2]).GetText() == "" )
    return False;
  
  if ( TGUITextEdit(Controls[1]).GetText() != "" )
    TryLogin(TGUITextEdit(Controls[1]).GetText() @ TGUITextEdit(Controls[2]).GetText());
  else
    TryLogin(TGUITextEdit(Controls[2]).GetText());
  return True;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=GUILabel Name=lblLoginInfo
    Caption="Please enter your login name and password.|Depending on the type of server administration used you have to enter either only the admin password or your username and password seperated by a space character."
    WinTop=0.05
    WinLeft=0.1
    WinWidth=0.8
    WinHeight=0.35
    TextAlign=TXTA_Center
    bAcceptsInput=False
    bNeverFocus=True
    bMultiLine=True
  End Object
  Controls(0)=GUILabel'lblLoginInfo'
  
  Begin Object Class=TGUITextEdit Name=txtUsername
    Caption="Username"
    Hint="Enter your login name."
    WinTop=0.4
    WinLeft=0.2
    WinWidth=0.6
    WinHeight=0.1
  End Object
  Controls(1)=TGUITextEdit'txtUsername'
  
  Begin Object Class=TGUITextEdit Name=txtPassword
    Caption="Password"
    Hint="Enter your login password."
    bMasked=True
    WinTop=0.55
    WinLeft=0.2
    WinWidth=0.6
    WinHeight=0.1
  End Object
  Controls(2)=TGUITextEdit'txtPassword'
  
  Begin Object Class=GUIButton Name=btnLogin
    Caption="Login"
    Hint="Log in with these parameters."
    WinTop=0.7
    WinLeft=0.35
    WinWidth=0.3
    WinHeight=0.08
  End Object
  Controls(3)=GUIButton'btnLogin'
}