//=============================================================================
// TGUISettingsPanelCrosshair
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for crosshair options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelCrosshair extends TGUISettingsPanel;


//=============================================================================
// InitPanel
//
// Assign OnChange delegates.
//=============================================================================

function InitPanel()
{
  local int i;
  
  TGUILabeledSlider(Controls[0]).OnDrawCaption = SelectorSliderCaption;
  TGUILabeledSlider(Controls[5]).OnDrawCaption = RotationSliderCaption;
  TGUILabeledSlider(Controls[6]).OnDrawCaption = SizeSliderCaption;
  
  Super.InitPanel();
  
  for (i = 0; i < Controls.Length; i++)
    LoadINISettings(Controls[i]);
  
  for (i = 0; i < Controls.Length; i++)
    Controls[i].OnChange = InternalOnChange;
}


//=============================================================================
// SelectorSliderCaption
//
// Returns the caption value for the crosshair selector slider.
//=============================================================================

function string SelectorSliderCaption(TGUILabeledSlider Sender)
{
  UpdateCrosshairPreview();
  return string(int(Sender.GetValue()));
}


//=============================================================================
// RotationSliderCaption
//
// Returns the caption value for the crosshair rotation slider.
//=============================================================================

function string RotationSliderCaption(TGUILabeledSlider Sender)
{
  UpdateCrosshairPreview();
  return "("$int(Sender.GetValue() * 90)$")";
}


//=============================================================================
// SizeSliderCaption
//
// Returns the caption value for the crosshair size slider.
//=============================================================================

function string SizeSliderCaption(TGUILabeledSlider Sender)
{
  UpdateCrosshairPreview();
  return "("$int(Sender.GetValue())$"%)";
}


//=============================================================================
// LoadINISettings
//
// Load INI values.
//=============================================================================

function LoadINISettings(GUIComponent Sender)
{
  Switch (Sender) {
  Case Controls[0]:
    TGUILabeledSlider(Sender).NumericRange(0, Class'TournamentHUD'.Default.Crosshairs.Length - 1);
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairStyle);
    break;
  Case Controls[1]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairColor.A);
    break;
  Case Controls[2]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairColor.R);
    break;
  Case Controls[3]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairColor.G);
    break;
  Case Controls[4]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairColor.B);
    break;
  Case Controls[5]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.CrosshairRotation % 4);
    break;
  Case Controls[6]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentCrosshairScale * 100);
    break;
  Case Controls[7]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bCrosshairShow);
    break;
  }
}


//=============================================================================
// InternalOnChange
//
// Save INI values.
//=============================================================================

function InternalOnChange(GUIComponent Sender)
{
  local TournamentHUD LocalHUD;

  LocalHUD = TournamentHUD(PlayerOwner().MyHud);
  if ( LocalHUD != None ) {
    LocalHUD.TournamentCrosshairStyle   = TGUILabeledSlider(Controls[0]).GetValue();
    LocalHUD.TournamentCrosshairColor.A = TGUILabeledSlider(Controls[1]).GetValue();
    LocalHUD.TournamentCrosshairColor.R = TGUILabeledSlider(Controls[2]).GetValue();
    LocalHUD.TournamentCrosshairColor.G = TGUILabeledSlider(Controls[3]).GetValue();
    LocalHUD.TournamentCrosshairColor.B = TGUILabeledSlider(Controls[4]).GetValue();
    LocalHUD.CrosshairRotation          = TGUILabeledSlider(Controls[5]).GetValue();
    LocalHUD.TournamentCrosshairScale   = TGUILabeledSlider(Controls[6]).GetValue() * 0.01;
    LocalHUD.bCrosshairShow             = moCheckBox(Controls[7]).IsChecked();
    LocalHUD.SaveConfig();
  }
  else {
    class'TournamentHUD'.default.TournamentCrosshairStyle   = TGUILabeledSlider(Controls[0]).GetValue();
    class'TournamentHUD'.default.TournamentCrosshairColor.A = TGUILabeledSlider(Controls[1]).GetValue();
    class'TournamentHUD'.default.TournamentCrosshairColor.R = TGUILabeledSlider(Controls[2]).GetValue();
    class'TournamentHUD'.default.TournamentCrosshairColor.G = TGUILabeledSlider(Controls[3]).GetValue();
    class'TournamentHUD'.default.TournamentCrosshairColor.B = TGUILabeledSlider(Controls[4]).GetValue();
    class'TournamentHUD'.default.CrosshairRotation          = TGUILabeledSlider(Controls[5]).GetValue();
    class'TournamentHUD'.default.TournamentCrosshairScale   = TGUILabeledSlider(Controls[6]).GetValue() * 0.01;
    class'TournamentHUD'.default.bCrosshairShow             = moCheckBox(Controls[7]).IsChecked();
    class'TournamentHUD'.static.StaticSaveConfig();
  }
  UpdateCrosshairPreview();
}


//=============================================================================
// UpdateCrosshairPreview
//
// Updates the TexRotator and GUIImage used for previewing the crosshair.
//=============================================================================

function UpdateCrosshairPreview()
{
  local GUIImage PreviewImage;
  local TexRotator PreviewRotator;
  local array<HudBase.SpriteWidget> Crosshairs;
  local int CrosshairStyle;
  local float CrosshairScale;
  local GUIComponent TopMostOwner;
  
  // find top-most owner
  for (TopMostOwner = Self; TopMostOwner.MenuOwner != None; TopMostOwner = TopMostOwner.MenuOwner);
  
  PreviewImage = GUIImage(Controls[9]);
  PreviewRotator = TexRotator(PreviewImage.Image);
  if ( TournamentHUD(PlayerOwner().MyHud) != None )
    Crosshairs = TournamentHUD(PlayerOwner().MyHud).Crosshairs;
  else
    Crosshairs = class'TournamentHUD'.default.Crosshairs;
  CrosshairStyle = Clamp(TGUILabeledSlider(Controls[0]).GetValue(), 0, Crosshairs.Length - 1);
  CrosshairScale = Crosshairs[CrosshairStyle].TextureScale * TGUILabeledSlider(Controls[6]).GetValue() * 0.01
                 * TopMostOwner.ActualWidth() / TopMostOwner.WinWidth / 640.0;
  
  PreviewRotator.Material = Crosshairs[CrosshairStyle].WidgetTexture;
  PreviewRotator.Rotation.Yaw = (TGUILabeledSlider(Controls[5]).GetValue() % 4) * 16384;
  
  if ( TournamentHUD(PlayerOwner().MyHud) != None )
    PreviewImage.ImageColor = TournamentHUD(PlayerOwner().MyHud).TournamentCrosshairColor;
  else
    PreviewImage.ImageColor = class'TournamentHUD'.default.TournamentCrosshairColor;
  PreviewImage.X1 = Crosshairs[CrosshairStyle].TextureCoords.X1;
  PreviewImage.Y1 = Crosshairs[CrosshairStyle].TextureCoords.Y1;
  PreviewImage.X2 = Crosshairs[CrosshairStyle].TextureCoords.X2;
  PreviewImage.Y2 = Crosshairs[CrosshairStyle].TextureCoords.Y2;
  if ( PreviewImage.X1 == PreviewImage.X2 || PreviewImage.Y1 == PreviewImage.Y2 ) {
    PreviewImage.X1 = 0;
    PreviewImage.Y1 = 0;
    PreviewImage.X2 = PreviewRotator.Material.MaterialUSize();
    PreviewImage.Y2 = PreviewRotator.Material.MaterialVSize();
  }
  PreviewImage.WinWidth = Abs(PreviewImage.X1 - PreviewImage.X2) * CrosshairScale;
  PreviewImage.WinHeight = Abs(PreviewImage.Y1 - PreviewImage.Y2) * CrosshairScale;
  PreviewImage.WinLeft = (TopMostOwner.WinLeft + TopMostOwner.WinWidth * 0.74) * TopMostOwner.ActualWidth()
                          / TopMostOwner.WinWidth - PreviewImage.WinWidth * 0.5;
  PreviewImage.WinTop = (TopMostOwner.WinTop + TopMostOwner.WinHeight * 0.755) * TopMostOwner.ActualHeight()
                          / TopMostOwner.WinHeight - PreviewImage.WinHeight * 0.5;
  PreviewImage.bScaleToParent = False;
  PreviewImage.bBoundToParent = False;
  
  PreviewRotator.UOffset = PreviewImage.X1 + (PreviewImage.X2 - PreviewImage.X1) / 2;
  PreviewRotator.VOffset = PreviewImage.Y1 + (PreviewImage.Y2 - PreviewImage.Y1) / 2;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  Begin Object Class=TGUILabeledSlider Name=CrosshairSelector
    Caption="Crosshair Style:"
    WinWidth=0.95
    WinLeft=0.025
    WinTop=0.0
    WinHeight=0.1
    MinValue=0
    MaxValue=17
    bIntegerOnly=True
    bVerticalLayout=False
    CaptionWidth=0.3
    Hint="Select a crosshair style."
  End Object
  Controls(0)=TGUILabeledSlider'CrosshairSelector'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairOpacity
    Caption="Crosshair Opacity"
    WinWidth=0.45
    WinLeft=0.025
    WinTop=0.15
    WinHeight=0.18
    MinValue=0
    MaxValue=255
    bIntegerOnly=True
    Hint="Changes the opacity level of the crosshair."
  End Object
  Controls(1)=TGUILabeledSlider'CrosshairOpacity'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairColorR
    Caption="Crosshair Color: Red"
    WinWidth=0.45
    WinLeft=0.025
    WinTop=0.35
    WinHeight=0.18
    MinValue=0
    MaxValue=255
    bIntegerOnly=True
    Hint="Changes the red component of the crosshair color."
  End Object
  Controls(2)=TGUILabeledSlider'CrosshairColorR'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairColorG
    Caption="Crosshair Color: Green"
    WinWidth=0.45
    WinLeft=0.025
    WinTop=0.55
    WinHeight=0.18
    MinValue=0
    MaxValue=255
    bIntegerOnly=True
    Hint="Changes the green component of the crosshair color."
  End Object
  Controls(3)=TGUILabeledSlider'CrosshairColorG'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairColorB
    Caption="Crosshair Color: Blue"
    WinWidth=0.45
    WinLeft=0.025
    WinTop=0.75
    MinValue=0
    WinHeight=0.18
    MaxValue=255
    bIntegerOnly=True
    Hint="Changes the blue component of the crosshair color."
  End Object
  Controls(4)=TGUILabeledSlider'CrosshairColorB'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairRotation
    Caption="Crosshair Rotation"
    WinWidth=0.45
    WinLeft=0.525
    WinTop=0.15
    WinHeight=0.18
    MinValue=0
    MaxValue=3
    bIntegerOnly=True
    Hint="Changes the rotation of the crosshair."
  End Object
  Controls(5)=TGUILabeledSlider'CrosshairRotation'
  
  Begin Object Class=TGUILabeledSlider Name=CrosshairSize
    Caption="Crosshair Size"
    WinWidth=0.45
    WinLeft=0.525
    WinTop=0.35
    WinHeight=0.18
    MinValue=40
    MaxValue=250
    bIntegerOnly=True
    Hint="Changes the size of the crosshair."
  End Object
  Controls(6)=TGUILabeledSlider'CrosshairSize'
  
  Begin Object Class=moCheckBox Name=CrosshairVisible
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.56
    Caption="Show Crosshair"
    Hint="Display the crosshair."
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(7)=moCheckbox'CrosshairVisible'
  
  Begin Object class=GUIImage Name=CrosshairPreviewFrame
    WinWidth=0.19
    WinHeight=0.35
    WinLeft=0.65
    WinTop=0.65
    Image=Material'InterfaceContent.Menu.BorderBoxA'
    ImageColor=(R=255,G=255,B=255,A=255);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(8)=GUIImage'CrosshairPreviewFrame'
  
  Begin Object Class=TexRotator Name=PreviewCrosshairRotator
    Outer=Tournament.GUI
  End Object
  
  Begin Object Class=GUIImage Name=CrosshairPreview
    Image=TexRotator'PreviewCrosshairRotator'
    ImageStyle=ISTY_Stretched
  End Object
  Controls(9)=GUIImage'CrosshairPreview'
}