//=============================================================================
// TGUISettingsPanelGame
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for client-side game options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelGame extends TGUISettingsPanel;


//=============================================================================
// Localization
//=============================================================================

var localized array<string> HitSoundCaptions;
var localized array<string> ColorNames;


//=============================================================================
// Variables
//=============================================================================

var string DefaultModels[12];


//=============================================================================
// InitPanel
//
// Assign OnChange delegates.
//=============================================================================

function InitPanel()
{
  local int i;
  
  TGUILabeledSlider(Controls[0]).OnDrawCaption = DrawHitSoundCaption;
  
  Super.InitPanel();
  
  for (i = 0; i < ColorNames.Length; i++) {
    moComboBox(Controls[3]).AddItem(ColorNames[i]);
    moComboBox(Controls[4]).AddItem(ColorNames[i]);
    moComboBox(Controls[5]).AddItem(ColorNames[i]);
    moComboBox(Controls[6]).AddItem(ColorNames[i]);
  }
  
  DefaultModels[0] = Localize("General", "None", "Core");
  for (i = 0; i < ArrayCount(DefaultModels); i++) {
    moComboBox(Controls[7]).AddItem(DefaultModels[i]);
    moComboBox(Controls[8]).AddItem(DefaultModels[i]);
  }
  
  for (i = 0; i < Controls.Length; i++)
    LoadINISettings(Controls[i]);
  
  for (i = 0; i < Controls.Length; i++)
    Controls[i].OnChange = InternalOnChange;
}


//=============================================================================
// DrawHitSoundCaption
//
// Returns the slider caption for the hit sound slider.
//=============================================================================

function string DrawHitSoundCaption(TGUILabeledSlider Sender)
{
  return HitSoundCaptions[Sender.GetValue()];
}


//=============================================================================
// LoadINISettings
//
// Load INI values.
//=============================================================================

function LoadINISettings(GUIComponent Sender)
{
  Switch (Sender) {
    Case Controls[0]:
      TGUILabeledSlider(Sender).NumericRange(0, class'HitSounds'.default.HitSoundEnemy.Length - 1);
      TGUILabeledSlider(Sender).SetValue(Class'TournamentClientSettings'.Default.HitSound);
      break;
    Case Controls[1]:
      TGUILabeledSlider(Sender).SetValue(Class'TournamentClientSettings'.Default.HitSoundVolume * 100);
      break;
    Case Controls[2]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bVariableHitSounds);
      break;
    Case Controls[3]:
      moComboBox(Sender).SetText(ColorNames[Class'TournamentClientSettings'.Default.EnemySkinColor]);
      break;
    Case Controls[4]:
      moComboBox(Sender).SetText(ColorNames[Class'TournamentClientSettings'.Default.EnemyEffectsColor]);
      break;
    Case Controls[5]:
      moComboBox(Sender).SetText(ColorNames[Class'TournamentClientSettings'.Default.TeamSkinColor]);
      break;
    Case Controls[6]:
      moComboBox(Sender).SetText(ColorNames[Class'TournamentClientSettings'.Default.TeamEffectsColor]);
      break;
    Case Controls[7]:
      if ( Class'TournamentPawn'.Default.EnemyCharacterName ~= "None" )
        moComboBox(Sender).SetText(DefaultModels[0]);
      else
        moComboBox(Sender).SetText(Class'TournamentPawn'.Default.EnemyCharacterName);
      break;
    Case Controls[8]:
      if ( Class'TournamentPawn'.Default.TeamCharacterName ~= "None" )
        moComboBox(Sender).SetText(DefaultModels[0]);
      else
        moComboBox(Sender).SetText(Class'TournamentPawn'.Default.TeamCharacterName);
      break;
    Case Controls[9]:
      moCheckBox(Sender).Checked(Class'TournamentPawn'.Default.bNoCustomModels);
      break;
    Case Controls[10]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bUsePlayerLights);
      break;
    Case Controls[11]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bInstantDeRes);
      break;
    Case Controls[12]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bTeamDeRes);
      break;
    Case Controls[13]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bRisingDeRes);
      break;
    Case Controls[14]:
      moCheckBox(Sender).Checked(Class'TournamentPawn'.Default.bPlayOwnFootsteps);
      break;
    Case Controls[15]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bAutoScreenshot);
      break;
    Case Controls[16]:
      moCheckBox(Sender).Checked(Class'TournamentClientSettings'.Default.bHideSpectatorBeacons);
      break;
    Case Controls[17]:
      moCheckBox(Sender).Checked(Class'DMMutator'.Default.bBrightSkins);
      break;
  }
}


//=============================================================================
// InternalOnChange
//
// Save INI values. This is simulated so PlaySound is executed only locally.
//=============================================================================

simulated function InternalOnChange(GUIComponent Sender)
{
  local int i;
  
  if ( Sender == Controls[0] || Sender == Controls[1] ) {
    PlayerOwner().PlaySound(class'HitSounds'.default.HitSoundEnemy[TGUILabeledSlider(Controls[0]).GetValue()],,
        TGUILabeledSlider(Controls[1]).GetValue() * 0.01,,,
        class'HitSounds'.default.HitSoundDefaultPitch[TGUILabeledSlider(Controls[0]).GetValue()], False);
  }
  
  for (i = 0; i < ColorNames.Length; i++) {
    if ( moComboBox(Controls[3]).GetText() ~= ColorNames[i] )
      class'TournamentClientSettings'.default.EnemySkinColor = i;
    if ( moComboBox(Controls[4]).GetText() ~= ColorNames[i] )
      class'TournamentClientSettings'.default.EnemyEffectsColor = i;
    if ( moComboBox(Controls[5]).GetText() ~= ColorNames[i] )
      class'TournamentClientSettings'.default.TeamSkinColor = i;
    if ( moComboBox(Controls[6]).GetText() ~= ColorNames[i] )
      class'TournamentClientSettings'.default.TeamEffectsColor = i;
  }
  
  class'TournamentClientSettings'.default.HitSound              = TGUILabeledSlider(Controls[0]).GetValue();
  class'TournamentClientSettings'.default.HitSoundVolume        = TGUILabeledSlider(Controls[1]).GetValue() * 0.01;
  class'TournamentClientSettings'.default.bVariableHitSounds    = moCheckBox(Controls[2]).IsChecked();
  class'TournamentClientSettings'.default.bUsePlayerLights      = moCheckBox(Controls[10]).IsChecked();
  class'TournamentClientSettings'.default.bInstantDeRes         = moCheckBox(Controls[11]).IsChecked();
  class'TournamentClientSettings'.default.bTeamDeRes            = moCheckBox(Controls[12]).IsChecked();
  class'TournamentClientSettings'.default.bRisingDeRes          = moCheckBox(Controls[13]).IsChecked();
  class'TournamentClientSettings'.default.bAutoScreenshot       = moCheckBox(Controls[15]).IsChecked();
  class'TournamentClientSettings'.default.bHideSpectatorBeacons = moCheckBox(Controls[16]).IsChecked();
  class'TournamentClientSettings'.static.StaticSaveConfig();
  
  if ( moComboBox(Controls[7]).GetText() == DefaultModels[0] )
    class'TournamentPawn'.default.EnemyCharacterName = "None";
  else
    class'TournamentPawn'.default.EnemyCharacterName = moComboBox(Controls[7]).GetText();
  
  if ( moComboBox(Controls[8]).GetText() == DefaultModels[0] )
    class'TournamentPawn'.default.TeamCharacterName = "None";
  else
    class'TournamentPawn'.default.TeamCharacterName  = moComboBox(Controls[8]).GetText();
  
  if ( UnrealPawn(PlayerOwner().Pawn) != None )
    UnrealPawn(PlayerOwner().Pawn).bPlayOwnFootsteps = moCheckBox(Controls[14]).IsChecked();
  PlayerOwner().bHideSpectatorBeacons = moCheckBox(Controls[16]).IsChecked();
  
  class'TournamentPawn'.default.bNoCustomModels   = moCheckBox(Controls[10]).IsChecked();
  class'TournamentPawn'.default.bPlayOwnFootsteps = moCheckBox(Controls[14]).IsChecked();
  class'TournamentPawn'.static.StaticSaveConfig();
  
  class'DMMutator'.default.bBrightSkins = moCheckBox(Controls[17]).IsChecked();
  class'DMMutator'.static.StaticSaveConfig();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  DefaultModels(1)  = "Gorge"
  DefaultModels(2)  = "Rylisa"
  DefaultModels(3)  = "Malcolm"
  DefaultModels(4)  = "Sapphire"
  DefaultModels(5)  = "Roc"
  DefaultModels(6)  = "Diva"
  DefaultModels(7)  = "Damarus"
  DefaultModels(8)  = "Mokara"
  DefaultModels(9)  = "Brutalis"
  DefaultModels(10) = "Lilith"
  DefaultModels(11) = "Rapier"
  
  HitSoundCaptions(0) = "Tournament (1)"
  HitSoundCaptions(1) = "Tournament (2)"
  HitSoundCaptions(2) = "TTM-like"
  HitSoundCaptions(3) = "Quake3-like"
  HitSoundCaptions(4) = "UTPure-like"
  
  ColorNames = ("Default","Red","Blue","Green","Gold","Cyan","Magenta","White")
  
  Begin Object Class=TGUILabeledSlider Name=HitSoundsStyle
    Caption="HitSound Style"
    WinWidth=0.4
    WinLeft=0.05
    WinTop=0.0
    WinHeight=0.18
    bIntegerOnly=True
    MinValue=0
    MaxValue=3
    Hint="Select the desired type of hitsounds."
  End Object
  Controls(0)=TGUILabeledSlider'HitSoundsStyle'

  Begin Object Class=TGUILabeledSlider Name=HitSoundsVolume
    Caption="HitSound Volume"
    WinWidth=0.4
    WinLeft=0.05
    WinTop=0.2
    WinHeight=0.18
    bIntegerOnly=True
    MinValue=0
    MaxValue=100
    Hint="Select the desired playback volume for hitsounds."
  End Object
  Controls(1)=TGUILabeledSlider'HitSoundsVolume'
  
  Begin Object Class=moCheckBox Name=VariableHitSounds
    WinWidth=0.45
    WinHeight=0.07
    WinLeft=0.05
    WinTop=0.4
    Caption="Variable HitSound Pitch"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Less damage results in a higher hit sound pitch."
  End Object
  Controls(2)=moCheckbox'VariableHitSounds'
  
  Begin Object Class=moComboBox Name=EnemySkinColor
    WinWidth=0.3
    WinHeight=0.1
    WinLeft=0.01
    WinTop=0.57
    Caption="Enemy"
    Hint="Select your enemy's skin color."
    CaptionWidth=0.333333
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(3)=moComboBox'EnemySkinColor'
  
  Begin Object Class=moComboBox Name=EnemyEffectColor
    WinWidth=0.2
    WinHeight=0.1
    WinLeft=0.31
    WinTop=0.57
    Caption=""
    Hint="Select your enemy's effect color."
    CaptionWidth=0.0
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(4)=moComboBox'EnemyEffectColor'
  
  Begin Object Class=moComboBox Name=TeamSkinColor
    WinWidth=0.3
    WinHeight=0.1
    WinLeft=0.01
    WinTop=0.68
    Caption="Team"
    Hint="Select your team's skin color."
    CaptionWidth=0.333333
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(5)=moComboBox'TeamSkinColor'
  
  Begin Object Class=moComboBox Name=TeamEffectColor
    WinWidth=0.2
    WinHeight=0.1
    WinLeft=0.31
    WinTop=0.68
    Caption=""
    Hint="Select your team's effect color."
    CaptionWidth=0.0
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(6)=moComboBox'TeamEffectColor'
  
  Begin Object Class=moComboBox Name=EnemyModel
    WinWidth=0.5
    WinHeight=0.1
    WinLeft=0.01
    WinTop=0.79
    Caption="Enemy Model"
    Hint="Select a model for enemy players."
    CaptionWidth=0.45
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(7)=moComboBox'EnemyModel'
  
  Begin Object Class=moComboBox Name=TeamModel
    WinWidth=0.5
    WinHeight=0.1
    WinLeft=0.01
    WinTop=0.9
    Caption="Team Model"
    Hint="Select a model for your team's players."
    CaptionWidth=0.45
    bReadOnly=True
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
  End Object
  Controls(8)=moComboBox'TeamModel'
  
  Begin Object Class=moCheckBox Name=NoCustomModels
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.02
    Caption="Disable Custom Models"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Replaces custom models with the Gorge model."
  End Object
  Controls(9)=moCheckbox'NoCustomModels'
  
  Begin Object Class=moCheckBox Name=AlwaysUsePlayerLights
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.13
    Caption="Always Use Player Lights"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Shoulder lights with bright skins and in non-team games."
  End Object
  Controls(10)=moCheckbox'AlwaysUsePlayerLights'
  
  Begin Object Class=moCheckBox Name=InstantDeRes
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.24
    Caption="Instant DeRes"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Killed players should de-res immediately."
  End Object
  Controls(11)=moCheckbox'InstantDeRes'
  
  Begin Object Class=moCheckBox Name=TeamDeRes
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.35
    Caption="Team-Colored DeRes"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="In team games the DeRes effect should be team-colored."
  End Object
  Controls(12)=moCheckbox'TeamDeRes'
  
  Begin Object Class=moCheckBox Name=RisingDeRes
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.46
    Caption="Rising DeRes"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Lets the body float upwards during the DeRes effect."
  End Object
  Controls(13)=moCheckbox'RisingDeRes'
  
  Begin Object Class=moCheckBox Name=PlayOwnFootsteps
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.57
    Caption="Play Own Footsteps"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Whether you can hear your own footsteps."
  End Object
  Controls(14)=moCheckbox'PlayOwnFootsteps'
  
  Begin Object Class=moCheckBox Name=AutoScreenshot
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.68
    Caption="End-Game Screenshot"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Take a screenshot of the scoreboard when the game ends."
  End Object
  Controls(15)=moCheckbox'AutoScreenshot'
  
  Begin Object Class=moCheckBox Name=HideSpectatorBeacons
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.79
    Caption="No Spectator Beacons"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Hides the team beacons in spectator mode."
  End Object
  Controls(16)=moCheckbox'HideSpectatorBeacons'
  
  Begin Object Class=moCheckBox Name=BrightSkins
    WinWidth=0.4
    WinHeight=0.07
    WinLeft=0.55
    WinTop=0.9
    Caption="Spectator Bright Skins"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Enables Epic's bright player skins in spectator mode."
  End Object
  Controls(17)=moCheckbox'BrightSkins'
  
  Begin Object class=GUILabel Name=SkinsLabel
    Caption="Skins"
    TextALign=TXTA_Left
    TextColor=(R=255,G=255,B=255,A=255)
    WinWidth=0.2
    WinHeight=32.000000
    WinLeft=0.11
    WinTop=0.48
  End Object
  Controls(18)=GUILabel'SkinsLabel'
  
  Begin Object class=GUILabel Name=EffectsLabel
    Caption="Effects"
    TextALign=TXTA_Left
    TextColor=(R=255,G=255,B=255,A=255)
    WinWidth=0.2
    WinHeight=32.000000
    WinLeft=0.31
    WinTop=0.48
  End Object
  Controls(19)=GUILabel'EffectsLabel'
}