//=============================================================================
// TGUISettingsPanelHUD
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for HUD options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelHUD extends TGUISettingsPanel;


//=============================================================================
// Localization
//=============================================================================

var localized string ClockLocationNames[7];


//=============================================================================
// InitPanel
//
// Assign OnChange delegates.
//=============================================================================

function InitPanel()
{
  local int i;
  
  Super.InitPanel();
  
  for (i = 0; i < ArrayCount(ClockLocationNames); i++)
	  moComboBox(Controls[8]).AddItem(ClockLocationNames[i]);
	moComboBox(Controls[8]).ReadOnly(True);
	
  for (i = 0; i < Controls.Length; i++)
    LoadINISettings(Controls[i]);
  
  for (i = 0; i < Controls.Length; i++)
    Controls[i].OnChange = InternalOnChange;
}


//=============================================================================
// LoadINISettings
//
// Load INI values.
//=============================================================================

function LoadINISettings(GUIComponent Sender)
{
  Switch (Sender) {
  Case Controls[0]:
    moCheckBox(Sender).Checked(class'TournamentHUD'.Default.bHideHud);
    break;
  Case Controls[1]:
    moCheckBox(Sender).Checked(class'TournamentHUD'.Default.bShowWeaponInfo);
    break;
  Case Controls[2]:
    moCheckBox(Sender).Checked(class'TournamentHUD'.Default.bShowPersonalInfo);
    break;
  Case Controls[3]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bShowPoints);
    break;
  Case Controls[4]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bShowWeaponBar);
    break;
  Case Controls[5]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bShowPortrait);
    break;
  Case Controls[6]:
    moCheckBox(Sender).Checked(!Class'TournamentHUD'.Default.bNoEnemyNames);
    break;
  Case Controls[7]:
    moCheckBox(Sender).Checked(!Class'TournamentHUD'.Default.bHideWeaponName);
    break;
  Case Controls[8]:
    if ( class'TournamentHUD'.Default.bShowClock )
      moComboBox(Sender).SetText(ClockLocationNames[class'TournamentHUD'.Default.ClockLocation]);
    else
      moComboBox(Sender).SetText(ClockLocationNames[6]);
    break;
  Case Controls[9]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bShowTimeRemaining);
    break;
  Case Controls[10]:
    moCheckBox(Sender).Checked(Class'TournamentHUD'.Default.bShowLocalEvents);
    break;
  Case Controls[11]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentHudScale * 100);
    break;
  Case Controls[12]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.HudOpacity * 100 / 255);
    break;
  Case Controls[13]:
    TGUILabeledSlider(Sender).SetValue(Class'TournamentHUD'.Default.TournamentHudCanvasScale * 100);
    break;
  }
}


//=============================================================================
// InternalOnChange
//
// Save INI values.
//=============================================================================

function InternalOnChange(GUIComponent Sender)
{
  local TournamentHUD LocalHUD;

  LocalHUD = TournamentHUD(PlayerOwner().MyHud);
  
  LocalHUD.bShowClock = True;
  LocalHUD.bHideHud = moCheckBox(Controls[0]).IsChecked();
  LocalHUD.bShowWeaponInfo = moCheckBox(Controls[1]).IsChecked();
  LocalHUD.bShowPersonalInfo = moCheckBox(Controls[2]).IsChecked();
  LocalHUD.bShowPoints = moCheckBox(Controls[3]).IsChecked();
  LocalHUD.bShowWeaponBar = moCheckBox(Controls[4]).IsChecked();
  LocalHUD.bShowPortrait = moCheckBox(Controls[5]).IsChecked();
  LocalHUD.bNoEnemyNames = !moCheckBox(Controls[6]).IsChecked();
  LocalHUD.bHideWeaponName = !moCheckBox(Controls[7]).IsChecked();
  switch (moComboBox(Controls[8]).GetText()) {
    case ClockLocationNames[0]:
      LocalHUD.ClockLocation = UpperLeft;
      break;
    case ClockLocationNames[1]:
      LocalHUD.ClockLocation = UpperRight;
      break;
    case ClockLocationNames[2]:
      LocalHUD.ClockLocation = UpperMiddle;
      break;
    case ClockLocationNames[3]:
      LocalHUD.ClockLocation = LowerLeft;
      break;
    case ClockLocationNames[4]:
      LocalHUD.ClockLocation = LowerRight;
      break;
    case ClockLocationNames[5]:
      LocalHUD.ClockLocation = LowerMiddle;
      break;
    default:
      LocalHUD.bShowClock = False;
  }
  LocalHUD.bShowTimeRemaining = moCheckBox(Controls[9]).IsChecked();
  LocalHUD.bShowLocalEvents = moCheckBox(Controls[10]).IsChecked();
  LocalHUD.TournamentHudScale = TGUILabeledSlider(Controls[11]).GetValue() / 100;
  LocalHUD.HudOpacity = TGUILabeledSlider(Controls[12]).GetValue() * 255 / 100;
  LocalHUD.TournamentHudCanvasScale = TGUILabeledSlider(Controls[13]).GetValue() / 100;
  LocalHUD.SaveConfig();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
	ClockLocationNames(0) = "Upper Left"
	ClockLocationNames(1) = "Upper Right"
	ClockLocationNames(2) = "Upper Middle"
	ClockLocationNames(3) = "Lower Left"
	ClockLocationNames(4) = "Lower Right"
	ClockLocationNames(5) = "Lower Middle"
	ClockLocationNames(6) = "Hidden"
	
	Begin Object Class=moCheckBox Name=GameHudVisible
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.05
		WinTop=0.01
		Caption="Hide HUD"
		Hint="Toggle drawing of the HUD, crosshair and messages."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(0)=moCheckbox'GameHudVisible'
  
	Begin Object Class=moCheckBox Name=GameHudShowWeaponInfo
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.14
		Caption="Show Weapon Info"
		Hint="Display ammo counter and weapon charge bar."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(1)=moCheckbox'GameHudShowWeaponInfo'
  
	Begin Object Class=moCheckBox Name=GameHudShowPersonalInfo
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.27
		Caption="Show Personal Info"
		Hint="Display health and shield amount."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(2)=moCheckbox'GameHudShowPersonalInfo'
  
  Begin Object Class=moCheckBox Name=GameHudShowScore
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.40
		Caption="Show Score"
		Hint="Display personal and team scores."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(3)=moCheckbox'GameHudShowScore'
  
	Begin Object Class=moCheckBox Name=GameHudShowWeaponBar
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.53
		Caption="Show Weapon Bar"
		Hint="Display the weapon bar."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(4)=moCheckbox'GameHudShowWeaponBar'
  
	Begin Object Class=moCheckBox Name=GameHudShowPortraits
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.66
		Caption="Show Portraits"
		Hint="Display players' portraits."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(5)=moCheckbox'GameHudShowPortraits'
  
	Begin Object Class=moCheckBox Name=GameHudShowEnemyNames
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.79
		Caption="Show Enemy Names"
		Hint="Draw the name of the player you look at."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(6)=moCheckbox'GameHudShowEnemyNames'
  
	Begin Object Class=moCheckBox Name=GameHudShowWeaponNames
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.050000
		WinTop=0.92
		Caption="Show Weapon Names"
		Hint="Draw the name of the weapon you switch to."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(7)=moCheckbox'GameHudShowWeaponNames'
  
	Begin Object Class=moComboBox Name=GameHudShowClock
		WinWidth=0.4
		WinHeight=0.1
		WinLeft=0.55
		WinTop=0.0
		Caption="Clock"
		Hint="Select the position of the game clock."
		CaptionWidth=0.3
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(8)=moComboBox'GameHudShowClock'
  
	Begin Object Class=moCheckBox Name=GameHudShowRemainingTime
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.55
		WinTop=0.14
		Caption="Show Remaining Time"
		Hint="Display the remaining time instead of the elapsed time."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(9)=moCheckbox'GameHudShowRemainingTime'
	
	Begin Object Class=moCheckBox Name=GameClockShowLocalEvents
		WinWidth=0.4
		WinHeight=0.07
		WinLeft=0.550000
		WinTop=0.26
		Caption="Show Local Events"
		Hint="Display flak monkey, head hunter etc. in the clock."
		CaptionWidth=0.9
		bSquare=true
		bHeightFromComponent=False
		ComponentJustification=TXTA_Left
	End Object
	Controls(10)=moCheckbox'GameClockShowLocalEvents'
  
	Begin Object Class=TGUILabeledSlider Name=GameHudScale
		Caption="Hud Scaling"
		WinWidth=0.4
		WinLeft=0.55
		WinTop=0.4
		WinHeight=0.18
		MinValue=50
		MaxValue=100
		Hint="Changes the size of the individual HUD elements."
	End Object
	Controls(11)=TGUILabeledSlider'GameHudScale'
  
	Begin Object Class=TGUILabeledSlider Name=GameHudOpacity
		Caption="Hud Opacity"
		WinWidth=0.4
		WinLeft=0.55
		WinTop=0.6
		WinHeight=0.18
		MinValue=0
		MaxValue=100
		Hint="Changes the opacity level of the HUD."
	End Object
	Controls(12)=TGUILabeledSlider'GameHudOpacity'
  
	Begin Object Class=TGUILabeledSlider Name=GameHudCanvasScale
		Caption="Hud Size"
		WinWidth=0.4
		WinLeft=0.55
		WinTop=0.8
		WinHeight=0.18
		MinValue=50
		MaxValue=100
		Hint="Changes the size of the whole HUD."
	End Object
	Controls(13)=TGUILabeledSlider'GameHudCanvasScale'
}