//=============================================================================
// TGUISettingsPanelMessages
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for HUD message options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelMessages extends TGUISettingsPanel;


//=============================================================================
// InitPanel
//
// Assign OnChange delegates.
//=============================================================================

function InitPanel()
{
  local int i;
  
  Super.InitPanel();
  
  for (i = 0; i < Controls.Length; i++)
    LoadINISettings(Controls[i]);
  
  for (i = 0; i < Controls.Length; i++)
    Controls[i].OnChange = InternalOnChange;
}


//=============================================================================
// LoadINISettings
//
// Load INI values.
//=============================================================================

function LoadINISettings(GUIComponent Sender)
{
  Switch (Sender) {
  case Controls[0]:
    TGUILabeledSlider(Sender).SetValue(class'HUD'.default.ConsoleMessageCount);
    break;
  case Controls[1]:
    TGUILabeledSlider(Sender).SetValue(8 - class'HUD'.default.ConsoleFontSize);
    break;
  case Controls[2]:
    TGUILabeledSlider(Sender).SetValue(class'TournamentHUD'.default.DeathMessageCount);
    break;
  case Controls[3]:
    moCheckbox(Sender).Checked(class'TournamentHUD'.default.bSeparateDeathMessages);
    break;
  case Controls[4]:
    moCheckbox(Sender).Checked(class'TournamentHUD'.default.bShortDeathMessages);
    break;
  case Controls[5]:
    moCheckbox(Sender).Checked(class'TournamentHUD'.default.bGraphicalDeathMessages);
    break;
  }
}


//=============================================================================
// InternalOnChange
//
// Save INI values.
//=============================================================================

function InternalOnChange(GUIComponent Sender)
{
  local TournamentHUD LocalHUD;

  LocalHUD = TournamentHUD(PlayerOwner().MyHud);
  LocalHUD.ConsoleMessageCount     = TGUILabeledSlider(Controls[0]).GetValue();
  LocalHUD.ConsoleFontSize         = 8 - TGUILabeledSlider(Controls[1]).GetValue();
  LocalHUD.DeathMessageCount       = TGUILabeledSlider(Controls[2]).GetValue();
  LocalHUD.bSeparateDeathMessages  = moCheckbox(Controls[3]).IsChecked();
  LocalHUD.bShortDeathMessages     = moCheckbox(Controls[4]).IsChecked();
  LocalHUD.bGraphicalDeathMessages = moCheckbox(Controls[5]).IsChecked();
  LocalHUD.SaveConfig();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
	Begin Object class=TGUILabeledSlider Name=ChatMessageCount
		WinWidth=0.4
		WinHeight=0.18
		WinLeft=0.05
		WinTop=0.0
		Caption="Max. Chat Count"
		MinValue=0
		MaxValue=8
		bIntegerOnly=True
		Hint="Maximum number of chat lines visible at a time."
	End Object
	Controls(0)=TGUILabeledSlider'ChatMessageCount'

	Begin Object class=TGUILabeledSlider Name=ChatMessageScale
		WinWidth=0.4
		WinHeight=0.18
		WinLeft=0.05
		WinTop=0.2
		Caption="Chat Font Size"
		MinValue=0
		MaxValue=8
		bIntegerOnly=True
		Hint="Relative font size for chat and death messages."
	End Object
	Controls(1)=TGUILabeledSlider'ChatMessageScale'

	Begin Object class=TGUILabeledSlider Name=DeathMessageCount
		WinWidth=0.4
		WinHeight=0.18
		WinLeft=0.05
		WinTop=0.4
		Caption="Death Message Count"
		MinValue=0
		MaxValue=8
		bIntegerOnly=True
		Hint="Maximum number of death messages visible at a time."
	End Object
	Controls(2)=TGUILabeledSlider'DeathMessageCount'
	
  Begin Object Class=moCheckBox Name=SeparateDeathMessages
    WinWidth=0.45
    WinHeight=0.07
    WinLeft=0.5
    WinTop=0.04
    Caption="Separate Death Messages"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Separate death messages from chat messages."
  End Object
  Controls(3)=moCheckbox'SeparateDeathMessages'
	
  Begin Object Class=moCheckBox Name=ShortDeathMessages
    WinWidth=0.45
    WinHeight=0.07
    WinLeft=0.5
    WinTop=0.16
    Caption="Short Death Messages"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Shorten death messages."
  End Object
  Controls(4)=moCheckbox'ShortDeathMessages'
	
  Begin Object Class=moCheckBox Name=GraphicalDeathMessages
    WinWidth=0.45
    WinHeight=0.07
    WinLeft=0.5
    WinTop=0.28
    Caption="Graphical Death Messages"
    CaptionWidth=0.9
    bSquare=true
    bHeightFromComponent=False
    ComponentJustification=TXTA_Left
    Hint="Display weapon icons instead of weapon names."
  End Object
  Controls(5)=moCheckbox'GraphicalDeathMessages'
}