//=============================================================================
// TGUISettingsPanelServer
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Panel for serverside options in the Tournament settings dialog box.
//=============================================================================


class TGUISettingsPanelServer extends TGUISettingsPanelServerBase;


//=============================================================================
// InitPanel
//
// Initialize the panel's components.
//=============================================================================

function InitPanel()
{
  local int i;
  
  Super.InitPanel();
  
  for (i = 0; i < Controls.Length; i++)
    LoadINISettings(Controls[i]);
  
  for (i = 0; i < Controls.Length; i++)
    Controls[i].OnChange = InternalOnChange;
}


//=============================================================================
// LoadINISettings
//
// Load INI values.
//=============================================================================

function LoadINISettings(GUIComponent Sender)
{
  Switch (Sender) {
    Case Controls[0]:
      Sender.MenuStateChange(MSAT_Disabled);
      break;
    Case Controls[1]:
      TGUILabeledSlider(Sender).SetValue(class'TournamentServerSettings'.Default.WarmupTime);
      break;
    Case Controls[3]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bWarmupWeapons);
      break;
    Case Controls[4]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bWarmupAmmo);
      break;
    Case Controls[5]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bWarmupHealth);
      break;
    Case Controls[6]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bWarmupShield);
      break;
    Case Controls[7]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bEnableNickFiltering);
      break;
    Case Controls[8]:
      TGUILabeledSlider(Sender).SetValue(class'TournamentServerSettings'.Default.MinNameLength);
      break;
    Case Controls[9]:
      TGUILabeledSlider(Sender).SetValue(class'TournamentServerSettings'.Default.SkinGlow);
      break;
    Case Controls[10]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bWeaponModifications);
      break;
    Case Controls[11]:
      moCheckBox(Sender).Checked(class'GameInfo'.default.bWeaponShouldViewShake);
      break;
    Case Controls[12]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.Default.bDisableLockdown);
      break;
    Case Controls[13]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowHitSounds);
      break;
    Case Controls[14]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowVariableHitSounds);
      break;
    Case Controls[15]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAutoServerDemo);
      break;
    Case Controls[16]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bBroadcastUDamageKills);
      break;
    Case Controls[17]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bNoHudClock);
      break;
    Case Controls[18]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowCarrierTelefrags);
      break;
    Case Controls[20]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowComboBerserk);
      break;
    Case Controls[21]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowComboBooster);
      break;
    Case Controls[22]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowComboInvisible);
      break;
    Case Controls[23]:
      moCheckBox(Sender).Checked(class'TournamentServerSettings'.default.bAllowComboSpeed);
      break;
  }
}


//=============================================================================
// InternalOnChange
//
// Save INI values. This is simulated so PlaySound is executed only locally.
//=============================================================================

simulated function InternalOnChange(GUIComponent Sender)
{
  class'TournamentServerSettings'.default.WarmupTime     = TGUILabeledSlider(Controls[1]).GetValue();
  class'TournamentServerSettings'.default.bWarmupWeapons = moCheckBox(Controls[3]).IsChecked();
  class'TournamentServerSettings'.default.bWarmupAmmo    = moCheckBox(Controls[4]).IsChecked();
  class'TournamentServerSettings'.default.bWarmupHealth  = moCheckBox(Controls[5]).IsChecked();
  class'TournamentServerSettings'.default.bWarmupShield  = moCheckBox(Controls[6]).IsChecked();
  
  class'TournamentServerSettings'.default.bEnableNickFiltering = moCheckBox(Controls[7]).IsChecked();
  class'TournamentServerSettings'.default.MinNameLength        = TGUILabeledSlider(Controls[8]).GetValue();
  
  class'TournamentServerSettings'.default.SkinGlow = TGUILabeledSlider(Controls[9]).GetValue();
  
  class'TournamentServerSettings'.default.bWeaponModifications = moCheckBox(Controls[10]).IsChecked();
  class'GameInfo'.default.bWeaponShouldViewShake               = moCheckBox(Controls[11]).IsChecked();
  class'TournamentServerSettings'.default.bDisableLockdown     = moCheckBox(Controls[12]).IsChecked();
  
  class'TournamentServerSettings'.default.bAllowHitSounds         = moCheckBox(Controls[13]).IsChecked();
  class'TournamentServerSettings'.default.bAllowVariableHitSounds = moCheckBox(Controls[14]).IsChecked();
  class'TournamentServerSettings'.default.bAutoServerDemo         = moCheckBox(Controls[15]).IsChecked();
  class'TournamentServerSettings'.default.bBroadcastUDamageKills  = moCheckBox(Controls[16]).IsChecked();
  class'TournamentServerSettings'.default.bNoHudClock             = moCheckBox(Controls[17]).IsChecked();
  class'TournamentServerSettings'.default.bAllowCarrierTelefrags  = moCheckBox(Controls[18]).IsChecked();
  
  class'TournamentServerSettings'.default.bAllowComboBerserk   = moCheckBox(Controls[20]).IsChecked();
  class'TournamentServerSettings'.default.bAllowComboBooster   = moCheckBox(Controls[21]).IsChecked();
  class'TournamentServerSettings'.default.bAllowComboInvisible = moCheckBox(Controls[22]).IsChecked();
  class'TournamentServerSettings'.default.bAllowComboSpeed     = moCheckBox(Controls[23]).IsChecked();
  
  class'GameInfo'.static.StaticSaveConfig();
  class'TournamentServerSettings'.static.StaticSaveConfig();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
}