//=============================================================================
// TGUISettingsPanelSpeech
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament's speech binder for species-specific phrases.
//=============================================================================


class TGUISettingsPanelSpeech extends TGUISettingsPanel
    dependson(Tab_ControlSettings);


var array<Tab_ControlSettings.KeyBinding> Bindings;  // Holds the array of key bindings

var GUIListBox MyListBox;
var Color      BkColor;
var bool       bSetNextKeyPress;
var int        NewIndex;
var int        NewSubIndex;
var GUIStyles  SelStyle;
var int        Row;
var int        HackIndex;               // Hacky Hacky

var localized string Header,Footer;
var float SectionLabelMargin;
var bool bListInitialised;

var transient bool bNoMatureLanguage;


function InitPanel()
{
  Super.InitPanel();
  
  GUILabel(Controls[4]).Caption = class'Tab_SpeechBinder'.default.SpeechLabel;
  LoadSpeechCommands();
  bNoMatureLanguage = PlayerOwner().bNoMatureLanguage;
  
  MyListBox = GUIListBox(Controls[0]);
  MyListBox.List.OnDrawItem      = DrawBinding;
  MyListBox.List.SelectedBkColor = BkColor;
  MyListBox.List.SelectedImage   = None;
  MyListbox.List.OnClick         = GetNewKey;
  MyListBox.List.OnChange        = ListChange;
  MyListBox.List.OnKeyEvent      = ListOnKeyEvent;
  MyListBox.List.bHotTrack       = true;
  MyListBox.List.OnClickSound    = CS_None;
  MyListBox.List.OnAdjustTop     = MyOnAdjustTop;
  
  SelStyle = Controller.GetStyle("RoundButton");
}

function ShowPanel(bool bShow)
{
  Super.ShowPanel(bShow);

  if ( bShow && !bListInitialised ) {
    InitBindings();
    
    MyListBox.List.Index = 1;
    HackIndex = 1;
    
    bListInitialised = true;
  }
}

function int Weight(int i)
{
  if ( i == 0x01 || i == 0x02 )
    return 100;
  
  if ( i >= 0x30 && i <= 0x5A )
    return 50;
  
  if ( i == 0x20 )
    return 45;
  
  if ( i == 0x04 )
    return 40;
  
  if ( i==0xEC || i==0xED )
    return 35;
  
  if ( i >= 0x21 && i <= 0x28 )
    return 30;
  
  if ( i >= 0x60 && i <= 0x6F)
    return 30;
  
  return 25;
}

function Swap(int Index, int a, int b)
{
  local int TempInt;
  local string TempStrA, TempStrB;
  
  TempInt  = Bindings[Index].Binds[a];
  TempStrA = Bindings[Index].BindKeyNames[a];
  TempStrB = Bindings[Index].BindLocalizedKeyNames[a];
  
  Bindings[Index].Binds[a]                 = Bindings[Index].Binds[B];
  Bindings[Index].BindKeyNames[a]          = Bindings[Index].BindKeyNames[b];
  Bindings[Index].BindLocalizedKeyNames[a] = Bindings[Index].BindLocalizedKeyNames[b];
  
  Bindings[Index].Binds[b]                 = TempInt;
  Bindings[Index].BindKeyNames[b]          = TempStrA;
  Bindings[Index].BindLocalizedKeyNames[b] = TempStrB;
}


function InitBindings()
{
  local int i, j, k, Index;
  local string KeyName, Alias, LocalizedKeyName;
  
  // Clear them all.
  for (i = 0; i < Bindings.Length; i++) {
    Bindings[i].Binds.Length = 0;
    Bindings[i].BindKeyNames.Length = 0;
    Bindings[i].BindLocalizedKeyNames.Length = 0;
    MyListBox.List.Add(Bindings[i].KeyLabel);
  }
  
  for (i = 0; i < 255; i++) {
    KeyName = PlayerOwner().ConsoleCommand("KEYNAME" @ i);
    LocalizedKeyName = PlayerOwner().ConsoleCommand("LOCALIZEDKEYNAME" @ i);
    if ( KeyName != "" ) {
      Alias = PlayerOwner().ConsoleCommand("KEYBINDING" @ KeyName);
      if ( Alias != "" )
        for (j = 0; j < Bindings.Length; j++)
          if ( Bindings[j].Alias ~= Alias ) {
            Index = Bindings[j].Binds.Length;
            
            Bindings[j].Binds[Index] = i;
            Bindings[j].BindKeyNames[Index] = KeyName;
            Bindings[j].BindLocalizedKeyNames[Index] = LocalizedKeyName;

            for (k = 0; k < Index; k++)
              if ( Weight(Bindings[j].Binds[k]) < Weight(Bindings[j].Binds[Index]) ) {
                Swap(j, k, Index);
                break;
              }
          } // if
    } // if
  } // for
}


function AddCommand(bool IsSection, string KeyLabel, string Alias)
{
  local int At;
  
  At = Bindings.Length;
  Bindings.Length = Bindings.Length + 1;
  
  Bindings[At].bIsSectionLabel = IsSection;
  Bindings[At].KeyLabel = KeyLabel;
  Bindings[At].Alias = Alias;
}



// We pass in the controller here
function LoadSpeechCommands()
{
  local int i;
  local class<xVoicePack> VoicePack;
  
  Bindings.Length = 0;
  if ( xPawn(PlayerOwner().Pawn) != None )
    VoicePack = class<xVoicePack>(xPawn(PlayerOwner().Pawn).VoiceClass);
  if ( VoicePack == None && PlayerOwner().PlayerReplicationInfo != None )
    VoicePack = class<xVoicePack>(PlayerOwner().PlayerReplicationInfo.VoiceType);
  if ( VoicePack == None )
    VoicePack = class'xVoicePack';
  
  //////////////////////////////// ACKNOWLEDGEMENTS /////////////////////////////////////////////
  AddCommand(true, class'ExtendedConsole'.default.SMStateName[1], "");
  for (i = 0; i < VoicePack.default.numAcks; i++)
    AddCommand(false, VoicePack.default.AckString[i], "speech ACK" @ i);
  
  //////////////////////////////// FRIENDLY FIRE /////////////////////////////////////////////
  AddCommand(true, class'ExtendedConsole'.default.SMStateName[2], "");
  for (i = 0; i < VoicePack.default.numFFires; i++) {
    if ( VoicePack.default.FFireAbbrev[i] != "" )
      AddCommand(false, VoicePack.default.FFireAbbrev[i], "speech FRIENDLYFIRE" @ i);
    else
      AddCommand(false, VoicePack.default.FFireString[i], "speech FRIENDLYFIRE" @ i);
  }
  
  //////////////////////////////// ORDERS /////////////////////////////////////////////
  AddCommand(true, class'ExtendedConsole'.default.SMStateName[3], "");
  for (i = 0; i < 16; i++) {
    if ( VoicePack.default.OrderString[i] == "" )
      continue;

    if ( VoicePack.default.OrderAbbrev[i] != "" )
      AddCommand(false, VoicePack.default.OrderAbbrev[i], "speech ORDER" @ i);
    else
      AddCommand(false, VoicePack.default.OrderString[i], "speech ORDER" @ i);
  }
  
  //////////////////////////////// OTHER /////////////////////////////////////////////
  AddCommand(true, class'ExtendedConsole'.default.SMStateName[4], "");
  for (i = 0; i < ArrayCount(VoicePack.default.OtherString); i++) {
    if ( VoicePack.default.OtherString[i] == "" )
      continue;
    
    if ( VoicePack.default.OtherAbbrev[i] != "" )
      AddCommand(false, VoicePack.default.OtherAbbrev[i], "speech OTHER" @ i);
    else
      AddCommand(false, VoicePack.default.OtherString[i], "speech OTHER" @ i);
  }
  
  //////////////////////////////// LOAD TAUNTS /////////////////////////////////////////////
  AddCommand(true, class'ExtendedConsole'.default.SMStateName[5], "");
  for (i = 0; i < VoicePack.default.numTaunts; i++) {
    if ( VoicePack.default.MatureTaunt[i] == 1 && bNoMatureLanguage )
      continue;
    
    if ( VoicePack.default.TauntAbbrev[i] != "" )
      AddCommand(false, VoicePack.default.TauntAbbrev[i], "speech TAUNT" @ i);
    else if ( VoicePack.default.TauntString[i] != "" )
      AddCommand(false, VoicePack.default.TauntString[i], "speech TAUNT" @ i);
    else  // no description, probably not used by this voice pack
      AddCommand(false, "(" $ class'MatSubAction'.default.Desc $ ")", "speech TAUNT" @ i);
  }
}


function string GetCurrentKeyBind(int Index, int bind)
{
  if ( Index >= bindings.Length )
    return "";
  
  if ( Bindings[Index].bIsSectionLabel )
    return "";
  
  if ( Index == NewIndex && Bind == NewSubIndex )
    return "???";
  
  if ( Bind >= Bindings[Index].Binds.Length )
    return "";
  
  return Bindings[Index].BindLocalizedKeyNames[Bind];
}

function DrawBinding(Canvas Canvas, int Item, float X, float Y, float W, float H, bool bSelected)
{
  local int x1,w1;
  local bool InBindArea;
  
  if ( Item >= Bindings.Length )
    return;
  
  if ( Controller.MouseX >= Controls[2].Bounds[0] && Controller.MouseX <= Controls[3].Bounds[2] )
    InBindArea = true;
  
  if ( Controller.HasMouseMoved() && !bSetNextKeyPress ) {
    if ( InBindArea ) {
      if ( Controller.MouseX >= Controls[2].Bounds[0] && Controller.MouseX <= Controls[2].Bounds[2] )
        Row = 0;
      else if ( Controller.MouseX >= Controls[3].Bounds[0] && Controller.MouseX <= Controls[3].Bounds[2] )
        Row = 1;
      
      InBindArea = true;
    }

    if ( InBindArea && MyListBox.List.Index != HackIndex && MyListBox.List.Index<Bindings.Length
        && Bindings[MyListBox.List.Index].bIsSectionLabel )
      MyListBox.List.SetIndex(HackIndex);
  }
  
  if ( InBindArea ) {
    if ( bSetNextKeyPress && MyListBox.List.Index != HackIndex )
      MyListBox.List.SetIndex(HackIndex);
    
    if ( HackIndex != MyListBox.List.Index )
      UpdateHint(MyListBox.List.Index);
    
    HackIndex = MyListBox.List.Index;
  }
  
  if ( Bindings[Item].bIsSectionLabel ) {
    Canvas.CurX = Controls[1].ClientBounds[0] + 3;
    Canvas.CurY = Y;
    Canvas.SetDrawColor(24, 24, 74, 200);
    Canvas.DrawTile(Controller.DefaultPens[0], Controls[1].ClientBounds[2] - Controls[1].ClientBounds[0] - 6,
        H, 0, 0, 1, 1);
    MyListBox.Style.DrawText(Canvas, MSAT_Pressed, Controls[1].ClientBounds[0] + SectionLabelMargin, Y,
        Controls[1].ClientBounds[2]-Controls[1].ClientBounds[0], H, TXTA_Left, Bindings[Item].KeyLabel);
  }
  else {
    MyListBox.Style.DrawText(Canvas,MenuState, Controls[1].ActualLeft(), Y, Controls[1].ActualWidth(), H,
        TXTA_Center, Bindings[Item].KeyLabel);
  }
  
  MyListBox.Style.DrawText(Canvas, MenuState, Controls[2].ActualLeft(), Y, Controls[2].ActualWidth(), H,
      TXTA_Center, GetCurrentKeyBind(Item,0));
  MyListBox.Style.DrawText(Canvas, MenuState, Controls[3].ActualLeft(), Y, Controls[3].ActualWidth(), H,
      TXTA_Center, GetCurrentKeyBind(Item,1));
  
  if ( bSelected && !Bindings[item].bIsSectionLabel ) {
    if ( Row == 0 ) {
      x1 = Controls[2].ActualLeft();
      w1 = Controls[2].ActualWidth();
    }
    else if ( Row == 1 ) {
      x1 = Controls[3].ActualLeft();
      w1 = Controls[3].ActualWidth();
    }
    
    if ( !bSetNextKeyPress )
      SelStyle.Draw(Canvas, MSAT_Watched, x1, y, w1, h);
    else
      SelStyle.Draw(Canvas, MSAT_Pressed, x1, y, w1, h);
    
    UpdateHint(Item);
  }
}

function bool ListOnKeyEvent(out byte Key, out byte State, float delta)
{
  local bool result;
  local int oldIndex;
  
  if ( Key == 0x0D && State == 3 ) {
    GetNewKey(None);
    return true;
  }
  
  if ( Key == 0x08 && State == 3 ) {  // Backsapce
    // Clear Over
    if ( Controller.MouseX >= Controls[2].Bounds[0] && Controller.MouseX <= Controls[2].Bounds[2] )
      RemoveExistingKey(MyListBox.List.Index, 0);
    else if ( Controller.MouseX>=Controls[3].Bounds[0] && Controller.MouseX<=Controls[3].Bounds[2] )
      RemoveExistingKey(MyListBox.List.Index, 1);
    
    UpdateHint(MyListBox.List.Index);
    
    return true;
  }
  
  // -- WARNING.. really big hack here
  if ( State == 1 && (Key == 0x25 || Key == 0x64 || Key == 0x27 || Key == 0x66) ) {
    if ( Row == 0 )
      Row = 1;
    else
      Row = 0;
    
    UpdateHint(MyListBox.List.Index);
    return true;
  }
  
  OldIndex = MyListBox.List.Index;
  result = MyListBox.List.InternalOnKeyEvent(Key, State, delta);
  if ( MyListBox.List.Index != OldIndex ) {
    if ( Bindings[MyListBox.List.Index].bIsSectionLabel ) {
      if ( MyListBox.List.Index < OldIndex )
        SearchUp(OldIndex);
      else
        SearchDown(OldIndex);
    }
  }
  UpdateHint(MyListBox.List.Index);
  return false;
}

function SearchUp(int OldIndex)
{
  local int CIndex;
  
  CIndex = MyListBox.List.Index;
  while (CIndex > 0) {
    if ( !Bindings[CIndex].bIsSectionLabel ) {
      MyListBox.List.SetIndex(CIndex);
      return;
    }
    CIndex--;
  }
  MyListBox.List.SetIndex(OldIndex);
}

function SearchDown(int OldIndex)
{
  local int CIndex;
  
  CIndex = MyListBox.List.Index;
  while (CIndex < MyListBox.List.ItemCount - 1) {
    if ( !Bindings[CIndex].bIsSectionLabel ) {
      MyListBox.List.SetIndex(CIndex);
      return;
    }
    CIndex++;
  }
  MyListBox.List.SetIndex(OldIndex);
}



function RemoveExistingKey(int Index, int SubIndex)
{
  if ( Index >= Bindings.Length || SubIndex >= Bindings[Index].Binds.Length || Bindings[Index].Binds[SubIndex] < 0 )
    return;
  
  // Clear the bind
  PlayerOwner().ConsoleCommand("SET Input" @ Bindings[Index].BindKeyNames[SubIndex]);
  
  // Clear the entry
  Bindings[Index].Binds.Remove(SubIndex, 1);
  Bindings[Index].BindKeyNames.Remove(SubIndex, 1);
  Bindings[Index].BindLocalizedKeyNames.Remove(SubIndex, 1);
}

function RemoveAllOccurance(byte NewKey)
{
  local int i, j;
  
  for (i = 0; i < Bindings.Length; i++)
    for (j = Bindings[i].Binds.Length - 1; j >= 0; j--)
      if ( Bindings[i].Binds[j] == NewKey )
        RemoveExistingKey(i, j);
}

function UpdateHint(int Index)
{
  local int i;
  local string t;
  
  if ( Index < 0 || Index >= Bindings.Length ) {
    MyListBox.List.Hint = "";
    Controller.ActivePage.ChangeHint(MyListBox.List.Hint);
    return;
  }
  else {
    t = "";
    for (i = 0; i < Bindings[Index].Binds.Length; i++) {
      if ( t == "" )
        t = "" $ GetCurrentKeyBind(Index, i);
      else
        t = t $ "," @ GetCurrentKeyBind(Index, i);
    }
  }
  MyListBox.List.Hint = class'Tab_ControlSettings'.default.Header $ t $ class'Tab_ControlSettings'.default.Footer;
  Controller.ActivePage.ChangeHint(MyListBox.List.Hint);
}

function AddNewKey(int Index, int SubIndex, byte NewKey)
{
  if ( Index >= Bindings.Length )
    return;
  
  if ( SubIndex < Bindings[Index].Binds.Length && Bindings[Index].Binds[SubIndex] == NewKey )
    return;
  
  RemoveAllOccurance(NewKey);
  
  if ( SubIndex >= Bindings[Index].Binds.Length ) {
    Bindings[Index].Binds.Length                 = Bindings[Index].Binds.Length + 1;
    Bindings[Index].BindKeyNames.Length          = Bindings[Index].BindKeyNames.Length + 1;
    Bindings[Index].BindLocalizedKeyNames.Length = Bindings[Index].BindLocalizedKeyNames.Length + 1;
    SubIndex = Bindings[Index].Binds.Length - 1;
  }
  Bindings[Index].Binds[SubIndex]                 = NewKey;
  Bindings[Index].BindKeyNames[SubIndex]          = PlayerOwner().ConsoleCommand("KeyName"@NewKey);
  Bindings[Index].BindLocalizedKeyNames[SubIndex] = PlayerOwner().ConsoleCommand("LOCALIZEDKEYNAME"@NewKey);
  PlayerOwner().ConsoleCommand("SET Input"@Bindings[Index].BindKeyNames[SubIndex]@Bindings[Index].Alias);
  
  UpdateHint(Index);
}

function bool GetNewKey(GUIComponent Sender)
{
  local int SubIndex;
  
  if ( Controller.MouseX >= Controls[2].Bounds[0] && Controller.MouseX <= Controls[2].Bounds[2] )
    SubIndex = 0;
  else if ( Controller.MouseX>=Controls[3].Bounds[0] && Controller.MouseX<=Controls[3].Bounds[2] )
    SubIndex = 1;
  else
    return true;
  
  NewIndex = MyListBox.List.Index;
  NewSubIndex = SubIndex;
  bSetNextKeyPress = true;
  Controller.OnNeedRawKeyPress = RawKey;
  Controller.Master.bRequireRawJoystick = true;
  
  PlayerOwner().ClientPlaySound(Controller.EditSound);
  PlayerOwner().ConsoleCommand("toggleime 0");
  
  return true;
}

function bool RawKey(byte NewKey)
{
  if ( NewKey != 0x1B )
    AddNewKey(NewIndex, NewSubIndex, NewKey);
  
  NewIndex = -1;
  NewSubIndex = -1;
  
  bSetNextKeyPress = false;
  Controller.OnNeedRawKeyPress = none;
  Controller.Master.bRequireRawJoystick = false;
  
  PlayerOwner().ClientPlaySound(Controller.ClickSound);
  
  return true;
}

function ListChange(GUIComponent Sender)
{
  UpdateHint(MyListBox.List.Index);
}

function MyOnAdjustTop(GUIComponent Sender)
{
  if ( MyListBox.List.Index < MyListBox.List.Top )
    MyListBox.List.SetIndex(MyListBox.List.Top);
  else if ( MyListBox.List.Index >= MyListBox.List.Top + MyListBox.List.ItemsPerPage )
    MyListBox.List.SetIndex(MyListBox.List.Top + MyListBox.List.ItemsPerPage);
}

defaultproperties
{
  Begin Object Class=GUIListBox Name=KeyConfigKeyList
    WinWidth=0.96
    WinHeight=0.9
    WinLeft=0.035
    WinTop=0.12
    bVisibleWhenEmpty=true
    StyleName="BindBox"
    Hint="Select Control to config"
  End Object
  Controls(0)=GUIListBox'KeyConfigKeyList'
  
  Begin Object class=GUIImage Name=KeyConfigBK1
    WinWidth=0.44
    WinLeft=0.01
    WinHeight=0.94
    WinTop=0.1
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
    bNeverFocus=true
    bAcceptsInput=true
  End Object
  Controls(1)=GUIImage'KeyConfigBK1'
  
  Begin Object class=GUIImage Name=KeyConfigBK2
    WinWidth=0.25
    WinLeft=0.45
    WinHeight=0.94
    WinTop=0.1
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(2)=GUIImage'KeyConfigBK2'
  
  Begin Object class=GUIImage Name=KeyConfigBK3
    WinWidth=0.25
    WinLeft=0.7
    WinHeight=0.94
    WinTop=0.1
    Image=Material'InterfaceContent.Menu.BorderBoxD'
    ImageColor=(R=255,G=255,B=255,A=160);
    ImageRenderStyle=MSTY_Alpha
    ImageStyle=ISTY_Stretched
  End Object
  Controls(3)=GUIImage'KeyConfigBK3'
  
  Begin Object class=GUILabel Name=KeyConfigAliasLabel
    TextALign=TXTA_Center
    TextColor=(R=255,G=255,B=255,A=255)
    WinWidth=0.44
    WinHeight=32.0
    WinLeft=0.01
    WinTop=0.0
    TextFont="UT2HeaderFont"
  End Object
  Controls(4)=GUILabel'KeyConfigAliasLabel'
  
  Begin Object class=GUILabel Name=KeyConfigAliasLabel2
    Caption="Key 1"
    TextALign=TXTA_Center
    TextColor=(R=255,G=255,B=255,A=255)
    WinWidth=0.25
    WinHeight=32.0
    WinLeft=0.45
    WinTop=0.0
    TextFont="UT2HeaderFont"
  End Object
  Controls(5)=GUILabel'KeyConfigAliasLabel2'
  
  Begin Object class=GUILabel Name=KeyConfigAliasLabel3
    Caption="Key 2"
    TextALign=TXTA_Center
    TextColor=(R=255,G=255,B=255,A=255)
    WinWidth=0.25
    WinHeight=32.0
    WinLeft=0.7
    WinTop=0.0
    TextFont="UT2HeaderFont"
  End Object
  Controls(6)=GUILabel'KeyConfigAliasLabel3'
  
  bAcceptsInput=false
  BKColor=(R=24,G=24,B=100,A=200)
  SectionLabelMargin=10
  Row=0
}