//=============================================================================
// TStatsAssaultGrenadeFire
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tracks weapon stats.
//=============================================================================


class TStatsAssaultGrenadeFire extends AssaultGrenade;


//=============================================================================
// Variables
//=============================================================================

var bool bWaiting;
var bool bWeaponModifications;


//=============================================================================
// PreBeginPlay
//
// Check whether weapon modifications are enabled.
//=============================================================================

function PreBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
    else
      mWaitTime = 0.3;
  
  Super.PreBeginPlay();
}


//=============================================================================
// SpawnProjectile
//
// Track weapon stats.
//=============================================================================

function Projectile SpawnProjectile(vector Start, rotator Dir)
{
  local Projectile p;
  local vector X, Y, Z;
  local float PawnSpeed;
  
  class'TournamentWeaponStats'.static.IncrementShots(Instigator.Controller, class'AssaultRifle', 1);
  
  p = Spawn(ProjectileClass, self,, Start, Dir);
  if ( p != None ) {
    Weapon.GetViewAxes(X,Y,Z);
    PawnSpeed = X dot Instigator.Velocity;
    
    if ( Bot(Instigator.Controller) != None )
      p.Speed = mHoldSpeedMax;
    else
      p.Speed = mHoldSpeedMin + HoldTime * mHoldSpeedGainPerSec;
    
    p.Speed = FClamp(p.Speed, mHoldSpeedMin, mHoldSpeedMax);
    p.Speed = PawnSpeed + p.Speed;
    p.Velocity = p.Speed * Vector(Dir);
    p.Damage *= DamageAtten;
  }
  
  return p;
}


//=============================================================================
// state Wait
// state LoadNext
//
// Allow charging while the next grenade is being loaded.
//=============================================================================

state Wait
{
  event ModeDoFire()
  {
    if ( !bWeaponModifications )
      Super.ModeDoFire();
    else if ( !bWaiting ) {
      Global.ModeDoFire();
      NextFireTime = Level.TimeSeconds;
      bWaiting = True;
    }
    else {
      HoldTime = 0;
    }
  }
  
  function EndState()
  {
    if ( !bWeaponModifications )
      Super.EndState();
    else
      bWaiting = False;
  }
}

state LoadNext
{
  event ModeDoFire()
  {
    if ( !bWeaponModifications )
      Super.EndState();
    else
      HoldTime = 0;
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  ProjectileClass=class'TStatsGrenade'
  mWaitTime=0.6
}
