//=============================================================================
// TStatsBioChargedFire
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tracks weapon stats.
//=============================================================================


class TStatsBioChargedFire extends BioChargedFire;


//=============================================================================
// Imports
//=============================================================================

#exec audio import file=Sounds\BioHoldLoad.wav group=WeaponSounds


//=============================================================================
// Variables
//=============================================================================

var bool bHoldLoadSound;
var bool bWeaponModifications;


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
}


//=============================================================================
// SpawnProjectile
//
// Track weapon stats.
//=============================================================================

function Projectile SpawnProjectile(vector Start, rotator Dir)
{
  local Projectile Glob;
  local int GroupID;
  
  GotoState('');
  
  if ( Instigator.AmbientSound == sound'BioHoldLoad' )
    Instigator.AmbientSound = None;
  
  if ( GoopLoad == 0 )
    return None;
  
  GroupID = class'TournamentWeaponStats'.static.IncrementShots(Instigator.Controller, Weapon.Class, 1);
  
  Glob = Spawn(ProjectileClass,,, Start, Dir);
  if ( BioGlob(Glob) != None ) {
    BioGlob(Glob).Damage *= DamageAtten;
    BioGlob(Glob).SetGoopLevel(GoopLoad);
    if ( TStatsBioGlob(Glob) != None ) {
      TStatsBioGlob(Glob).GroupID = GroupID;
      TStatsBioGlob(Glob).ModeNum = 1;
    }
  }
  GoopLoad = 0;
  if ( Weapon.Ammo[ThisModeNum].AmmoAmount <= 0 )
    Weapon.OutOfAmmo();
  return Glob;
}


//=============================================================================
// state Hold
//
// Add an ambient sound while the bio rifle is charged up.
//=============================================================================

state Hold
{
  simulated function BeginState()
  {
    Super.BeginState();
    if ( Level.NetMode != NM_Client && bWeaponModifications )
      Instigator.AmbientSound = sound'BioHoldLoad';
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  ProjectileClass=TStatsBioGlob
}
