//=============================================================================
// TStatsBioGlob
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Bio glob.
//=============================================================================


class TStatsBioGlob extends BioGlob;


//=============================================================================
// Variables
//=============================================================================

var int GroupID;
var byte ModeNum;


//=============================================================================
// state Flying
// ProcessTouch
//
// Transfer GroupID when merging globs.
//=============================================================================

auto state Flying
{
  simulated function ProcessTouch(Actor Other, Vector HitLocation)
  {
    local TStatsBioGlob Glob;
    
    Glob = TStatsBioGlob(Other);
    if ( Glob != None && (Glob.Owner == None || Glob.Owner != Owner && Glob.Owner != self) && bMergeGlobs ) {
      Glob.GroupID = GroupID;
      Glob.ModeNum = 2;
    }
    
    Super.ProcessTouch(Other, HitLocation);
  }
}


//=============================================================================
// state OnGround
// BeginState
//
// Use special fire mode for tracking hits with globs on ground.
//=============================================================================

state OnGround
{
  simulated function BeginState()
  {
    ModeNum = 2;
    Super.BeginState();
  }
}


//=============================================================================
// BlowUp
//
// Track weapon hit stats.
//=============================================================================

function BlowUp(Vector HitLocation)
{
  if ( Role == ROLE_Authority ) {
    Damage = BaseDamage + Damage * GoopLevel;
    DamageRadius = DamageRadius * GoopVolume;
    MomentumTransfer = MomentumTransfer * GoopVolume;
    if ( Physics == PHYS_Flying )
      MomentumTransfer *= 0.5;
    if ( class'TournamentUtil'.static.StatsHurtRadius(Self, Damage, DamageRadius, MyDamageType,
        MomentumTransfer, HitLocation, ModeNum) )
      class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'BioRifle', ModeNum, GroupID);
  }
  
  PlaySound(ExplodeSound, SLOT_Misc);
  
  Destroy();
}


//=============================================================================
// SplashGlobs
//
// Transfer GroupID to splashed globlings.
//=============================================================================

function SplashGlobs(int NumGloblings)
{
  local int g;
  local TStatsBioGlob NewGlob;
  local Vector VNorm;
  
  for (g = 0; g < NumGloblings; g++) {
    NewGlob = Spawn(Class, self,, Location + GoopVolume * (CollisionHeight + 4.0) * SurfaceNormal);
    if ( NewGlob != None ) {
      NewGlob.Velocity = (GloblingSpeed + FRand() * 150.0) * (SurfaceNormal + VRand() * 0.8);
      NewGlob.GroupID = GroupID;
      NewGlob.ModeNum = 2;
      if ( Physics == PHYS_Falling ) {
        VNorm = (Velocity dot SurfaceNormal) * SurfaceNormal;
        NewGlob.Velocity += (-VNorm + (Velocity - VNorm)) * 0.1;
      }
    }
  }
}
