//=============================================================================
// TStatsFlakChunk
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Flak chunk.
//=============================================================================


class TStatsFlakChunk extends FlakChunk;


//=============================================================================
// Variables
//=============================================================================

var int GroupID;  // ID for tracking multi-projectile shot stats
var byte ModeNum; // whether this chunk was spawned from a flak shell
var bool bWeaponModifications;


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
  if ( Instigator != None && Instigator.Controller != None )
    InstigatorController = Instigator.Controller;
  
//  if ( Bounces < 2 )
//    Bounces++;
  if ( bWeaponModifications )
    SetTimer(0.5, False);
  else
    Disable('Tick');
}


//=============================================================================
// Tick
//
// Fade out the trail.
//=============================================================================

simulated function Tick(float DeltaTime)
{
  if ( Trail != None ) {
    Trail.mColorRange[0].A = Clamp(255 - 200 * FMax(0, default.LifeSpan - LifeSpan - 1), 0, 20 + 0.1 * VSize(Velocity));
    Trail.mColorRange[1].A = Trail.mColorRange[0].A;
    if ( Trail.mColorRange[0].A == 0 ) {
      Trail.mRegen = False;
      Disable('Tick');
    }
  }
  else
    Disable('Tick');
}


//=============================================================================
// Timer
//
// Set falling physics.
//=============================================================================

simulated function Timer()
{
  if ( Physics != PHYS_None )
    SetPhysics(PHYS_Falling);
  
//  Bounces--;
}


//=============================================================================
// ProcessTouch
//
// Track weapon stats and fix delayed damage.
//=============================================================================

simulated function ProcessTouch(Actor Other, vector HitLocation)
{
  if ( FlakChunk(Other) == None && (Physics == PHYS_Falling && bWeaponModifications || Other != Instigator) ) {
    speed = VSize(Velocity);
    if ( speed > 200 ) {
      if ( Role == ROLE_Authority ) {
        if ( Instigator == None && Pawn(Other) != None )
          Pawn(Other).DelayedDamageInstigatorController = InstigatorController;
        if ( Pawn(Other) != None && Pawn(Other).Controller != None && Pawn(Other).Controller.bIsPlayer )
          class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'FlakCannon', ModeNum, GroupID);
        if ( bWeaponModifications )
          Other.TakeDamage(FMax(5, Damage - DamageAtten * FMax(0, default.LifeSpan - LifeSpan - 1))
              * Speed / default.Speed, Instigator, HitLocation,
              MomentumTransfer * Normal(Velocity) * Speed / default.Speed, MyDamageType);
        else
          Other.TakeDamage(Max(5, Damage - DamageAtten * FMax(0, default.LifeSpan - LifeSpan - 1)),
              Instigator, HitLocation, MomentumTransfer * Velocity / Speed, MyDamageType);
      }
      Destroy();
    }
  }
}
