//=============================================================================
// TStatsFlakShell
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Flak shell.
//=============================================================================


class TStatsFlakShell extends FlakShell;


//=============================================================================
// Variables
//=============================================================================

var transient int GroupID;  // ID for tracking multi-projectile shot stats
var bool bCanHitInstigator;
var() int ChunkSpread;
var bool bWeaponModifications;


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
  if ( Instigator != None && Instigator.Controller != None )
    InstigatorController = Instigator.Controller;
  
  if ( bWeaponModifications )
    SetTimer(0.7, False);
}


//=============================================================================
// Timer
//
// Set falling physics.
//=============================================================================

simulated function Timer()
{
  bCanHitInstigator = True;
}


//=============================================================================
// ProcessTouch
//
// Fix owner hits.
//=============================================================================

simulated function ProcessTouch(Actor Other, Vector HitLocation)
{
  if ( Other != Instigator || bCanHitInstigator ) {
    SpawnEffects(HitLocation, vect(0,0,0));
    Explode(HitLocation, Normal(HitLocation - Other.Location));
  }
}


//=============================================================================
// SpawnEffects
//
// Spawn a cool shiny new Emitter-based explosion effect.
//=============================================================================

simulated function SpawnEffects(vector HitLocation, vector HitNormal)
{
  PlaySound(Sound'WeaponSounds.BExplosion1',, 3 * TransientSoundVolume);
  if ( EffectIsRelevant(Location, false) ) {
    if ( HitNormal == vect(0,0,0) )
      Spawn(class'TStatsFlakExplosion',,, HitLocation);
    else {
      Spawn(class'TStatsFlakWallExplosion',,, HitLocation + HitNormal * 16, rotator(HitNormal) + rot(16384,0,16384));
      if ( ExplosionDecal != None && Level.NetMode != NM_DedicatedServer )
        Spawn(ExplosionDecal, self,, HitLocation, rotator(-HitNormal));
    }
  }
}


//=============================================================================
// Explode
//
// Track weapon stats and fix delayed damage.
//=============================================================================

simulated function Explode(vector HitLocation, vector HitNormal)
{
  local vector Start;
  local rotator rot;
  local int i;
  local TStatsFlakChunk NewChunk;
  
  Start = Location + 10 * HitNormal;
  if ( class'TournamentUtil'.static.StatsHurtRadius(Self, Damage, DamageRadius, MyDamageType,
      MomentumTransfer, HitLocation, 1) && Role == ROLE_Authority )
    class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'FlakCannon', 1, GroupID);
  
  if ( Role == ROLE_Authority ) {
    MakeNoise(1.0);
    for (i = 0; i < 6; i++) {
      rot = rotator(Velocity);
      rot.Yaw   += RandRange(-ChunkSpread, ChunkSpread);
      rot.Pitch += RandRange(-ChunkSpread, ChunkSpread);
      rot.Roll  += RandRange(-ChunkSpread, ChunkSpread);
      NewChunk = Spawn(class'TStatsFlakChunk',,, Start, rot);
      if ( NewChunk != None && !NewChunk.bDeleteMe ) {
        NewChunk.InstigatorController = InstigatorController;
        NewChunk.GroupID = GroupID;
        NewChunk.ModeNum = 1;
        if ( bWeaponModifications ) {
          NewChunk.SetPhysics(PHYS_Falling);
          NewChunk.Velocity *= 0.5;
          NewChunk.SetLocation(NewChunk.Location);  // force touch event
        }
      }
    }
  }
  Destroy();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  DamageRadius=220.0
  ChunkSpread=16000
  bGameRelevant=False // allow Mutator.CheckReplacement() calls
}
