//=============================================================================
// TStatsGrenade
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Assault rifle grenade.
//=============================================================================


class TStatsGrenade extends Grenade;


//=============================================================================
// Variables
//=============================================================================

var bool bWeaponModifications;


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
}


//=============================================================================
// TornOff
//
// Called when exploding on the server. Cause clientside damage.
//=============================================================================

simulated function TornOff()
{
  if ( Level.NetMode != NM_Client )
    return;
  
  BlowUp(Location);
  Destroy();
}


//=============================================================================
// HitWall
//
// Allow grenade to hit Instigator.
//=============================================================================

simulated function HitWall(vector HitNormal, Actor Wall)
{
  Super.HitWall(HitNormal, Wall);
  bCanHitOwner = bWeaponModifications;
}


//=============================================================================
// ProcessTouch
//
// Made non-simulated.
//=============================================================================

function ProcessTouch(Actor Other, vector HitLocation)
{
  Super.ProcessTouch(Other, HitLocation);
}


//=============================================================================
// BlowUp
//
// Track weapon stats.
//=============================================================================

simulated function BlowUp(vector HitLocation)
{
  if ( class'TournamentUtil'.static.StatsHurtRadius(Self, Damage, DamageRadius, MyDamageType,
      MomentumTransfer, HitLocation, 1) && Role == ROLE_Authority )
    class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'AssaultRifle', 1, -1);
  if ( Role == ROLE_Authority )
    MakeNoise(1.0);
}


//=============================================================================
// Explode
//
// Blow up and fix clientside simulation errors.
//=============================================================================

function Explode(vector HitLocation, vector HitNormal)
{
  BlowUp(HitLocation);
  PlaySound(sound'WeaponSounds.BExplosion3',,2.5 * TransientSoundVolume);
  Spawn(class'TStatsRocketExplosion',,, HitLocation);
  Spawn(ExplosionDecal,self,,HitLocation, rotator(-HitNormal));
  if ( bWeaponModifications ) {
    bUpdateSimulatedPosition = True;
    SetPhysics(PHYS_None);
    SetCollision(false, false, false);
    LifeSpan = 0.1;
    bTearOff = True;
  }
  else
    Destroy();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bNetTemporary=False
}
