//=============================================================================
// TStatsLinkBeamFire
// Copyright 2003-2004 by Wormbo
//
// Fixes serverside Accessed Nones in LinkFire.ModeTick() and tracks stats.
//=============================================================================


class TStatsLinkBeamFire extends LinkFire;


//=============================================================================
// ModeTick
//
// Track weapon hits.
//=============================================================================

simulated function ModeTick(float DeltaTime)
{
  local Vector StartTrace, EndTrace, X,Y,Z, V;
  local Vector HitLocation, HitNormal, EndEffect;
  local Actor Other;
  local Rotator Aim;
  local LinkGun LinkGun;
  local float MT, Step, ls;
  local bot B;
  local bool bShouldStop;
  local int AdjustedDamage;
  local LinkBeamEffect LB;
  
  if ( !bIsFiring ) {
    bInitAimError = true;
    return;
  }
  LinkGun = LinkGun(Weapon);
  
  if ( LinkGun.Links < 0 ) {
    log("warning:"@Instigator@"linkgun had"@LinkGun.Links@"links");
    LinkGun.Links = 0;
  }
  
  ls = LinkScale[Min(LinkGun.Links, 5)];
  
  if ( FlashEmitter != None ) {
    // set fuzzle flash color
    if ( LinkGun.Linking ) {
      if ( Instigator.PlayerReplicationInfo == None || Instigator.PlayerReplicationInfo.Team == None
          || Instigator.PlayerReplicationInfo.Team.TeamIndex == 0 )
        FlashEmitter.Skins[0] = Texture'XEffectMat.link_muz_red';
      else
        FlashEmitter.Skins[0] = Texture'XEffectMat.link_muz_blue';
    }
    else {
      if ( LinkGun.Links > 0 )
        FlashEmitter.Skins[0] = Texture'XEffectMat.link_muz_yellow';
      else
        FlashEmitter.Skins[0] = Texture'XEffectMat.link_muz_green';
    }
    
    // adjust muzzle flash size to link size
    FlashEmitter.mSizeRange[0] = FlashEmitter.default.mSizeRange[0] * (ls * 0.5 + 1);
    FlashEmitter.mSizeRange[1] = FlashEmitter.mSizeRange[0];
  }
  
  if ( LinkGun.Ammo[ThisModeNum] != None && LinkGun.Ammo[ThisModeNum].AmmoAmount >= AmmoPerFire
      && (UpTime > 0.0 || Instigator.Role < ROLE_Authority) ) {
    UpTime -= DeltaTime;
    LinkGun.GetViewAxes(X,Y,Z);
    
    // the to-hit trace always starts right in front of the eye
    StartTrace = Instigator.Location + Instigator.EyePosition() + X * Instigator.CollisionRadius;
    
    TraceRange = default.TraceRange + LinkGun.Links * 250;
    
    if ( Instigator.Role < ROLE_Authority ) {
      if ( Beam == None ) {
        ForEach DynamicActors(class'LinkBeamEffect', LB ) {
          if ( !LB.bDeleteMe && LB.Instigator != None && LB.Instigator == Instigator ) {
            Beam = LB;
            break;
          }
        }
      }
      if ( Beam != None )
        LockedPawn = Beam.LinkedPawn;
    }  
    if ( LockedPawn != None ) 
      TraceRange *= 1.5;
    
    if ( Instigator.Role == ROLE_Authority ) {
      B = Bot(Instigator.Controller);
      if ( B != None && B.Squad != None && PlayerController(B.Squad.SquadLeader) != None
          && B.Squad.SquadLeader.Pawn != None ) {
        if ( IsLinkable(B.Squad.SquadLeader.Pawn) && B.Squad.SquadLeader.Pawn.Weapon.FireMode[1].bIsFiring
            && VSize(B.Squad.SquadLeader.Pawn.Location - StartTrace) < TraceRange ) {
          Other = Trace(HitLocation, HitNormal, B.Squad.SquadLeader.Pawn.Location, StartTrace, true); 
          if ( Other == B.Squad.SquadLeader.Pawn ) {
            B.Focus = B.Squad.SquadLeader.Pawn;
            if ( B.Focus != LockedPawn )
              SetLinkTo(B.Squad.SquadLeader.Pawn); 
            B.SetRotation(Rotator(B.Focus.Location - StartTrace));
            X = Normal(B.Focus.Location - StartTrace);
          }
          else if ( B.Focus == B.Squad.SquadLeader.Pawn )
            bShouldStop = true;
        }
        else if ( B.Focus == B.Squad.SquadLeader.Pawn )
          bShouldStop = true;
      }
    }  
    if ( LockedPawn != None ) {
      EndTrace = LockedPawn.Location + LockedPawn.EyeHeight * vect(0,0,0.5); // beam ends at approx gun height
      if ( Instigator.Role == ROLE_Authority ) {
        V = Normal(EndTrace - StartTrace);
        if ( V dot X < LinkFlexibility || LockedPawn.Health <= 0 || LockedPawn.bDeleteMe
            || VSize(EndTrace - StartTrace) > 1.5 * TraceRange ) {
          SetLinkTo(None);
        }
      }
    }
    
    if ( LockedPawn == None ) {
      // FIXED HERE: Instigator.Controller.Target might be None
      if ( Bot(Instigator.Controller) != None && Instigator.Controller.Target != None ) {
        if ( bInitAimError ) {
          CurrentAimError = AdjustAim(StartTrace, AimError);
          bInitAimError = false;
        }
        else {
          BoundError();
          CurrentAimError.Yaw = CurrentAimError.Yaw + Instigator.Rotation.Yaw;
        }
        // smooth aim error changes
        Step = 7500.0 * DeltaTime;
        if ( DesiredAimError.Yaw ClockWiseFrom CurrentAimError.Yaw ) {
          CurrentAimError.Yaw += Step;
          if ( !(DesiredAimError.Yaw ClockWiseFrom CurrentAimError.Yaw) ) {
            CurrentAimError.Yaw = DesiredAimError.Yaw;
            DesiredAimError = AdjustAim(StartTrace, AimError);
          }
        }
        else {
          CurrentAimError.Yaw -= Step;
          if ( DesiredAimError.Yaw ClockWiseFrom CurrentAimError.Yaw ) {
            CurrentAimError.Yaw = DesiredAimError.Yaw;
            DesiredAimError = AdjustAim(StartTrace, AimError);
          }
        }
        CurrentAimError.Yaw = CurrentAimError.Yaw - Instigator.Rotation.Yaw;
        if ( BoundError() )
          DesiredAimError = AdjustAim(StartTrace, AimError);
        CurrentAimError.Yaw = CurrentAimError.Yaw + Instigator.Rotation.Yaw;
        
        Aim = Rotator(Instigator.Controller.Target.Location - StartTrace);
        
        Aim.Yaw = CurrentAimError.Yaw;
        
        // save difference
        CurrentAimError.Yaw = CurrentAimError.Yaw - Instigator.Rotation.Yaw;
      }
      else
        Aim = AdjustAim(StartTrace, AimError);
      
      X = Vector(Aim);
      EndTrace = StartTrace + TraceRange * X;
    }
    
    Other = Trace(HitLocation, HitNormal, EndTrace, StartTrace, true);
    if ( Other != None && Other != Instigator )
      EndEffect = HitLocation;
    else
      EndEffect = EndTrace;
    if ( Beam != None )
      Beam.EndEffect = EndEffect;
    
    if ( Instigator.Role < ROLE_Authority )
      return;
    
    if ( Other != None && Other != Instigator ) {
      // target can be linked to
      if ( IsLinkable(Other) ) {
        if ( Other != LockedPawn )
          SetLinkTo(Pawn(Other));
      
        if ( LockedPawn != None )
          LinkBreakTime = LinkBreakDelay;
      }
      else {
        // stop linking
        if ( LockedPawn != None ) {
          if ( LinkBreakTime <= 0.0 )
            SetLinkTo(None);
          else
            LinkBreakTime -= DeltaTime;
        }
        
        // beam is updated every frame, but damage is only done based on the firing rate
        if ( bDoHit ) {
          if ( Beam != None )
            Beam.bLockedOn = false;
          
          Instigator.MakeNoise(1.0);
          
          AdjustedDamage = Damage * (1.5 * Linkgun.Links + 1);
          
          if ( !Other.bWorldGeometry ) {
            if ( Level.Game.bTeamGame && Pawn(Other) != None && Pawn(Other).PlayerReplicationInfo != None
                && Pawn(Other).PlayerReplicationInfo.Team == Instigator.PlayerReplicationInfo.Team )
              AdjustedDamage = 0; // so even if friendly fire is on you can't hurt teammates
            
            /*if ( Other.Physics == PHYS_Falling || Other.Physics == PHYS_Flying || Other.Physics == PHYS_Swimming )
              MT = DeathMatch(Level.Game).LinkLockDownFactor * MomentumTransfer;
            else
              MT = 0.0;*/
            if ( Instigator.Controller != None )
              class'TournamentWeaponStats'.static.IncrementHits(Instigator.Controller, Weapon.Class, 1, -1);
            if ( TournamentPawn(Other) != None )
              TournamentPawn(Other).NextDamageMode = 1;
            Other.TakeDamage(AdjustedDamage, Instigator, HitLocation, MT * X, DamageType);
            
            if ( Beam != None )
              Beam.bLockedOn = true;
          }
        }
      } 
    }
    
    // FIXED HERE: B might be None
    if ( B != None && bShouldStop )
      B.StopFiring();
    else {
      // beam effect is created and destroyed when firing starts and stops 
      if ( Beam == None && bIsFiring )
        Beam = Spawn(class'TStatsLinkBeamEffect', Instigator);
      
      if ( Beam != None ) {
        if ( LinkGun.Linking )
          Beam.LinkColor = Instigator.PlayerReplicationInfo.Team.TeamIndex + 1;
        else
          Beam.LinkColor = 0;
        Beam.Links = LinkGun.Links;
        Beam.LinkedPawn = LockedPawn;
        Beam.bHitSomething = Other != None;
        Beam.EndEffect = EndEffect;
      }
    }      
  }
  else {
    StopFiring();
  }
  
  bStartFire = false;
  bDoHit = false;
}


//=============================================================================
// DoFireEffect
//
// Track weapon shots.
//=============================================================================

function DoFireEffect()
{
  if ( Instigator.Controller != None )
    class'TournamentWeaponStats'.static.IncrementShots(Instigator.Controller, Weapon.Class, 1);
  
  Super.DoFireEffect();
}
