//=============================================================================
// TStatsRocketProj
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Rocket.
//=============================================================================


class TStatsRocketProj extends RocketProj;


//=============================================================================
// Variables
//=============================================================================

var transient int GroupID;  // ID for tracking multi-projectile shot stats
var transient byte ModeNum; // whether the rocket was fired in primary or secondary mode

var int Countdown;
var bool bCanHitInstigator;
var vector WallNormal;
var bool bWeaponModifications;


//=============================================================================
// Replication
//=============================================================================

replication
{
  unreliable if ( Role == ROLE_Authority )
    WallNormal;
}


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
  if ( Instigator != None && Instigator.Controller != None )
    InstigatorController = Instigator.Controller;
}


//=============================================================================
// Timer
//
// Set falling physics.
//=============================================================================

simulated function Timer()
{
  Super.Timer();
  if ( bWeaponModifications && Countdown-- < 0 )
    bCanHitInstigator = True;
}


//=============================================================================
// TornOff
//
// Called when exploding on the server. Cause clientside damage.
//=============================================================================

simulated function TornOff()
{
  if ( Level.NetMode != NM_Client )
    return;
  
  BlowUp(Location);
  Destroy();
}


//=============================================================================
// BlowUp
//
// Track weapon stats and fix delayed damage.
//=============================================================================

simulated function BlowUp(vector HitLocation)
{
  local TStatsRocketWallExplosion Explosion;
  
  PlaySound(sound'WeaponSounds.BExplosion3',, 2.5 * TransientSoundVolume);
  if ( WallNormal != vect(0,0,0) ) {
    Explosion = Spawn(class'TStatsRocketWallExplosion',,, HitLocation + WallNormal * 16,
        rotator(WallNormal) + rot(16384,0,16384));  
    Explosion.RemoteRole = ROLE_None;
    if ( ExplosionDecal != None )
      Spawn(ExplosionDecal, Self,, Location, rotator(-WallNormal));
  }
  else {
    Spawn(class'TStatsRocketExplosion',,, HitLocation);  
  }
  if ( class'TournamentUtil'.static.StatsHurtRadius(Self, Damage, DamageRadius, MyDamageType,
      MomentumTransfer, HitLocation, ModeNum) && Role == ROLE_Authority )
    class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'RocketLauncher', ModeNum, GroupID);
	if ( Role == ROLE_Authority )
    MakeNoise(1.0);
}


//=============================================================================
// ProcessTouch
//
// Made non-simulated.
//=============================================================================

function ProcessTouch(Actor Other, Vector HitLocation)
{
  if ( (Other != Instigator || bCanHitInstigator) && (!Other.IsA('Projectile') || Other.bProjTarget) ) 
		Explode(HitLocation, vect(0,0,0));
}


//=============================================================================
// Explode
//
// Made non-simulated.
//=============================================================================

function Explode(vector HitLocation, vector HitNormal) 
{
  WallNormal = HitNormal;
  BlowUp(HitLocation);
  bUpdateSimulatedPosition = True;
  SetPhysics(PHYS_None);
  SetCollision(false, false, false);
  LifeSpan = 0.1;
  bTearOff = True;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bNetTemporary=False
  Countdown=5
}
