//=============================================================================
// TStatsRocketWallExplosion
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Emitter that creates an explosion effect with a smoke ring and small stuff
// flying around.
//=============================================================================


class TStatsRocketWallExplosion extends Emitter;


//=============================================================================
// PostBeginPlay
//
// Handle low framerate conditions.
//=============================================================================

simulated event PostBeginPlay()
{
  if ( Level.DetailMode == DM_Low || Level.DetailMode == DM_High && Level.bDropDetail )
    Emitters[2] = None;
  else if ( Emitters[2] != None && (Level.DetailMode == DM_High
      || Level.DetailMode == DM_SuperHigh && Level.bDropDetail) ) {
    Emitters[2].UseCollision = False;
    Emitters[2].LifetimeRange.Min = 1;
    Emitters[2].LifetimeRange.Max = 1.5;
  }
  
  if ( Level.bDropDetail )
    LightRadius = 7;
}


//=============================================================================
// PostNetBeginPlay
//
// Handle late replication.
//=============================================================================

simulated event PostNetBeginPlay()
{
  if ( Level.NetMode == NM_Client && LifeSpan < default.LifeSpan / 2 )
    Destroy();
}


//=============================================================================
// Tick
//
// Fade out the light.
//=============================================================================

simulated event Tick(float DeltaTime)
{
  LightBrightness = 600 * FMax(0.5 + LifeSpan - Default.LifeSpan, 0);
  if ( LightBrightness < 10 ) {
    LightType = LT_None;
    Disable('Tick');
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  bNoDelete=False
  RemoteRole=ROLE_SimulatedProxy
  bNetTemporary=True
  
  AutoDestroy=True
  LifeSpan=3.0
  
  bDynamicLight=true
  LightEffect=LE_QuadraticNonIncidence
  LightType=LT_Steady
  LightBrightness=255
  LightHue=28
  LightSaturation=90
  LightRadius=9
  LightPeriod=32
  LightCone=128
  
  Begin Object Class=SpriteEmitter Name=ExplosionParticles
    FadeOutStartTime=0.200000
    FadeOut=True
    RespawnDeadParticles=False
    StartLocationShape=PTLS_Sphere
    SphereRadiusRange=(Min=0.000000,Max=15.000000)
    SpinParticles=True
    StartSpinRange=(X=(Max=65535.000000),Y=(Max=65535.000000),Z=(Max=65535.000000))
    UseSizeScale=True
    UseRegularSizeScale=False
    SizeScale(0)=(RelativeSize=0.500000)
    SizeScale(1)=(RelativeTime=0.400000,RelativeSize=0.800000)
    SizeScale(2)=(RelativeTime=1.000000,RelativeSize=1.000000)
    StartSizeRange=(X=(Min=70.000000,Max=100.000000),Y=(Min=70.000000,Max=100.000000),Z=(Min=70.000000,Max=100.000000))
    UniformSize=True
    InitialParticlesPerSecond=1000.000000
    AutomaticInitialSpawning=False
    Texture=Texture'fEXpt'
    LifetimeRange=(Min=0.300000,Max=0.800000)
    StartVelocityRadialRange=(Min=-15.000000,Max=-5.000000)
    VelocityLossRange=(X=(Min=2.000000,Max=2.500000),Y=(Min=2.000000,Max=2.500000),Z=(Min=2.000000,Max=2.500000))
    GetVelocityDirectionFrom=PTVD_AddRadial
  End Object
  Emitters(0)=SpriteEmitter'ExplosionParticles'

  Begin Object Class=SpriteEmitter Name=ExplosionSmokeRing
    FadeOutStartTime=0.000000
    FadeOut=True
    FadeOutFactor=(X=0,Y=0,Z=0,W=1)
    ColorMultiplierRange=(X=(Min=0.800000,Max=0.900000),Y=(Min=0.800000,Max=0.900000),Z=(Min=0.800000,Max=0.900000))
    CoordinateSystem=PTCS_Relative
    MaxParticles=30
    RespawnDeadParticles=False
    StartLocationShape=PTLS_Polar
    StartLocationPolarRange=(Y=(Max=65535.000000),Z=(Min=40.000000,Max=40.000000))
    UseSizeScale=True
    UseRegularSizeScale=False
    SizeScale(0)=(RelativeSize=0.300000)
    SizeScale(1)=(RelativeTime=0.200000,RelativeSize=1.500000)
    SizeScale(2)=(RelativeTime=1.000000,RelativeSize=2.000000)
    StartSizeRange=(X=(Min=25.000000,Max=25.000000),Y=(Min=25.000000,Max=25.000000),Z=(Min=25.000000,Max=25.000000))
    UniformSize=True
    InitialParticlesPerSecond=1000.000000
    AutomaticInitialSpawning=False
    DrawStyle=PTDS_AlphaBlend
    Texture=Texture'SmokeAlphab_t'
    TextureUSubdivisions=4
    TextureVSubdivisions=4
    UseRandomSubdivision=True
    LifetimeRange=(Min=1.200000,Max=1.400000)
    StartVelocityRadialRange=(Min=100.000000,Max=100.000000)
    VelocityLossRange=(X=(Min=2.500000,Max=2.500000),Y=(Min=2.500000,Max=2.500000),Z=(Min=2.500000,Max=2.500000))
    GetVelocityDirectionFrom=PTVD_AddRadial
  End Object
  Emitters(1)=SpriteEmitter'ExplosionSmokeRing'
  
  Begin Object Class=SpriteEmitter Name=ExplosionCrapParticles
    Acceleration=(Z=-900.000000)
    UseCollision=True
    UseMaxCollisions=True
    DampingFactorRange=(X=(Min=0.300000,Max=0.500000),Y=(Min=0.300000,Max=0.500000),Z=(Min=0.300000,Max=0.500000))
    MaxCollisions=(Min=5.000000,Max=15.000000)
    UseColorScale=True
    ColorScale(0)=(Color=(B=255,G=255,R=255,A=255))
    ColorScale(1)=(RelativeTime=0.900000,Color=(B=255,G=255,R=255,A=255))
    ColorScale(2)=(RelativeTime=1.000000,Color=(B=255,G=255,R=255))
    FadeOutFactor=(X=0.000000,Y=0.000000,Z=0.000000)
    MaxParticles=50
    RespawnDeadParticles=False
    StartLocationShape=PTLS_Sphere
    SphereRadiusRange=(Min=10.000000,Max=10.000000)
    StartSizeRange=(X=(Min=2.000000,Max=6.000000),Y=(Min=2.000000,Max=6.000000),Z=(Min=2.000000,Max=6.000000))
    UniformSize=True
    InitialParticlesPerSecond=1000.000000
    AutomaticInitialSpawning=False
    DrawStyle=PTDS_AlphaBlend
    Texture=Texture'EmitterTextures.MultiFrame.rockchunks02'
    TextureUSubdivisions=4
    TextureVSubdivisions=4
    UseRandomSubdivision=True
    LifetimeRange=(Min=2.000000,Max=3.000000)
    StartVelocityRadialRange=(Min=-200.000000,Max=-700.000000)
    GetVelocityDirectionFrom=PTVD_AddRadial
  End Object
  Emitters(2)=SpriteEmitter'ExplosionCrapParticles'
}