//=============================================================================
// TStatsRocketProj
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Seeking Rocket.
//=============================================================================


class TStatsSeekingRocketProj extends TStatsRocketProj;


//=============================================================================
// Variables
//=============================================================================

var Actor Seeking;
var vector InitialDir;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( bNetInitial && Role==ROLE_Authority )
    Seeking, InitialDir;
}


//=============================================================================
// Timer
//
// Do the homing calculation.
//=============================================================================

simulated function Timer()
{
  local vector ForceDir;
  local float VelMag;
  
  if ( InitialDir == vect(0,0,0) )
    InitialDir = Normal(Velocity);
  
  Acceleration = vect(0,0,0);
  
  Super.Timer();
  
  if ( Seeking != None && Seeking != Instigator ) {
    // Do normal guidance to target.
    ForceDir = Normal(Seeking.Location - Location);
    
    if ( ForceDir Dot InitialDir > 0 ) {
      VelMag = VSize(Velocity);
      
      ForceDir = Normal(ForceDir * 0.5 * VelMag + Velocity);
      Velocity =  VelMag * ForceDir;  
      Acceleration += 5 * ForceDir; 
    }
    // Update rocket so it faces in the direction its going.
    SetRotation(rotator(Velocity));
  }
}


//=============================================================================
// PostBeginPlay
//
// Start the homing timer.
//=============================================================================

simulated function PostBeginPlay()
{
  Super.PostBeginPlay();
  SetTimer(0.1, true);
}


//=============================================================================
// BlowUp
//
// Track weapon stats and fix delayed damage.
//=============================================================================

simulated function BlowUp(vector HitLocation)
{
  if ( Seeking != None && VSize(Seeking.Location - HitLocation) - FMax(Seeking.CollisionRadius, CollisionHeight)
      < 1.5 * DamageRadius && FastTrace(Seeking.Location, HitLocation) )
    ModeNum = 2;  // seeking rocket might have hit its target
  
  Super.BlowUp(HitLocation);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  MyDamageType=class'DamTypeRocketHoming'
  LifeSpan=10.000000
  Countdown=7
}
