//=============================================================================
// TStatsShieldGun
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Implements a variable protection angle.
//=============================================================================


class TStatsShieldGun extends ShieldGun;


//=============================================================================
// Variables
//=============================================================================

var() float ProtectionAngle;
var() float MaxAbsorption;


//=============================================================================
// CheckReflect
//
// Check whether a hit was reflected by the shield.
//=============================================================================

function bool CheckReflect(vector HitLocation, out vector RefNormal, int AmmoDrain)
{
  local vector HitDir;
  local vector FaceDir;
  
  if ( !FireMode[1].bIsFiring || Ammo[0].AmmoAmount == 0 )
    return false;
  
  FaceDir = Vector(Instigator.Controller.Rotation);
  HitDir = Normal(Instigator.Location - HitLocation + vect(0,0,8));
  
  RefNormal = FaceDir;
  
  if ( FaceDir dot HitDir < ProtectionAngle ) {
    if ( AmmoDrain > 0 )
      Ammo[0].UseAmmo(AmmoDrain);
    return true;
  }
  return false;
}


//=============================================================================
// AdjustPlayerDamage
//
// Variable maximum damage absorption.
//=============================================================================

function AdjustPlayerDamage(out int Damage, Pawn InstigatedBy, vector HitLocation, out vector Momentum, class<DamageType> DamageType)
{
  local int Drain;
  local vector Reflect;
  local vector HitNormal;
  local float DamageMax;
  
  DamageMax = MaxAbsorption;
  if ( DamageType == class'Fell' )
    DamageMax = 20.0;
  else if ( !DamageType.default.bArmorStops || DamageType == class'DamTypeShieldImpact' && InstigatedBy == Instigator )
    return;
  
  if ( CheckReflect(HitLocation, HitNormal, 0) ) {
    Drain = Min(Ammo[1].AmmoAmount * 2, Damage);
    Drain = Min(Drain, DamageMax);
    Reflect = MirrorVectorByNormal(Normal(Location - HitLocation), vector(Instigator.Rotation));
    Damage -= Drain;
    Momentum *= 1.25;
    Ammo[1].UseAmmo(Drain / 2);
    DoReflectEffect(Drain / 2);
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  ProtectionAngle=-0.37
  MaxAbsorption=100.0
}