//=============================================================================
// TStatsShockProjectile
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Fixes bugs in the ShockProjectile class and tracks weapon stats.
//=============================================================================


class TStatsShockProjectile extends ShockProjectile;


//=============================================================================
// Variables
//=============================================================================

var bool bCanHitInstigator;
var bool bWeaponModifications;


//=============================================================================
// PostBeginPlay
//
// Set InstigatorController.
//=============================================================================

simulated function PostBeginPlay()
{
  local TournamentGameReplicationInfo TGRI;
  
  foreach DynamicActors(class'TournamentGameReplicationInfo', TGRI)
    if ( TGRI.bWeaponModifications )
      bWeaponModifications = true;
  
  Super.PostBeginPlay();
  if ( Instigator != None && Instigator.Controller != None )
    InstigatorController = Instigator.Controller;
}


//=============================================================================
// Timer
//
// Allow instigator hits.
//=============================================================================

simulated function Timer()
{
  Super.Timer();
  bCanHitInstigator = bWeaponModifications;
}


//=============================================================================
// ProcessTouch
//
// Allow instigator hits.
//=============================================================================

simulated function ProcessTouch(Actor Other, vector HitLocation)
{
  local vector X, RefNormal, RefDir;
  
  if ( Other == Instigator && !bCanHitInstigator)
    return;
  
  if ( Other.IsA('xPawn') && xPawn(Other).CheckReflect(HitLocation, RefNormal, Damage * 0.25) ) {
    if ( Role == ROLE_Authority ) {
      X = Normal(Velocity);
      RefDir = RefNormal;
      Spawn(Class, Other,, HitLocation + RefDir * 20, rotator(RefDir));
    }
    DestroyTrails();
    Destroy();
  }
  else if ( !Other.IsA('Projectile') || Other.bProjTarget ) {
    Explode(HitLocation, Normal(HitLocation - Other.Location));
  }
}


//=============================================================================
// Explode
//
// Track weapon stats and fix collision with other projectiles.
//=============================================================================

simulated function Explode(vector HitLocation, vector HitNormal)
{
  if ( class'TournamentUtil'.static.StatsHurtRadius(Self, Damage, DamageRadius, MyDamageType,
      MomentumTransfer, HitLocation, 1) && Role == ROLE_Authority )
    class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'ShockRifle', 1, -1);
  
  PlaySound(ImpactSound, SLOT_Misc);
  if ( EffectIsRelevant(Location,false) )
    Spawn(class'ShockExplosion',,, Location);
  
  if ( bWeaponModifications )
    LifeSpan = 0.001;  // destroy after all this so the other projectile explodes as well
  else
    Destroy();
}


//=============================================================================
// TornOff
//
// Called when exploding on the server. Cause clientside damage.
//=============================================================================

simulated function TornOff()
{
  if ( Level.NetMode != NM_Client )
    return;
  
  HurtRadius(ComboDamage, ComboRadius, class'DamTypeShockCombo', ComboMomentumTransfer, Location);
  Destroy();
}


//=============================================================================
// SuperExplosion
//
// Track weapon stats.
//=============================================================================

function SuperExplosion()
{
  local actor HitActor;
  local vector HitLocation, HitNormal;
  
  if ( class'TournamentUtil'.static.StatsHurtRadius(Self, ComboDamage, ComboRadius, class'DamTypeShockCombo',
      ComboMomentumTransfer, Location, 2) && Role == ROLE_Authority )
    class'TournamentWeaponStats'.static.IncrementHits(InstigatorController, class'ShockRifle', 2, -1);
  
  Spawn(class'ShockCombo');
  if ( Level.NetMode != NM_DedicatedServer && EffectIsRelevant(Location,false) ) {
    HitActor = Trace(HitLocation, HitNormal, Location - Vect(0,0,120), Location, false);
    if ( HitActor != None )
      Spawn(class'ComboDecal', self,, HitLocation, rotator(vect(0,0,-1)));
  }
  PlaySound(ComboSound, SLOT_None, 1.0,, 800);
  DestroyTrails();
  if ( bWeaponModifications ) {
    LifeSpan = 0.1;  // stay around for a while so clients can do damage to ragdolls etc.
    SetPhysics(PHYS_None);
    SetCollision(false, false, false);
    bUpdateSimulatedPosition = True;
    bTearOff = True;
  }
  else
    Destroy();
}


//=============================================================================
// state WaitForCombo
// Tick
//
// Wait for combo.
//=============================================================================

state WaitForCombo
{
  function Tick(float DeltaTime)
  {
    if ( !bWeaponModifications ) {
      Super.Tick(DeltaTime);
      return;
    }
    
    if ( ComboTarget == None || ComboTarget.bDeleteMe || Instigator == None || Instigator.Health <= 0 ) {
      GotoState('');
      return;
    }
    else if ( ShockRifle(Instigator.Weapon) == None )
      return; // might still switch back to shock rifle
    
    if ( VSize(ComboTarget.Location - Location) <= 0.5 * ComboRadius + ComboTarget.CollisionRadius 
        || Velocity Dot (ComboTarget.Location - Location) <= 0 ) {
      ShockRifle(Instigator.Weapon).DoCombo();
      GotoState('');
      return;
    }
  }
}
