//=============================================================================
// TStatsSniperFire
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tracks weapon stats.
//=============================================================================


class TStatsSniperFire extends SniperFire;


//=============================================================================
// DoTrace
//
// Track weapon stats.
//=============================================================================

function DoTrace(vector Start, rotator Dir)
{
  local vector X, Y, Z, End, HitLocation, HitNormal, RefNormal;
  local Actor Other, mainArcHitTarget;
  local int Damage, ReflectNum, arcsRemaining;
  local bool bDoReflect;
  local xEmitter hitEmitter;
  local class<Actor> tmpHitEmitClass;
  local float tmpTraceRange, dist;
  local vector arcEnd, mainArcHit;
  local byte ModeNum;
  local int GroupID;
  
  GroupID = class'TournamentWeaponStats'.static.IncrementShots(Instigator.Controller, Weapon.Class, 0);
  
  Weapon.GetViewAxes(X, Y, Z);
  if ( Weapon.WeaponCentered() )
    arcEnd = Instigator.Location + Weapon.EffectOffset.X * X + 1.5 * Weapon.EffectOffset.Z * Z; 
  else
    arcEnd = Instigator.Location + Instigator.CalcDrawOffset(Weapon) + Weapon.EffectOffset.X * X
        + Weapon.Hand * Weapon.EffectOffset.Y * Y + Weapon.EffectOffset.Z * Z; 
  
  arcsRemaining = NumArcs;
  
  tmpHitEmitClass = HitEmitterClass;
  tmpTraceRange = TraceRange;
  
  ReflectNum = 0;
  while (true) {
    bDoReflect = false;
    X = vector(Dir);
    End = Start + tmpTraceRange * X;
    Other = Trace(HitLocation, HitNormal, End, Start, true);
    
    if ( Other != None && (Other != Instigator || ReflectNum > 0) ) {
      if ( bReflective && Other.IsA('xPawn') && xPawn(Other).CheckReflect(HitLocation, RefNormal, DamageMin * 0.25) ) {
        bDoReflect = true;
      }
      else if ( Other != mainArcHitTarget ) {
        if ( !Other.bWorldGeometry ) {
          Damage = (DamageMin + Rand(DamageMax - DamageMin)) * DamageAtten;
          if ( Pawn(Other) != None && arcsRemaining == NumArcs 
              && Other.GetClosestBone(HitLocation, X, dist, 'head', HeadShotRadius) == 'head' ) {
            if ( Pawn(Other) != None && Pawn(Other).Controller != None && Pawn(Other).Controller.bIsPlayer )
              class'TournamentWeaponStats'.static.IncrementHits(Instigator.Controller,
                  Weapon.Class, 2, GroupID);
            if ( TournamentPawn(Other) != None )
              TournamentPawn(Other).NextDamageMode = 2;
            Other.TakeDamage(Damage * HeadShotDamageMult, Instigator, HitLocation, Momentum * X, DamageTypeHeadShot);
          }
          else {
            if ( arcsRemaining < NumArcs ) {
              Damage *= SecDamageMult;
              ModeNum = 1;
            }
            if ( Pawn(Other) != None && Pawn(Other).Controller != None && Pawn(Other).Controller.bIsPlayer )
              class'TournamentWeaponStats'.static.IncrementHits(Instigator.Controller,
                  Weapon.Class, ModeNum, GroupID);
            if ( TournamentPawn(Other) != None )
              TournamentPawn(Other).NextDamageMode = ModeNum;
            Other.TakeDamage(Damage, Instigator, HitLocation, Momentum * X, DamageType);
          }
        }
        else
          HitLocation = HitLocation + 2.0 * HitNormal;
      }
    }
    else {
      HitLocation = End;
      HitNormal = Normal(Start - End);
    }
    hitEmitter = xEmitter(Spawn(tmpHitEmitClass,,, arcEnd, rotator(HitNormal)));
    if ( hitEmitter != None )
      hitEmitter.mSpawnVecA = HitLocation;
    
    if ( arcsRemaining == NumArcs ) {
      mainArcHit = HitLocation + (HitNormal * 2.0);
      if ( Other != None && !Other.bWorldGeometry )
        mainArcHitTarget = Other;
    }
    
    if ( bDoReflect && ++ReflectNum < 4 ) {
      Start = HitLocation;
      Dir = rotator(X - 2.0 * RefNormal * (X dot RefNormal));
    }
    else if ( arcsRemaining > 0 ) {
      arcsRemaining--;
      
      // done parent arc, now move trace point to arc trace hit location and try child arcs from there
      Start = mainArcHit;
      Dir = rotator(VRand());
      tmpHitEmitClass = SecHitEmitterClass;
      tmpTraceRange = SecTraceDist;
      arcEnd = mainArcHit;
    }
    else {
      break;
    }
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  DamageTypeHeadShot=class'Tournament.DamTypeSniperHeadShot'
}