//=============================================================================
// TStatsSuperShockBeam
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Super shock beam with correct team colors.
//=============================================================================


class TStatsSuperShockBeam extends SuperShockBeamEffect;


//=============================================================================
// Variables
//=============================================================================

var class<ShockBeamCoil>      BeamCoilClasses[8];
var class<ShockMuzFlash>      MuzFlashClasses[8];
var class<ShockMuzFlash3rd>   MuzFlash3Classes[8];
var class<ShockImpactFlare>   ImpactFlareClasses[8];
var class<ShockImpactRing>    ImpactRingClasses[8];
var class<ShockExplosionCore> ExplosionCoreClasses[8];
var byte                      LightHues[8];
var Material                  BeamSkins[8];

var int ColorIndex;
var PlayerReplicationInfo MyPRI;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    MyPRI;
}


//=============================================================================
// SetColorIndex
//
// Returns the beam's color index.
//=============================================================================

simulated function SetColorIndex()
{
  local PlayerReplicationInfo LocalPRI;
  local int LocalTeamIndex, MyTeamIndex;
  
  if ( Level.GetLocalPlayerController() != None )
    LocalPRI = Level.GetLocalPlayerController().PlayerReplicationInfo;
  
  if ( LocalPRI == None || LocalPRI.bOnlySpectator )
    LocalTeamIndex = -2;
  else if ( LocalPRI.Team == None )
    LocalTeamIndex = -1;
  else
    LocalTeamIndex = LocalPRI.Team.TeamIndex;
  
  if ( MyPRI == None )
    MyTeamIndex = -2;
  else if ( MyPRI.Team == None )
    MyTeamIndex = -1;
  else
    MyTeamIndex = MyPRI.Team.TeamIndex;
  
  if ( LocalTeamIndex < 0 && MyTeamIndex >= 0 || LocalTeamIndex == -2 ) { // spectating
    ColorIndex = MyTeamIndex + 1;
  }
  else if ( LocalTeamIndex == -1 && MyTeamIndex < 0 ) { // playing deathmatch
    if ( LocalPRI == MyPRI )
      ColorIndex = class'TournamentClientSettings'.default.TeamEffectsColor + 1;
    else
      ColorIndex = class'TournamentClientSettings'.default.EnemyEffectsColor + 1;
  }
  else {
    if ( LocalTeamIndex == MyTeamIndex )
      ColorIndex = class'TournamentClientSettings'.default.TeamEffectsColor + 1;
    else
      ColorIndex = class'TournamentClientSettings'.default.EnemyEffectsColor + 1;
  }
}


//=============================================================================
// PostBeginPlay
//
// Sets the beam color serverside.
//=============================================================================

simulated function PostBeginPlay()
{
  if ( Role == ROLE_Authority ) {
    if ( TournamentPawn(Owner) != None )
      ColorIndex = TournamentPawn(Owner).EffectsColorIndex;
    if ( Pawn(Owner) != None )
      MyPRI = Pawn(Owner).PlayerReplicationInfo;
  }
  Super.PostBeginPlay();
}


//=============================================================================
// PostNetBeginPlay
//
// Sets the beam color clientside.
//=============================================================================

simulated function PostNetBeginPlay()
{
  SetColorIndex();
  mRegen = True;
  Skins[0] = BeamSkins[ColorIndex];
  LightHue = LightHues[ColorIndex];
  if ( ColorIndex == 7 )
    LightSaturation = 0;
  Super.PostNetBeginPlay();
}


//=============================================================================
// SpawnImpactEffects
//
// Spawn impact effects based on the team color.
//=============================================================================

simulated function SpawnImpactEffects(rotator HitRot, vector EffectLoc)
{
  Spawn(ImpactFlareClasses[ColorIndex],,, EffectLoc, HitRot);
  Spawn(ImpactRingClasses[ColorIndex],,, EffectLoc, HitRot);
  Spawn(class'ShockImpactScorch',,, EffectLoc, Rotator(-HitNormal));
  Spawn(ExplosionCoreClasses[ColorIndex],,, EffectLoc + HitNormal * 8, HitRot);
}


//=============================================================================
// SpawnEffects
//
// Spawn beam effects based on the team color.
//=============================================================================

simulated function SpawnEffects()
{
  local ShockBeamCoil Coil;
  local xWeaponAttachment Attachment;
  
  if ( Instigator != None ) {
    if ( Instigator.IsFirstPerson() ) {
      if ( Instigator.Weapon != None && Instigator.Weapon.Instigator == Instigator )
        SetLocation(Instigator.Weapon.GetEffectStart());
      else
        SetLocation(Instigator.Location);
      Spawn(MuzFlashClasses[ColorIndex],,, Location);
    }
    else {
      Attachment = xPawn(Instigator).WeaponAttachment;
      if ( Attachment != None && Level.TimeSeconds - Attachment.LastRenderTime < 1 )
        SetLocation(Attachment.GetTipLocation());
      else
        SetLocation(Instigator.Location + Instigator.EyeHeight * vect(0,0,1)
            + Normal(mSpawnVecA - Instigator.Location) * 25.0); 
      Spawn(MuzFlash3Classes[ColorIndex]);
    }
  }
  
  if ( EffectIsRelevant(mSpawnVecA + HitNormal * 2, false) && HitNormal != vect(0,0,0) )
    SpawnImpactEffects(Rotator(HitNormal), mSpawnVecA + HitNormal * 2);
  
  if ( !Level.bDropDetail && Level.DetailMode != DM_Low && VSize(Location - mSpawnVecA) > 40 ) {
    Coil = Spawn(BeamCoilClasses[ColorIndex],,, Location, Rotation);
    if ( Coil != None )
      Coil.mSpawnVecA = mSpawnVecA;
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  mRegen=False
  
  BeamCoilClasses(0)=class'ShockBeamCoilB'
  BeamCoilClasses(1)=class'ShockBeamCoilRed'
  BeamCoilClasses(2)=class'Tournament.ShockBeamCoilBlue'
  BeamCoilClasses(3)=class'ShockBeamCoilGreen'
  BeamCoilClasses(4)=class'ShockBeamCoilGold'
  BeamCoilClasses(5)=class'ShockBeamCoilCyan'
  BeamCoilClasses(6)=class'ShockBeamCoilMagenta'
  BeamCoilClasses(7)=class'ShockBeamCoilWhite'
  
  MuzFlashClasses(0)=class'ShockMuzFlashB'
  MuzFlashClasses(1)=class'ShockMuzFlashRed'
  MuzFlashClasses(2)=class'ShockMuzFlashBlue'
  MuzFlashClasses(3)=class'ShockMuzFlashGreen'
  MuzFlashClasses(4)=class'ShockMuzFlashGold'
  MuzFlashClasses(5)=class'ShockMuzFlashCyan'
  MuzFlashClasses(6)=class'ShockMuzFlashMagenta'
  MuzFlashClasses(7)=class'ShockMuzFlashWhite'
  
  MuzFlash3Classes(0)=class'ShockMuzFlashB3rd'
  MuzFlash3Classes(1)=class'ShockMuzFlash3rdRed'
  MuzFlash3Classes(2)=class'ShockMuzFlash3rdBlue'
  MuzFlash3Classes(3)=class'ShockMuzFlash3rdGreen'
  MuzFlash3Classes(4)=class'ShockMuzFlash3rdGold'
  MuzFlash3Classes(5)=class'ShockMuzFlash3rdCyan'
  MuzFlash3Classes(6)=class'ShockMuzFlash3rdMagenta'
  MuzFlash3Classes(7)=class'ShockMuzFlash3rdWhite'
  
  ImpactFlareClasses(0)=class'ShockImpactFlareB'
  ImpactFlareClasses(1)=class'ShockImpactFlareRed'
  ImpactFlareClasses(2)=class'ShockImpactFlareBlue'
  ImpactFlareClasses(3)=class'ShockImpactFlareGreen'
  ImpactFlareClasses(4)=class'ShockImpactFlareGold'
  ImpactFlareClasses(5)=class'ShockImpactFlareCyan'
  ImpactFlareClasses(6)=class'ShockImpactFlareMagenta'
  ImpactFlareClasses(7)=class'ShockImpactFlareWhite'
  
  ImpactRingClasses(0)=class'ShockImpactRingB'
  ImpactRingClasses(1)=class'ShockImpactRingRed'
  ImpactRingClasses(2)=class'ShockImpactRingBlue'
  ImpactRingClasses(3)=class'ShockImpactRingGreen'
  ImpactRingClasses(4)=class'ShockImpactRingGold'
  ImpactRingClasses(5)=class'ShockImpactRingCyan'
  ImpactRingClasses(6)=class'ShockImpactRingMagenta'
  ImpactRingClasses(7)=class'ShockImpactRingWhite'
  
  ExplosionCoreClasses(0)=class'ShockExplosionCoreB'
  ExplosionCoreClasses(1)=class'ShockExplosionCoreRed'
  ExplosionCoreClasses(2)=class'ShockExplosionCoreBlue'
  ExplosionCoreClasses(3)=class'ShockExplosionCoreGreen'
  ExplosionCoreClasses(4)=class'ShockExplosionCoreGold'
  ExplosionCoreClasses(5)=class'ShockExplosionCoreCyan'
  ExplosionCoreClasses(6)=class'ShockExplosionCoreMagenta'
  ExplosionCoreClasses(7)=class'ShockExplosionCoreWhite'
  
  LightHues(0)=0
  LightHues(1)=0
  LightHues(2)=170
  LightHues(3)=85
  LightHues(4)=45
  LightHues(5)=128
  LightHues(6)=230
  LightHues(7)=128
  
  BeamSkins(0)=Material'RedSuperShockBeam'
  BeamSkins(1)=Material'SuperShockBeamRed'
  BeamSkins(2)=Material'SuperShockBeamBlue'
  BeamSkins(3)=Material'SuperShockBeamGreen'
  BeamSkins(4)=Material'SuperShockBeamGold'
  BeamSkins(5)=Material'SuperShockBeamCyan'
  BeamSkins(6)=Material'SuperShockBeamMagenta'
  BeamSkins(7)=Material'SuperShockBeamWhite'
}