//=============================================================================
// TStatsTransRecall
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tracks weapon stats.
//=============================================================================


class TStatsTransRecall extends TransRecall;


//=============================================================================
// Variables
//=============================================================================

var Material TransOverlayMaterials[8];


//=============================================================================
// Translocate
//
// Teleport the player.
//=============================================================================

function Translocate()
{
  local TransBeacon TransBeacon;
  local Actor HitActor;
  local Vector HitNormal,HitLocation,dest,Vel2D;
  local Vector PrevLocation,Diff;
  local xPawn XP;
  local controller C;
  local bool bFailedTransloc;
  local int EffectNum;
  
  if ( Instigator == None || Translauncher(Weapon) == None )
    return;
  TransBeacon = TransLauncher(Weapon).TransBeacon;
  
  // gib if the translocator is disrupted
  if ( TransBeacon.Disrupted() ) {
    UnrealMPGameInfo(Level.Game).SpecialEvent(Instigator.PlayerReplicationInfo, "translocate_gib");
    bGibMe = true; // delay gib to avoid destroying player and weapons right away in the middle of all this
    return;
  }
  
  dest = TransBeacon.Location;
  if ( TransBeacon.Physics == PHYS_None )
    dest += vect(0,0,1) * Instigator.CollisionHeight;
  else
    dest += vect(0,0,0.5) * Instigator.CollisionHeight;
  HitActor = Trace(HitLocation, HitNormal, dest, TransBeacon.Location, true);
  if ( HitActor != None )
    dest = TransBeacon.Location;    
  
  TransBeacon.SetCollision(false, false, false);    
  
  if ( Instigator.PlayerReplicationInfo.HasFlag != None )
    Instigator.PlayerReplicationInfo.HasFlag.Drop(0.5 * Instigator.Velocity);
  
  PrevLocation = Instigator.Location;
  
  // verify won't telefrag teammate or recently spawned player
  for (C = Level.ControllerList; C != None; C = C.NextController)
    if ( C.Pawn != None && C.Pawn != Instigator && (C.SameTeamAs(Instigator.Controller)
        || Level.TimeSeconds - C.Pawn.SpawnTime < DeathMatch(Level.Game).SpawnProtectionTime) ) { 
      Diff = Dest - C.Pawn.Location;
      if ( Abs(Diff.Z) < C.Pawn.CollisionHeight + 2 * Instigator.CollisionHeight ) {
        Diff.Z = 0;
        if ( VSize(Diff) < C.Pawn.CollisionRadius + Instigator.CollisionRadius + 8 ) {
          bFailedTransloc = true;
          break;
        }
      }
    }
  
  if ( !bFailedTransloc && AttemptTranslocation(dest, TransBeacon) ) {
    TransLauncher(Weapon).ReduceAmmo();
    
    // spawn out
    XP = xPawn(Instigator);
    if ( XP != None )
      XP.DoTranslocateOut(PrevLocation);
    
    // bound XY velocity to prevent cheats
    Vel2D = Instigator.Velocity * vect(1,1,0);
    Vel2D = Normal(Instigator.Velocity) * FMin(Instigator.GroundSpeed, VSize(Vel2D));
    Vel2D.Z = Instigator.Velocity.Z;
    Instigator.Velocity = Vel2D;
    
    if ( TournamentPawn(Instigator) != None )
      EffectNum = TournamentPawn(Instigator).EffectsColorIndex;
    else if ( Instigator.PlayerReplicationInfo.Team != None )
      EffectNum = Instigator.PlayerReplicationInfo.Team.TeamIndex;
    
    Instigator.SetOverlayMaterial(TransOverlayMaterials[EffectNum], 1.0, false);
    Instigator.PlayTeleportEffect(false, false);
    
    if ( !Instigator.PhysicsVolume.bWaterVolume ) {
      if ( Bot(Instigator.Controller) != None ) {
        Instigator.Velocity.X = 0;
        Instigator.Velocity.Y = 0;
        Instigator.Velocity.Z = -150;
        Instigator.Acceleration = vect(0,0,0);
      }
      Instigator.SetPhysics(PHYS_Falling);
    }
  }
  else if ( PlayerController(Instigator.Controller) != None )
    PlayerController(Instigator.Controller).ClientPlaySound(TransFailedSound);
  
  TransBeacon.Destroy();
  TransLauncher(Weapon).TransBeacon = None;
  TransLauncher(Weapon).ViewPlayer();
}


//=============================================================================
// ModeDoFire
//
// Check for disruption kill and track weapon stats.
//=============================================================================

function ModeDoFire()
{
  local TransBeacon TransBeacon;
  local TournamentPawn Disruptor;
  
  Super(ProjectileFire).ModeDoFire();
  
  if ( Role == ROLE_Authority && bGibMe ) {
    TransBeacon = TransLauncher(Weapon).TransBeacon;
    TransLauncher(Weapon).TransBeacon = None;
    TransLauncher(Weapon).ViewPlayer();
    Disruptor = TournamentPawn(TransBeacon.Disruptor);
    if ( Disruptor != None ) {
      Disruptor.TelefragDamage = class'DamTypeTelefrag';
      Disruptor.TelefragMode = 1;
    }
    Instigator.GibbedBy(TransBeacon.Disruptor);
    if ( Disruptor != None )
      Disruptor.TelefragDamage = None;
    TransBeacon.Destroy();
    bGibMe = false;
  }
}


//=============================================================================
// DoFireEffect
//
// Set instigator's for catching telefrag stats points.
//=============================================================================

function DoFireEffect()
{
  if ( TournamentPawn(Instigator) != None ) {
    TournamentPawn(Instigator).TelefragDamage = class'DamTypeTelefrag';
    TournamentPawn(Instigator).TelefragMode = 0;
  }
  Super.DoFireEffect();
  
  if ( TournamentPawn(Instigator) != None )
    TournamentPawn(Instigator).TelefragDamage = None;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  TransOverlayMaterials(0)=Material'PlayerTransRed'
  TransOverlayMaterials(1)=Material'PlayerTransRed'
  TransOverlayMaterials(2)=Material'PlayerTrans'
  TransOverlayMaterials(3)=Material'PlayerTransShaderGreen'
  TransOverlayMaterials(4)=Material'PlayerTransShaderGold'
  TransOverlayMaterials(5)=Material'PlayerTransShaderCyan'
  TransOverlayMaterials(6)=Material'PlayerTransShaderMagenta'
  TransOverlayMaterials(7)=Material'PlayerTransShaderWhite'
}