//=============================================================================
// TournamentDeathMessage
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Death message class for Tournament gametypes.
//=============================================================================


class TournamentDeathMessage extends xDeathMessage
    abstract;


//=============================================================================
// Variables
//=============================================================================

var array<class<DamageType> > DamageTypes;      // so death messages are only reset once


//=============================================================================
// GetString
//
// Returns the display string for this message.
//=============================================================================

static function string GetString(optional int Switch, optional PlayerReplicationInfo RelatedPRI_1, 
    optional PlayerReplicationInfo RelatedPRI_2, optional Object OptionalObject)
{
  local class<DamageType> DamageType;
  
  DamageType = class<DamageType>(OptionalObject);
  if ( DamageType != None )
    ResetDamageType(DamageType);
  return Super.GetString(Switch % 2, RelatedPRI_1, RelatedPRI_2, OptionalObject);
}



//=============================================================================
// ResetDamageType
//
// Resets the damage type's death messages.
//=============================================================================

static function ResetDamageType(class<DamageType> DamageType)
{
  local int i;
  local string s;
  
  for (i = 0; i < default.DamageTypes.Length; i++)
    if ( default.DamageTypes[i] == DamageType )
      return;
  
  default.DamageTypes[default.DamageTypes.Length] = DamageType;
  s = class'TournamentUtil'.static.GetLocalizedString(DamageType, "DeathString");
  if ( s != "" )
    DamageType.default.DeathString = s;
  s = class'TournamentUtil'.static.GetLocalizedString(DamageType, "MaleSuicide");
  if ( s != "" )
    DamageType.default.MaleSuicide = s;
  s = class'TournamentUtil'.static.GetLocalizedString(DamageType, "FemaleSuicide");
  if ( s != "" )
    DamageType.default.FemaleSuicide = s;
}


//=============================================================================
// ClientReceive
//
// Called when the message is received on a client.
//=============================================================================

static function ClientReceive(PlayerController P, optional int Switch, optional PlayerReplicationInfo RelatedPRI_1,
    optional PlayerReplicationInfo RelatedPRI_2, optional Object OptionalObject)
{
  if ( Switch % 2 == 1 ) {
    if ( !Default.bNoConsoleDeathMessages && P.myHUD != None )
      P.myHUD.LocalizedMessage(Default.Class, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
  }
  else {
    if ( RelatedPRI_1 == P.PlayerReplicationInfo || P.IsA('DemoRecSpectator') && P.RealViewTarget != None
        && P.RealViewTarget.PlayerReplicationInfo == RelatedPRI_1 ) {
      // Interdict and send the child message instead.
      P.ReceiveLocalizedMessage(Default.ChildMessage, Switch % 2, RelatedPRI_1, RelatedPRI_2, OptionalObject);
      if ( !Default.bNoConsoleDeathMessages && P.myHUD != None )
        P.myHUD.LocalizedMessage(Default.Class, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
      
      // check multikills
      if ( P.Role == ROLE_Authority && UnrealPlayer(P) != None ) {
        // multikills checked already in LogMultiKills()
        if ( UnrealPlayer(P).MultiKillLevel > 0 )
          P.ReceiveLocalizedMessage(class'MultiKillMessage', UnrealPlayer(P).MultiKillLevel);
      }
      else if ( UnrealPlayer(P) != None ) {
        if ( RelatedPRI_1 != RelatedPRI_2 && RelatedPRI_2 != None
            && (RelatedPRI_2.Team == None || RelatedPRI_1.Team != RelatedPRI_2.Team) ) {
          if ( P.Level.TimeSeconds - UnrealPlayer(P).LastKillTime < 4 ) {
            UnrealPlayer(P).MultiKillLevel++;
            P.ReceiveLocalizedMessage(class'MultiKillMessage', UnrealPlayer(P).MultiKillLevel);
          } 
          else
            UnrealPlayer(P).MultiKillLevel = 0;
          UnrealPlayer(P).LastKillTime = P.Level.TimeSeconds;
        }
        else
          UnrealPlayer(P).MultiKillLevel = 0;
      }
    } 
    else if ( RelatedPRI_2 == P.PlayerReplicationInfo || P.IsA('DemoRecSpectator') && P.RealViewTarget != None
        && P.RealViewTarget.PlayerReplicationInfo == RelatedPRI_2 ) {
      P.ReceiveLocalizedMessage(class'xVictimMessage', 0, RelatedPRI_1);
      if ( !Default.bNoConsoleDeathMessages && P.myHUD != None )
        P.myHUD.LocalizedMessage(Default.Class, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
    } 
    else if ( !Default.bNoConsoleDeathMessages && P.myHUD != None )
      P.myHUD.LocalizedMessage(Default.Class, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
  }
}