//=============================================================================
// TournamentDemoRecSpectator
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// 
//=============================================================================


class TournamentDemoRecSpectator extends DemoRecSpectator;

var bool bTempNoBehindView;


//=============================================================================
// ClientVoiceMessage
//
// Fix for Accessed None log spam while recording server demos.
//=============================================================================

function ClientVoiceMessage(PlayerReplicationInfo Sender, PlayerReplicationInfo Recipient, name MessageType, byte MessageID)
{
  if ( Player != None )
    Super.ClientVoiceMessage(Sender, Recipient, MessageType, MessageID);
}


//=============================================================================
// state Spectating
// PlayerTick
//
// Fix for Accessed None log spam while recording server demos.
//=============================================================================

auto state Spectating
{
  event PlayerTick(float DeltaTime)
  {
    Super(UnrealPlayer).PlayerTick(DeltaTime);
    
    // attempt to find a player to view.
    if ( Role == ROLE_AutonomousProxy && (RealViewTarget == None || RealViewTarget == Self) && !bFoundPlayer ) {
      DemoViewNextPlayer();
      if ( RealViewTarget != None && RealViewTarget != Self ) 
        bFoundPlayer = true;    
    }
    
    // hack to go to 3rd person during deaths
    if ( RealViewTarget != None && RealViewTarget.Pawn == None ) {
      if( !bTempBehindView ) {
        bTempBehindView = true;
        bBehindView = true;
      }
    }
    else if ( bTempBehindView ) {
      bBehindView = false;
      bTempBehindView = false;
    }
    
    // hack to go to 1st person while piloting a redeemer
    if ( (bBehindView || bTempNoBehindView) && (ViewTarget.IsA('Projectile') || ViewTarget.IsA('RedeemerWarhead')) ) {
      if( !bTempNoBehindView ) {
        bTempNoBehindView = true;
        bBehindView = false;
      }
    }
    else if ( bTempNoBehindView ) {
      bBehindView = true;
      bTempNoBehindView = false;
    }
  }   
}


event PlayerCalcView(out actor ViewActor, out vector CameraLocation, out rotator CameraRotation )
{
  local Pawn PTarget;
  
  if ( ViewTarget == None || ViewTarget.bDeleteMe || ViewTarget.IsA('Pawn') && RealViewTarget != None
      && RealViewTarget.Pawn != None && RealViewTarget.Pawn != ViewTarget ) {
    if ( RealViewTarget != None && RealViewTarget.Pawn != None && !RealViewTarget.Pawn.bDeleteMe )
      SetViewTarget(RealViewTarget.Pawn);
    else
      SetViewTarget(self);
  }
  
  ViewActor = ViewTarget;
  CameraLocation = ViewTarget.Location;
  if ( ViewTarget != Self )
    SetLocation(ViewTarget.Location);
  
  if ( RealViewTarget != None && ViewTarget == RealViewTarget.Pawn ) {
    if ( bBehindView )
      CalcBehindView(CameraLocation, CameraRotation, CameraDist * RealViewTarget.Pawn.Default.CollisionRadius);
    else
      CalcFirstPersonView(CameraLocation, CameraRotation);
  }
  else if ( ViewTarget == self ) {
    CameraRotation = Rotation;
  }
  else if ( ViewTarget.IsA('Projectile') && !bBehindView ) {
    CameraLocation += (ViewTarget.CollisionHeight) * vect(0,0,1);
    CameraRotation = Rotation;
  }
  else {
    CameraRotation = ViewTarget.Rotation;
    PTarget = Pawn(ViewTarget);
    if ( PTarget != None ) {
      if ( Level.NetMode == NM_Client ) {
        PTarget.SetViewRotation(TargetViewRotation);
        CameraRotation = BlendedTargetViewRotation;
        
        PTarget.EyeHeight = TargetEyeHeight;
        if ( PTarget.Weapon != None )
          PTarget.Weapon.PlayerViewOffset = TargetWeaponViewOffset;
      }
      else if ( PTarget.IsPlayerPawn() )
        CameraRotation = PTarget.GetViewRotation();
      if ( !bBehindView )
        CameraLocation += PTarget.EyePosition();
    }
    if ( bBehindView ) {
      CameraLocation = CameraLocation + (ViewTarget.Default.CollisionHeight - ViewTarget.CollisionHeight) * vect(0,0,1);
      CalcBehindView(CameraLocation, CameraRotation, CameraDist * ViewTarget.Default.CollisionRadius);
    }
  }
}


function CalcFirstPersonView(out vector CameraLocation, out rotator CameraRotation)
{
  local vector X, Y, Z;
  
  GetAxes(RealViewTarget.Rotation, X, Y, Z);
  
  // First-person view.
  CameraRotation = Normalize(RealViewTarget.Rotation + ShakeRot);
  CameraLocation = CameraLocation + vect(0,0,1) * Pawn(ViewTarget).BaseEyeHeight + Pawn(ViewTarget).WalkBob + 
      ShakeOffset.X * Y + ShakeOffset.Y * Y + ShakeOffset.Z * Z;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  
}