//=============================================================================
// TournamentGameReplicationInfo
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// 
//=============================================================================


class TournamentGameReplicationInfo extends GameReplicationInfo
    dependson(TournamentPlayerReplicationInfo);


//=============================================================================
// Structs
//=============================================================================

struct TGameEventMarker {
  var() int ElapsedGameTime;              // the time index of the game event
  var() int Team;                         // team index associated with the game event
};

struct TDisconnectedPlayer {
  var() string IP;
  var() string PlayerID;
  var() TournamentPlayerReplicationInfo.TPRIData PlayerData;
};


//=============================================================================
// Variables
//=============================================================================

var private TGameEventMarker GameEvents[512];
var private int GameEventsIndex;
var private int GameEventsCount;

var int GameStartTime;

var array<TDisconnectedPlayer> DisconnectedPlayers;

var float SpawnProtectionTime;
var float MaxFOV;
var bool bNoHudClock;
var bool bWeaponModifications;


//=============================================================================
// Replication
//=============================================================================

replication
{
  reliable if ( Role == ROLE_Authority )
    GameEvents, GameEventsIndex, GameEventsCount,
    SpawnProtectionTime, bNoHudClock, MaxFOV, bWeaponModifications;
}


//=============================================================================
// PostBeginPlay
//
// Initialize the SpawnProtectionTime value.
//=============================================================================

simulated function PostBeginPlay()
{
  MaxFOV = 0;
  bWeaponModifications = class'TournamentServerSettings'.default.bWeaponModifications;
  if ( Role == ROLE_Authority && DeathMatch(Level.Game) != None )
    SpawnProtectionTime = DeathMatch(Level.Game).SpawnProtectionTime;
  if ( Role == ROLE_Authority ) {
    bNoHudClock = class'TournamentServerSettings'.default.bNoHudClock;
    MaxFOV = Clamp(class'TournamentServerSettings'.default.MaxFOV, 80, 170);
  }
  Super.PostBeginPlay();
}


//=============================================================================
// Timer
//
// Fix clientside time display.
//=============================================================================

simulated function Timer()
{
  Super.Timer();
  if ( Level.NetMode == NM_Client && !bMatchHasBegun )
    GameStartTime = ElapsedTime;
}


//=============================================================================
// AddGameEvent
//
// Adds a new game event to the list.
//=============================================================================

function AddGameEvent(int ElapsedTime, int TeamIndex)
{
  local int EventNumber;
  local TGameEventMarker EventData;
  
  if ( GameEventsCount < ArrayCount(GameEvents) )
    GameEventsCount++;
  else
    GameEventsIndex++;
  
  EventData.ElapsedGameTime = ElapsedTime - GameStartTime;
  EventData.Team            = TeamIndex;
  
  EventNumber = (GameEventsIndex + GameEventsCount - 1) % ArrayCount(GameEvents);
  GameEvents[EventNumber] = EventData;
}


//=============================================================================
// CountGameEvents
//
// Returns the actual number of game events in the list.
//=============================================================================

simulated function int CountGameEvents()
{
  return GameEventsCount;
}


//=============================================================================
// GetEventTime
//
// Returns the elapsed time for the specified game event.
//=============================================================================

simulated function int GetEventTime(int EventNumber)
{
  if ( EventNumber < 0 || EventNumber > GameEventsCount )
    return -1;
  
  return GameEvents[(GameEventsIndex + EventNumber) % ArrayCount(GameEvents)].ElapsedGameTime;
}


//=============================================================================
// GetEventTeam
//
// Returns the team index accociated with a game event.
//=============================================================================

simulated function int GetEventTeam(int EventNumber)
{
  if ( EventNumber < 0 || EventNumber > GameEventsCount )
    return -1;
  
  return GameEvents[(GameEventsIndex + EventNumber) % ArrayCount(GameEvents)].Team;
}


//=============================================================================
// Reset
//
// Reset the GRI after warmup.
//=============================================================================

function Reset()
{
  GameEventsCount = 0;
  GameEventsIndex = 0;
  DisconnectedPlayers.Length = 0;
  
  Super.Reset();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  MaxFOV=0
}