//=============================================================================
// TournamentHUD
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament DeathMatch HUD and base class for all other Tournament HUDs.
//=============================================================================


class TournamentHUD extends HudBDeathMatch
  config(User);


//=============================================================================
// Imports
//=============================================================================

#exec texture import file=Textures\DamageFlash.dds           mips=off alpha=on  group=Effects lodset=LODSET_None
#exec texture import file=Textures\DamageFlashRing.dds       mips=off alpha=on  group=Effects lodset=LODSET_None
#exec texture import file=Textures\TournamentHUDElements.dds mips=off alpha=on  group=GUI     lodset=LODSET_Interface
#exec texture import file=Textures\DisconnectWarn.pcx        mips=off masked=on group=GUI     lodset=LODSET_Interface

#exec obj load file=..\Textures\TournamentCrosshairs1.utx

#exec new truetypefontfactory name=LucidaConsoleSmall fontname="Lucida Console" height=7 antialias=0 dropshadowx=1 dropshadowy=1 chars=" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"


//=============================================================================
// Structs
//=============================================================================

struct RotatedWidget {
  var() SpriteWidget SpriteWidget;      // SpriteWidget to rotate
  
  var() bool bFixedAngle;               // DrawRotatedWidget should only offset the widget, not actually rotate it
  var() int Angle;                      // rotation angle in Unreal units
  var() float OffsetCenterX;            // offset of rotation center, relative
  var() float OffsetCenterY;            // to unrotated upper-left corner
  var() float OffsetRotatedX;           // horizontal and vertical displacement
  var() float OffsetRotatedY;           // scaled and rotated with the sprite
};

struct TDeathMessage
{
  var bool bSuicide;
  var bool bHeadShot;
  var bool bUDamage;
  var string Message;
  var PlayerReplicationInfo Killer, Victim;
  var string KillerName, VictimName;
  var color KillerColor, VictimColor;
  var class<DamageType> DamageType;
  var float EndOfLife, MessageTime, LifeTime;
};

struct TChatMessage
{
  var string Text;
  var color TextColor;
  var float EndOfLife, MessageTime, LifeTime;
  var PlayerReplicationInfo PRI;
  var string Name;
  var string Location;
  var color NameColor;
  var enum EChatMessageType {
    CMT_Other,
    CMT_Say,
    CMT_TeamSay
  } MessageType;
};

struct TWidgetLocation {
  var() bool bFlipH, bFlipV;
  
  var() EDrawPivot DrawPivot;
  var() float PosX, PosY;
  var() int OffsetX, OffsetY;
};


//=============================================================================
// Enums
//=============================================================================

enum EClockLocation {
  UpperLeft,
  UpperRight,
  UpperMiddle,
  LowerLeft,
  LowerRight,
  LowerMiddle
};


//=============================================================================
// Configuration
//=============================================================================

var() globalconfig float CursorBlinkingRate;
var() globalconfig bool  bSeparateDeathMessages;
var() globalconfig bool  bShortDeathMessages;
var() globalconfig bool  bGraphicalDeathMessages;
var() globalconfig int   DeathMessageCount;

var() globalconfig bool           bShowClock;
var() globalconfig bool           bShowTimeRemaining;
var() globalconfig bool           bShowLocalEvents;
var() globalconfig EClockLocation ClockLocation;

var() globalconfig float  TournamentHudScale;
var() globalconfig float  TournamentHudCanvasScale;
var() globalconfig byte   TournamentCrosshairStyle;
var() globalconfig float  TournamentCrosshairScale;
var() globalconfig color  TournamentCrosshairColor;
var() globalconfig byte   CrosshairRotation;
var() globalconfig string CustomCrosshairTextureName;
var() globalconfig bool   bRotateCustomWeaponCrosshair;


//=============================================================================
// Variables
//=============================================================================

var() SpriteWidget ClockBGFill;         // visual clock anchor fill
var() SpriteWidget ClockBGTint;         // visual clock anchor tint
var() SpriteWidget ClockBGFrame;        // visual clock anchor frame
var() SpriteWidget ClockCircle;         // clock background circle
var() SpriteWidget ClockSeperator;      // seperator between minutes and seconds display
var() SpriteWidget ClockFilledFirst;    // complete fill, first half
var() RotatedWidget ClockFillFirst;     // gradual fill, first half
var() RotatedWidget ClockFillSecond;    // gradual fill, second half
var() RotatedWidget ClockTick;          // clock section tick
var() RotatedWidget ClockMarker;        // game event marker
var() NumericWidget ClockMinutes;       // minutes of gametime
var() NumericWidget ClockSeconds;       // seconds of gametime

var() color DefaultClockColor;
var() color OverTimeClockColor;

var() TWidgetLocation LocationsClockBG[12];
var() TWidgetLocation LocationsClockCircle[12];
var() TWidgetLocation LocationsClockFilledFirst[12];
var() TWidgetLocation LocationsClockFillFirst[12];
var() TWidgetLocation LocationsClockFillSecond[12];
var() TWidgetLocation LocationsClockTick[12];
var() TWidgetLocation LocationsClockMarker[12];
var() TWidgetLocation LocationsClockSeperator[12];
var() TWidgetLocation LocationsClockMinutes[12];
var() TWidgetLocation LocationsClockSeconds[12];
var transient int CurrentClockLocation; // index for LocationsClock* array
var transient bool bDrawClock;          // whether the clock should actually be drawn

var() TWidgetLocation LocationsShieldCount[2];
var() TWidgetLocation LocationsShieldIcon[2];
var() TWidgetLocation LocationsShieldIconGlow[2];

var() array<color> EventColors;         // colors used for the event markers
var() array<int>   UDamagePickups;
var() int          FlakMonkeyTime;
var() int          ComboWhoreTime;
var() int          HeadHunterTime;
var() IntBox       UDamagePickCoords;
var() IntBox       FlakMonkeyCoords;
var() IntBox       ComboWhoreCoords;
var() IntBox       HeadHunterCoords;
var() array<TournamentGameReplicationInfo.TGameEventMarker> LocalEvents;  // other local events

var() SpriteWidget BadConnectionIcon;   // warning icon for bad connection alert
var() SpriteWidget SpawnProtFill;       // spawn protection timer
var() NumericWidget LivesLeft;          // lives left

var GameReplicationInfo GRI;
var float ClockPulseTime;
var() color ClockColorPulse;

var() string ConfigMenuClass;           // GUIPage class for client-side configuration

var bool bMakeScreenshot;               // when set to True a screenshot will be made after the next frame
var bool bScoresScreenshot;             // whether the next screenshot should display the scoreboard
var bool bCleanScreenshot;              // whether the next screenshot shouldn't show any HUD elements

var transient array<TChatMessage>  ChatMessages;
var transient array<TDeathMessage> DeathMessages;
var() Material NoWeaponKillIcon;
var() Material HeadshotKillIcon;
var() Material UDamageKillIcon;
var() IntBox   NoWeaponIconCoords;
var() IntBox   HeadshotIconCoords;
var() IntBox   UDamageIconCoords;
var() Material CustomCrosshairTexture;

// internal variables for DrawRotatedWidget() function
var private int iTexRotatorPool;        // index into TexRotator object pool
var private float TimeTexRotatorPool;   // time of last TexRotator pool reset
var private array<TexRotator> ListTexRotatorPool;  // TexRotator object pool

var transient float ConsoleMessagePosAlpha;
var float ScoreBoardConsoleMessagePosX, ScoreBoardConsoleMessagePosY;


//=============================================================================
// PostBeginPlay
//
// Clean up TTM's "leftovers" (Modified default properties etc.) and fix the
// smallest font not supporting lowercase.
//=============================================================================

function PostBeginPlay()
{
  local int i;
  
  for (i = 0; i < ArrayCount(FontArrayNames); i++) {
    FontArrayFonts[i] = None;
    default.FontArrayFonts[i] = None;
  }
  Super.PostBeginPlay();
  
  class'TournamentUtil'.static.CleanUpTTMLeftOvers();
}


//=============================================================================
// Reset
//
// Reset local events.
//=============================================================================

function Reset()
{
  LocalEvents.Length = 0;
  UDamagePickups.Length = 0;
  FlakMonkeyTime = 0;
  ComboWhoreTime = 0;
  HeadHunterTime = 0;
}


//=============================================================================
// exec Menu
//
// Brings up the configuration menu.
//=============================================================================

exec function Menu()
{
  if ( Level.NetMode == NM_StandAlone ) {
    PlayerOwner.SetPause(True);
    PlayerOwner.ClientOpenMenu(ConfigMenuClass,, "SetPause");
  }
  else
    PlayerOwner.ClientOpenMenu(ConfigMenuClass);
}


//=============================================================================
// exec NextStats
//
// Call stats screen's and scoreboard's NextStats() functions.
//=============================================================================

exec function NextStats()
{
  if ( bShowLocalStats && LocalStatsScreen != None )
    LocalStatsScreen.NextStats();
  if ( bShowScoreBoard && ScoreBoard != None )
    ScoreBoard.NextStats();
}


//=============================================================================
// exec Screenshot
//
// Makes a screenshot with with gametype, player name and timestamp.
//=============================================================================

exec function Screenshot()
{
  bMakeScreenshot = True;
}


//=============================================================================
// exec ScoresShot
//
// Makes a screenshot of the current scoreboard.
//=============================================================================

exec function ScoresShot()
{
  bMakeScreenshot = True;
  bScoresScreenshot = True;
}


//=============================================================================
// exec CleanShot
//
// Makes a screenshot without any HUD elements.
//=============================================================================

exec function CleanShot()
{
  bMakeScreenshot = True;
  bCleanScreenshot = True;
}


//=============================================================================
// exec Echo
//
// Just display the string as a console message.
//=============================================================================

exec function Echo(string S)
{
  AddTextMessage(S, class'StringMessagePlus', None);
}


//=============================================================================
// exec ShowClock
//
// Toggles the game clock.
//=============================================================================

exec function ShowClock()
{
  bShowClock = !bShowClock;
}


//=============================================================================
// exec ShowRemainingTime
//
// Displays the game clock and toggles between elapsed and remaining time.
//=============================================================================

exec function ShowRemainingTime()
{
  bShowClock = True;
  bShowTimeRemaining = !bShowTimeRemaining;
}


//=============================================================================
// exec ShowPoints
//
// Toggles the personal score and team scores.
//=============================================================================

exec function ShowPoints()
{
  bShowPoints = !bShowPoints;
}


//=============================================================================
// exec ShowHealth
//
// Toggles the health, shield and UDamage status.
//=============================================================================

exec function ShowHealth()
{
  bShowPersonalInfo = !bShowPersonalInfo;
}


//=============================================================================
// exec ShowWeaponInfo
//
// Toggles the ammo and weapon status.
//=============================================================================

exec function ShowWeaponInfo()
{
  bShowWeaponInfo = !bShowWeaponInfo;
}


//=============================================================================
// exec ShowWeaponBar
//
// Toggles the weapon bar.
//=============================================================================

exec function ShowWeaponBar()
{
  bShowWeaponBar = !bShowWeaponBar;
}


//=============================================================================
// exec GrowHUD
//
// Displays more HUD elements.
//=============================================================================

exec function GrowHUD()
{
  if ( !bShowWeaponInfo )
    bShowWeaponInfo = true;
  else if ( !bShowPersonalInfo )
    bShowPersonalInfo = true;
  else if ( !bShowClock )
    bShowClock = true;
  else if ( !bShowPoints )
    bShowPoints = true;
  else if ( !bShowWeaponBar )
    bShowWeaponBar = true;
  SaveConfig();
}


//=============================================================================
// exec ShrinkHUD
//
// Displays less HUD elements.
//=============================================================================

exec function ShrinkHUD()
{
  if ( bShowWeaponBar )
    bShowWeaponBar = false;
  else if ( bShowPoints )
    bShowPoints = false;
  else if ( bShowClock )
    bShowClock = false;
  else if ( bShowPersonalInfo )
    bShowPersonalInfo = false;
  else if ( bShowWeaponInfo )
    bShowWeaponInfo = false;
  SaveConfig();
}


//=============================================================================
// exec SetCrosshairStyle
//
// Sets a different crosshair style.
//=============================================================================

exec function SetCrosshairStyle(int NewStyle)
{
  CrosshairStyle = Abs(NewStyle) % Crosshairs.Length;
}


//=============================================================================
// Tick
//
// Update the clock pulse time and the console message position.
//=============================================================================

function Tick(float DeltaTime)
{
  if ( bShowScoreBoard && ConsoleMessagePosAlpha < 1 )
    ConsoleMessagePosAlpha = FMin(ConsoleMessagePosAlpha + 4 * DeltaTime, 1);
  else if ( !bShowScoreBoard && ConsoleMessagePosAlpha > 0 )
    ConsoleMessagePosAlpha = FMax(ConsoleMessagePosAlpha - 4 * DeltaTime, 0);
  
  ConsoleMessagePosX = Lerp(ConsoleMessagePosAlpha, default.ConsoleMessagePosX, ScoreBoardConsoleMessagePosX);
  ConsoleMessagePosY = Lerp(ConsoleMessagePosAlpha, default.ConsoleMessagePosY, ScoreBoardConsoleMessagePosY);
  
  Super.Tick(DeltaTime);
  
  if ( ClockPulseTime > 0 )
    ClockPulseTime -= DeltaTime / Level.TimeDilation;
  if ( ClockPulseTime < 0 )
    ClockPulseTime = 0;
}


//=============================================================================
// CheckCountdown
//
// Pulse the clock.
//=============================================================================

function CheckCountdown(GameReplicationInfo GRI)
{
  if ( GRI == None || !GRI.bMatchHasBegun || GRI.RemainingTime == 0 && OldRemainingTime == 0
      || GRI.RemainingTime == OldRemainingTime || GRI.Winner != None )
    return;
  
  OldRemainingTime = GRI.RemainingTime;
  switch (OldRemainingTime) {
    case 300:
      PlayerOwner.PlayAnnouncement(LongCount[0], 1, true);
      break;
    case 180:
      PlayerOwner.PlayAnnouncement(LongCount[1], 1, true);
      break;
    case 120:
      PlayerOwner.PlayAnnouncement(LongCount[2], 1, true);
      break;
    case 60:
      PlayerOwner.PlayAnnouncement(LongCount[3], 1, true);
      break;
    case 30:
      PlayerOwner.PlayAnnouncement(LongCount[4], 1, true);
      break;
    case 20:
      PlayerOwner.PlayAnnouncement(LongCount[5], 1, true);
      break;
    default:
      if ( OldRemainingTime <= 10 && OldRemainingTime > 0 )
        PlayerOwner.PlayAnnouncement(CountDown[OldRemainingTime - 1], 1, true);
      else if ( OldRemainingTime > 0 )
        return;
  }
  ClockPulseTime = 1.0;
}


//=============================================================================
// LinkActors
//
// Find the GameReplicationInfo.
//=============================================================================

function LinkActors()
{
  Super.LinkActors();
  
  if ( GRI == None )
    foreach DynamicActors(class'GameReplicationInfo', GRI)
      break;
}


//=============================================================================
// Message
//
// Find the GameReplicationInfo.
//=============================================================================

function Message(PlayerReplicationInfo PRI, coerce string Msg, name MsgType)
{
  local Class<LocalMessage> MessageClass;
  
  switch( MsgType )
  {
    case 'Say':
      MessageClass = class'SayMessagePlus';
      break;
    case 'TeamSay':
      MessageClass = class'TeamSayMessagePlus';
      break;
    case 'CriticalEvent':
      MessageClass = class'CriticalEventPlus';
      LocalizedMessage(MessageClass, 0, None, None, None, Msg);
      return;
    case 'DeathMessage':
      MessageClass = class'xDeathMessage';
      break;
    default:
      MessageClass = class'StringMessagePlus';
      break;
  }

  AddTextMessage(Msg, MessageClass, PRI);
}


//=============================================================================
// LocalizedMessage
//
// Seperately handle death messages.
//=============================================================================

function LocalizedMessage(
    class<LocalMessage> Message,
    optional int Switch,
    optional PlayerReplicationInfo RelatedPRI_1,
    optional PlayerReplicationInfo RelatedPRI_2,
    optional Object OptionalObject,
    optional String CriticalString)
{
  if ( class<xDeathMessage>(Message) != None )
    AddDeathMessage(Message, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject);
  else
    Super.LocalizedMessage(Message, Switch, RelatedPRI_1, RelatedPRI_2, OptionalObject, CriticalString);
}


//=============================================================================
// AddTextMessage
//
// Re-implemented to support multi-colored string messages.
//=============================================================================

function AddTextMessage(string M, class<LocalMessage> MessageClass, PlayerReplicationInfo PRI)
{
  local int i, NumMessages;
  
  while (i < ChatMessages.Length) {
    if ( ChatMessages[i].Text == "" || ChatMessages[i].EndOfLife - Level.TimeSeconds <= 0 )
      ChatMessages.Remove(i, 1);
    else
      i++;
  }
  
  for (i = 0; i < ChatMessages.Length; i++) {
    if ( ChatMessages[i].EndOfLife - Level.TimeSeconds > 0.3 )
      NumMessages++;
    if ( NumMessages > ConsoleMessageCount )
      ChatMessages[i].EndOfLife = FMin(ChatMessages[i].EndOfLife,
          Level.TimeSeconds + 0.3 + 0.1 * (1 + ConsoleMessageCount - NumMessages));
  }
  ChatMessages.Insert(0, 1);
  ChatMessages[0].Text = M;
  ChatMessages[0].LifeTime = MessageClass.Default.LifeTime;
  ChatMessages[0].MessageTime = Level.TimeSeconds;
  ChatMessages[0].EndOfLife = Level.TimeSeconds + ChatMessages[i].LifeTime;
  ChatMessages[0].TextColor = MessageClass.static.GetConsoleColor(PRI);
  ChatMessages[0].PRI = PRI;
  if ( PRI != None ) {
    ChatMessages[0].Name = PRI.PlayerName;
    ChatMessages[0].Location = PRI.GetLocationName();
    switch (MessageClass) {
      case class'SayMessagePlus':
        ChatMessages[0].NameColor = class'SayMessagePlus'.static.GetConsoleColor(PRI);
        ChatMessages[0].TextColor = GreenColor;
        ChatMessages[0].MessageType = CMT_Say;
        M = PRI.PlayerName $ ":" @ M;
        break;
      case class'TeamSayMessagePlus':
        ChatMessages[0].NameColor = class'SayMessagePlus'.static.GetConsoleColor(PRI);
        ChatMessages[0].MessageType = CMT_TeamSay;
        M = PRI.PlayerName $ "(" $ PRI.GetLocationName() $ "):" @ M;
        break;
      default:
        ChatMessages[0].NameColor = MessageClass.static.GetConsoleColor(PRI);
        ChatMessages[0].MessageType = CMT_Other;
    }
  }
  else {
    ChatMessages[0].MessageType = CMT_Other;
  }
  
  Super.AddTextMessage(M, MessageClass, PRI);
}


//=============================================================================
// AddDeathMessage
//
// Adds a death message to the DeathMessages stack.
//=============================================================================

function AddDeathMessage(
    class<LocalMessage> Message,
    optional int Switch,
    optional PlayerReplicationInfo RelatedPRI_1,
    optional PlayerReplicationInfo RelatedPRI_2,
    optional Object OptionalObject)
{
  local int i, NumMessages;
  
  if ( bMessageBeep && MessageClass.Default.bBeep )
    PlayerOwner.PlayBeepSound();
  
  while (i < DeathMessages.Length) {
    if ( DeathMessages[i].DamageType == None || DeathMessages[i].EndOfLife - Level.TimeSeconds <= 0 )
      DeathMessages.Remove(i, 1);
    else
      i++;
  }
  
  for (i = 0; i < DeathMessages.Length; i++) {
    if ( DeathMessages[i].EndOfLife - Level.TimeSeconds > 0.3 )
      NumMessages++;
    if ( NumMessages > DeathMessageCount )
      DeathMessages[i].EndOfLife = FMin(DeathMessages[i].EndOfLife,
          Level.TimeSeconds + 0.3 + 0.1 * (1 + DeathMessageCount - NumMessages));
  }
  DeathMessages.Insert(0, 1);
  if ( class<DamageType>(OptionalObject) == None )
    OptionalObject = class'DamageType';
  DeathMessages[0].DamageType = class<DamageType>(OptionalObject);
  DeathMessages[0].LifeTime = MessageClass.Default.LifeTime;
  DeathMessages[0].MessageTime = Level.TimeSeconds;
  DeathMessages[0].EndOfLife = Level.TimeSeconds + DeathMessages[0].LifeTime;
  DeathMessages[0].bSuicide = Switch % 2 != 0;
  DeathMessages[0].bUDamage = Switch > 1;
  DeathMessages[0].bHeadshot = DeathMessages[0].DamageType.default.bSpecial;
  DeathMessages[0].Killer = RelatedPRI_1;
  DeathMessages[0].Victim = RelatedPRI_2;
  if ( RelatedPRI_1 != None )
    DeathMessages[0].KillerName = RelatedPRI_1.PlayerName;
  if ( RelatedPRI_2 != None )
    DeathMessages[0].VictimName = RelatedPRI_2.PlayerName;
  DeathMessages[0].KillerColor = class'SayMessagePlus'.static.GetConsoleColor(RelatedPRI_1);
  DeathMessages[0].VictimColor = class'SayMessagePlus'.static.GetConsoleColor(RelatedPRI_2);
  if ( !DeathMessages[0].bSuicide )
    DeathMessages[0].Message = DeathMessages[0].DamageType.Static.DeathMessage(RelatedPRI_1, RelatedPRI_2);
  else
    DeathMessages[0].Message = DeathMessages[0].DamageType.Static.SuicideMessage(RelatedPRI_2);
  if ( DeathMessages[0].Message == "" ) {
    if ( !DeathMessages[0].bSuicide )
      DeathMessages[0].Message = class'DamageType'.Static.DeathMessage(RelatedPRI_1, RelatedPRI_2);
    else
      DeathMessages[0].Message = class'DamageType'.Static.SuicideMessage(RelatedPRI_2);
  }
}


//=============================================================================
// DisplayMessages
//
// Display chat and death messages.
//=============================================================================

function DisplayMessages(Canvas C)
{
  local int i, XPos, YPos;
  local float XL, YL, OldAlphaModulate;
  local int DeathMessageIndex, ChatMessageIndex;
  
  // count and clean up death messages
  i = 0;
  while (i < DeathMessages.Length) {
    if ( DeathMessages[i].DamageType == None || DeathMessages[i].EndOfLife - Level.TimeSeconds <= 0 )
      DeathMessages.Remove(i, 1);
    else
      i++;
  }
  
  // count and clean up other text messages
  i = 0;
  while (i < ChatMessages.Length) {
    if ( ChatMessages[i].Text == "" || ChatMessages[i].EndOfLife - Level.TimeSeconds <= 0 )
      ChatMessages.Remove(i, 1);
    else
      i++;
  }
  
  XPos = ConsoleMessagePosX * HudCanvasScale * C.SizeX + (1.0 - HudCanvasScale) * 0.5 * C.SizeX;
  YPos = ConsoleMessagePosY * HudCanvasScale * C.SizeY + (1.0 - HudCanvasScale) * 0.5 * C.SizeY;
  
  C.Reset();
  C.Style = ERenderStyle.STY_Alpha;
  C.Font = GetConsoleFont(C);
  C.TextSize("A", XL, YL);
  OldAlphaModulate = C.ColorModulate.W;
  
  if ( bSeparateDeathMessages ) {
    YPos -= YL * (ConsoleMessageCount + 1);     // room for text messages and typing prompt
    for (i = 0; i < DeathMessages.Length; i++)
      DrawDeathMessage(C, i, XPos, YPos);
    
    YPos = ConsoleMessagePosY * HudCanvasScale * C.SizeY + (1.0 - HudCanvasScale) * 0.5 * C.SizeY - YL;
    for (i = 0; i < ChatMessages.Length; i++)
      DrawChatMessage(C, i, XPos, YPos);
  }
  else {
    YPos -= YL; // room for typing prompt
    while (DeathMessageIndex < DeathMessages.Length || ChatMessageIndex < ChatMessages.Length) {
      if ( ChatMessageIndex < ChatMessages.Length && (DeathMessageIndex == DeathMessages.Length
          || DeathMessages[DeathMessageIndex].MessageTime < ChatMessages[ChatMessageIndex].MessageTime) ) {
        DrawChatMessage(C, ChatMessageIndex, XPos, YPos);
        ChatMessageIndex++;
      }
      else if ( DeathMessageIndex < DeathMessages.Length && (ChatMessageIndex == ChatMessages.Length
          || ChatMessages[ChatMessageIndex].MessageTime < DeathMessages[DeathMessageIndex].MessageTime) ) {
        DrawDeathMessage(C, DeathMessageIndex, XPos, YPos);
        DeathMessageIndex++;
      }
      else {
        //log("=== FIXME: Break executed! ===", 'TournamentDebug');
        break;
      }
    }
  }
  C.ColorModulate.W = OldAlphaModulate;
}


//=============================================================================
// DrawChatMessage
//
// Displays one chat line or text message.
//=============================================================================

function DrawChatMessage(Canvas C, int Index, out int XPos, out int YPos)
{
  local float OffsetX;
  local float XL, YL;
  local float FadeIn, FadeOut;
  
  if ( ChatMessages[Index].Text == "" )
    return;
  
  FadeIn = FMin((Level.TimeSeconds - ChatMessages[Index].MessageTime) / 0.3, 1.0);
  FadeOut = FMin((ChatMessages[Index].EndOfLife - Level.TimeSeconds) / 0.3, 1.0);
  C.ColorModulate.W = FMin(FadeIn, FadeOut);
  
  switch (ChatMessages[Index].MessageType) {
  case CMT_Say:
    C.DrawColor = ChatMessages[Index].NameColor;
    C.StrLen(ChatMessages[Index].Name $ ": ", XL, YL);
    C.SetPos(XPos, YPos - YL * FadeIn);
    C.DrawText(ChatMessages[Index].Name $ ": ", false);
    OffsetX = XL;
    break;
  case CMT_TeamSay:
    C.DrawColor = ChatMessages[Index].NameColor;
    C.StrLen(ChatMessages[Index].Name, XL, YL);
    C.SetPos(XPos, YPos - YL * FadeIn);
    C.DrawText(ChatMessages[Index].Name, false);
    OffsetX = XL;
    C.DrawColor = GreenColor;
    C.StrLen("(" $ ChatMessages[Index].Location $ "): ", XL, YL);
    C.SetPos(XPos + OffsetX, YPos - YL * FadeIn);
    C.DrawText("(" $ ChatMessages[Index].Location $ "): ", false);
    OffsetX += XL;
    break;
  }
  C.DrawColor = ChatMessages[Index].TextColor;
  C.StrLen(ChatMessages[Index].Text, XL, YL);
  C.SetPos(XPos + OffsetX, YPos - YL * FadeIn);
  C.DrawText(ChatMessages[Index].Text, false);
  
  YPos -= YL * FadeIn * FadeOut;
}


//=============================================================================
// DrawDeathMessage
//
// Displays one death message.
//=============================================================================

function DrawDeathMessage(Canvas C, int Index, out int XPos, out int YPos)
{
  local float XL, YL;
  local float OffsetX, IconWidth;
  local class<Weapon> KillerWeapon;
  local Material WeaponIcon;
  local IntBox IconCoords;
  local color IconColor;
  local string Message, DisplayString;
  local int KillerLoc, OtherLoc;
  local float FadeIn, FadeOut;
  
  if ( DeathMessages[Index].DamageType == None )
    return;

  FadeIn = FMin((Level.TimeSeconds - DeathMessages[Index].MessageTime) / 0.3, 1.0);
  FadeOut = FMin((DeathMessages[Index].EndOfLife - Level.TimeSeconds) / 0.3, 1.0);
  C.ColorModulate.W = FMin(FadeIn, FadeOut);
  
  if ( bGraphicalDeathMessages && bSeparateDeathMessages ) {
    if ( DeathMessages[Index].bSuicide ) {
      C.StrLen("A", XL, YL);
      XL = 0;
    }
    else {
      if ( DeathMessages[Index].KillerName != "" ) {
        C.DrawColor = DeathMessages[Index].KillerColor;
        C.StrLen(DeathMessages[Index].KillerName, XL, YL);
        C.SetPos(XPos, YPos + YL * (0.3 - 1.7 * FadeIn));
        C.DrawText(DeathMessages[Index].KillerName, false);
      }
      else {
        C.DrawColor = GreenColor;
        C.StrLen(class'xDeathMessage'.default.SomeoneString, XL, YL);
        C.SetPos(XPos, YPos + YL * (0.3 - 1.7 * FadeIn));
        C.DrawText(class'xDeathMessage'.default.SomeoneString, false);
      }
    } // else
    
    if ( class<WeaponDamageType>(DeathMessages[Index].DamageType) != None )
      KillerWeapon = class<WeaponDamageType>(DeathMessages[Index].DamageType).default.WeaponClass;
    else
      KillerWeapon = None;
    
    OffsetX = XL + 2;
    if ( DeathMessages[Index].bUDamage ) {
      IconWidth = YL * Abs(UDamageIconCoords.X2 - UDamageIconCoords.X1)
                     / Abs(UDamageIconCoords.Y2 - UDamageIconCoords.Y1);
      
      C.SetDrawColor(0, 0, 0);
      C.SetPos(XPos + OffsetX + 1, YPos + YL * (0.2 - 1.7 * FadeIn) + 1);
      C.DrawTile(UDamageKillIcon, IconWidth, YL, UDamageIconCoords.X2, UDamageIconCoords.Y1,
          UDamageIconCoords.X1 - UDamageIconCoords.X2, UDamageIconCoords.Y2 - UDamageIconCoords.Y1);
      C.DrawColor = PurpleColor;
      C.SetPos(XPos + OffsetX, YPos + YL * (0.2 - 1.7 * FadeIn));
      C.DrawTile(UDamageKillIcon, IconWidth, YL, UDamageIconCoords.X2, UDamageIconCoords.Y1,
          UDamageIconCoords.X1 - UDamageIconCoords.X2, UDamageIconCoords.Y2 - UDamageIconCoords.Y1);
      
      OffsetX += IconWidth;
    }
    
    if ( DeathMessages[Index].DamageType.default.Texture != None
        && DeathMessages[Index].DamageType.default.Texture != class'DamageType'.Default.Texture ) {
      WeaponIcon = DeathMessages[Index].DamageType.default.Texture;
      IconCoords.X1 = 0;
      IconCoords.Y1 = 0;
      IconCoords.X2 = WeaponIcon.MaterialUSize();
      IconCoords.Y2 = WeaponIcon.MaterialVSize();
      if ( KillerWeapon != None )
        IconColor  = KillerWeapon.default.HUDColor;
      else
        IconColor  = WhiteColor;
    }
    else if ( KillerWeapon != None && KillerWeapon.default.IconMaterial != None ) {
      WeaponIcon = KillerWeapon.default.IconMaterial;
      IconCoords = KillerWeapon.default.IconCoords;
      // mirror weapon icons
      IconCoords.X1 = KillerWeapon.default.IconCoords.X2;
      IconCoords.X2 = KillerWeapon.default.IconCoords.X1;
      IconColor  = KillerWeapon.default.HUDColor;
    }
    else {
      WeaponIcon = NoWeaponKillIcon;
      IconCoords = NoWeaponIconCoords;
      IconColor  = WhiteColor;
    }
    IconColor.A = 255;
    IconWidth = YL * Abs(IconCoords.X2 - IconCoords.X1) / Abs(IconCoords.Y2 - IconCoords.Y1);
    
    C.SetDrawColor(0, 0, 0);
    C.SetPos(XPos + OffsetX + 1, YPos - YL * (0.35 + 1.7 * FadeIn) + 1);
    C.DrawTile(WeaponIcon, 2 * IconWidth, 2 * YL, IconCoords.X1, IconCoords.Y1,
        IconCoords.X2 - IconCoords.X1, IconCoords.Y2 - IconCoords.Y1);
    C.DrawColor = BrightenColor(IconColor);
    C.SetPos(XPos + OffsetX, YPos - YL * (0.35 + 1.7 * FadeIn));
    C.DrawTile(WeaponIcon, 2 * IconWidth, 2 * YL, IconCoords.X1, IconCoords.Y1,
        IconCoords.X2 - IconCoords.X1, IconCoords.Y2 - IconCoords.Y1);
    
    OffsetX += IconWidth * 2;
    
    if ( DeathMessages[Index].bHeadshot ) {
      IconWidth = YL * Abs(HeadshotIconCoords.X2 - HeadshotIconCoords.X1)
                     / Abs(HeadshotIconCoords.Y2 - HeadshotIconCoords.Y1);
      
      C.SetDrawColor(0, 0, 0);
      C.SetPos(XPos + OffsetX + 1, YPos + YL * (0.2 - 1.7 * FadeIn) + 1);
      C.DrawTile(HeadshotKillIcon, IconWidth, YL, HeadshotIconCoords.X2, HeadshotIconCoords.Y1,
          HeadshotIconCoords.X1 - HeadshotIconCoords.X2, HeadshotIconCoords.Y2 - HeadshotIconCoords.Y1);
      C.DrawColor = RedColor;
      C.SetPos(XPos + OffsetX, YPos + YL * (0.2 - 1.7 * FadeIn));
      C.DrawTile(HeadshotKillIcon, IconWidth, YL, HeadshotIconCoords.X2, HeadshotIconCoords.Y1,
          HeadshotIconCoords.X1 - HeadshotIconCoords.X2, HeadshotIconCoords.Y2 - HeadshotIconCoords.Y1);
      
      OffsetX += IconWidth;
    }
    
    C.SetPos(XPos + OffsetX + 2, YPos + YL * (0.3 - 1.7 * FadeIn));
    if ( DeathMessages[Index].VictimName != "" ) {
      C.DrawColor = DeathMessages[Index].VictimColor;
      C.DrawText(DeathMessages[Index].VictimName, false);
    }
    else {
      C.DrawColor = GreenColor;
      C.DrawText(class'xDeathMessage'.default.SomeoneString, false);
    }
    YPos -= YL * 1.7 * FadeIn * FadeOut;
  }
  else {  // text death messages ============================================
    if ( bShortDeathMessages ) {
      if ( class<WeaponDamageType>(DeathMessages[Index].DamageType) != None )
        KillerWeapon = class<WeaponDamageType>(DeathMessages[Index].DamageType).default.WeaponClass;
      else
        KillerWeapon = None;
      
      Message = "> %o";
      if ( DeathMessages[Index].bHeadshot )
        Message = "-o-" $ Message;
      if ( DeathMessages[Index].bSuicide ) {
        if ( KillerWeapon != None && KillerWeapon.default.ItemName != "" )
          Message = KillerWeapon.default.ItemName @ Message;
      }
      else if ( KillerWeapon != None && KillerWeapon.default.ItemName != "" )
        Message = "%k >" @ KillerWeapon.default.ItemName @ Message;
      else
        Message = "%k >" $ Message;
    }
    else
      Message = DeathMessages[Index].Message;
    
    if ( DeathMessages[Index].KillerName == "" )
      ReplaceText(Message, "%k", class'xDeathMessage'.default.SomeoneString);
    if ( DeathMessages[Index].VictimName == "" )
      ReplaceText(Message, "%o", class'xDeathMessage'.default.SomeoneString);
    
    do {
      KillerLoc = InStr(Message, "%k");
      OtherLoc = InStr(Message, "%o");
      
      if ( KillerLoc == 0 ) {
        Message = Mid(Message, 2);
        DisplayString = DeathMessages[Index].KillerName;
        C.DrawColor = DeathMessages[Index].KillerColor;
      }
      else if ( OtherLoc == 0 ) {
        Message = Mid(Message, 2);
        DisplayString = DeathMessages[Index].VictimName;
        C.DrawColor = DeathMessages[Index].VictimColor;
      }
      else {
        if ( DeathMessages[Index].bUDamage )
          C.DrawColor = PurpleColor;
        else
          C.DrawColor = GreenColor;
        if ( KillerLoc == -1 ) {
          if ( OtherLoc == -1 ) {
            DisplayString = Message;
            Message = "";
          }
          else {
            DisplayString = Left(Message, OtherLoc);
            Message = Mid(Message, OtherLoc);
          }
        }
        else if ( OtherLoc == -1 ) {
          DisplayString = Left(Message, KillerLoc);
          Message = Mid(Message, KillerLoc);
        }
        else {
          DisplayString = Left(Message, Min(KillerLoc, OtherLoc));
          Message = Mid(Message, Min(KillerLoc, OtherLoc));
        }
      } // else
      
      C.StrLen(DisplayString, XL, YL);
      C.SetPos(XPos + OffsetX, YPos - YL * FadeIn);
      C.DrawText(DisplayString, false);
      OffsetX += XL;
    } until (Message == "");
    YPos -= YL * FadeIn * FadeOut;
  } // text death messages
}


//=============================================================================
// BrightenColor
//
// Brightens up a given color so it stands out from a black background.
//=============================================================================

function color BrightenColor(color C)
{
  local float R, G, B;
  local float f;
  
  R = C.R + 32;
  G = C.G + 32;
  B = C.B + 32;
  
  f = FMin(FMax(R, FMax(G, B)), 255);
  f = 255 / f;
  C.R = FMin(R * f, 255);
  C.G = FMin(G * f, 255);
  C.B = FMin(B * f, 255);
  C.A = 255;
  
  return C;
}


//=============================================================================
// MakeScreenshot
//
// Draws time, date and game information to the HUD and makes a screenshot.
//=============================================================================

function MakeScreenshot(Canvas C)
{
  local float TextX, TextY;
  local string InfoString;
  
  if ( !bShowScoreboard ) {
    // assemble the info string
    InfoString = "Tournament" @ Level.GetGameClass().Name $ "," @ PlayerOwner.PlayerReplicationInfo.PlayerName
        $ "," @ class'TournamentUtil'.static.CurrentDate(Level);
    
    // prepare the canvas and write the info string centered at the top of the screen
    C.Reset();
    C.DrawColor = WhiteColor;
    if ( C.ClipX < 800 )
      C.Font = C.SmallFont;
    else
      C.Font = C.MedFont;
    C.TextSize(InfoString, TextX, TextY);
    C.SetPos(0.5 * (C.ClipX - TextX), 1);
    C.DrawText(InfoString);
  }
  // now make the screenshot
  PlayerOwner.ConsoleCommand("SHOT");
  bMakeScreenshot = False;
}


//=============================================================================
// PostRender
//
// Draw player beacons before rendering anything else.
//=============================================================================

function PostRender(Canvas C)
{
  local byte OldCrosshairStyle;
  local float OldHudScale, OldHudCanvasScale, OldCrosshairScale;
  local color OldCrosshairColor;
  local bool bOldShowScoreboard, bOldShowLocalStats, bOldShowDebugInfo, bOldHideHud;
  
  if ( bMakeScreenshot && bCleanScreenshot ) {
    PlayerOwner.ConsoleCommand("SHOT");
    bCleanScreenshot = False;
    bMakeScreenshot  = bScoresScreenshot;
    return;
  }
  else if ( bMakeScreenshot && bScoresScreenshot ) {
    bOldHideHud = bHideHud;
    bOldShowScoreboard = bShowScoreboard;
    bOldShowLocalStats = bShowLocalStats;
    bOldShowDebugInfo  = bShowDebugInfo;
    
    bHideHud = False;
    bShowScoreboard = True;
    bShowLocalStats = False;
    bShowDebugInfo  = False;
  }
  OldHudScale = HudScale;
  OldHudCanvasScale = HudCanvasScale;
  OldCrosshairStyle = CrosshairStyle;
  OldCrosshairScale = CrosshairScale;
  OldCrosshairColor = CrosshairColor;
  
  HudScale = TournamentHudScale;
  HudCanvasScale = TournamentHudCanvasScale;
  CrosshairStyle = TournamentCrosshairStyle;
  CrosshairScale = TournamentCrosshairScale;
  CrosshairColor = TournamentCrosshairColor;
  
  // for compatibility with AntiTCC's Llama Effect
  Crosshairs[CrosshairStyle].PosX = Crosshairs[OldCrosshairStyle].PosX;
  Crosshairs[CrosshairStyle].PosY = Crosshairs[OldCrosshairStyle].PosY;
  
  DrawPlayerBeacons(C);
  Super.PostRender(C);
  
  if ( bMakeScreenshot && bScoresScreenshot ) {
    bShowScoreboard   = bOldShowScoreboard;
    bShowLocalStats   = bOldShowLocalStats;
    bShowDebugInfo    = bOldShowDebugInfo;
    bHideHud          = bOldHideHud;
    bScoresScreenshot = False;
  }
  
  if ( bMakeScreenshot )
    MakeScreenshot(C);
  
  HudScale = OldHudScale;
  HudCanvasScale = OldHudCanvasScale;
  CrosshairStyle = OldCrosshairStyle;
  CrosshairScale = OldCrosshairScale;
  CrosshairColor = OldCrosshairColor;
}


//=============================================================================
// DrawSpectatingHud
//
// Draw spectator HUD.
//=============================================================================

function DrawSpectatingHud(Canvas C)
{
  Super.DrawSpectatingHud(C);
  return;
  
  if ( FontsPrecached < 2 )
    PrecacheFonts(C);
  
  UpdateSpectatingHud();
  
  PassStyle = STY_Alpha;
  DrawSpectatingHudPassA(C);
  PassStyle = STY_Additive;
  DrawSpectatingHudPassB(C);
  PassStyle = STY_Alpha;
  DrawSpectatingHudPassC(C);
  PassStyle = STY_None;
  DrawSpectatingHudPassD(C);
  
  DisplayLocalMessages(C);
}


//=============================================================================
// DrawSpectatingHudPass*
//
// Actual drawing functions for the spectator HUD.
//=============================================================================

function DrawSpectatingHudPassA(Canvas C); // Alpha Pass
function DrawSpectatingHudPassB(Canvas C); // Additive Pass
function DrawSpectatingHudPassC(Canvas C); // Alpha Pass
function DrawSpectatingHudPassD(Canvas C); // Alternate Texture Pass


//=============================================================================
// UpdateSpectatingHud
//
// Update HUD elements of the spectator's HUD.
//=============================================================================

function UpdateSpectatingHud();


//=============================================================================
// DrawHudPassC
//
// Second alpha drawing pass.
//=============================================================================

function DrawHudPassC(Canvas C)
{
  local TournamentGameReplicationInfo TGRI;
  
  Super.DrawHudPassC(C);
  DrawClock(C);
  
  TGRI = TournamentGameReplicationInfo(GRI);
  if ( bShowPersonalInfo && PawnOwner != None && Level.TimeSeconds - PawnOwner.SpawnTime < TGRI.SpawnProtectionTime ) {
    SpawnProtFill.Scale = FClamp(1 - (Level.TimeSeconds - PawnOwner.SpawnTime) / TGRI.SpawnProtectionTime, 0, 1);
    
    DrawSpriteWidget(C, SpawnProtFill);
    DrawSpriteWidget(C, UDamageTeamTint);
    DrawSpriteWidget(C, UDamageTrim);
  }
}


//=============================================================================
// DisplayBadConnectionAlert
//
// Warn user that net connection is bad.
//=============================================================================

function DisplayBadConnectionAlert(Canvas C)
{
  DrawSpriteWidget(C, BadConnectionIcon);
}


//=============================================================================
// DrawPlayerBeacons
//
// Draws team beacons.
//=============================================================================

function DrawPlayerBeacons(Canvas C);


//=============================================================================
// DrawCrosshair
//
// Draws the crosshair.
//=============================================================================

function DrawCrosshair(Canvas C)
{
  local float CurrentCrosshairRotation;
  local int CurrentCrosshair;
  local float OldScale, OldCanvasScale, CurrentCrosshairScale;
  local plane OldModulate;
  local color CurrentCrosshairColor;
  local RotatedWidget CHTexture;
  
  if ( !bCrosshairShow )
    return;
  
  if ( PawnOwner != None && PawnOwner.Weapon != None && PawnOwner.Weapon.CustomCrosshair >= 0 ) {
    CurrentCrosshairColor = PawnOwner.Weapon.CustomCrosshairColor;
    CurrentCrosshair = PawnOwner.Weapon.CustomCrosshair;
    CurrentCrosshairScale = PawnOwner.Weapon.CustomCrosshairScale;
    if ( bRotateCustomWeaponCrosshair )
      CurrentCrosshairRotation = (CrosshairRotation % 4) * 16384;
    if ( PawnOwner.Weapon.CustomCrosshairTextureName != "" ) {
      if ( PawnOwner.Weapon.CustomCrosshairTexture == None ) {
        PawnOwner.Weapon.CustomCrosshairTexture = Texture(DynamicLoadObject(PawnOwner.Weapon.CustomCrosshairTextureName, class'Material'));
        if ( PawnOwner.Weapon.CustomCrosshairTexture == None ) {
          log(PawnOwner.Weapon@"custom crosshair texture not found!");
          PawnOwner.Weapon.CustomCrosshairTextureName = "";
        }
      }
      CHTexture.SpriteWidget = Crosshairs[0];
      CHTexture.SpriteWidget.WidgetTexture = PawnOwner.Weapon.CustomCrosshairTexture;
    }
  }
  else {
    if ( CustomCrosshairTextureName != "" ) {
      if ( CustomCrosshairTexture == None ) {
        CustomCrosshairTexture = Material(DynamicLoadObject(CustomCrosshairTextureName, class'Material'));
        if ( CustomCrosshairTexture == None ) {
          log(Self@"custom crosshair texture not found!");
          CustomCrosshairTextureName = "";
        }
      }
      CHTexture.SpriteWidget = Crosshairs[0];
      CHTexture.SpriteWidget.WidgetTexture = CustomCrosshairTexture;
    }
    CurrentCrosshair = CrosshairStyle;
    CurrentCrosshairColor = CrosshairColor;
    CurrentCrosshairScale = CrosshairScale;
    CurrentCrosshairRotation = (CrosshairRotation % 4) * 16384;
  }  
    
  CurrentCrosshair = Clamp(CurrentCrosshair, 0, Crosshairs.Length - 1);
  
  if ( CHTexture.SpriteWidget.WidgetTexture == None )
    CHTexture.SpriteWidget = Crosshairs[CurrentCrosshair];
  CHTexture.SpriteWidget.TextureScale *= CurrentCrosshairScale;
  
  CHTexture.SpriteWidget.Tints[0] = CurrentCrossHairColor;
  CHTexture.SpriteWidget.Tints[1] = CurrentCrossHairColor;
  CHTexture.OffsetCenterX = Abs(CHTexture.SpriteWidget.TextureCoords.X1 - CHTexture.SpriteWidget.TextureCoords.X2) / 2;
  CHTexture.OffsetCenterY = Abs(CHTexture.SpriteWidget.TextureCoords.Y1 - CHTexture.SpriteWidget.TextureCoords.Y2) / 2;
  CHTexture.Angle = CurrentCrosshairRotation;
  
  OldScale = HudScale;
  OldCanvasScale = HudCanvasScale;
  HudScale = 1;
  HudCanvasScale = 1;
  OldModulate = C.ColorModulate;
  C.ColorModulate.X = 1;
  C.ColorModulate.Y = 1;
  C.ColorModulate.Z = 1;
  C.ColorModulate.W = 1;
  DrawRotatedWidget(C, CHTexture);
  C.ColorModulate = OldModulate;
  HudCanvasScale = OldCanvasScale;
  HudScale = OldScale;
  
  DrawEnemyName(C);
}


//=============================================================================
// DrawClock
//
// Draws the elapsed or remaining match time and the round clock.
//=============================================================================

function DrawClock(Canvas C)
{
  local int nSecondsMax;                // number of seconds displayable (time limit in seconds)
  local int nSecondsCurrent;            // elapsed game time in seconds
  local int nSecondsDisplayed;          // displayed number of seconds
  local int nSecondsInterval;           // interval between ticks
  local int nSecondsTick;
  local int ElapsedGameTime, GameStartTime;
  
  local TournamentGameReplicationInfo TGRI;
  local int i;
  local int GameEventCount;
  
  if ( !bDrawClock )
    return;
  
  TGRI = TournamentGameReplicationInfo(GRI);
  if ( TGRI != None )
    GameStartTime = TGRI.GameStartTime;
  ElapsedGameTime = GRI.ElapsedTime - GameStartTime;
  
  nSecondsCurrent = ElapsedGameTime;
  
  if ( GRI.TimeLimit > 0 ) {
    nSecondsMax = 60 * GRI.TimeLimit;
  }
  else {
    nSecondsMax = 60 * 12;
    while (nSecondsMax <= ElapsedGameTime)
      nSecondsMax *= 2;
  }
  
  // draw background for numeric time display
  DrawSpriteWidget(C, ClockBGFill);
  DrawSpriteWidget(C, ClockBGTint);
  DrawSpriteWidget(C, ClockBGFrame);
  
  // calculate minutes and seconds to display
  if ( GRI.TimeLimit > 0 && nSecondsCurrent > nSecondsMax ) {
    // overtime
    ClockSeconds.Value = (nSecondsCurrent - nSecondsMax) % 60;
    ClockMinutes.Value = (nSecondsCurrent - nSecondsMax) / 60;
    ClockSeperator.Tints[0] = OverTimeClockColor;
    ClockSeperator.Tints[1] = OverTimeClockColor;
    ClockSeconds.Tints[0] = OverTimeClockColor;
    ClockSeconds.Tints[1] = OverTimeClockColor;
    ClockMinutes.Tints[0] = OverTimeClockColor;
    ClockMinutes.Tints[1] = OverTimeClockColor;
  }
  else {
    if ( bShowTimeRemaining && GRI.TimeLimit > 0 ) {
      ClockSeconds.Value = (nSecondsMax - nSecondsCurrent) % 60;
      ClockMinutes.Value = (nSecondsMax - nSecondsCurrent) / 60;
    }
    else {
      ClockSeconds.Value = nSecondsCurrent % 60;
      ClockMinutes.Value = nSecondsCurrent / 60;
    }
    if ( TGRI == None ) {
      ClockSeperator.Tints[0] = DefaultClockColor;
      ClockSeperator.Tints[1] = DefaultClockColor;
      ClockSeconds.Tints[0] = DefaultClockColor;
      ClockSeconds.Tints[1] = DefaultClockColor;
      ClockMinutes.Tints[0] = DefaultClockColor;
      ClockMinutes.Tints[1] = DefaultClockColor;
    }
  }
  if ( ClockSeconds.Value < 0 )
    ClockSeconds.Value += 60;
  ClockMinutes.Value = Abs(ClockMinutes.Value) % 100;
  
  ClockBGFill.Tints[0] = LerpColor(ClockPulseTime, Default.ClockBGFill.Tints[0], ClockColorPulse);
  ClockBGFill.Tints[1] = LerpColor(ClockPulseTime, Default.ClockBGFill.Tints[1], ClockColorPulse);
  ClockBGTint.Tints[0] = LerpColor(ClockPulseTime, Default.ClockBGTint.Tints[0], ClockColorPulse);
  ClockBGTint.Tints[1] = LerpColor(ClockPulseTime, Default.ClockBGTint.Tints[1], ClockColorPulse);
  
  // draw numeric time display
  DrawNumericWidget(C, ClockMinutes, DigitsBig);
  DrawSpriteWidget(C, ClockSeperator);
  DrawNumericWidget(C, ClockSeconds, DigitsBig);
  
  
  // draw background for clock  
  DrawSpriteWidget(C, ClockCircle);
  
  nSecondsInterval  = nSecondsMax / 15 + 59;
  nSecondsInterval -= nSecondsInterval % 60;
  
  nSecondsDisplayed = Min(nSecondsCurrent, nSecondsMax);
  
  // draw clock
  if ( nSecondsDisplayed > nSecondsMax / 2 ) {
    DrawSpriteWidget(C, ClockFilledFirst);
    ClockFillSecond.Angle = (nSecondsMax - nSecondsDisplayed) * 65536 / nSecondsMax;
    DrawRotatedWidget(C, ClockFillSecond);
  }
  else {
    ClockFillFirst.Angle = -nSecondsDisplayed * 65536 / nSecondsMax;
    DrawRotatedWidget(C, ClockFillFirst);
  }
  
  for (nSecondsTick = 0; nSecondsTick < nSecondsMax; nSecondsTick += nSecondsInterval) {
    ClockTick.Angle = -65536 * nSecondsTick / nSecondsMax;
    DrawRotatedWidget(C, ClockTick);
  }
  
  // draw game events
  if ( TGRI != None ) {
    GameEventCount = TGRI.CountGameEvents();
    for (i = 0; i < GameEventCount; i++) {
      ClockMarker.Angle = -65536 * TGRI.GetEventTime(i) / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = EventColors[TGRI.GetEventTeam(i)];
      ClockMarker.SpriteWidget.Tints[1] = ClockMarker.SpriteWidget.Tints[0];
      
      DrawRotatedWidget(C, ClockMarker);
    }
  }
  
  if ( bShowLocalEvents ) {
    ClockMarker.bFixedAngle = True;
    ClockMarker.OffsetRotatedY *= 0.9;
    ClockMarker.SpriteWidget.OffsetX /= 0.7;
    ClockMarker.SpriteWidget.OffsetY /= 0.7;
    ClockMarker.SpriteWidget.TextureScale *= 0.7;
    
    ClockMarker.SpriteWidget.TextureCoords = UDamagePickCoords;
    ClockMarker.SpriteWidget.Tints[0] = EventColors[3];
    ClockMarker.SpriteWidget.Tints[1] = EventColors[3];
    for (i = 0; i < UDamagePickups.Length; i++) {
      ClockMarker.Angle = -65536 * UDamagePickups[i] / nSecondsMax;
      
      DrawRotatedWidget(C, ClockMarker);
    }
    
    if ( FlakMonkeyTime > 0 ) {
      ClockMarker.SpriteWidget.TextureCoords = FlakMonkeyCoords;
      ClockMarker.Angle = -65536 * FlakMonkeyTime / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = EventColors[4];
      ClockMarker.SpriteWidget.Tints[1] = EventColors[4];
      
      DrawRotatedWidget(C, ClockMarker);
    }
    
    if ( ComboWhoreTime > 0 ) {
      ClockMarker.SpriteWidget.TextureCoords = ComboWhoreCoords;
      ClockMarker.Angle = -65536 * ComboWhoreTime / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = EventColors[5];
      ClockMarker.SpriteWidget.Tints[1] = EventColors[5];
      
      DrawRotatedWidget(C, ClockMarker);
    }
    
    if ( HeadHunterTime > 0 ) {
      ClockMarker.SpriteWidget.TextureCoords = HeadHunterCoords;
      ClockMarker.Angle = -65536 * HeadHunterTime / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = EventColors[6];
      ClockMarker.SpriteWidget.Tints[1] = EventColors[6];
      
      DrawRotatedWidget(C, ClockMarker);
    }
    ClockMarker = default.ClockMarker;
    for (i = 0; i < LocalEvents.Length; i++) {
      ClockMarker.Angle = -65536 * LocalEvents[i].ElapsedGameTime / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = EventColors[LocalEvents[i].Team];
      ClockMarker.SpriteWidget.Tints[1] = ClockMarker.SpriteWidget.Tints[0];
      
      DrawRotatedWidget(C, ClockMarker);
    }
  }
}


//=============================================================================
// GetClockLocation
//
// Updates the CurrentClockLocation according to the ClockLocation setting.
//=============================================================================

function GetClockLocation()
{
  local bool bSpectating;
  
  bSpectating = PlayerOwner == None || PawnOwner == None || PawnOwnerPRI == None
      || PlayerOwner.IsSpectating() && PlayerOwner.bBehindView;
  
  bDrawClock = bShowClock && (TournamentGameReplicationInfo(GRI) == None
      || !TournamentGameReplicationInfo(GRI).bNoHudClock);
  
  switch (ClockLocation) {
    case UpperLeft:
      if ( bShowPoints && !bSpectating )
        CurrentClockLocation = 0;
      else
        CurrentClockLocation = 1;
      break;
    case UpperRight:
      if ( bShowPersonalInfo && !bSpectating )
        CurrentClockLocation = 2;
      else
        CurrentClockLocation = 3;
      break;
    case UpperMiddle:
      if ( bShowPoints )
        CurrentClockLocation = 4;
      else
        CurrentClockLocation = 5;
      break;
    case LowerLeft:
      if ( bShowPersonalInfo && !bSpectating )
        CurrentClockLocation = 6;
      else
        CurrentClockLocation = 7;
      break;
    case LowerRight:
      if ( bShowWeaponInfo && !bSpectating )
        CurrentClockLocation = 8;
      else
        CurrentClockLocation = 9;
      break;
    case LowerMiddle:
      if ( bShowPersonalInfo && PawnOwner != None && PawnOwner.GetShieldStrength() > 0 && !bSpectating )
        CurrentClockLocation = 10;
      else
        CurrentClockLocation = 11;
      break;
  }
}


//=============================================================================
// UpdateClockLocation
//
// Updates the clock's location.
//=============================================================================

function UpdateClockLocation()
{
  GetClockLocation();
  
  GetSpriteWidgetLocation(ClockBGFill,      default.ClockBGFill,
      LocationsClockBG[CurrentClockLocation]);
  GetSpriteWidgetLocation(ClockBGTint,      default.ClockBGTint,
      LocationsClockBG[CurrentClockLocation]);
  GetSpriteWidgetLocation(ClockBGFrame,     default.ClockBGFrame,
      LocationsClockBG[CurrentClockLocation]);
  GetSpriteWidgetLocation(ClockCircle,      default.ClockCircle,
      LocationsClockCircle[CurrentClockLocation]);
  GetSpriteWidgetLocation(ClockSeperator,   default.ClockSeperator,
      LocationsClockSeperator[CurrentClockLocation]);
  GetSpriteWidgetLocation(ClockFilledFirst, default.ClockFilledFirst,
      LocationsClockFilledFirst[CurrentClockLocation]);
  
  GetRotatedWidgetLocation(ClockFillFirst,  default.ClockFillFirst,
      LocationsClockFillFirst[CurrentClockLocation]);
  GetRotatedWidgetLocation(ClockFillSecond, default.ClockFillSecond,
      LocationsClockFillSecond[CurrentClockLocation]);
  GetRotatedWidgetLocation(ClockTick,       default.ClockTick,
      LocationsClockTick[CurrentClockLocation]);
  GetRotatedWidgetLocation(ClockMarker,     default.ClockMarker,
      LocationsClockMarker[CurrentClockLocation]);
  
  GetNumericWidgetLocation(ClockMinutes, LocationsClockMinutes[CurrentClockLocation]);
  GetNumericWidgetLocation(ClockSeconds, LocationsClockSeconds[CurrentClockLocation]);
}


//=============================================================================
// UpdateHud
//
// Update the clock and shield icon locations.
//=============================================================================

function UpdateHud()
{
  Super.UpdateHud();
  
  UpdateClockLocation();
  
  if ( bDrawClock && ClockLocation == LowerMiddle ) {
    GetSpriteWidgetLocation(ShieldIcon,     default.ShieldIcon,     LocationsShieldIcon[1]);
    GetSpriteWidgetLocation(ShieldIconGlow, default.ShieldIconGlow, LocationsShieldIconGlow[1]);
    GetNumericWidgetLocation(ShieldCount, LocationsShieldCount[1]);
  }
  else {
    GetSpriteWidgetLocation(ShieldIcon,     default.ShieldIcon,     LocationsShieldIcon[0]);
    GetSpriteWidgetLocation(ShieldIconGlow, default.ShieldIconGlow, LocationsShieldIconGlow[0]);
    GetNumericWidgetLocation(ShieldCount, LocationsShieldCount[0]);
  }
}


//=============================================================================
// DrawRotatedWidget
//
// Draws a RotatedWidget sprite on the given Canvas.
//=============================================================================

simulated final function DrawRotatedWidget(Canvas C, RotatedWidget RotatedWidget)
{
  local float AngleInRadian;
  local float SinAngle;
  local float CosAngle;
  local TexRotator TexRotator;
  local SpriteWidget RotatedSpriteWidget;
  
  if ( TimeTexRotatorPool != Level.TimeSeconds )
    iTexRotatorPool = 0;
  else
    iTexRotatorPool++;
  
  TimeTexRotatorPool = Level.TimeSeconds;
  if ( iTexRotatorPool >= ListTexRotatorPool.Length )
    ListTexRotatorPool[iTexRotatorPool] = new Class'TexRotator';
  
  TexRotator = ListTexRotatorPool[iTexRotatorPool];
  RotatedSpriteWidget = RotatedWidget.SpriteWidget;
  if ( !RotatedWidget.bFixedAngle ) {
    RotatedSpriteWidget.WidgetTexture = TexRotator;
    
    TexRotator.Material        = RotatedWidget.SpriteWidget.WidgetTexture;
    TexRotator.UOffset         = RotatedWidget.OffsetCenterX + RotatedWidget.SpriteWidget.TextureCoords.X1;
    TexRotator.VOffset         = RotatedWidget.OffsetCenterY + RotatedWidget.SpriteWidget.TextureCoords.Y1;
    TexRotator.Rotation.Yaw    = RotatedWidget.Angle;
    TexRotator.Rotation.Pitch  = 0;
    TexRotator.Rotation.Roll   = 0;
    TexRotator.TexRotationType = TR_FixedRotation;
  }
  if ( RotatedWidget.OffsetRotatedX != 0 || RotatedWidget.OffsetRotatedY != 0 ) {
    AngleInRadian = Pi * RotatedWidget.Angle / 32768;
    SinAngle = Sin(AngleInRadian);
    CosAngle = Cos(AngleInRadian);
    
    RotatedSpriteWidget.OffsetX += RotatedWidget.OffsetRotatedX * CosAngle + RotatedWidget.OffsetRotatedY * SinAngle;
    RotatedSpriteWidget.OffsetY += RotatedWidget.OffsetRotatedY * CosAngle - RotatedWidget.OffsetRotatedX * SinAngle;
  }
  
  DrawSpriteWidget(C, RotatedSpriteWidget);
}


//=============================================================================
// GetSpriteWidgetLocation
//
// Applies a WidgetLocation to a SpriteWidget.
//=============================================================================

function GetSpriteWidgetLocation(out SpriteWidget W,
    SpriteWidget WDefault, TWidgetLocation L)
{
  W.PosX = L.PosX;
  W.PosY = L.PosY;
  W.OffsetX = L.OffsetX;
  W.OffsetY = L.OffsetY;
  W.DrawPivot = L.DrawPivot;
  
  if ( L.bFlipH ) {
    W.TextureCoords.X1 = WDefault.TextureCoords.X2;
    W.TextureCoords.X2 = WDefault.TextureCoords.X1;
  }
  if ( L.bFlipV ) {
    W.TextureCoords.Y1 = WDefault.TextureCoords.Y2;
    W.TextureCoords.Y2 = WDefault.TextureCoords.Y1;
  }
}


//=============================================================================
// GetRotatedWidgetLocation
//
// Applies a WidgetLocation to a RotatedWidget's SpriteWidget.
//=============================================================================

function GetRotatedWidgetLocation(out RotatedWidget W,
    RotatedWidget WDefault, TWidgetLocation L)
{
  GetSpriteWidgetLocation(W.SpriteWidget, WDefault.SpriteWidget, L);
}


//=============================================================================
// GetNumericWidgetLocation
//
// Applies a WidgetLocation to a NumericWidget.
//=============================================================================

function GetNumericWidgetLocation(out NumericWidget W, TWidgetLocation L)
{
  W.PosX = L.PosX;
  W.PosY = L.PosY;
  W.OffsetX = L.OffsetX;
  W.OffsetY = L.OffsetY;
  W.DrawPivot = L.DrawPivot;
}


//=============================================================================
// Draw2DDirection
//
// Draws a direction indicator to the given world co-ordinates based on the
// player's rotation. (replacement for Draw2DLocationDot)
//=============================================================================

function Draw2DDirection(Canvas C, vector Loc, RotatedWidget DirectionIndicator)
{
  local rotator Dir;
  local Actor Start;
  
  if ( PawnOwner == None )
    Start = PlayerOwner;
  else
    Start = PawnOwner;
  
  Dir = rotator(Loc - Start.Location);
  DirectionIndicator.Angle = PlayerOwner.Rotation.Yaw - Dir.Yaw;
  DrawRotatedWidget(C, DirectionIndicator);
}


//=============================================================================
// GetScreenCoords
//
// Move messages down a bit if clock is in the upper middle or up if it's in
// the lower middle of the screen.
//=============================================================================

function GetScreenCoords(float PosX, float PosY, out float ScreenX, out float ScreenY,
    out HudLocalizedMessage Message, Canvas C)
{
  if ( bDrawClock && ClockLocation == UpperMiddle && PosY < 0.5 )
    PosY += 0.05;
  else if ( bDrawClock && ClockLocation == LowerMiddle && PosY > 0.5 )
    PosY -= 0.06;
  Super.GetScreenCoords(PosX, PosY, ScreenX, ScreenY, Message, C);
}


//=============================================================================
// GetScreenCoords
//
// Move messages down a bit if clock is in the upper middle or up if it's in
// the lower middle of the screen.
//=============================================================================

simulated function LayoutMessage(out HudLocalizedMessage Message, Canvas C)
{
  local int FontSize;
  
  FontSize = Message.Message.static.GetFontSize(Message.Switch, Message.RelatedPRI, Message.RelatedPRI2,
      PlayerOwner.PlayerReplicationInfo);
  FontSize += MessageFontOffset + 1;
  Message.StringFont = GetFontSizeIndex(C, FontSize);
  Message.DrawColor = Message.Message.static.GetColor(Message.Switch);
  Message.Message.static.GetPos(Message.Switch, Message.DrawPivot, Message.StackMode, Message.PosX, Message.PosY);
  C.Font = Message.StringFont;
  C.TextSize(Message.StringMessage, Message.DX, Message.DY);
}


//=============================================================================
// DrawTypingPrompt
//
// Draws a typing prompt with a blinking cursor.
//=============================================================================

function DrawTypingPrompt(Canvas C, string Text)
{
  local float XPos, YPos;
  local float XL, YL;
  
  C.Font = GetConsoleFont(C);
  C.Style = ERenderStyle.STY_Alpha;
  C.DrawColor = ConsoleColor;
  
  C.TextSize("A", XL, YL);
  
  XPos = ConsoleMessagePosX * HudCanvasScale * C.SizeX + (1.0 - HudCanvasScale) * 0.5 * C.SizeX;
  YPos = ConsoleMessagePosY * HudCanvasScale * C.SizeY + (1.0 - HudCanvasScale) * 0.5 * C.SizeY - YL;
  
  C.SetPos(XPos, YPos);
  if ( int((60.0 * Level.Minute + Level.Second + 0.001 * Level.MilliSecond) * 2 * CursorBlinkingRate % 2) == 0 )
    Text = "(>" @ Text;
  else
    Text = "(>" @ Text $ "_";
  C.DrawTextClipped(Text, false);
}


//=============================================================================
// LerpColor
//
// Lerp() for colors.
//=============================================================================

static final function color LerpColor(float Alpha, color A, color B, optional bool bLerpAlpha)
{
  local color result;
  
  Alpha = FClamp(Alpha, 0, 1);
  result.R = Lerp(Alpha, A.R, B.R);
  result.G = Lerp(Alpha, A.G, B.G);
  result.B = Lerp(Alpha, A.B, B.B);
  if ( bLerpAlpha )
    result.A = Lerp(Alpha, A.A, B.A);
  else
    result.A = A.A;
  
  return result;
}


//=============================================================================
// Defaults
//=============================================================================

defaultproperties
{
  //===========================================================================
  // Default Configuration
  //===========================================================================
  
  bShowClock=True
  bShowTimeRemaining=False
  bShowLocalEvents=False
  ClockLocation=UpperMiddle
  
  TournamentHudScale=1.0
  TournamentHudCanvasScale=1.0
  TournamentCrosshairStyle=0
  TournamentCrosshairScale=1.0
  TournamentCrosshaircolor=(R=255,G=255,B=255,A=255)
  CrosshairRotation=0
  
  bSeparateDeathMessages=True
  bShortDeathMessages=True
  bGraphicalDeathMessages=True
  DeathMessageCount=4
  CursorBlinkingRate=2.0
  
  
  //===========================================================================
  // Default Properties
  //===========================================================================
  
  ConfigMenuClass      = "Tournament.TGUISettingsIngameMenu"
  LevelActionPositionY = 0.17
  
  NoWeaponKillIcon   = Material'InterfaceContent.Hud.SkinA'
  NoWeaponIconCoords = (X1=946,Y1=428,X2=1024,Y2=538)
  HeadshotKillIcon   = Material'TournamentHUDElements'
  HeadshotIconCoords = (X1=0,Y1=451,X2=64,Y2=510)
  UDamageKillIcon    = Material'TournamentHUDElements'
  UDamageIconCoords  = (X1=64,Y1=451,X2=128,Y2=510)
  
  UDamagePickCoords = (X1=064,Y1=451,X2=128,Y2=510)
  FlakMonkeyCoords  = (X1=064,Y1=424,X2=128,Y2=448)
  ComboWhoreCoords  = (X1=000,Y1=386,X2=064,Y2=446)
  HeadHunterCoords  = (X1=000,Y1=451,X2=064,Y2=510)
  
  ScoreBoardConsoleMessagePosX = 0.0
  ScoreBoardConsoleMessagePosY = 0.95
  
  FontArrayNames(0)="UT2003Fonts.FontEurostile29"
  FontArrayNames(1)="UT2003Fonts.FontEurostile24"
  FontArrayNames(2)="UT2003Fonts.FontEurostile21"
  FontArrayNames(3)="UT2003Fonts.jFontMedium800x600"
  FontArrayNames(4)="UT2003Fonts.jFontMedium"
  FontArrayNames(5)="UT2003Fonts.FontEurostile12"
  FontArrayNames(6)="UT2003Fonts.jFontSmallText800x600"
  FontArrayNames(7)="UT2003Fonts.FontMono"
  FontArrayNames(8)="Tournament.LucidaConsoleSmall"
  
  FontScreenWidthMedium(0)=1600
  FontScreenWidthMedium(1)=1280
  FontScreenWidthMedium(2)=1024
  FontScreenWidthMedium(3)=800
  FontScreenWidthMedium(4)=640
  FontScreenWidthMedium(5)=512
  FontScreenWidthMedium(6)=400
  FontScreenWidthMedium(7)=320
  FontScreenWidthMedium(8)=256
  
  ClockBGFrame = (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=274,Y1=440,X2=511,Y2=0511),TextureScale=0.33,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=80,OffsetY=145,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ClockBGTint  = (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=274,Y1=368,X2=511,Y2=0439),TextureScale=0.33,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=80,OffsetY=145,ScaleMode=SM_Left,Scale=1.00,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=000,B=000,A=100),Tints[1]=(R=037,G=066,B=102,A=150))
  ClockBGFill  = (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=274,Y1=295,X2=511,Y2=0366),TextureScale=0.33,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=80,OffsetY=145,ScaleMode=SM_Left,Scale=1.00,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=000,B=000,A=200),Tints[1]=(R=48,G=75,B=120,A=200))
  ClockCircle  = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=0,Y1=880,X2=142,Y2=1023),TextureScale=0.35,DrawPivot=DP_UpperLeft,PosX=0,PosY=0,OffsetX=00,OffsetY=100,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  
  ClockSeperator   =               (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=014,Y1=266,X2=038,Y2=320),TextureScale=0.1900,DrawPivot=DP_UpperMiddle,PosX=00,PosY=0,OffsetX=442,OffsetY=292,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ClockFilledFirst =               (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=128,Y1=038,X2=038,Y2=218),TextureScale=0.2380,DrawPivot=DP_UpperLeft,PosX=0000,PosY=0,OffsetX=131,OffsetY=230,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=056,G=056,B=056,A=255),Tints[1]=(R=056,G=056,B=056,A=255))
  ClockFillFirst   = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=128,Y1=038,X2=218,Y2=218),TextureScale=0.2380,DrawPivot=DP_UpperLeft,PosX=0000,PosY=0,OffsetX=131,OffsetY=230,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=056,G=056,B=056,A=255),Tints[1]=(R=056,G=056,B=056,A=255)),OffsetCenterX=00,OffsetCenterY=90)
  ClockFillSecond  = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=038,Y1=038,X2=128,Y2=218),TextureScale=0.2380,DrawPivot=DP_UpperRight,PosX=000,PosY=0,OffsetX=132,OffsetY=230,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=056,G=056,B=056,A=255),Tints[1]=(R=056,G=056,B=056,A=255)),OffsetCenterX=90,OffsetCenterY=90)
  ClockTick        = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=010,Y1=328,X2=028,Y2=350),TextureScale=0.2875,DrawPivot=DP_MiddleMiddle,PosX=0,PosY=0,OffsetX=087,OffsetY=210,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=96,G=96,B=96,A=192),Tints[1]=(R=96,G=96,B=96,A=192)),OffsetCenterX=09,OffsetCenterY=11,OffsetRotatedX=0,OffsetRotatedY=-78)
  ClockMarker      = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=007,Y1=357,X2=037,Y2=383),TextureScale=0.2875,DrawPivot=DP_MiddleMiddle,PosX=0,PosY=0,OffsetX=087,OffsetY=210,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=192,G=192,B=192,A=255),Tints[1]=(R=192,G=192,B=192,A=255)),OffsetCenterX=15,OffsetCenterY=13,OffsetRotatedX=0,OffsetRotatedY=-80)
  
  ClockMinutes = (TextureScale=0.15,MinDigitCount=2,DrawPivot=DP_UpperRight,PosX=0,PosY=0,OffsetX=550,OffsetY=350,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
  ClockSeconds = (TextureScale=0.15,MinDigitCount=2,DrawPivot=DP_UpperLeft,PosX=00,PosY=0,OffsetX=580,OffsetY=350,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255),bPadWithZeroes=1)
  
  DefaultClockColor  = (R=255,G=255,B=255,A=255)
  OverTimeClockColor = (R=255,G=000,B=000,A=255)
  
  // upper left with points
  LocationsClockBG(0)          = (OffsetX=130,OffsetY=145,DrawPivot=DP_UpperLeft)
  LocationsClockCircle(0)      = (OffsetX=000,OffsetY=100,DrawPivot=DP_UpperLeft)
  LocationsClockSeperator(0)   = (OffsetX=442,OffsetY=292,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(0) = (OffsetX=104,OffsetY=254,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(0)   = (OffsetX=104,OffsetY=254,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(0)  = (OffsetX=106,OffsetY=254,DrawPivot=DP_MiddleRight)
  LocationsClockTick(0)        = (OffsetX=087,OffsetY=210,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(0)      = (OffsetX=087,OffsetY=210,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(0)     = (OffsetX=550,OffsetY=350,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(0)     = (OffsetX=580,OffsetY=350,DrawPivot=DP_UpperLeft)
  
  // upper left without points
  LocationsClockBG(1)          = (OffsetX=130,OffsetY=040,DrawPivot=DP_UpperLeft)
  LocationsClockCircle(1)      = (OffsetX=000,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsClockSeperator(1)   = (OffsetX=442,OffsetY=105,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(1) = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(1)   = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(1)  = (OffsetX=106,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(1)        = (OffsetX=087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(1)      = (OffsetX=087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(1)     = (OffsetX=550,OffsetY=115,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(1)     = (OffsetX=580,OffsetY=115,DrawPivot=DP_UpperLeft)
  
  // upper right with adrenaline
  LocationsClockBG(2)          = (PosX=1.0,OffsetX=-130,OffsetY=145,DrawPivot=DP_UpperRight,bFlipH=True)
  LocationsClockCircle(2)      = (PosX=1.0,OffsetX=-000,OffsetY=100,DrawPivot=DP_UpperRight)
  LocationsClockSeperator(2)   = (PosX=1.0,OffsetX=-442,OffsetY=292,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(2) = (PosX=1.0,OffsetX=-106,OffsetY=254,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(2)   = (PosX=1.0,OffsetX=-106,OffsetY=254,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(2)  = (PosX=1.0,OffsetX=-104,OffsetY=254,DrawPivot=DP_MiddleRight)
  LocationsClockTick(2)        = (PosX=1.0,OffsetX=-087,OffsetY=210,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(2)      = (PosX=1.0,OffsetX=-087,OffsetY=210,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(2)     = (PosX=1.0,OffsetX=-580,OffsetY=350,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(2)     = (PosX=1.0,OffsetX=-540,OffsetY=350,DrawPivot=DP_UpperLeft)
  
  // upper right without adrenaline
  LocationsClockBG(3)          = (PosX=1.0,OffsetX=-130,OffsetY=040,DrawPivot=DP_UpperRight,bFlipH=True)
  LocationsClockCircle(3)      = (PosX=1.0,OffsetX=-000,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsClockSeperator(3)   = (PosX=1.0,OffsetX=-442,OffsetY=105,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(3) = (PosX=1.0,OffsetX=-106,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(3)   = (PosX=1.0,OffsetX=-106,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(3)  = (PosX=1.0,OffsetX=-104,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(3)        = (PosX=1.0,OffsetX=-087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(3)      = (PosX=1.0,OffsetX=-087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(3)     = (PosX=1.0,OffsetX=-580,OffsetY=115,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(3)     = (PosX=1.0,OffsetX=-540,OffsetY=115,DrawPivot=DP_UpperLeft)
  
  // upper middle with points
  LocationsClockBG(4)          = (PosX=0.5,OffsetX=000,OffsetY=140,DrawPivot=DP_UpperMiddle)
  LocationsClockCircle(4)      = (PosX=0.5,OffsetX=000,OffsetY=000,DrawPivot=DP_UpperMiddle)
  LocationsClockSeperator(4)   = (PosX=0.5,OffsetX=000,OffsetY=280,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(4) = (PosX=0.5,OffsetX=-01,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(4)   = (PosX=0.5,OffsetX=-01,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(4)  = (PosX=0.5,OffsetX=001,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(4)        = (PosX=0.5,OffsetX=000,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(4)      = (PosX=0.5,OffsetX=000,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(4)     = (PosX=0.5,OffsetX=-15,OffsetY=335,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(4)     = (PosX=0.5,OffsetX=020,OffsetY=335,DrawPivot=DP_UpperLeft)
  
  // upper middle without points
  LocationsClockBG(5)          = (PosX=0.5,OffsetX=000,OffsetY=140,DrawPivot=DP_UpperMiddle)
  LocationsClockCircle(5)      = (PosX=0.5,OffsetX=000,OffsetY=000,DrawPivot=DP_UpperMiddle)
  LocationsClockSeperator(5)   = (PosX=0.5,OffsetX=000,OffsetY=280,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(5) = (PosX=0.5,OffsetX=-01,OffsetY=105,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(5)   = (PosX=0.5,OffsetX=-01,OffsetY=105,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(5)  = (PosX=0.5,OffsetX=001,OffsetY=105,DrawPivot=DP_MiddleRight)
  LocationsClockTick(5)        = (PosX=0.5,OffsetX=000,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(5)      = (PosX=0.5,OffsetX=000,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(5)     = (PosX=0.5,OffsetX=-15,OffsetY=335,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(5)     = (PosX=0.5,OffsetX=020,OffsetY=335,DrawPivot=DP_UpperLeft)
  
  // lower left with health
  LocationsClockBG(6)          = (PosY=0.835,OffsetX=0600,OffsetY=075,DrawPivot=DP_UpperLeft)
  LocationsClockCircle(6)      = (PosY=0.835,OffsetX=0445,OffsetY=030,DrawPivot=DP_UpperLeft)
  LocationsClockSeperator(6)   = (PosY=0.835,OffsetX=1260,OffsetY=167,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(6) = (PosY=0.835,OffsetX=0757,OffsetY=151,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(6)   = (PosY=0.835,OffsetX=0757,OffsetY=151,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(6)  = (PosY=0.835,OffsetX=0759,OffsetY=151,DrawPivot=DP_MiddleRight)
  LocationsClockTick(6)        = (PosY=0.835,OffsetX=0629,OffsetY=125,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(6)      = (PosY=0.835,OffsetX=0629,OffsetY=125,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(6)     = (PosY=0.835,OffsetX=1585,OffsetY=192,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(6)     = (PosY=0.835,OffsetX=1615,OffsetY=192,DrawPivot=DP_UpperLeft)
  
  // lower left without health
  LocationsClockBG(7)          = (PosY=0.95,OffsetX=130,OffsetY=-10,DrawPivot=DP_LowerLeft)
  LocationsClockCircle(7)      = (PosY=0.95,OffsetX=000,OffsetY=028,DrawPivot=DP_LowerLeft)
  LocationsClockSeperator(7)   = (PosY=0.95,OffsetX=442,OffsetY=-50,DrawPivot=DP_LowerMiddle)
  LocationsClockFilledFirst(7) = (PosY=0.95,OffsetX=104,OffsetY=-64,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(7)   = (PosY=0.95,OffsetX=104,OffsetY=-64,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(7)  = (PosY=0.95,OffsetX=106,OffsetY=-64,DrawPivot=DP_MiddleRight)
  LocationsClockTick(7)        = (PosY=0.95,OffsetX=087,OffsetY=-53,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(7)      = (PosY=0.95,OffsetX=087,OffsetY=-53,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(7)     = (PosY=0.95,OffsetX=550,OffsetY=-50,DrawPivot=DP_LowerRight)
  LocationsClockSeconds(7)     = (PosY=0.95,OffsetX=580,OffsetY=-50,DrawPivot=DP_LowerLeft)
  
  // lower right with ammo
  LocationsClockBG(8)          = (PosX=1.0,PosY=0.835,OffsetX=-0600,OffsetY=075,DrawPivot=DP_UpperRight)
  LocationsClockCircle(8)      = (PosX=1.0,PosY=0.835,OffsetX=-0445,OffsetY=030,DrawPivot=DP_UpperRight)
  LocationsClockSeperator(8)   = (PosX=1.0,PosY=0.835,OffsetX=-1262,OffsetY=167,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(8) = (PosX=1.0,PosY=0.835,OffsetX=-0759,OffsetY=151,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(8)   = (PosX=1.0,PosY=0.835,OffsetX=-0759,OffsetY=151,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(8)  = (PosX=1.0,PosY=0.835,OffsetX=-0757,OffsetY=151,DrawPivot=DP_MiddleRight)
  LocationsClockTick(8)        = (PosX=1.0,PosY=0.835,OffsetX=-0629,OffsetY=125,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(8)      = (PosX=1.0,PosY=0.835,OffsetX=-0629,OffsetY=125,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(8)     = (PosX=1.0,PosY=0.835,OffsetX=-1615,OffsetY=192,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(8)     = (PosX=1.0,PosY=0.835,OffsetX=-1575,OffsetY=192,DrawPivot=DP_UpperLeft)
  
  // lower right without ammo
  LocationsClockBG(9)          = (PosX=1.0,PosY=0.95,OffsetX=-130,OffsetY=-10,DrawPivot=DP_LowerRight)
  LocationsClockCircle(9)      = (PosX=1.0,PosY=0.95,OffsetX=0000,OffsetY=028,DrawPivot=DP_LowerRight)
  LocationsClockSeperator(9)   = (PosX=1.0,PosY=0.95,OffsetX=-442,OffsetY=-50,DrawPivot=DP_LowerMiddle)
  LocationsClockFilledFirst(9) = (PosX=1.0,PosY=0.95,OffsetX=-106,OffsetY=-64,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(9)   = (PosX=1.0,PosY=0.95,OffsetX=-106,OffsetY=-64,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(9)  = (PosX=1.0,PosY=0.95,OffsetX=-104,OffsetY=-64,DrawPivot=DP_MiddleRight)
  LocationsClockTick(9)        = (PosX=1.0,PosY=0.95,OffsetX=-087,OffsetY=-53,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(9)      = (PosX=1.0,PosY=0.95,OffsetX=-087,OffsetY=-53,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(9)     = (PosX=1.0,PosY=0.95,OffsetX=-580,OffsetY=-50,DrawPivot=DP_LowerRight)
  LocationsClockSeconds(9)     = (PosX=1.0,PosY=0.95,OffsetX=-540,OffsetY=-50,DrawPivot=DP_LowerLeft)
  
  // lower middle with shield
  LocationsClockBG(10)          = (PosX=0.5,PosY=0.95,OffsetX=-108,OffsetY=0000,DrawPivot=DP_LowerMiddle)
  LocationsClockCircle(10)      = (PosX=0.5,PosY=0.95,OffsetX=-098,OffsetY=-055,DrawPivot=DP_LowerMiddle)
  LocationsClockSeperator(10)   = (PosX=0.5,PosY=0.95,OffsetX=-180,OffsetY=-030,DrawPivot=DP_LowerMiddle)
  LocationsClockFilledFirst(10) = (PosX=0.5,PosY=0.95,OffsetX=-146,OffsetY=-186,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(10)   = (PosX=0.5,PosY=0.95,OffsetX=-146,OffsetY=-186,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(10)  = (PosX=0.5,PosY=0.95,OffsetX=-144,OffsetY=-186,DrawPivot=DP_MiddleRight)
  LocationsClockTick(10)        = (PosX=0.5,PosY=0.95,OffsetX=-120,OffsetY=-154,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(10)      = (PosX=0.5,PosY=0.95,OffsetX=-120,OffsetY=-154,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(10)     = (PosX=0.5,PosY=0.95,OffsetX=-260,OffsetY=-030,DrawPivot=DP_LowerRight)
  LocationsClockSeconds(10)     = (PosX=0.5,PosY=0.95,OffsetX=-200,OffsetY=-030,DrawPivot=DP_LowerLeft)
  
  // lower middle without shield
  LocationsClockBG(11)          = (PosX=0.5,PosY=0.95,OffsetX=000,OffsetY=0000,DrawPivot=DP_LowerMiddle)
  LocationsClockCircle(11)      = (PosX=0.5,PosY=0.95,OffsetX=000,OffsetY=-055,DrawPivot=DP_LowerMiddle)
  LocationsClockSeperator(11)   = (PosX=0.5,PosY=0.95,OffsetX=000,OffsetY=-030,DrawPivot=DP_LowerMiddle)
  LocationsClockFilledFirst(11) = (PosX=0.5,PosY=0.95,OffsetX=-01,OffsetY=-186,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(11)   = (PosX=0.5,PosY=0.95,OffsetX=-01,OffsetY=-186,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(11)  = (PosX=0.5,PosY=0.95,OffsetX=001,OffsetY=-186,DrawPivot=DP_MiddleRight)
  LocationsClockTick(11)        = (PosX=0.5,PosY=0.95,OffsetX=000,OffsetY=-154,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(11)      = (PosX=0.5,PosY=0.95,OffsetX=000,OffsetY=-154,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(11)     = (PosX=0.5,PosY=0.95,OffsetX=-20,OffsetY=-030,DrawPivot=DP_LowerRight)
  LocationsClockSeconds(11)     = (PosX=0.5,PosY=0.95,OffsetX=030,OffsetY=-030,DrawPivot=DP_LowerLeft)
  
  // shield location without clock
  LocationsShieldIcon(0)     = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=0,OffsetY=-90)
  LocationsShieldIconGlow(0) = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=0,OffsetY=-85)
  LocationsShieldCount(0)    = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=0,OffsetY=-80)
  
  // shield location with clock
  LocationsShieldIcon(1)     = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=204,OffsetY=-90)
  LocationsShieldIconGlow(1) = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=195,OffsetY=-85)
  LocationsShieldCount(1)    = (DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.95,OffsetX=208,OffsetY=-80)
  
  // colors for event markers
  EventColors(0)   = (R=255,G=000,B=000,A=255)  // red team event
  EventColors(1)   = (R=064,G=096,B=255,A=255)  // blue team event
  EventColors(2)   = (R=255,G=255,B=000,A=255)
  EventColors(3)   = (R=160,G=000,B=160,A=200)  // UDamage
  EventColors(4)   = (R=160,G=096,B=000,A=200)  // flak monkey
  EventColors(5)   = (R=064,G=000,B=160,A=200)  // combo whore
  EventColors(6)   = (R=160,G=048,B=000,A=200)  // head hunter
  ClockColorPulse  = (R=255,G=255,B=000,A=255)
  
  BadConnectionIcon = (WidgetTexture=Material'DisconnectWarn',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.5,DrawPivot=DP_MiddleRight,PosX=1.0,PosY=0.7,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=180),Tints[1]=(R=255,G=255,B=255,A=180))
  SpawnProtFill     = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=450,Y1=490,X1=836,Y2=454),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.0,PosY=0.835,OffsetX=137,OffsetY=15,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=0,A=255),Tints[1]=(R=255,G=255,B=0,A=255))
  
  Crosshairs(0)  = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(1)  = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(2)  = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(3)  = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(4)  = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(5)  = (TextureCoords=(X1=30,Y1=29,X2=37,Y2=36))
  Crosshairs(10) = (TextureCoords=(X1=1,Y1=1))
  Crosshairs(11) = (TextureCoords=(X1=1,Y1=1))
  
  Crosshairs(14) = (WidgetTexture=Material'TCrosshair14',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(15) = (WidgetTexture=Material'TCrosshair15',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(16) = (WidgetTexture=Material'TCrosshair16',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(17) = (WidgetTexture=Material'TCrosshair17',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(18) = (WidgetTexture=Material'TCrosshair18',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(19) = (WidgetTexture=Material'TCrosshair19',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(20) = (WidgetTexture=Material'TCrosshair20',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(21) = (WidgetTexture=Material'TCrosshair21',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  Crosshairs(22) = (WidgetTexture=Material'TCrosshair22',TextureCoords=(X1=0,Y1=0,X2=64,Y2=64),TextureScale=0.4,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.5,OffsetX=0,OffsetY=0,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
}