//=============================================================================
// TournamentHUDBombingRun
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Bombing Run HUD
//=============================================================================


class TournamentHUDBombingRun extends TournamentHUDTeamDM;


//=============================================================================
// Imports
//=============================================================================

#exec obj load file=IndoorAmbience.uax


//=============================================================================
// Variables
//=============================================================================

var() SpriteWidget BombBG;
var() HudBBombingRun.FBombWidget BombWidget;
var() Sound PassTargetLocked;
var() RotatedWidget GoalIndicator;

var() TWidgetLocation LocationsBombBG[2];
var() TWidgetLocation LocationsBombWidget[2];
var() TWidgetLocation LocationsGoalIndicator[2];

var transient xBombFlag BombFlag;
var transient xBombDelivery MyBombDelivery;


//=============================================================================
// PostBeginPlay
//
// Start the timer that will handle the bomb warning messages.
//=============================================================================

function PostBeginPlay()
{
  Super.PostBeginPlay();
  SetTimer(1.0, True);
}


//=============================================================================
// ShowPointBarBottom
// ShowPointBarTop
// ShowPersonalScore
//
// Personal score is disabled in BR.
//=============================================================================

function ShowPointBarBottom(Canvas C);
function ShowPointBarTop(Canvas C);
function ShowPersonalScore(Canvas C);


//=============================================================================
// DrawHudPassD
//
// Draw the bomb state widget.
// This must be done in the alternate HUD drawing pass because it uses not only
// STY_Alpha, but also STY_Additive in some cases and DrawSpriteWidget() would
// spam the log if this was done in pass C.
//=============================================================================

function DrawHudPassD(Canvas C)
{
  local vector Pos;
  
  Super.DrawHudPassD(C);
  
  if ( !bDrawClock || ClockLocation != UpperMiddle || !bShowPoints )
    DrawSpriteWidget(C, BombBG);
  DrawSpriteWidget(C, BombWidget.Widgets[BombWidget.BombState]);
  
  if ( BombFlag == None )
    ForEach DynamicActors(Class'xBombFlag', BombFlag)
      break;
  
  if ( PawnOwner != None && BallLauncher(PawnOwner.Weapon) != None ) {
    if ( MyBombDelivery == None || MyBombDelivery.Team == PlayerOwner.PlayerReplicationInfo.Team.TeamIndex )
      ForEach DynamicActors(Class'xBombDelivery', MyBombDelivery)
        if ( MyBombDelivery.Team != PlayerOwner.PlayerReplicationInfo.Team.TeamIndex )
          break;
    if ( MyBombDelivery == None )
      return;
    Pos = MyBombDelivery.Location;
  }
  else if ( BombFlag != None ) {
    if ( Pawn(BombFlag.Base) != None )
      Pos = BombFlag.Base.Location;
    else
      Pos = BombFlag.Location;
  }
  else if ( PlayerOwner.GameReplicationInfo != None )
    Pos = PlayerOwner.GameReplicationInfo.FlagPos;
  Draw2DDirection(C, Pos, GoalIndicator);
}


//=============================================================================
// Timer
//
// Display the flag messages.
//=============================================================================

function Timer()
{
  Super.Timer();
  
  if ( PawnOwnerPRI == None )
    return;
  
  // offsets returned in Y can be the same for xBombHUDMessage
  if ( BombFlag != None && BombFlag.HolderPRI != None && BombFlag.HolderPRI.Team != PawnOwnerPRI.Team)
    PlayerOwner.ReceiveLocalizedMessage(class'xBombHUDMessage', 1);
  else if ( PawnOwnerPRI.HasFlag != None )
    PlayerOwner.ReceiveLocalizedMessage(class'xBombHUDMessage', 0);
}


//=============================================================================
// UpdateTeamHud
//
// Update the bomb state and bomb widget location.
//=============================================================================

function UpdateTeamHud()
{
  Super.UpdateTeamHUD();
  
  if ( PlayerOwner.PlayerReplicationInfo.Team == None )
    BombWidget.BombState = GRI.FlagState[0];
  else
    BombWidget.BombState = GRI.FlagState[PlayerOwner.PlayerReplicationInfo.Team.TeamIndex];

  if ( bDrawClock && ClockLocation == UpperMiddle && !bShowPoints ) {
    GetSpriteWidgetLocation(BombBG, default.BombBG, LocationsBombBG[1]);
    GetSpriteWidgetLocation(BombWidget.Widgets[BombWidget.BombState],
        default.BombWidget.Widgets[BombWidget.BombState], LocationsBombWidget[1]);
    GetRotatedWidgetLocation(GoalIndicator, default.GoalIndicator, LocationsGoalIndicator[1]);
  }
  else {
    GetSpriteWidgetLocation(BombBG, default.BombBG, LocationsBombBG[0]);
    GetSpriteWidgetLocation(BombWidget.Widgets[BombWidget.BombState],
        default.BombWidget.Widgets[BombWidget.BombState], LocationsBombWidget[0]);
    GetRotatedWidgetLocation(GoalIndicator, default.GoalIndicator, LocationsGoalIndicator[0]);
  }
  
  if ( bDrawClock && ClockLocation == UpperMiddle && bShowPoints ) {
    BombWidget.Widgets[BombWidget.BombState].RenderStyle = STY_Additive;
    BombWidget.Widgets[BombWidget.BombState].Tints[0].A = 160;
    BombWidget.Widgets[BombWidget.BombState].Tints[1].A = 160;
  }
  else {
    BombWidget.Widgets[BombWidget.BombState].RenderStyle = STY_Alpha;
    BombWidget.Widgets[BombWidget.BombState].Tints[0].A = 255;
    BombWidget.Widgets[BombWidget.BombState].Tints[1].A = 255;
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  TargetMaterial=Material'InterfaceContent.Hud.fbBombFocus'
  BombBG = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=0,Y1=880,X1=142,Y2=1023),TextureScale=0.35,DrawPivot=DP_UpperMiddle,PosX=0.5,PosY=0.01,OffsetX=0,OffsetY=-15,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  BombWidget = (BombState=FLAG_Home,Widgets[0]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=624,Y1=538,X2=728,Y2=655),TextureScale=0.3,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0085,OffsetX=0,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)),Widgets[1]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=744,Y1=538,X2=848,Y2=655),TextureScale=0.3,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0085,OffsetX=0,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[2]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=744,Y1=538,X2=848,Y2=655),TextureScale=0.3,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0085,OffsetX=0,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[3]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=624,Y1=538,X2=728,Y2=655),TextureScale=0.3,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0085,OffsetX=0,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255)))
  
  GoalIndicator = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=48,Y1=256,X2=146,Y2=354),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0,OffsetX=0,OffsetY=215,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=0,A=255),Tints[1]=(R=255,G=255,B=0,A=255)),bFixedAngle=True,OffsetCenterX=49,OffsetCenterY=49,OffsetRotatedX=0,OffsetRotatedY=-230)
  
  // team scores without clock
  LocationsLTeamScoreBG(0) = (PosX=0.5,OffsetX=-040,OffsetY=20,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(0) = (PosX=0.5,OffsetX=0040,OffsetY=20,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(0)   = (PosX=0.5,OffsetX=-340,OffsetY=100,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(0)   = (PosX=0.5,OffsetX=0340,OffsetY=100,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(0)  = (PosX=0.5,OffsetX=-360,OffsetY=200,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(0)  = (PosX=0.5,OffsetX=0360,OffsetY=200,DrawPivot=DP_UpperLeft)
  TeamSymbolScale(0)       = 0.075;
  
  // bomb widget locations
  LocationsBombBG(0)        = (PosX=0.5,PosY=0.0,OffsetX=000,OffsetY=000,DrawPivot=DP_UpperMiddle)
  LocationsBombBG(1)        = (PosX=0.5,PosY=0.0,OffsetX=095,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsBombWidget(0)    = (PosX=0.5,PosY=0.0,OffsetX=000,OffsetY=085,DrawPivot=DP_MiddleMiddle)
  LocationsBombWidget(1)    = (PosX=0.5,PosY=0.0,OffsetX=141,OffsetY=085,DrawPivot=DP_MiddleLeft)
  LocationsGoalIndicator(0) = (PosX=0.5,PosY=0.0,OffsetX=000,OffsetY=250,DrawPivot=DP_MiddleMiddle)
  LocationsGoalIndicator(1) = (PosX=0.5,PosY=0.0,OffsetX=580,OffsetY=250,DrawPivot=DP_MiddleMiddle)
  
  // upper left with points
  LocationsClockBG(0)          = (OffsetX=130,OffsetY=040,DrawPivot=DP_UpperLeft)
  LocationsClockCircle(0)      = (OffsetX=000,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsClockSeperator(0)   = (OffsetX=442,OffsetY=105,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(0) = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(0)   = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(0)  = (OffsetX=106,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(0)        = (OffsetX=087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(0)      = (OffsetX=087,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(0)     = (OffsetX=550,OffsetY=115,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(0)     = (OffsetX=580,OffsetY=115,DrawPivot=DP_UpperLeft)
  
  // upper middle without points
  LocationsClockBG(5)          = (PosX=0.5,OffsetX=0000,OffsetY=040,DrawPivot=DP_UpperMiddle)
  LocationsClockCircle(5)      = (PosX=0.5,OffsetX=-095,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsClockSeperator(5)   = (PosX=0.5,OffsetX=0000,OffsetY=108,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(5) = (PosX=0.5,OffsetX=-247,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(5)   = (PosX=0.5,OffsetX=-247,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(5)  = (PosX=0.5,OffsetX=-245,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(5)        = (PosX=0.5,OffsetX=-203,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(5)      = (PosX=0.5,OffsetX=-203,OffsetY=088,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(5)     = (PosX=0.5,OffsetX=-015,OffsetY=115,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(5)     = (PosX=0.5,OffsetX=0020,OffsetY=115,DrawPivot=DP_UpperLeft)
}