//=============================================================================
// TournamentHUDCTF
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Capture The Flag HUD
//=============================================================================


class TournamentHUDCTF extends TournamentHUDTeamDM;


//=============================================================================
// Variables
//=============================================================================

var() SpriteWidget FlagBG[2];
var() HudBCaptureTheFlag.FFlagWidget FlagWidgets[2];
var() RotatedWidget BaseIndicator[2];

var() TWidgetLocation LocationsLFlagBG[2];
var() TWidgetLocation LocationsLFlagWidget[2];
var() TWidgetLocation LocationsLBaseIndicator[2];
var() TWidgetLocation LocationsRFlagBG[2];
var() TWidgetLocation LocationsRFlagWidget[2];
var() TWidgetLocation LocationsRBaseIndicator[2];

var transient CTFBase RedBase;
var transient CTFBase BlueBase;


//=============================================================================
// PostBeginPlay
//
// Start the timer that will handle the flag warning messages.
//=============================================================================

function PostBeginPlay()
{
  Super.PostBeginPlay();
  SetTimer(1.0, True);
}


//=============================================================================
// ShowPointBarBottom
// ShowPointBarTop
// ShowPersonalScore
//
// Personal score is disabled in CTF.
//=============================================================================

function ShowPointBarBottom(Canvas C);
function ShowPointBarTop(Canvas C);
function ShowPersonalScore(Canvas C);


//=============================================================================
// ShowTeamScorePassA
//
// First alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassA(Canvas C)
{
  Super.ShowTeamScorePassA(C);
  
  DrawSpriteWidget(C, FlagBG[0]);
  DrawSpriteWidget(C, FlagBG[1]);
}


//=============================================================================
// ShowTeamScorePassC
//
// Second alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassC(Canvas C)
{
  local CTFBase B;
  
  Super.ShowTeamScorePassC(C);
  DrawSpriteWidget(C, FlagWidgets[0].Widgets[FlagWidgets[0].FlagState]);
  DrawSpriteWidget(C, FlagWidgets[1].Widgets[FlagWidgets[1].FlagState]);
  
  if ( RedBase == None ) {
    ForEach DynamicActors(Class'CTFBase', B) {
      if ( B.IsA('xRedFlagBase') )
        RedBase = B;
      else
        BlueBase = B;
    }
  }
  if ( RedBase != None )
    Draw2DDirection(C, RedBase.Location, BaseIndicator[0]);
  
  if ( BlueBase != None )
    Draw2DDirection(C, BlueBase.Location, BaseIndicator[1]);
}


//=============================================================================
// Timer
//
// Display the flag messages.
//=============================================================================

function Timer()
{
  Super.Timer();
  
  if ( PawnOwnerPRI == None || PlayerOwner.IsSpectating()
      && (PlayerOwner.bBehindView || PlayerOwner.ViewTarget == PlayerOwner) )
    return;
  
  if ( PawnOwnerPRI.HasFlag != None )
    PlayerOwner.ReceiveLocalizedMessage(class'CTFHUDMessage', 0);
  
  if ( PlayerOwner.GameReplicationInfo != None
      && PlayerOwner.GameReplicationInfo.FlagState[PlayerOwner.PlayerReplicationInfo.Team.TeamIndex]
      == EFlagState.FLAG_HeldEnemy )
    PlayerOwner.ReceiveLocalizedMessage(class'CTFHUDMessage', 1);
}


//=============================================================================
// UpdateTeamHud
//
// Update the FlagWidgets' FlagStates.
//=============================================================================

function UpdateTeamHud()
{
  local int i;
  
  for (i = 0; i < 2; i++)
    FlagWidgets[i].FlagState = PlayerOwner.GameReplicationInfo.FlagState[i];
  
  Super.UpdateTeamHUD();
  
  if ( bDrawClock && ClockLocation == UpperMiddle ) {
    GetSpriteWidgetLocation(FlagBG[0], default.FlagBG[0], LocationsLFlagBG[1]);
    GetSpriteWidgetLocation(FlagWidgets[0].Widgets[FlagWidgets[0].FlagState],
        default.FlagWidgets[0].Widgets[FlagWidgets[0].FlagState], LocationsLFlagWidget[1]);
    GetRotatedWidgetLocation(BaseIndicator[0], default.BaseIndicator[0], LocationsLBaseIndicator[1]);
    GetSpriteWidgetLocation(FlagBG[1], default.FlagBG[1], LocationsRFlagBG[1]);
    GetSpriteWidgetLocation(FlagWidgets[1].Widgets[FlagWidgets[1].FlagState],
        default.FlagWidgets[0].Widgets[FlagWidgets[1].FlagState], LocationsRFlagWidget[1]);
    GetRotatedWidgetLocation(BaseIndicator[1], default.BaseIndicator[1], LocationsRBaseIndicator[1]);
  }
  else {
    GetSpriteWidgetLocation(FlagBG[0], default.FlagBG[0], LocationsLFlagBG[0]);
    GetSpriteWidgetLocation(FlagWidgets[0].Widgets[FlagWidgets[0].FlagState],
        default.FlagWidgets[0].Widgets[FlagWidgets[0].FlagState], LocationsLFlagWidget[0]);
    GetRotatedWidgetLocation(BaseIndicator[0], default.BaseIndicator[0], LocationsLBaseIndicator[0]);
    GetSpriteWidgetLocation(FlagBG[1], default.FlagBG[1], LocationsRFlagBG[0]);
    GetSpriteWidgetLocation(FlagWidgets[1].Widgets[FlagWidgets[1].FlagState],
        default.FlagWidgets[0].Widgets[FlagWidgets[1].FlagState], LocationsRFlagWidget[0]);
    GetRotatedWidgetLocation(BaseIndicator[1], default.BaseIndicator[1], LocationsRBaseIndicator[0]);
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FlagBG(0) = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=0,Y1=880,X1=142,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  FlagBG(1) = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=0,Y1=880,X1=142,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  
  ScoreBg(0) = (TextureCoords=(X2=0,Y1=0,X1=0,Y2=0))
  ScoreBg(1) = (TextureCoords=(X2=0,Y1=0,X1=0,Y2=0))
  ScoreBg(2) = (TextureCoords=(X2=0,Y1=0,X1=0,Y2=0))
  
  FlagWidgets(0) = (FlagState=FLAG_Home,Widgets[0]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=743,Y1=97,X2=834,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=-67,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[1]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=926,Y1=97,X2=1017,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=-67,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[2]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=926,Y1=97,X2=1017,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=-67,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[3]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=835,Y1=97,X2=925,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=-67,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=255,G=60,B=60,A=255)))
  FlagWidgets(1) = (FlagState=FLAG_Home,Widgets[0]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=743,Y1=97,X1=834,Y2=147),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=68,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[1]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=926,Y1=97,X1=1017,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=68,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[2]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=926,Y1=97,X1=1017,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=68,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[3]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=835,Y1=97,X1=925,Y2=148),TextureScale=0.31,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=68,OffsetY=70,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=100,G=205,B=255,A=255)))
  
  BaseIndicator(0) = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=48,Y1=256,X2=146,Y2=354),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0,OffsetX=-215,OffsetY=215,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=0,B=0,A=255),Tints[1]=(R=255,G=0,B=0,A=255)),bFixedAngle=True,OffsetCenterX=49,OffsetCenterY=49,OffsetRotatedX=0,OffsetRotatedY=-190)
  BaseIndicator(1) = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=48,Y1=256,X2=146,Y2=354),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0,OffsetX=0215,OffsetY=215,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=0,G=0,B=255,A=255),Tints[1]=(R=0,G=0,B=255,A=255)),bFixedAngle=True,OffsetCenterX=49,OffsetCenterY=49,OffsetRotatedX=0,OffsetRotatedY=-190)
  
  // team scores without clock
  LocationsLTeamScoreBG(0) = (PosX=0.5,OffsetX=-095,OffsetY=12,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(0) = (PosX=0.5,OffsetX=0095,OffsetY=12,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(0)   = (PosX=0.5,OffsetX=-420,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(0)   = (PosX=0.5,OffsetX=0420,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(0)  = (PosX=0.5,OffsetX=-600,OffsetY=165,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(0)  = (PosX=0.5,OffsetX=0600,OffsetY=165,DrawPivot=DP_UpperLeft)
  TeamSymbolScale(0)       = 0.075;
  
  // team scores with clock
  LocationsLTeamScoreBG(1) = (PosX=0.5,OffsetX=-175,OffsetY=12,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(1) = (PosX=0.5,OffsetX=0175,OffsetY=12,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(1)   = (PosX=0.5,OffsetX=-500,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(1)   = (PosX=0.5,OffsetX=0500,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(1)  = (PosX=0.5,OffsetX=-920,OffsetY=165,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(1)  = (PosX=0.5,OffsetX=0920,OffsetY=165,DrawPivot=DP_UpperLeft)
  
  // flag widget locations without clock
  LocationsLFlagBG(0)        = (PosX=0.5,PosY=0.0,OffsetX=0000,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsLFlagWidget(0)    = (PosX=0.5,PosY=0.0,OffsetX=-068,OffsetY=070,DrawPivot=DP_MiddleMiddle)
  LocationsLBaseIndicator(0) = (PosX=0.5,PosY=0.0,OffsetX=-215,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  LocationsRFlagBG(0)        = (PosX=0.5,PosY=0.0,OffsetX=0000,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsRFlagWidget(0)    = (PosX=0.5,PosY=0.0,OffsetX=0068,OffsetY=070,DrawPivot=DP_MiddleMiddle)
  LocationsRBaseIndicator(0) = (PosX=0.5,PosY=0.0,OffsetX=0215,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  
  // flag widget locations with clock
  LocationsLFlagBG(1)        = (PosX=0.5,PosY=0.0,OffsetX=-080,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsLFlagWidget(1)    = (PosX=0.5,PosY=0.0,OffsetX=-145,OffsetY=070,DrawPivot=DP_MiddleMiddle)
  LocationsLBaseIndicator(1) = (PosX=0.5,PosY=0.0,OffsetX=-450,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  LocationsRFlagBG(1)        = (PosX=0.5,PosY=0.0,OffsetX=0080,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsRFlagWidget(1)    = (PosX=0.5,PosY=0.0,OffsetX=0145,OffsetY=070,DrawPivot=DP_MiddleMiddle)
  LocationsRBaseIndicator(1) = (PosX=0.5,PosY=0.0,OffsetX=0450,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  
  // upper left with points
  LocationsClockBG(0)          = (OffsetX=130,OffsetY=040,DrawPivot=DP_UpperLeft)
  LocationsClockCircle(0)      = (OffsetX=000,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsClockSeperator(0)   = (OffsetX=442,OffsetY=105,DrawPivot=DP_UpperMiddle)
  LocationsClockFilledFirst(0) = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillFirst(0)   = (OffsetX=104,OffsetY=107,DrawPivot=DP_MiddleLeft)
  LocationsClockFillSecond(0)  = (OffsetX=106,OffsetY=107,DrawPivot=DP_MiddleRight)
  LocationsClockTick(0)        = (OffsetX=087,OffsetY=090,DrawPivot=DP_MiddleMiddle)
  LocationsClockMarker(0)      = (OffsetX=087,OffsetY=090,DrawPivot=DP_MiddleMiddle)
  LocationsClockMinutes(0)     = (OffsetX=550,OffsetY=115,DrawPivot=DP_UpperRight)
  LocationsClockSeconds(0)     = (OffsetX=580,OffsetY=115,DrawPivot=DP_UpperLeft)
}
