//=============================================================================
// TournamentHUDDoubleDom
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Double Domination HUD
//=============================================================================


class TournamentHUDDoubleDom extends TournamentHUDTeamDM;


//=============================================================================
// Variables
//=============================================================================

var() SpriteWidget PointBG[2];
var() HudBDoubleDomination.FPointWidget PointWidgets[2];
var() RotatedWidget BaseIndicator[2];

var() TWidgetLocation LocationsLPointBG[2];
var() TWidgetLocation LocationsLPointWidget[2];
var() TWidgetLocation LocationsLBaseIndicator[2];
var() TWidgetLocation LocationsRPointBG[2];
var() TWidgetLocation LocationsRPointWidget[2];
var() TWidgetLocation LocationsRBaseIndicator[2];

var transient xDomPoint DP1;
var transient xDomPoint DP2;


//=============================================================================
// PostBeginPlay
//
// Start the timer that will handle the domination warning messages.
//=============================================================================

function PostBeginPlay()
{
  Super.PostBeginPlay();
  SetTimer(1.0, True);
}


//=============================================================================
// ShowTeamScorePassA
//
// Second alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassA(Canvas C)
{
  Super.ShowTeamScorePassA(C);
  
  DrawSpriteWidget(C, PointBG[0]);
  DrawSpriteWidget(C, PointBG[1]);
}


//=============================================================================
// ShowTeamScorePassC
//
// Second alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassC(Canvas C)
{
  Super.ShowTeamScorePassC(C);
  
  DrawSpriteWidget(C, PointWidgets[0].Widgets[PointWidgets[0].PointState]);
  DrawSpriteWidget(C, PointWidgets[1].Widgets[PointWidgets[1].PointState]);
  
  if ( DP1 != None )
    Draw2DDirection(C, DP1.Location, BaseIndicator[0]);
  
  if ( DP2 != None )
    Draw2DDirection(C, DP2.Location, BaseIndicator[1]);
}


//=============================================================================
// UpdateTeamHud
//
// Update the PointWidgets' PointStates.
//=============================================================================

function UpdateTeamHud()
{
  local NavigationPoint N;
  
  if ( DP1 == None || DP2 == None ) {
    for (N = Level.NavigationPointList; N != None; N = N.NextNavigationPoint) {
      if ( !N.IsA('xDomPoint') )
        continue;
      
      if ( N.IsA('xDomPointA') && DP1 == None )
        DP1 = xDomPoint(N);
      else
        DP2 = xDomPoint(N);
    }
  }
  
  // Point A state
  if ( DP1 == None || !DP1.bControllable )
    PointWidgets[0].PointState = PS_Disabled;
  else if ( DP1.ControllingTeam == None )
    PointWidgets[0].PointState = PS_Neutral;
  else if ( PlayerOwner.PlayerReplicationInfo.Team == None ) {
    if ( DP1.ControllingTeam.TeamIndex == 0 )
      PointWidgets[0].PointState = PS_HeldFriendly; 
    else
      PointWidgets[0].PointState = PS_HeldEnemy;
  }
  else if ( DP1.ControllingTeam == PawnOwnerPRI.Team )
    PointWidgets[0].PointState = PS_HeldFriendly;
  else
    PointWidgets[0].PointState = PS_HeldEnemy;
  
  // Point B state
  if ( DP2 == None || !DP2.bControllable )
    PointWidgets[1].PointState = PS_Disabled;
  else if ( DP2.ControllingTeam == None )
    PointWidgets[1].PointState = PS_Neutral;
  else if ( PlayerOwner.PlayerReplicationInfo.Team == None ) {
    if ( DP2.ControllingTeam.TeamIndex == 0 )
      PointWidgets[0].PointState = PS_HeldFriendly;
    else
      PointWidgets[0].PointState = PS_HeldEnemy;
  }
  else if (DP2.ControllingTeam == PawnOwnerPRI.Team)
    PointWidgets[1].PointState = PS_HeldFriendly;
  else
    PointWidgets[1].PointState = PS_HeldEnemy;
  
  Super.UpdateTeamHUD();
  
  if ( bDrawClock && ClockLocation == UpperMiddle ) {
    GetSpriteWidgetLocation(PointBG[0], default.PointBG[0], LocationsLPointBG[1]);
    GetSpriteWidgetLocation(PointWidgets[0].Widgets[PointWidgets[0].PointState],
        default.PointWidgets[0].Widgets[PointWidgets[0].PointState], LocationsLPointWidget[1]);
    GetRotatedWidgetLocation(BaseIndicator[0], default.BaseIndicator[0], LocationsLBaseIndicator[1]);
    GetSpriteWidgetLocation(PointBG[1], default.PointBG[1], LocationsRPointBG[1]);
    GetSpriteWidgetLocation(PointWidgets[1].Widgets[PointWidgets[1].PointState],
        default.PointWidgets[0].Widgets[PointWidgets[1].PointState], LocationsRPointWidget[1]);
    GetRotatedWidgetLocation(BaseIndicator[1], default.BaseIndicator[1], LocationsRBaseIndicator[1]);
  }
  else {
    GetSpriteWidgetLocation(PointBG[0], default.PointBG[0], LocationsLPointBG[0]);
    GetSpriteWidgetLocation(PointWidgets[0].Widgets[PointWidgets[0].PointState],
        default.PointWidgets[0].Widgets[PointWidgets[0].PointState], LocationsLPointWidget[0]);
    GetRotatedWidgetLocation(BaseIndicator[0], default.BaseIndicator[0], LocationsLBaseIndicator[0]);
    GetSpriteWidgetLocation(PointBG[1], default.PointBG[1], LocationsRPointBG[0]);
    GetSpriteWidgetLocation(PointWidgets[1].Widgets[PointWidgets[1].PointState],
        default.PointWidgets[0].Widgets[PointWidgets[1].PointState], LocationsRPointWidget[0]);
    GetRotatedWidgetLocation(BaseIndicator[1], default.BaseIndicator[1], LocationsRBaseIndicator[0]);
  }
}


//=============================================================================
// Timer
//
// Display the domination messages.
//=============================================================================

function Timer()
{
  local NavigationPoint N;
  
  Super.Timer();
  
  if ( PlayerOwner == None || PawnOwner == None || PawnOwnerPRI == None )
    return;
  
  if ( DP1 == None || DP2 == None ) {
    for (N = Level.NavigationPointList; N != None; N = N.NextNavigationPoint) {
      if ( !N.IsA ('xDomPoint') )
        continue;
      
      if ( N.IsA('xDomPointA') )
        DP1 = xDomPoint(N);
      else
        DP2 = xDomPoint(N);
    }
  }
  
  if ( DP1.ControllingTeam == DP2.ControllingTeam ) {
    if ( DP1.ControllingTeam != None ) {
      if ( PawnOwnerPRI.Team.TeamIndex == DP1.ControllingTeam.TeamIndex )
        PlayerOwner.ReceiveLocalizedMessage (class'xDomMessage', 0);
      else
        PlayerOwner.ReceiveLocalizedMessage (class'xDomMessage', 1);
    }
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  PointBG[0] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=0,Y1=880,X1=142,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  PointBG[1] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=0,Y1=880,X1=142,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  
  PointWidgets(0) = (PointState=PS_Neutral,Widgets[0]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=334,Y1=512,X2=391,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=-34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=195,G=195,B=195,A=255),Tints[1]=(R=195,G=195,B=195,A=255)),Widgets[1]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=334,Y1=512,X2=391,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=-34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[2]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=334,Y1=512,X2=391,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=-34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[3]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=334,Y1=512,X2=391,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=-34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=195,G=195,B=195,A=55),Tints[1]=(R=255,G=255,B=255,A=0)))
  PointWidgets(1) = (PointState=PS_Neutral,Widgets[0]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=392,Y1=512,X2=449,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=195,G=195,B=195,A=255),Tints[1]=(R=195,G=195,B=195,A=255)),Widgets[1]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=392,Y1=512,X2=449,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=255),Tints[1]=(R=100,G=205,B=255,A=255)),Widgets[2]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=392,Y1=512,X2=449,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=255),Tints[1]=(R=255,G=60,B=60,A=255)),Widgets[3]=(WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=392,Y1=512,X2=449,Y2=567),TextureScale=0.6,DrawPivot=DP_MiddleMiddle,OffsetX=34,PosX=0.5,PosY=0.045,ScaleMode=SM_None,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=195,G=195,B=195,A=55),Tints[1]=(R=255,G=255,B=255,A=0)))
  
  BaseIndicator(0) = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=48,Y1=256,X2=146,Y2=354),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0,OffsetX=-215,OffsetY=215,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=0,A=255),Tints[1]=(R=255,G=255,B=0,A=255)),bFixedAngle=True,OffsetCenterX=49,OffsetCenterY=49,OffsetRotatedX=0,OffsetRotatedY=-190)
  BaseIndicator(1) = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=48,Y1=256,X2=146,Y2=354),TextureScale=0.1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0,OffsetX=0215,OffsetY=215,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=0,A=255),Tints[1]=(R=255,G=255,B=0,A=255)),bFixedAngle=True,OffsetCenterX=49,OffsetCenterY=49,OffsetRotatedX=0,OffsetRotatedY=-190)
  
  // team scores without clock
  LocationsLTeamScoreBG(0) = (PosX=0.5,OffsetX=-095,OffsetY=12,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(0) = (PosX=0.5,OffsetX=0095,OffsetY=12,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(0)   = (PosX=0.5,OffsetX=-420,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(0)   = (PosX=0.5,OffsetX=0420,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(0)  = (PosX=0.5,OffsetX=-600,OffsetY=165,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(0)  = (PosX=0.5,OffsetX=0600,OffsetY=165,DrawPivot=DP_UpperLeft)
  TeamSymbolScale(0)       = 0.075;
  
  // team scores with clock
  LocationsLTeamScoreBG(1) = (PosX=0.5,OffsetX=-175,OffsetY=12,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(1) = (PosX=0.5,OffsetX=0175,OffsetY=12,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(1)   = (PosX=0.5,OffsetX=-500,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(1)   = (PosX=0.5,OffsetX=0500,OffsetY=90,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(1)  = (PosX=0.5,OffsetX=-920,OffsetY=165,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(1)  = (PosX=0.5,OffsetX=0920,OffsetY=165,DrawPivot=DP_UpperLeft)
  
  // point widget locations without clock
  LocationsLPointBG(0)       = (PosX=0.5,PosY=0.0,OffsetX=0000,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsLPointWidget(0)   = (PosX=0.5,PosY=0.0,OffsetX=-006,OffsetY=008,DrawPivot=DP_UpperRight)
  LocationsLBaseIndicator(0) = (PosX=0.5,PosY=0.0,OffsetX=-215,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  LocationsRPointBG(0)       = (PosX=0.5,PosY=0.0,OffsetX=0000,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsRPointWidget(0)   = (PosX=0.5,PosY=0.0,OffsetX=0006,OffsetY=008,DrawPivot=DP_UpperLeft)
  LocationsRBaseIndicator(0) = (PosX=0.5,PosY=0.0,OffsetX=0215,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  
  // point widget locations with clock
  LocationsLPointBG(1)       = (PosX=0.5,PosY=0.0,OffsetX=-080,OffsetY=000,DrawPivot=DP_UpperRight)
  LocationsLPointWidget(1)   = (PosX=0.5,PosY=0.0,OffsetX=-046,OffsetY=008,DrawPivot=DP_UpperRight)
  LocationsLBaseIndicator(1) = (PosX=0.5,PosY=0.0,OffsetX=-450,OffsetY=215,DrawPivot=DP_MiddleMiddle)
  LocationsRPointBG(1)       = (PosX=0.5,PosY=0.0,OffsetX=0080,OffsetY=000,DrawPivot=DP_UpperLeft)
  LocationsRPointWidget(1)   = (PosX=0.5,PosY=0.0,OffsetX=0046,OffsetY=008,DrawPivot=DP_UpperLeft)
  LocationsRBaseIndicator(1) = (PosX=0.5,PosY=0.0,OffsetX=0450,OffsetY=215,DrawPivot=DP_MiddleMiddle)
}