//=============================================================================
// TournamentHUDTeamDM
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Team DeathMatch HUD
//=============================================================================


class TournamentHUDTeamDM extends TournamentHUD;


//=============================================================================
// Variables
//=============================================================================

var() NumericWidget ScoreTeam[2];
var() NumericWidget Score;
var() NumericWidget TotalLinks;
var() SpriteWidget  LinkIcon;
var() SpriteWidget  LTeamHud[3];
var() SpriteWidget  RTeamHud[3];
var() SpriteWidget  TeamSymbols[2];

var() TWidgetLocation LocationsLTeamScoreBG[2];
var() TWidgetLocation LocationsRTeamScoreBG[2];
var() TWidgetLocation LocationsLTeamScore[2];
var() TWidgetLocation LocationsRTeamScore[2];
var() TWidgetLocation LocationsLTeamSymbol[2];
var() TWidgetLocation LocationsRTeamSymbol[2];
var() float           TeamSymbolScale[2];

var int    Links;


//=============================================================================
// DrawSpectatingHud
//
// Draw the spectator's HUD.
//=============================================================================

function DrawSpectatingHud(Canvas C)
{
  Super.DrawSpectatingHud(C);
  
  if ( PlayerOwner == None || PlayerOwner.PlayerReplicationInfo == None
      || !PlayerOwner.PlayerReplicationInfo.bOnlySpectator )
    return;
  
  UpdateRankAndSpread(C);
  ShowTeamScorePassA(C);
  ShowTeamScorePassC(C);
  UpdateTeamHUD();
}


//=============================================================================
// ShowLinks
//
// Update the number of LinkGun links.
//=============================================================================

function ShowLinks()
{
  local Inventory Inv;
  
  Inv = PawnOwner.FindInventoryType(class'LinkGun');
  
  if ( Inv != None )
    Links = LinkGun(Inv).Links;
  else
    Links = 0;
}


//=============================================================================
// UpdateRankAndSpread
//
// Rank and Spread don't get drawn in other gametypes than DM.
//=============================================================================

function UpdateRankAndSpread(Canvas C);


//=============================================================================
// DrawHudPassA
//
// First alpha pass.
//=============================================================================

function DrawHudPassA(Canvas C)
{
  Super.DrawHudPassA (C);
  UpdateRankAndSpread(C);
  ShowTeamScorePassA(C);
  if ( bShowWeaponInfo ) {
    TotalLinks.Value = Links;
    if ( Links > 0 ) {
      LinkIcon.Tints[TeamIndex].A = 255 - Abs(Level.Millisecond * 0.2 - 100);
      DrawSpriteWidget(C, LinkIcon);
      DrawNumericWidget(C, TotalLinks, DigitsBig);
    }
  }
}


//=============================================================================
// ShowTeamScorePassA
//
// First alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassA(Canvas C)
{
  if ( bShowPoints ) {
    DrawSpriteWidget(C, LTeamHud[2]);
    DrawSpriteWidget(C, LTeamHud[1]);
    DrawSpriteWidget(C, LTeamHud[0]);
    
    DrawSpriteWidget(C, RTeamHud[2]);
    DrawSpriteWidget(C, RTeamHud[1]);
    DrawSpriteWidget(C, RTeamHud[0]);
  }
}


//=============================================================================
// ShowTeamScorePassC
//
// Second alpha pass of team scores.
//=============================================================================

function ShowTeamScorePassC(Canvas C)
{
  if ( bShowPoints ) {
    if (TeamSymbols[0].WidgetTexture != None)
      DrawSpriteWidget (C, TeamSymbols[0]);
    
    if (TeamSymbols[1].WidgetTexture != None)
      DrawSpriteWidget (C, TeamSymbols[1]);
    
    DrawNumericWidget (C, ScoreTeam[0], DigitsBig);
    DrawNumericWidget (C, ScoreTeam[1], DigitsBig);
  }
}


//=============================================================================
// ShowPersonalScore
//
// Draw the player's personal score.
//=============================================================================

function ShowPersonalScore(Canvas C)
{
  DrawNumericWidget(C, Score, DigitsBig);
}


//=============================================================================
// DrawHudPassC
//
// Second alpha pass.
//=============================================================================

function DrawHudPassC(Canvas C)
{
  Super.DrawHudPassC (C);
  
  if ( bShowPoints )
    ShowPersonalScore(C);
  ShowTeamScorePassC(C);
}


//=============================================================================
// UpdateTeamHud
//
// Update the team game-relevant parts of the HUD.
//=============================================================================

function UpdateTeamHud()
{
  local int i;
  
  if ( !bShowPoints )
    return;
  
  if ( bDrawClock && ClockLocation == UpperMiddle ) {
    for (i = 0; i < 3; i++) {
      GetSpriteWidgetLocation(RTeamHud[i], default.RTeamHud[i], LocationsRTeamScoreBG[1]);
      GetSpriteWidgetLocation(LTeamHud[i], default.LTeamHud[i], LocationsLTeamScoreBG[1]);
    }
    GetSpriteWidgetLocation(TeamSymbols[0], default.TeamSymbols[0], LocationsLTeamSymbol[1]);
    GetSpriteWidgetLocation(TeamSymbols[1], default.TeamSymbols[1], LocationsRTeamSymbol[1]);
    TeamSymbols[0].TextureScale = TeamSymbolScale[1];
    TeamSymbols[1].TextureScale = TeamSymbolScale[1];
    GetNumericWidgetLocation(ScoreTeam[0], LocationsLTeamScore[1]);
    GetNumericWidgetLocation(ScoreTeam[1], LocationsRTeamScore[1]);
  }
  else {
    for (i = 0; i < 3; i++) {
      GetSpriteWidgetLocation(RTeamHud[i], default.RTeamHud[i], LocationsRTeamScoreBG[0]);
      GetSpriteWidgetLocation(LTeamHud[i], default.LTeamHud[i], LocationsLTeamScoreBG[0]);
    }
    GetSpriteWidgetLocation(TeamSymbols[0], default.TeamSymbols[0], LocationsLTeamSymbol[0]);
    GetSpriteWidgetLocation(TeamSymbols[1], default.TeamSymbols[1], LocationsRTeamSymbol[0]);
    TeamSymbols[0].TextureScale = TeamSymbolScale[0];
    TeamSymbols[1].TextureScale = TeamSymbolScale[0];
    GetNumericWidgetLocation(ScoreTeam[0], LocationsLTeamScore[0]);
    GetNumericWidgetLocation(ScoreTeam[1], LocationsRTeamScore[0]);
  }
  
  Score.Value = Clamp(CurScore, -999, 999);    // drawing limitation
  
  for (i = 0; i < 2; i++) {
    if ( GRI.Teams[i] == None )
      continue;
    
    ScoreTeam[i].Value = Clamp(GRI.Teams[i].Score, -99, 999);  // max space in hud
    
    if ( GRI.TeamSymbols[i] != None )
      TeamSymbols[i].WidgetTexture = GRI.TeamSymbols[i];
  }
}


//=============================================================================
// UpdateHud
//
// Update the HUD variables and link status.
//=============================================================================

function UpdateHud()
{
  Super.UpdateHud();
  UpdateTeamHUD();
  ShowLinks();
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  RTeamHud[0] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=611,Y1=900,X2=979,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=200),Tints[1]=(R=255,G=255,B=255,A=200))
  RTeamHud[1] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=611,Y1=777,X2=979,Y2=899),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=0,B=0,A=100),Tints[1]=(R=37,G=66,B=102,A=150))
  RTeamHud[2] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X1=611,Y1=654,X2=979,Y2=776),TextureScale=0.3,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=0,B=0,A=200),Tints[1]=(R=48,G=75,B=120,A=200))
  
  LTeamHud[0] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=611,Y1=900,X1=979,Y2=1023),TextureScale=0.3,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  LTeamHud[1] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=611,Y1=777,X1=979,Y2=899),TextureScale=0.3,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=0,B=0,A=100),Tints[1]=(R=37,G=66,B=102,A=150))
  LTeamHud[2] = (WidgetTexture=Material'InterfaceContent.Hud.SkinA',TextureCoords=(X2=611,Y1=654,X1=979,Y2=776),TextureScale=0.3,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=0,OffsetY=0,ScaleMode=SM_Left,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=0,B=0,A=200),Tints[1]=(R=48,G=75,B=120,A=200))
  
  LinkIcon = (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=64,Y1=355,X2=181,Y2=423),TextureScale=0.65,DrawPivot=DP_LowerRight,PosX=1.0,PosY=0.835,OffsetX=0,OffsetY=0,ScaleMode=SM_Left,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=128,G=255,B=128,A=255),Tints[1]=(R=128,G=255,B=128,A=255))
  TotalLinks = (TextureScale=0.3,MinDigitCount=1,DrawPivot=DP_LowerMiddle,PosX=1.0,PosY=0.835,OffsetX=-130,OffsetY=-40,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  
  ScoreTeam(0) = (TextureScale=0.24,MinDigitCount=1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=-290,OffsetY=80,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ScoreTeam(1) = (TextureScale=0.24,MinDigitCount=1,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.0,OffsetX=290,OffsetY=80,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  
  Score = (TextureScale=0.18,MinDigitCount=2,DrawPivot=DP_UpperRight,PosX=0,PosY=0,OffsetX=450,OffsetY=40,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ScoreBG(0)=(OffsetX=10,OffsetY=10)
  ScoreBG(1)=(OffsetX=10,OffsetY=10)
  ScoreBG(2)=(OffsetX=10,OffsetY=10)
  ScoreBG(3)=(TextureCoords=(X2=0,Y1=0,X1=0,Y2=0))  // hide circle
  
  TeamSymbols(0) = (TextureCoords=(X1=0,Y1=0,X2=256,Y2=256),TextureScale=0.1,DrawPivot=DP_UpperRight,PosX=0.5,PosY=0.0,OffsetX=-60,OffsetY=60,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=60,B=60,A=150),Tints[1]=(R=255,G=60,B=60,A=150))
  TeamSymbols(1) = (TextureCoords=(X1=0,Y1=0,X2=256,Y2=256),TextureScale=0.1,DrawPivot=DP_UpperLeft,PosX=0.5,PosY=0.0,OffsetX=90,OffsetY=60,RenderStyle=STY_Alpha,Tints[0]=(R=100,G=205,B=255,A=150),Tints[1]=(R=100,G=205,B=255,A=150))
  
  // team scores without clock
  LocationsLTeamScoreBG(0) = (PosX=0.5,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(0) = (PosX=0.5,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(0)   = (PosX=0.5,OffsetX=-290,OffsetY=80,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(0)   = (PosX=0.5,OffsetX=0290,OffsetY=80,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(0)  = (PosX=0.5,OffsetX=-060,OffsetY=60,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(0)  = (PosX=0.5,OffsetX=0060,OffsetY=60,DrawPivot=DP_UpperLeft)
  TeamSymbolScale(0)       = 0.1;
  
  // team scores with clock
  LocationsLTeamScoreBG(1) = (PosX=0.5,OffsetX=-040,OffsetY=020,DrawPivot=DP_UpperRight)
  LocationsRTeamScoreBG(1) = (PosX=0.5,OffsetX=0040,OffsetY=020,DrawPivot=DP_UpperLeft)
  LocationsLTeamScore(1)   = (PosX=0.5,OffsetX=-340,OffsetY=100,DrawPivot=DP_MiddleMiddle)
  LocationsRTeamScore(1)   = (PosX=0.5,OffsetX=0340,OffsetY=100,DrawPivot=DP_MiddleMiddle)
  LocationsLTeamSymbol(1)  = (PosX=0.5,OffsetX=-360,OffsetY=200,DrawPivot=DP_UpperRight)
  LocationsRTeamSymbol(1)  = (PosX=0.5,OffsetX=0360,OffsetY=200,DrawPivot=DP_UpperLeft)
  TeamSymbolScale(1)       = 0.075;
}