//=============================================================================
// TournamentMutator
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Base mutator for Tournament gametypes.
//=============================================================================


class TournamentMutator extends DMMutator;


//=============================================================================
// Structs
//=============================================================================

struct TWeaponFireReplacements {
  var class<WeaponFire> WFClass;
  var class<WeaponFire> StatsReplacement;
};

//=============================================================================
// Variables
//=============================================================================

var array<TWeaponFireReplacements> WeaponFireReplacements;
var bool bInitialized;
var bool bWeaponStats;


//=============================================================================
// PostBeginPlay
//
// Upgrade the translocator for SuperBots.
//=============================================================================

function PostBeginPlay()
{
  local TournamentGameRules MyGameRules;
  
  if ( bInitialized )
    return;
  bInitialized = True;
  bWeaponStats = class'TournamentServerSettings'.default.bWeaponModifications;
  
  MyGameRules = Spawn(class'TournamentGameRules');
  if ( MyGameRules != None ) {
    MyGameRules.NextGameRules = Level.Game.GameRulesModifiers;
    Level.Game.GameRulesModifiers = MyGameRules;
    log("Base GameRules is"@MyGameRules);
  }
}


//=============================================================================
// ModifyPlayer
//
// Give the player all weapons and full ammo/health/shield in warmup.
//=============================================================================

function ModifyPlayer(Pawn Other)
{
  local xWeaponBase W;
  local Inventory Inv;
  
  Super.ModifyPlayer(Other);
  
  if ( xPawn(Other) != None && Level.Game.IsInState('PendingMatch') ) {
    if ( class'TournamentServerSettings'.default.bWarmupWeapons ) {
      foreach AllActors(class'xWeaponBase', W)
        if ( W.WeaponType != None && W.WeaponType.default.InventoryGroup != 0 )
          Other.CreateInventory(string(W.WeaponType));
      if ( class'TournamentServerSettings'.default.bWarmupAmmo )
        for (Inv = Other.Inventory; Inv != None; Inv = Inv.Inventory)
          if ( Ammunition(Inv) != None ) {
            Ammunition(Inv).AmmoAmount = 999;
            Ammunition(Inv).MaxAmmo = 999;
          }
    }
    if ( class'TournamentServerSettings'.default.bWarmupShield )
      Other.AddShieldStrength(150);
    if ( class'TournamentServerSettings'.default.bWarmupHealth )
      Other.GiveHealth(199, 199);
  }
}


//=============================================================================
// CheckReplacement
//
// Make sure the correct PRI is used, change KILLZ_None to KILLZ_Suicide and
// maybe modify weapons.
//=============================================================================

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
  if ( Controller(Other) != None ) {
    if ( Controller(Other).PlayerReplicationInfoClass == class'xPlayerReplicationInfo' )
      Controller(Other).PlayerReplicationInfoClass = class'TournamentPlayerReplicationInfo';
    if ( Controller(Other).PawnClass == class'xPawn' )
      Controller(Other).PawnClass = class'TournamentPawn';
  }
  else if ( Other.IsA('ZoneInfo') && ZoneInfo(Other).KillZType == KILLZ_None ) {
    // KILLZ_None looks stupid in almost all cases, especially on BR-Skyline
    ZoneInfo(Other).KillZType = KILLZ_Suicide;
  }
  
  return Super.CheckReplacement(Other, bSuperRelevant);
}


//=============================================================================
// CheckRelevance
//
// Replace the WeaponFire classes for stats tracking.
//=============================================================================

function bool CheckRelevance(Actor Other)
{
  local bool bResult;
  local byte bSuperRelevant;
  local int i, j;
  
  if ( !AlwaysKeep(Other) ) {
    bResult = IsRelevant(Other, bSuperRelevant);
    if ( bResult && bWeaponStats && Other.Class == class'IonCannon' ) {
      ReplaceWith(Other, string(class'TStatsIonCannon'));
      return false;
    }
  }
  else
    bResult = true;
  
  if ( bResult && bWeaponStats && Other.IsA('Weapon') )
    for (i = 0; i < 2; i++) {
      for (j = 0; j < WeaponFireReplacements.Length; j++)
        if ( Weapon(Other).FireModeClass[i] == WeaponFireReplacements[j].WFClass ) {
          Weapon(Other).FireModeClass[i] = WeaponFireReplacements[j].StatsReplacement;
          break;
        }
      if ( Weapon(Other).FireModeClass[i] != None && Weapon(Other).FireModeClass[i].IsA('FixedLinkFire') )
        Weapon(Other).FireModeClass[i] = class'TStatsLinkBeamFire';
    }
  
  return bResult;
}


//=============================================================================
// MutatorFillPlayInfo
//
// Adds the Tournament server configuration to the webadmin interface.
//=============================================================================

function MutatorFillPlayInfo(PlayInfo PlayInfo)
{
  PlayInfo.AddClass(class'TournamentServerSettings');
  
  PlayInfo.AddSetting("Adrenaline", "bAllowComboBerserk",   "Allow Berserk Combo",   1, 0, "Check");
  PlayInfo.AddSetting("Adrenaline", "bAllowComboBooster",   "Allow Booster Combo",   1, 0, "Check");
  PlayInfo.AddSetting("Adrenaline", "bAllowComboInvisible", "Allow Invisible Combo", 1, 0, "Check");
  PlayInfo.AddSetting("Adrenaline", "bAllowComboSpeed",     "Allow Speed Combo",     1, 0, "Check");
  
  PlayInfo.AddSetting("Warmup", "WarmupTime",     "Maximum Warmup Time", 0, 10, "Text", "2;1:15");
  PlayInfo.AddSetting("Warmup", "bWarmupWeapons", "All Map Weapons",     0, 20, "Check");
  PlayInfo.AddSetting("Warmup", "bWarmupAmmo",    "Full Ammo",           0, 21, "Check");
  PlayInfo.AddSetting("Warmup", "bWarmupHealth",  "Full Health",         0, 30, "Check");
  PlayInfo.AddSetting("Warmup", "bWarmupShield",  "Full Shield",         0, 40, "Check");
  
  PlayInfo.AddSetting("Tournament", "bDisableLockdown",           "Disable Lockdown",              1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bAllowHitSounds",            "Allow Hit Sounds",              1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bAllowVariableHitSounds",    "Allow Variable Hit Sounds",     1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bBroadcastUDamageKills",     "Broadcast Double Damage Kills", 1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bNoHudClock",                "Disallow HUD Clock",            1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bWeaponModifications",       "Track Weapon Stats",            1, 0, "Check");
  PlayInfo.AddSetting("Tournament", "bAllowCarrierTelefrags",     "Telefrag Flag/Bomb Carrier",    1, 0, "Check");
  
  PlayInfo.AddSetting("Tournament", "SkinGlow",      "Skin Glow (default = 40)",   241, 300, "Text", "3;0:200");
  PlayInfo.AddSetting("Tournament", "MaxFOV",        "Maximum Default FOV",        241, 300, "Text", "3;80:170");
  PlayInfo.AddSetting("Tournament", "MinNameLength", "Minimum Player Name Length", 241, 300, "Text", "1;1:5");
  
//  PlayInfo.AddSetting("Tournament", "bAutoServerDemo", "Auto-Record Server Demos", 255, 301, "Check");
//  PlayInfo.AddSetting("Tournament", "ServerDemoName",  "Server Demo File Name",    255, 302, "Text", "50");
  
  PlayInfo.PopClass();
  
  if ( NextMutator != None )
    NextMutator.MutatorFillPlayInfo(PlayInfo);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  FriendlyName="Tournament"
  GroupName="TTM2003"   // disallow TTM, it only messes things up
  
  WeaponFireReplacements(0)  = (WFClass=class'ShieldFire',StatsReplacement=class'TStatsShieldFire')
  WeaponFireReplacements(1)  = (WFClass=class'AssaultFire',StatsReplacement=class'TStatsAssaultFire')
  WeaponFireReplacements(2)  = (WFClass=class'AssaultGrenade',StatsReplacement=class'TStatsAssaultGrenadeFire')
  WeaponFireReplacements(3)  = (WFClass=class'BioFire',StatsReplacement=class'TStatsBioFire')
  WeaponFireReplacements(4)  = (WFClass=class'BioChargedFire',StatsReplacement=class'TStatsBioChargedFire')
  WeaponFireReplacements(5)  = (WFClass=class'ShockBeamFire',StatsReplacement=class'TStatsShockBeamFire')
  WeaponFireReplacements(6)  = (WFClass=class'ShockProjFire',StatsReplacement=class'TStatsShockProjFire')
  WeaponFireReplacements(7)  = (WFClass=class'LinkAltFire',StatsReplacement=class'TStatsLinkProjFire')
  WeaponFireReplacements(8)  = (WFClass=class'LinkFire',StatsReplacement=class'TStatsLinkBeamFire')
  WeaponFireReplacements(9)  = (WFClass=class'MinigunFire',StatsReplacement=class'TStatsMinigunFire')
  WeaponFireReplacements(10) = (WFClass=class'MinigunAltFire',StatsReplacement=class'TStatsMinigunAltFire')
  WeaponFireReplacements(11) = (WFClass=class'FlakFire',StatsReplacement=class'TStatsFlakFire')
  WeaponFireReplacements(12) = (WFClass=class'FlakAltFire',StatsReplacement=class'TStatsFlakAltFire')
  WeaponFireReplacements(13) = (WFClass=class'RocketFire',StatsReplacement=class'TStatsRocketFire')
  WeaponFireReplacements(14) = (WFClass=class'RocketMultiFire',StatsReplacement=class'TStatsRocketMultiFire')
  WeaponFireReplacements(15) = (WFClass=class'SniperFire',StatsReplacement=class'TStatsSniperFire')
  WeaponFireReplacements(16) = (WFClass=class'TransFire',StatsReplacement=class'TStatsTransFire')
  WeaponFireReplacements(17) = (WFClass=class'TransRecall',StatsReplacement=class'TStatsTransRecall')
  WeaponFireReplacements(18) = (WFClass=class'SuperShockBeamFire',StatsReplacement=class'TStatsSuperShockBeamFire')
  WeaponFireReplacements(19) = (WFClass=class'ZoomSuperShockBeamFire',StatsReplacement=class'TStatsZoomSuperShockBeamFire')
  WeaponFireReplacements(20) = (WFClass=class'RedeemerFire',StatsReplacement=class'TStatsRedeemerFire')
  WeaponFireReplacements(21) = (WFClass=class'RedeemerGuidedFire',StatsReplacement=class'TStatsRedeemerGuidedFire')
}