//=============================================================================
// TournamentPawn
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Pawn class for Tournament gametypes
//=============================================================================


class TournamentPawn extends xPawn;


//=============================================================================
// Imports
//=============================================================================

#exec obj load file=..\Textures\TournamentPlayerSkins.utx


//=============================================================================
// Structs
//=============================================================================

struct TSkinCache {
  var Material DefaultSkin;
  var Material TeamSkins[7];
};


//=============================================================================
// Configuration
//=============================================================================

var globalconfig string TeamCharacterName;
var globalconfig string EnemyCharacterName;
var globalconfig bool bNoCustomModels;


//=============================================================================
// Variables
//=============================================================================

var int SkinColorIndex, EffectsColorIndex;
var Material DeResMatBody[8], DeResMatHead[8];
var class<Emitter> DeResEmitterClass[8];
var Material PlayerLights[8];
var class<Actor> TeleportEffects[8];
var class<Actor> TranslocOutEffects[8];
var bool bPawnIsStupidEnoughToLeapToItsDeathOffOfLedgesWhilstDucking;
var bool bBrightSkins;
var xUtil.PlayerRecord PlayerRecord;
var TournamentGameReplicationInfo TGRI;

var transient byte NextDamageMode;
var transient class<DamageType> TelefragDamage;
var transient byte TelefragMode;
var transient int LastHealthAdd;
var transient int LastShieldAdd;
var transient int LastAdrenalineAmount;

// performance optimization
var array<TSkinCache> SkinCache;  // global skin texture cache for all pawns, faster than DynamicLoadObject()
var array<string> SkinCacheNames; // skin names for the global cache
var Material BodySkins[8];        // this pawn's skin cache, prevents slower global cache hits every tick


//=============================================================================
// Replication
//=============================================================================

replication
{
  unreliable if ( Role < ROLE_Authority )
    ServerStartFiring;
}


//=============================================================================
// PostBeginPlay
//
// Initialize the SpawnTime clientside.
//=============================================================================

simulated function PostBeginPlay()
{
  Super.PostBeginPlay();
  SpawnTime = Level.TimeSeconds;
}


//=============================================================================
// PostNetReceive
//
// Don't reset AmbientGlow.
//=============================================================================

simulated event PostNetReceive()
{
  if ( PlayerReplicationInfo != None )
    Setup(class'xUtil'.static.FindPlayerRecord(PlayerReplicationInfo.CharacterName));
}


//=============================================================================
// Setup
//
// Replace default voice packs with their extended Tournament versions.
//=============================================================================

simulated function Setup(xUtil.PlayerRecord Rec, optional bool bLoadNow)
{
  local PlayerController LocalPlayer;
  local string ForcedModel;
  local string MeshPackage;
  local int i;
  local byte SavedGlow;
  
  LocalPlayer = Level.GetLocalPlayerController();
  if ( Level.Game != None )
    TGRI = TournamentGameReplicationInfo(Level.Game.GameReplicationInfo);
  if ( TGRI == None && LocalPlayer != None )
    TGRI = TournamentGameReplicationInfo(LocalPlayer.GameReplicationInfo);
  
  if ( Level.NetMode != NM_DedicatedServer ) {
    if ( !(default.EnemyCharacterName ~= GetPlacedCharacterName(default.EnemyCharacterName)) )
      default.EnemyCharacterName = "None";
    if ( !(default.TeamCharacterName ~= GetPlacedCharacterName(default.TeamCharacterName)) )
      default.TeamCharacterName = "None";
    
    MeshPackage = Caps(Left(Rec.MeshName, InStr(Rec.MeshName, ".")));
    if ( bNoCustomModels && MeshPackage != "ALIENS" && MeshPackage != "BOT" && MeshPackage != "JUGG"
        && MeshPackage != "HUMANFEMALEA" && MeshPackage != "HUMANMALEA" )
      Rec = class'xUtil'.static.FindPlayerRecord("Gorge");
    if ( Rec.Species == None || !(default.EnemyCharacterName ~= "None" && default.TeamCharacterName ~= "None") ) {
      if ( LocalPlayer != None && (PlayerReplicationInfo.Team != None
          && LocalPlayer.PlayerReplicationInfo.Team == PlayerReplicationInfo.Team
          || LocalPlayer.PlayerReplicationInfo == PlayerReplicationInfo) ) {
        if ( !(default.TeamCharacterName ~= "None") ) {
          ForcedModel = GetPlacedCharacterName(default.TeamCharacterName);
          Rec = class'xUtil'.static.FindPlayerRecord(ForcedModel);
        }
      }
      else if ( !(default.EnemyCharacterName ~= "None") ) {
        ForcedModel = GetPlacedCharacterName(default.EnemyCharacterName);
        Rec = class'xUtil'.static.FindPlayerRecord(ForcedModel);
      }
    }
  }
  Species = Rec.Species;
  RagdollOverride = Rec.Ragdoll;
  
  if ( Level.NetMode != NM_Client )
    SavedGlow = class'TournamentServerSettings'.default.SkinGlow;
  else
    SavedGlow = AmbientGlow;
  if ( Species.static.Setup(self, Rec) )
    ResetPhysicsBasedAnim();
  AmbientGlow = SavedGlow;
  
  PlayerRecord = Rec;
  SetTeamIndex();
  for (i = 0; i < 8; i++)
    BodySkins[i] = None;
  if ( Level.NetMode != NM_DedicatedServer )
    SetBodySkin();
  RepSkin = None;
  
  switch (VoiceClass) {
    case class'AlienMaleVoice':
      VoiceClass = class'TournamentVoiceAlienMale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'AlienFemaleVoice':
      VoiceClass = class'TournamentVoiceAlienFemale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'JuggMaleVoice':
      VoiceClass = class'TournamentVoiceJuggMale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'JuggFemaleVoice':
      VoiceClass = class'TournamentVoiceJuggFemale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'MercMaleVoice':
      VoiceClass = class'TournamentVoiceMercMale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'MercFemaleVoice':
      VoiceClass = class'TournamentVoiceMercFemale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'NightMaleVoice':
      VoiceClass = class'TournamentVoiceNightMale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'NightFemaleVoice':
      VoiceClass = class'TournamentVoicenightFemale';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
    case class'RobotVoice':
      VoiceClass = class'TournamentVoiceRobot';
      VoiceType = string(VoiceClass);
      PlayerReplicationInfo.VoiceType = VoiceClass;
      break;
  }
}


//=============================================================================
// SetTeamIndex
//
// Returns the pawn's team index for the skin.
//=============================================================================

simulated function SetTeamIndex()
{
  local PlayerReplicationInfo LocalPRI;
  local int LocalTeamIndex, MyTeamIndex;
  
  if ( Level.GetLocalPlayerController() != None )
    LocalPRI = Level.GetLocalPlayerController().PlayerReplicationInfo;
  
  if ( LocalPRI == None || LocalPRI.bOnlySpectator )
    LocalTeamIndex = -2;
  else if ( LocalPRI.Team == None )
    LocalTeamIndex = -1;
  else
    LocalTeamIndex = LocalPRI.Team.TeamIndex;
  
  if ( PlayerReplicationInfo == None )
    MyTeamIndex = -2;
  else if ( PlayerReplicationInfo.Team == None )
    MyTeamIndex = -1;
  else
    MyTeamIndex = PlayerReplicationInfo.Team.TeamIndex;
  
  if ( LocalTeamIndex < 0 && MyTeamIndex >= 0 || LocalTeamIndex == -2 ) { // spectating
    SkinColorIndex = MyTeamIndex + 1;
    EffectsColorIndex = MyTeamIndex + 1;
  }
  else if ( LocalTeamIndex == -1 && MyTeamIndex < 0 ) { // playing deathmatch
    if ( LocalPRI == PlayerReplicationInfo ) {
      SkinColorIndex = class'TournamentClientSettings'.default.TeamSkinColor;
      EffectsColorIndex = class'TournamentClientSettings'.default.TeamEffectsColor;
    }
    else {
      SkinColorIndex = class'TournamentClientSettings'.default.EnemySkinColor;
      EffectsColorIndex = class'TournamentClientSettings'.default.EnemyEffectsColor;
    }
  }
  else {
    if ( LocalTeamIndex == MyTeamIndex ) {
      if ( class'TournamentClientSettings'.default.TeamSkinColor == 0 )
        SkinColorIndex = MyTeamIndex + 1;
      else
        SkinColorIndex = class'TournamentClientSettings'.default.TeamSkinColor;
      if ( class'TournamentClientSettings'.default.TeamEffectsColor == 0 )
        EffectsColorIndex = MyTeamIndex + 1;
      else
        EffectsColorIndex = class'TournamentClientSettings'.default.TeamEffectsColor;
    }
    else {
      if ( class'TournamentClientSettings'.default.EnemySkinColor == 0 )
        SkinColorIndex = MyTeamIndex + 1;
      else
        SkinColorIndex = class'TournamentClientSettings'.default.EnemySkinColor;
      if ( class'TournamentClientSettings'.default.EnemyEffectsColor == 0 )
        EffectsColorIndex = MyTeamIndex + 1;
      else
        EffectsColorIndex = class'TournamentClientSettings'.default.EnemyEffectsColor;
    }
  }
}


//=============================================================================
// GetPlacedCharacterName
//
// Makes sure the desired character name is valid.
//=============================================================================

simulated function string GetPlacedCharacterName(string DesiredCharacter)
{
  if (   DesiredCharacter ~= "Gorge"    || DesiredCharacter ~= "Rylisa"
      || DesiredCharacter ~= "Malcolm"  || DesiredCharacter ~= "Sapphire"
      || DesiredCharacter ~= "Roc"      || DesiredCharacter ~= "Diva"
      || DesiredCharacter ~= "Damarus"  || DesiredCharacter ~= "Mokara"
      || DesiredCharacter ~= "Brutalis" || DesiredCharacter ~= "Lilith"
      || DesiredCharacter ~= "Rapier" )
    return DesiredCharacter;
  else
    return "Gorge";
}


//=============================================================================
// SetBodySkin
//
// Loads and sets the body (and DeRes) skin.
//=============================================================================

simulated function SetBodySkin()
{
  local PlayerController LocalPlayer;
  local bool bSpectating;
  
  LocalPlayer = Level.GetLocalPlayercontroller();
  if ( LocalPlayer != None && LocalPlayer.PlayerReplicationInfo != None
      && LocalPlayer.PlayerReplicationInfo.bOnlySpectator ) {
    if ( Skins[0] != None && bBrightSkins != class'DMMutator'.default.bBrightSkins ) {
      BodySkins[SkinColorIndex] = None;
      bBrightSkins = class'DMMutator'.default.bBrightSkins;
    }
    bSpectating = True;
  }
  
  if ( Skins[0] != None && Skins[0] == BodySkins[SkinColorIndex] )
    return;
  
  if ( BodySkins[SkinColorIndex] == None )
    BodySkins[SkinColorIndex] = GetSkin(PlayerRecord.BodySkinName, SkinColorIndex - 1, bSpectating);
  
  Skins[0] = BodySkins[SkinColorIndex];
  
  if ( default.DeResMatBody[SkinColorIndex] == None ) {
    default.DeResMatBody[SkinColorIndex] = Material(DynamicLoadObject("TournamentPlayerSkins" $ SkinColorIndex - 1
        $ ".DeResFinalBody" $ SkinColorIndex - 1, class'Material', true));
    if ( default.DeResMatBody[SkinColorIndex] == None ) {
      default.DeResMatBody[SkinColorIndex] = DeResMatBody[SkinColorIndex % 2 + 1];
      default.DeResEmitterClass[SkinColorIndex] = DeResEmitterClass[SkinColorIndex % 2 + 1];
      DeResEmitterClass[SkinColorIndex] = DeResEmitterClass[SkinColorIndex % 2 + 1];
    }
    default.DeResMatHead[SkinColorIndex] = Material(DynamicLoadObject("TournamentPlayerSkins" $ SkinColorIndex - 1
        $ ".DeResFinalHead" $ SkinColorIndex - 1, class'Material', true));
    if ( default.DeResMatHead[SkinColorIndex] == None )
      default.DeResMatHead[SkinColorIndex] = DeResMatHead[SkinColorIndex % 2 + 1];
  }
}


//=============================================================================
// GetSkin
//
// Looks up a skin in the static skin cache and returns it if found or
// dynamically loads the skin.
//=============================================================================

simulated function Material GetSkin(string SkinTexture, int TeamIndex, bool bSpectating)
{
  local int SkinIndex, i;
  local string SkinPackage, SkinName;
  local Material NewSkin;
  
  if ( bSpectating && TeamIndex > 1 )
    TeamIndex = TeamIndex % 2;
  
  SkinTexture = Caps(SkinTexture);
  if ( default.SkinCacheNames.Length > 0 ) {
    SkinIndex = class'TournamentUtil'.static.BinarySearch(default.SkinCacheNames, SkinTexture, True);
    if ( SkinIndex == default.SkinCacheNames.Length || default.SkinCacheNames[SkinIndex] != SkinTexture ) {
      default.SkinCacheNames.Insert(SkinIndex, 1);
      default.SkinCacheNames[SkinIndex] = SkinTexture;
      default.SkinCache.Insert(SkinIndex, 1);
      if ( string(Skins[0]) ~= SkinTexture )
        default.SkinCache[SkinIndex].DefaultSkin = Skins[0];
    }
  }
  else {
    default.SkinCacheNames[0] = SkinTexture;
    default.SkinCache.Length = 1;
    if ( string(Skins[0]) ~= SkinTexture )
      default.SkinCache[0].DefaultSkin = Skins[0];
  }
  
  if ( TeamIndex != -1 ) {
    if ( !bSpectating || bBrightSkins ) {
      if ( default.SkinCache[SkinIndex].TeamSkins[TeamIndex] != None )
        return default.SkinCache[SkinIndex].TeamSkins[TeamIndex];
      
      i = InStr(PlayerRecord.BodySkinName, ".");
      SkinPackage = Left(SkinTexture, i);
      SkinName = Mid(SkinTexture, i + 1);
      if ( TeamIndex >= 2 || Caps(SkinPackage) != "PLAYERSKINS" ) {
        NewSkin = Material(DynamicLoadObject("Tournament" $ SkinPackage $ TeamIndex $ "." $ SkinName
            $ "_" $ TeamIndex, class'Material', true));
      }
      if ( NewSkin != None ) {
        default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = NewSkin;
        return NewSkin;
      }
    }
    if ( TeamIndex < 2 || Caps(SkinPackage) != "PLAYERSKINS" ) {
      if ( !bSpectating || bBrightSkins ) {
        NewSkin = Material(DynamicLoadObject("Tournament" $ SkinTexture $ "_" $ TeamIndex, class'Material', true));
        if ( NewSkin != None ) {
          default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = NewSkin;
          return NewSkin;
        }
        NewSkin = Material(DynamicLoadObject("Bright" $ SkinTexture $ "_" $ TeamIndex $ "B", class'Material', true));
        if ( NewSkin != None ) {
          default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = NewSkin;
          return NewSkin;
        }
      }
      NewSkin = Material(DynamicLoadObject(SkinTexture $ "_" $ TeamIndex, class'Material', true));
    }
    if ( NewSkin != None ) {
      default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = NewSkin;
      return NewSkin;
    }
    else if ( TeamIndex >= 2 ) {
      default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = GetSkin(SkinTexture, TeamIndex % 2, bSpectating);
      return default.SkinCache[SkinIndex].TeamSkins[TeamIndex];
    }
  }
  if ( default.SkinCache[SkinIndex].DefaultSkin == None )
    default.SkinCache[SkinIndex].DefaultSkin = Material(DynamicLoadObject(SkinTexture, class'Material', true));
  if ( TeamIndex != -1 )
    default.SkinCache[SkinIndex].TeamSkins[TeamIndex] = default.SkinCache[SkinIndex].DefaultSkin;
  return default.SkinCache[SkinIndex].DefaultSkin;
}


//=============================================================================
// PlayTeleportEffect
//
// Creates a teleport or respawn effect.
//=============================================================================

function PlayTeleportEffect(bool bOut, bool bSound)
{
  Spawn(TeleportEffects[EffectsColorIndex],,, Location + CollisionHeight * vect(0,0,0.75));
  Super(UnrealPawn).PlayTeleportEffect(bOut, bSound);
}


//=============================================================================
// Tick
//
// Helps tracking the amount of adrenaline picked up and sets the correct skins
// and shoulder light colors.
//=============================================================================

simulated function Tick(float DeltaTime)
{
  if ( Controller != None )
    LastAdrenalineAmount = Controller.Adrenaline;
  else if ( Controller(Owner) != None )
    LastAdrenalineAmount = Controller(Owner).Adrenaline;
  else if ( LastAdrenalineAmount > 0 )
    LastAdrenalineAmount = 0;
  
  Super.Tick(DeltaTime);
  
  if ( bInvis || Health <= 0 || Level.NetMode == NM_DedicatedServer )
    return;
  
  if ( LeftMarker != None && RightMarker != None && LeftMarker.Texture != PlayerLights[EffectsColorIndex] ) {
    LeftMarker.Texture  = PlayerLights[EffectsColorIndex];
    RightMarker.Texture = PlayerLights[EffectsColorIndex];
  }
  SetBodySkin();
}


//=============================================================================
// TickFX
//
// Update visual effects.
//=============================================================================

simulated function TickFX(float DeltaTime)
{
  local xPlayerReplicationInfo xPRI;
  local int i, NumSkins;
  
  if ( SimHitFxTicker != HitFxTicker )
    ProcessHitFX();
  
  if ( PlayerReplicationInfo != None )
    SetTeamIndex();
  
  // do translocate-out effect
  if ( TransEffectTicker != SimTransEffectTicker && Level.NetMode != NM_DedicatedServer ) {
    SimTransEffectTicker = TransEffectTicker;
    Spawn(TransOutEffect[EffectsColorIndex], self,, TransEffectOrigin, rot(16384,0,0));
  }

  if ( Level.NetMode != NM_DedicatedServer && (LeftMarker != None || RightMarker != None)
      && (bInvis || !class'TournamentClientSettings'.default.bUsePlayerLights) ) {
    if ( LeftMarker != None )
      LeftMarker.Destroy();
    if ( RightMarker != None )
      RightMarker.Destroy();
  }
  
  if ( bInvis && !bOldInvis) { // Going invisible
    // Save the 'real' non-invis skin
    NumSkins = Clamp(Skins.Length, 2, 4);
    
    for (i = 0; i < NumSkins; i++) {
      RealSkins[i] = Skins[i];
      Skins[i] = InvisMaterial;
    }
    
    // Remove/disallow projectors on invisible people
    Projectors.Remove(0, Projectors.Length);
    bAcceptsProjectors = false;
    
    // Invisible - no shadow
    if ( PlayerShadow != None )
      PlayerShadow.bShadowActive = false;
    
    // No giveaway flames either
    RemoveFlamingEffects();
  }
  else if ( !bInvis && bOldInvis ) { // Going visible
    NumSkins = Clamp(Skins.Length, 2, 4);
    
    for (i = 0; i < NumSkins; i++)
      Skins[i] = RealSkins[i];
    
    bAcceptsProjectors = Default.bAcceptsProjectors;
    
    if ( PlayerShadow != None )
      PlayerShadow.bShadowActive = true;
  }
  
  bOldInvis = bInvis;
  
  xPRI = xPlayerReplicationInfo(PlayerReplicationInfo);
  
  if ( Level.NetMode == NM_DedicatedServer || bPlayedDeath || PlayerReplicationInfo == None
      || !class'TournamentClientSettings'.default.bUsePlayerLights
      && (PlayerReplicationInfo.Team == None || class'DMMutator'.Default.bBrightSkins
          || class'DeathMatch'.Default.bNoCoronas || xPRI != None && xPRI.bForceNoPlayerLights) )
    return;
  
  if ( LeftMarker == None && !bInvis ) {
    LeftMarker = Spawn(class'PlayerLight', self,, Location);
    if ( !AttachToBone(LeftMarker,'lshoulder') ) {
      log("Couldn't attach LeftMarker to lshoulder", 'Error' );
      LeftMarker.Destroy();
      return;
    }
    
    RightMarker = Spawn(class'PlayerLight',self,,Location);
    if ( !AttachToBone(RightMarker,'rshoulder') ) {
      log("Couldn't attach RightMarker to rshoulder", 'Error' );
      LeftMarker.Destroy();
      RightMarker.Destroy();
      return;
    }
    
    LeftMarker.SetRelativeLocation(LeftOffset);
    LeftMarker.Style = STY_Translucent;
    LeftMarker.SetDrawScale(0.15);
    LeftMarker.SetDrawScale3D(vect(1,1,1));
    RightMarker.SetRelativeLocation(RightOffset);
    RightMarker.Style = STY_Translucent;
    RightMarker.SetDrawScale(0.15);
    RightMarker.SetDrawScale3D(vect(1,1,1));
  }
}


//=============================================================================
// StartDeRes
//
// Start DeRes-ing.
//=============================================================================

simulated function StartDeRes()
{
  local KarmaParamsSkel skelParams;
  local int i;
  
  if ( LifeSpan > DeResTime || LifeSpan == 0 )
    LifeSpan = DeResTime;
  
  if ( Level.NetMode == NM_DedicatedServer )
    return;
  
  AmbientGlow = 254;
  MaxLights = 0;
  
  if ( !class'TournamentClientSettings'.default.bTeamDeRes )
    SkinColorIndex = 0;
  
  DeResFX = Spawn(DeResEmitterClass[SkinColorIndex], self,, Location);
  if ( DeResFX != None ) {
    DeResFX.Emitters[0].SkeletalMeshActor = self;
    DeResFX.SetBase(self);
  }
  
  Skins[0] = default.DeResMatBody[SkinColorIndex];
  Skins[1] = default.DeResMatHead[SkinColorIndex];
  for (i = 2; i < Skins.Length; i++) {
    if ( i % 2 == 0 )
      Skins[i] = default.DeResMatBody[SkinColorIndex];
    else
      Skins[1] = default.DeResMatHead[SkinColorIndex];
  }
  
  if ( Physics == PHYS_KarmaRagdoll ) {
    KWake();
    if ( class'TournamentClientSettings'.default.bRisingDeRes ) {
      // Attach bone lifter to raise body
      KAddBoneLifter('bip01 Spine', DeResLiftVel, DeResLateralFriction, DeResLiftSoftness);
      KAddBoneLifter('bip01 Spine2', DeResLiftVel, DeResLateralFriction, DeResLiftSoftness);
      
      // Turn off gravity while de-res-ing
      KSetActorGravScale(DeResGravScale);
    }
    
    // Turn off collision with the world for the ragdoll.
    KSetBlockKarma(false);
    
    // Turn off convulsions during de-res
    skelParams = KarmaParamsSkel(KParams);
    skelParams.bKDoConvulsions = false;        
  }
  
  AmbientSound = Sound'GeneralAmbience.Texture19';
  SoundRadius = 100.0;
  
  // Turn off collision when we de-res (avoids rockets etc. hitting corpse!)
  SetCollision(false, false, false);
  
  // Remove/disallow projectors
  Projectors.Remove(0, Projectors.Length);
  bAcceptsProjectors = false;
  
  // Remove shadow
  if ( PlayerShadow != None )
    PlayerShadow.bShadowActive = false;
  
  // Remove flames
  RemoveFlamingEffects();
  
  // Turn off any overlays
  SetOverlayMaterial(None, 0, true);
  
  bDeRes = true;
}


//=============================================================================
// StartFiring
//
// Reset the SpawnTime clientside and fix UDamage sound in demos.
//=============================================================================

simulated function StartFiring(bool bHeavy, bool bRapid)
{
  local name FireAnim;
  
  if ( Role != ROLE_Authority )
    SpawnTime = -10000;
  
  if ( Level.TimeSeconds - LastUDamageSoundTime > 0.25 ) {
    LastUDamageSoundTime = Level.TimeSeconds;
    if ( IsLocallyControlled() )
      ServerStartFiring();
    if ( HasUDamage() )
      PlaySound(UDamageSound, SLOT_None, 1.5 * TransientSoundVolume,, 700);
  }
  
  if ( Physics == PHYS_Swimming )
    return;
  
  if ( bHeavy ) {
    if ( bRapid )
      FireAnim = FireHeavyRapidAnim;
    else
      FireAnim = FireHeavyBurstAnim;
  }
  else {
    if ( bRapid )
      FireAnim = FireRifleRapidAnim;
    else
      FireAnim = FireRifleBurstAnim;
  }
  
  AnimBlendParams(1, 1.0, 0.0, 0.2, FireRootBone);
  
  if ( bRapid ) {
    if ( FireState != FS_Looping ) {
      LoopAnim(FireAnim,, 0.0, 1);
      FireState = FS_Looping;
    }
  }
  else {
    PlayAnim(FireAnim,, 0.0, 1);
    FireState = FS_PlayOnce;
  }
  
  IdleTime = Level.TimeSeconds;
}


//=============================================================================
// ServerStartFiring
//
// Called by the client on the server when firing.
//=============================================================================

function ServerStartFiring()
{
  if ( HasUDamage() )
    PlayUDamageSound();
  
  if ( Inventory != None )
    Inventory.OwnerEvent('WeaponFired');
}


//=============================================================================
// PlayUDamageSound
//
// Play the UDamage sound.
//=============================================================================

function PlayUDamageSound()
{
  PlayOwnedSound(UDamageSound, SLOT_None, 1.5 * TransientSoundVolume,, 700);
}


//=============================================================================
// ClientSetUDamageTime
//
// Add pickup time to HUD local events.
//=============================================================================

function ClientSetUDamageTime(float NewUDam)
{
  Super.ClientSetUDamageTime(NewUDam);
  if ( NewUDam > 0 && PlayerController(Controller) != None
      && TournamentHUD(PlayerController(Controller).MyHud) != None )
    TournamentHUD(PlayerController(Controller).MyHud).UDamagePickups
        [TournamentHUD(PlayerController(Controller).MyHud).UDamagePickups.Length]
        = PlayerController(Controller).GameReplicationInfo.ElapsedTime
        - TournamentGameReplicationInfo(PlayerController(Controller).GameReplicationInfo).GameStartTime;
}


//=============================================================================
// GiveHealth
//
// Tracks the amount of health added.
//=============================================================================

function bool GiveHealth(int HealAmount, int HealMax)
{
  local int PrevHealth;
  
  PrevHealth = Health;
  if ( Super.GiveHealth(HealAmount, HealMax) ) {
    LastHealthAdd = Health - PrevHealth;
    return true;
  }
  return false;
}


//=============================================================================
// AddShieldStrength
//
// Tracks the amount of shield added.
//=============================================================================

function bool AddShieldStrength(int Amount)
{
  local int PrevShieldStrength;
  
  PrevShieldStrength = GetShieldStrength();
  if ( Super.AddShieldStrength(Amount) ) {
    LastShieldAdd = GetShieldStrength() - PrevShieldStrength;
    return true;
  }
  return false;
}


//=============================================================================
// HandlePickup
//
// Counts the number of pickups.
//=============================================================================

function HandlePickup(Pickup Pick)
{
  Super.HandlePickup(Pick);
  if ( Pick != None && TournamentPlayerReplicationInfo(PlayerReplicationInfo) != None )
    TournamentPlayerReplicationInfo(PlayerReplicationInfo).CountPickup(Self, Pick.Class);
  LastHealthAdd = 0;
  LastShieldAdd = 0;
  if ( Controller != None )
    LastAdrenalineAmount = Controller.Adrenaline;
  else if ( Controller(Owner) != None )
    LastAdrenalineAmount = Controller(Owner).Adrenaline;
  else
    LastAdrenalineAmount = 0;
}


//=============================================================================
// TakeDamage
//
// Add hit sounds.
//=============================================================================

function TakeDamage(int Damage, Pawn InstigatedBy, vector HitLocation, vector Momentum, class<DamageType> DamageType)
{
  local int ActualDamage;
  local bool bAlreadyDead;
  local Controller Killer;
  
  if ( Role < ROLE_Authority ) {
    log(self @ "client damage type" @ DamageType @ "by" @ InstigatedBy);
    return;
  }
  
  if ( DamageType == None ) {
    if ( InstigatedBy != None )
      warn("No DamageType for damage by" @ InstigatedBy @ "with weapon" @ InstigatedBy.Weapon);
    DamageType = class'DamageType';
  }
  
  bAlreadyDead = Health <= 0;
  
  if ( Physics == PHYS_None )
    SetMovementPhysics();
  if ( Physics == PHYS_Walking )
    Momentum.Z = FMax(Momentum.Z, 0.4 * VSize(Momentum));
  if ( InstigatedBy == self )
    Momentum *= 0.6;
  Momentum = Momentum / Mass;
  
  if ( Weapon != None )
    Weapon.AdjustPlayerDamage(Damage, InstigatedBy, HitLocation, Momentum, DamageType);
  if ( InstigatedBy != None ) {
    if ( InstigatedBy.HasUDamage() )
      Damage *= 2;
  }
  ActualDamage = Level.Game.ReduceDamage(Damage, self, InstigatedBy, HitLocation, Momentum, DamageType);
  
  if ( Damage > 0 && !bAlreadyDead ) {
    if ( InstigatedBy != None )
      Killer = InstigatedBy.GetKillerController();
    if ( Killer == None )
      Killer = DelayedDamageInstigatorController;
    if ( Killer != None && Killer.PlayerReplicationInfo != PlayerReplicationInfo ) {
      class'HitSounds'.static.CalcHitSoundsFor(PlayerController(Killer), Damage, Self);
      if ( class<WeaponDamageType>(DamageType) != None )
        class'TournamentWeaponStats'.static.IncrementDamage(Killer,
            class<WeaponDamageType>(DamageType).default.WeaponClass, Damage, NextDamageMode);
      else
        class'TournamentWeaponStats'.static.IncrementDamage(Killer, None, Damage, NextDamageMode);
      NextDamageMode = 0;
    }
  }
  if ( DamageType.default.bArmorStops && ActualDamage > 0 )
    ActualDamage = ShieldAbsorb(ActualDamage); 
  
  Health -= ActualDamage;
  if ( HitLocation == vect(0,0,0) )
    HitLocation = Location;
  
  if ( bAlreadyDead ) {
    DelayedDamageInstigatorController = None;
    return;
  }
  PlayHit(ActualDamage, InstigatedBy, HitLocation, DamageType, Momentum);
  if ( Health <= 0 ) {
    // pawn died
    if ( InstigatedBy != None )
      Killer = InstigatedBy.GetKillerController();
    if ( Killer == None )
      Killer = DelayedDamageInstigatorController;
    if ( bPhysicsAnimUpdate )
      TearOffMomentum = Momentum;
    if ( PlayerController(Controller) != None && PlayerController(Controller).DemoViewer != None ) {
      // hack for server-demo behindview
      PlayerController(Controller).SetLocation(Location);
      PlayerController(Controller).DemoViewer.SetLocation(Location);
    }
    Died(Killer, DamageType, HitLocation);
  }
  else {
    // don't switch to falling physics if not enough momentum
    if ( VSize(Momentum) > 1 && (!class'TournamentServerSettings'.default.bDisableLockdown
        || Physics != PHYS_Walking || VSize(Momentum) > 50) )
      AddVelocity(Momentum);
    if ( Controller != None )
      Controller.NotifyTakeHit(InstigatedBy, HitLocation, ActualDamage, DamageType, Momentum);
  }
  MakeNoise(1.0); 
  DelayedDamageInstigatorController = None;
}


//=============================================================================
// GibbedBy
//
// Check for telefrags against flag carriers and track telefrag damage stats.
//=============================================================================

function GibbedBy(actor Other)
{
  local int OldHealth;
  local class<WeaponDamageType> DamageType;
  
  if ( Role < ROLE_Authority )
    return;
  if ( Pawn(Other) != None ) {
    OldHealth = Health;
    if ( class'TournamentServerSettings'.default.bAllowCarrierTelefrags || PlayerReplicationInfo.HasFlag == None ) {
    	if ( TournamentPawn(Other) != None && TournamentPawn(Other).TelefragDamage != None ) {
    		DamageType = class<WeaponDamageType>(TournamentPawn(Other).TelefragDamage);
    		Died(Pawn(Other).Controller, TournamentPawn(Other).TelefragDamage, Location);
        if ( Health <= 0 && DamageType != None ) {
          class'TournamentWeaponStats'.static.IncrementHits(Pawn(Other).Controller,
              DamageType.default.WeaponClass, TelefragMode, -1);
          class'TournamentWeaponStats'.static.IncrementDamage(Pawn(Other).Controller,
              DamageType.default.WeaponClass, OldHealth, TelefragMode);
        }
        else if ( Health <= 0 ) {
          class'TournamentWeaponStats'.static.IncrementHits(Pawn(Other).Controller, None, TelefragMode, -1);
          class'TournamentWeaponStats'.static.IncrementDamage(Pawn(Other).Controller, None, OldHealth, TelefragMode);
        }
    	}
    	else if ( Pawn(Other) != None ) {
    		Died(Pawn(Other).Controller, class'DamTypeTelefragged', Location);
    		if ( Health <= 0 ) {
          class'TournamentWeaponStats'.static.IncrementHits(Pawn(Other).Controller, None, 0, -1);
          class'TournamentWeaponStats'.static.IncrementDamage(Pawn(Other).Controller, None, OldHealth, 0);
        }
    	}
    	else
    		Died(None, class'Gibbed', Location);
    }
  }
}


//=============================================================================
// FootStepping
//
// Make sure bPlayOwnFootSteps can really turn off footstep sounds.
//=============================================================================

simulated function FootStepping(int Side)
{
  if ( bPlayOwnFootSteps || PlayerController(Controller) == None
      || ViewPort(PlayerController(Controller).Player) == None )
    Super.FootStepping(Side);
}


//=============================================================================
// state Dying
//
// Immediately start DeRes-ing if desired.
//=============================================================================

simulated state Dying
{
Begin:
  if ( Level.NetMode != NM_DedicatedServer && class'TournamentClientSettings'.default.bInstantDeRes
      && (PlayerController(OldController) == None || PlayerController(OldController).ViewTarget != Self) )
    StartDeRes();
  else
    AmbientGlow = 40;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  TeamCharacterName="None"
  EnemyCharacterName="None"
  
  RequiredEquipment(0)="Tournament.TStatsAssaultRifle"
  RequiredEquipment(1)="Tournament.TStatsShieldGun"
  
  DeResMatBody(0)=FinalBlend'DeRez.DeRezFinalBody'
  DeResMatBody(1)=FinalBlend'DeResFinalBodyRed'
  DeResMatBody(2)=FinalBlend'DeResFinalBodyBlue'
  DeResMatBody(3)=FinalBlend'DeRez.DeRezFinalBody'
  
  DeResMatHead(0)=FinalBlend'DeRez.DeRezFinalHead'
  DeResMatHead(1)=FinalBlend'DeResFinalHeadRed'
  DeResMatHead(2)=FinalBlend'DeResFinalHeadBlue'
  DeResMatHead(3)=FinalBlend'DeRez.DeRezFinalHead'
  
  DeResEmitterClass(0)=class'DeResPart'
  DeResEmitterClass(1)=class'DeResPartRed'
  DeResEmitterClass(2)=class'DeResPartBlue'
  DeResEmitterClass(3)=class'DeResPart'
  DeResEmitterClass(4)=class'DeResPartGold'
  DeResEmitterClass(5)=class'DeResPartCyan'
  DeResEmitterClass(6)=class'DeResPartMagenta'
  DeResEmitterClass(7)=class'DeResPartWhite'
  
  PlayerLights(0)=Texture'PlayerLightBlue'
  PlayerLights(1)=Texture'PlayerLightRed'
  PlayerLights(2)=Texture'PlayerLightBlue'
  PlayerLights(3)=Texture'PlayerLightGreen'
  PlayerLights(4)=Texture'PlayerLightGold'
  PlayerLights(5)=Texture'PlayerLightCyan'
  PlayerLights(6)=Texture'PlayerLightMagenta'
  PlayerLights(7)=Texture'PlayerLightWhite'
  
  TranslocOutEffects(0)=class'TransDeRes'
  TranslocOutEffects(1)=class'TransDeRes'
  TranslocOutEffects(2)=class'TranseDeResBlue'
  TranslocOutEffects(3)=class'TransDeResGreen'
  TranslocOutEffects(4)=class'TransDeResGold'
  TranslocOutEffects(5)=class'TransDeResCyan'
  TranslocOutEffects(6)=class'TransDeResMagenta'
  TranslocOutEffects(7)=class'TransDeResWhite'
  
  TeleportEffects(0)=class'TransEffectRed'
  TeleportEffects(1)=class'TransEffectRed'
  TeleportEffects(2)=class'TranslocEffectBlue'
  TeleportEffects(3)=class'TranslocEffectGreen'
  TeleportEffects(4)=class'TranslocEffectGold'
  TeleportEffects(5)=class'TranslocEffectCyan'
  TeleportEffects(6)=class'TranslocEffectMagenta'
  TeleportEffects(7)=class'TranslocEffectWhite'
}