//=============================================================================
// TournamentScoreboard
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament DeathMatch score board.
//=============================================================================


class TournamentScoreboard extends ScoreBoard
    dependson(TournamentHUD)
    dependson(TournamentWeaponStats);


//=============================================================================
// Imports
//=============================================================================

#exec texture import file=Textures\FlagIcon.dds mips=off alpha=on group=GUI lodset=LODSET_Interface
#exec texture import file=Textures\BombIcon.dds mips=off alpha=on group=GUI lodset=LODSET_Interface


//=============================================================================
// Structs
//=============================================================================

struct TCoordinates {
  var() float X;
  var() float Y;
};

struct TColorSet {
  var() color MainColor;
  var() color AltColor;
};


//=============================================================================
// Enums
//=============================================================================

enum ESortOrder {
  SO_Score,
  SO_Efficiency,
  SO_Kills,
  SO_Deaths,
  SO_FPH,
  SO_Name,
  SO_Ping,
};


//=============================================================================
// Configuration
//=============================================================================

var() config ESortOrder SortOrder;


//=============================================================================
// Localization
//=============================================================================

var localized string HeaderText;
var localized string OnlineString;

var localized string GoalScoreString;
var localized string MaxLivesString;

var localized string ClockElapsedTime;
var localized string ClockRemainingTime;
var localized string ClockOverTime;

var localized string CurrentSortOrderString;
var localized array<string> SortOrderStrings;
var localized string PlayerNotShown;
var localized string PlayersNotShown;

var localized string ScoreCaption;
var localized string GoalsCaption;
var localized string TouchesCaption;
var localized string ReturnsCaption;
var localized string KillsCaption;
var localized string DeathsCaption;
var localized string SuicidesCaption;
var localized string EfficiencyCaption;
var localized string FPHCaption;
var localized string PingCaption;
var localized string PacketLossCaption;
var localized string NetSpeedCaption;
var localized string BotString;
var localized string AdminString;
var localized string OutString;

var localized string StatsForString;
var localized string OtherString;
var localized string TotalString;

var localized string WeaponCaption;
var localized string ShotsCaption;
var localized string HitsCaption;
var localized string AccuracyCaption;
var localized string DamageCaption;
var localized string DamagePerHitCaption;

var localized string PickupCaption;
var localized string PickupCountCaption;
var localized string PickupAmountCaption;


//=============================================================================
// Variables
//=============================================================================

var transient TournamentHUD LocalHud;
var transient TournamentWeaponStats WeaponStats;

var() int MaxDisplayedPlayers;
var() int MaxFullDetailPlayers;

var() float                       ClockScale;
var() HudBase.SpriteWidget        ClockCircle;      // clock background circle
var() HudBase.SpriteWidget        ClockFilledFirst; // complete fill, first half
var() TournamentHUD.RotatedWidget ClockFillFirst;   // gradual fill, first half
var() TournamentHUD.RotatedWidget ClockFillSecond;  // gradual fill, second half
var() TournamentHUD.RotatedWidget ClockTick;        // clock section tick
var() TournamentHUD.RotatedWidget ClockMarker;      // game event marker
var() TCoordinates                ClockPosition;

var() bool bShowGoalCount;
var() bool bShowSuicideCount;
var() bool bShowRank;
var() bool bShowTouches;
var() bool bShowReturns;

var() color WhiteColor, OverTimeColor, HighlightColor;
var() color TeamColors[4], OutColor, AdminColor;
var() color ScoresBG, StatsHeaderBG, StatsBG;


//=============================================================================
// DrawScoreboard
//
// Draws the Scoreboard.
//=============================================================================

function DrawScoreboard(Canvas C)
{
  local float OldHudScale, OldHudCanvasScale;
  local plane OldColorModulate;
  local ERenderStyle OldPassStyle;
  
  if ( LocalHud == None )
    foreach DynamicActors(class'TournamentHUD', LocalHud)
      break;
  
  if ( LocalHud != None ) {
    OldHudScale       = LocalHud.HudScale;
    OldHudCanvasScale = LocalHud.HudCanvasScale;
    OldPassStyle      = LocalHud.PassStyle;
    
    LocalHud.HudScale       = 1;
    LocalHud.HudCanvasScale = 1;
    LocalHud.PassStyle      = STY_None;
  }
  OldColorModulate = C.ColorModulate;
  C.ColorModulate.X = 1;
  C.ColorModulate.Y = 1;
  C.ColorModulate.Z = 1;
  C.ColorModulate.W = 1;
  
  Super.DrawScoreboard(C);
  
  if ( LocalHud != None ) {
    LocalHud.HudScale       = OldHudScale;
    LocalHud.HudCanvasScale = OldHudCanvasScale;
    LocalHud.PassStyle      = OldPassStyle;
  }
  C.ColorModulate = OldColorModulate;
}


//=============================================================================
// UpdateScoreBoard
//
// Actually draws the scoreboard.
//=============================================================================

function UpdateScoreBoard(Canvas C)
{
  local float OldHudScale;
  
  if ( LocalHud != None && LocalHud.PawnOwner != None && !Level.GetLocalPlayerController().bBehindView )
    LocalHud.DrawCrosshair(C);
  
  C.Style = 5;  // STY_Alpha
  DrawTitle(C);
  DrawScores(C);
  DrawStats(C);
  if ( LocalHud != None ) {
    OldHudScale = LocalHud.HudScale;
    LocalHud.HudScale *= ClockScale;
  }
  DrawClock(C);
  if ( LocalHud != None )
    LocalHud.HudScale = OldHudScale;
}


//=============================================================================
// DrawTitle
//
// Should be subclassed to draw the gametype info.
//=============================================================================

function DrawTitle(Canvas C);


//=============================================================================
// DrawScores
//
// Should be subclassed to draw the actual scores.
//=============================================================================

function DrawScores(Canvas C);


//=============================================================================
// DrawStats
//
// Should be subclassed to draw the weapon and pickup stats.
//=============================================================================

function DrawStats(Canvas C);


//=============================================================================
// NextStats
//
// Toggles the sorting order.
//=============================================================================

function NextStats()
{
  switch (SortOrder) {
    case SO_Score:      SortOrder = SO_Efficiency; break;
    case SO_Efficiency: SortOrder = SO_Kills;      break;
    case SO_Kills:      SortOrder = SO_Deaths;     break;
    case SO_Deaths:     SortOrder = SO_FPH;        break;
    case SO_FPH:        SortOrder = SO_Name;       break;
    case SO_Name:
      if ( Level.NetMode == NM_Standalone )
        SortOrder = SO_Score;
      else
        SortOrder = SO_Ping;
      break;
    case SO_Ping:       SortOrder = SO_Score;      break;
  }
  SaveConfig();
}


//=============================================================================
// SortPRIArray
//
// Sorts the GRI's PRIArray by score.
//=============================================================================

function SortPRIArray()
{
  SortPRIs(GRI.PRIArray, 0, GRI.PRIArray.Length - 1, SO_Score);
}


//=============================================================================
// SortPRIs
//
// Sorts a dynamic array of PRIs via (optimized) Quick Sort.
//=============================================================================

function SortPRIs(out array<PlayerReplicationInfo> PRIArray, int lwb, int upb, ESortOrder SortBy)
{
  local int up, down, iPivot;
  local PlayerReplicationInfo Pivot, tmp;
  
  if ( lwb < upb ) {
    up = lwb;
    down = upb;
    
    // partition
    iPivot = (lwb + upb) / 2;
    Pivot = PRIArray[iPivot];
    while (up <= down) {
      while (up <= upb   && ComparePRIs(PRIArray[up], Pivot, SortBy))   up++;
      while (lwb <= down && ComparePRIs(Pivot, PRIArray[down], SortBy)) down--;
      if ( up < down ) {
        tmp = PRIArray[up];
        PRIArray[up] = PRIArray[down];
        PRIArray[down] = tmp;
        up++;
        down--;
      }
    }
    // adjust pivot
    if ( up < iPivot ) {
      PRIArray[iPivot] = PRIArray[up];
      PRIArray[up] = Pivot;
      up++;
    }
    else if ( iPivot < down ) {
      PRIArray[iPivot] = PRIArray[down];
      PRIArray[down] = Pivot;
      down--;
    }
    
    // recursion
    if ( lwb < down )
      SortPRIs(PRIArray, lwb, down, SortBy);
    if ( up < upb )
      SortPRIs(PRIArray, up, upb, SortBy);
  }
}

//=============================================================================
// ComparePRIs
//
// Returns whether P1 ranks higher than P2 with the specified sorting order.
//=============================================================================

function bool ComparePRIs(PlayerReplicationInfo P1, PlayerReplicationInfo P2, ESortOrder SortBy)
{
  local TournamentPlayerReplicationInfo TP1, TP2;
  
  if ( P1.bOnlySpectator && P2.bOnlySpectator )
    SortBy = SO_Name;
  else if ( P1.bOnlySpectator )
    return false;
  else if ( P2.bOnlySpectator )
    return true;
  
  if ( !P1.bOutOfLives && P2.bOutOfLives )
    return true;
  else if ( P1.bOutOfLives && !P2.bOutOfLives )
    return false;
  
  TP1 = TournamentPlayerReplicationInfo(P1);
  TP2 = TournamentPlayerReplicationInfo(P2);
  
  switch (SortBy) {
  case SO_Score:
    if ( TP1.Score < TP2.Score )
      return false;
    else if ( TP1.Score == TP2.Score ) {
      if ( TP1.KillCount < TP2.KillCount )
        return false;
      else if ( TP1.KillCount == TP2.KillCount ) {
        if ( TP1.Deaths > TP2.Deaths )
          return false;
        else if ( TP1.Deaths == TP2.Deaths ) {
          if ( TP1.SuicideCount > TP2.SuicideCount )
            return false;
          else if ( TP1.SuicideCount == TP2.SuicideCount )
            if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
              return false;
            else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
              return false;
        }
      }
    }
    return true;
  case SO_Efficiency:
    if ( GetEfficiency(TP1) < GetEfficiency(TP2) )
      return false;
    else if ( GetEfficiency(TP1) == GetEfficiency(TP2) ) {
      if ( TP1.Score < TP2.Score )
        return false;
      else if ( TP1.Score == TP2.Score ) {
        if ( TP1.KillCount < TP2.KillCount )
          return false;
        else if ( TP1.KillCount == TP2.KillCount ) {
          if ( TP1.SuicideCount > TP2.SuicideCount )
            return false;
          else if ( TP1.SuicideCount == TP2.SuicideCount )
            if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
              return false;
            else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
              return false;
        }
      }
    }
    return true;
  case SO_Kills:
    if ( TP1.KillCount < TP2.KillCount )
      return false;
    else if ( TP1.KillCount == TP2.KillCount ) {
      if ( TP1.Score < TP2.Score )
        return false;
      else if ( TP1.Score == TP2.Score ) {
        if ( TP1.Deaths > TP2.Deaths )
          return false;
        else if ( TP1.Deaths == TP2.Deaths ) {
          if ( TP1.SuicideCount > TP2.SuicideCount )
            return false;
          else if ( TP1.SuicideCount == TP2.SuicideCount )
            if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
              return false;
            else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
              return false;
        }
      }
    }
    return true;
  case SO_Deaths:
    if ( TP1.Deaths > TP2.Deaths )
      return false;
    else if ( TP1.Deaths == TP2.Deaths ) {
      if ( TP1.SuicideCount > TP2.SuicideCount )
        return false;
      else if ( TP1.SuicideCount == TP2.SuicideCount ) {
        if ( TP1.Deaths > TP2.Deaths )
          return false;
        else if ( TP1.Deaths == TP2.Deaths ) {
          if ( TP1.Score > TP2.Score )
            return false;
          else if ( TP1.Score == TP2.Score )
            if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
              return false;
            else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
              return false;
        }
      }
    }
    return true;
  case SO_FPH:
    if ( GetFPH(TP1) < GetFPH(TP2) )
      return false;
    else if ( GetFPH(TP1) == GetFPH(TP2) ) {
      if ( TP1.Score < TP2.Score )
        return false;
      else if ( TP1.Score == TP2.Score ) {
        if ( TP1.KillCount < TP2.KillCount )
          return false;
        else if ( TP1.KillCount == TP2.KillCount ) {
          if ( TP1.Deaths > TP2.Deaths )
            return false;
          else if ( TP1.Deaths == TP2.Deaths ) {
            if ( TP1.SuicideCount > TP2.SuicideCount )
              return false;
            else if ( TP1.SuicideCount == TP2.SuicideCount )
              if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
                return false;
              else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
                return false;
          }
        }
      }
    }
    return true;
  case SO_Name:
    if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
      return false;
    else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
      return false;
    return true;
  case SO_Ping:
    if ( TP1.Ping > TP2.Ping )
      return false;
    else if ( TP1.Ping == TP2.Ping ) {
      if ( Caps(TP1.PlayerName) > Caps(TP2.PlayerName) )
        return false;
      else if ( TP1.PlayerName ~= TP2.PlayerName && TP1.PlayerName > TP2.PlayerName )
        return false;
    }
    return true;
  }
}


//=============================================================================
// Efficiency
//
// Returns the player's efficiency rating.
//=============================================================================

function float GetEfficiency(TournamentPlayerReplicationInfo PRI)
{
  if ( PRI.KillCount == 0 )
    return 0;
  return PRI.KillCount / (PRI.KillCount + PRI.Deaths);
}


//=============================================================================
// FPH
//
// Returns the player's frags per hour rating.
//=============================================================================

function int GetFPH(TournamentPlayerReplicationInfo PRI)
{
  if ( PRI.KillCount == 0 || GRI.ElapsedTime - PRI.StartTime <= 0 )
    return 0;
  
  return PRI.KillCount * 3600 / (GRI.ElapsedTime - PRI.StartTime);
}


//=============================================================================
// DrawPlayerScore
//
// Draws a player's name and score at the specified location.
//=============================================================================

function DrawPlayerScore(Canvas C, PlayerReplicationInfo PRI, int Rank, float YPos, bool bRightSide, bool bSmallInfo,
  optional bool bNoDetails, optional Decoration HasFlag)
{
  local byte Team;
  local TournamentPlayerReplicationInfo TPRI;
  local PlayerController LocalPlayer;
  local Texture FlagIcon;
  
  local string PlayerName, PlayerLocation, PlayerOrders;
  local string Score, Goals, Touches, Returns, Kills, Deaths, Suicides;
  local string Efficiency, FPH, Ping, PacketLoss, NetSpeed;
  
  local float XPos;
  local float XL, YL;
  local EDrawPivot LowerOuter, LowerInner, UpperInner, UpperOuter;
  local float AddFactor;
  local int LineDir;
  
  TPRI = TournamentPlayerReplicationInfo(PRI);
  LocalPlayer = PlayerController(Owner);
  if ( TPRI != None ) {
    if ( GRI != None && GRI.bTeamGame ) {
      if ( TPRI.bBot && !TPRI.bOutOfLives && TPRI.Squad != None && TPRI.Team != None
          && (LocalPlayer.PlayerReplicationInfo.bOnlySpectator
          || TPRI.Team == LocalPlayer.PlayerReplicationInfo.Team) )
        PlayerOrders = TPRI.Squad.GetOrderStringFor(TPRI);
      else if ( TPRI.bAdmin )
        PlayerOrders = AdminString;
      if ( TPRI.bOutOfLives ) {
        if ( PlayerOrders == "" )
          PlayerOrders = OutString;
        else
          PlayerOrders = OutString $ "/" $ PlayerOrders;
      }
      if ( TPRI.bOnlySpectator || LocalPlayer.PlayerReplicationInfo.bOnlySpectator
          || TPRI.Team != None && TPRI.Team == LocalPlayer.PlayerReplicationInfo.Team
          || TPRI == LocalPlayer.PlayerReplicationInfo )
        PlayerLocation = TPRI.GetLocationName();
    }
    else
      bSmallInfo = False;
    
    if ( TPRI.Team == None )
      Team = 2;
    else
      Team = Clamp(TPRI.Team.TeamIndex, 0, ArrayCount(TeamColors) - 1);
    
    PlayerName = TPRI.PlayerName;
    Score      = string(int(TPRI.Score));
    Goals      = string(TPRI.Goals);
    Touches    = string(TPRI.TouchCount);
    Returns    = string(TPRI.ReturnCount);
    Kills      = string(TPRI.KillCount);
    Deaths     = string(int(TPRI.Deaths));
    Suicides   = string(TPRI.SuicideCount);
    Efficiency = int(100 * GetEfficiency(TPRI)) $ "%";
    FPH        = string(GetFPH(TPRI));
    if ( Level.NetMode != NM_Standalone && !TPRI.bBot ) {
      Ping       = string(TPRI.Ping);
      PacketLoss = string(TPRI.PacketLoss);
      NetSpeed   = class'TournamentUtil'.static.FloatToString(float(TPRI.NetSpeed) * 0.001) $ "k";
    }
  }
  else {
    if ( GRI != None && !GRI.bTeamGame )
      bSmallInfo = False;
    Team = 3;
    Score = ScoreCaption;
    Goals = GoalsCaption;
    Touches = TouchesCaption;
    Returns = ReturnsCaption;
    Kills = KillsCaption;
    Deaths = DeathsCaption;
    Suicides = SuicidesCaption;
    Efficiency = EfficiencyCaption;
    FPH = FPHCaption;
    if ( Level.NetMode != NM_Standalone ) {
      Ping       = PingCaption;
      PacketLoss = PacketLossCaption;
      NetSpeed   = NetSpeedCaption;
    }
  }
  
  if ( !bRightSide ) {
    XPos = 0.46;
    AddFactor = -1;
    LineDir = 2;
    LowerOuter = DP_LowerLeft;
    LowerInner = DP_LowerRight;
    UpperInner = DP_UpperRight;
    UpperOuter = DP_UpperLeft;
  }
  else {
    XPos = 0.54;
    AddFactor = 1;
    LineDir = 3;
    LowerOuter = DP_LowerRight;
    LowerInner = DP_LowerLeft;
    UpperInner = DP_UpperLeft;
    UpperOuter = DP_UpperRight;
  }
  
  C.Font = GetFontSizeIndex(C, -3);
  C.TextSize("/", XL, YL);
  C.DrawColor = ScoresBG;
  C.SetPos(XPos * C.ClipX, YPos * C.ClipY - 2 * (YL + 1));
  C.DrawRect(Texture'WhiteTexture', AddFactor * C.ClipX * 0.45, 2 * (YL + 1));
  if ( PRI != None && PRI.HasFlag != None )
    HasFlag = PRI.HasFlag;
  if ( HasFlag != None ) {
    if ( PlayerName != "" )
      C.SetPos(XPos * C.ClipX, YPos * C.ClipY - 2 * YL);
    else
      C.SetPos((XPos + AddFactor * 0.08) * C.ClipX, YPos * C.ClipY - 2 * YL);
    if ( HasFlag.IsA('xBombFlag') ) {
      C.DrawColor = WhiteColor;
      FlagIcon = Texture'BombIcon';
    }
    else if ( HasFlag.IsA('CTFFlag') ) {
      C.DrawColor = TeamColors[CTFFlag(HasFlag).TeamNum];
      FlagIcon = Texture'FlagIcon';
    }
    else if ( HasFlag.IsA('TournamentPlayerIcon') ) {
      C.DrawColor = TournamentPlayerIcon(HasFlag).DrawColor;
      FlagIcon = TournamentPlayerIcon(HasFlag).Icon;
    }
    if ( FlagIcon != None ) {
      if ( bRightSide )
        C.DrawRect(FlagIcon, -2 * YL * FlagIcon.USize / FlagIcon.VSize,
            2 * YL);
      else
        C.DrawIcon(FlagIcon, 2 * YL / FlagIcon.VSize);
    }
  }
  C.DrawColor = TeamColors[Team];
  C.SetPos(XPos * C.ClipX, YPos * C.ClipY);
  C.DrawLine(LineDir, C.ClipX * 0.45);
  if ( LocalPlayer.PlayerReplicationInfo == TPRI )
    C.DrawColor = HighlightColor;
  else
    C.DrawColor = WhiteColor;
  C.Font = GetFontSizeIndex(C, -1);
  if ( !bNoDetails && bShowRank && Rank > 0 && HasFlag == None )
    C.DrawScreenText(string(Rank), XPos - AddFactor * 0.005, YPos, LowerOuter);
  C.Font = GetFontSizeIndex(C, 0);
  if ( PlayerName != "" )
    C.DrawScreenText(PlayerName, XPos + AddFactor * 0.002, YPos, LowerInner);
  
  if ( bNoDetails || PRI != None && PRI.bOnlySpectator && !PRI.bWaitingPlayer )
    return;
  
  if ( !bSmallInfo ) {
    if ( Score != "" ) {
      if ( bShowGoalCount )
        C.DrawScreenText(Score, XPos + AddFactor * 0.2, YPos, DP_LowerRight);
      else
        C.DrawScreenText(Score, XPos + AddFactor * 0.2, YPos, DP_LowerMiddle);
    }
    C.Font = GetFontSizeIndex(C, -3);
    C.TextSize("/", XL, YL);
    if ( bShowGoalCount ) {
      if ( bShowTouches ) {
        C.DrawScreenText(" " $ Goals, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_LowerLeft);
        if ( bShowReturns )
          C.DrawScreenText(" " $ Touches $ "/" $ Returns, XPos + AddFactor * 0.2, YPos, DP_LowerLeft);
        else
          C.DrawScreenText(" " $ Touches, XPos + AddFactor * 0.2, YPos, DP_LowerLeft);
      }
      else
        C.DrawScreenText("/" $ Goals, XPos + AddFactor * 0.2, YPos - 0.25 * YL / C.ClipY, DP_LowerLeft);
    }
    if ( PlayerLocation != "" )
      C.DrawScreenText(PlayerLocation, XPos + AddFactor * 0.002, YPos + 4 / C.ClipY, UpperInner);
    if ( PlayerOrders != "" )
      C.DrawScreenText(PlayerOrders, XPos + AddFactor * 0.45, YPos + 4 / C.ClipY, UpperOuter);
    
    if ( bShowSuicideCount ) {
      C.DrawScreenText(Kills $ "/" $ Deaths $ "/" $ Suicides, XPos + AddFactor * 0.325, YPos - YL / C.ClipY,
          DP_LowerMiddle);
    }
    else {
      C.DrawScreenText(Kills, XPos + AddFactor * 0.325 - 0.5 * XL / C.ClipX, YPos - YL / C.ClipY, DP_LowerRight);
      C.DrawScreenText("/", XPos + AddFactor * 0.325, YPos - YL / C.ClipY, DP_LowerMiddle);
      C.DrawScreenText(Deaths, XPos + AddFactor * 0.325 + 0.5 * XL / C.ClipX, YPos - YL / C.ClipY, DP_LowerLeft);
    }
    C.DrawScreenText(Efficiency, XPos + AddFactor * 0.325 - 0.5 * XL / C.ClipX, YPos, DP_LowerRight);
    C.DrawScreenText(FPH, XPos + AddFactor * 0.325 + 0.5 * XL / C.ClipX, YPos, DP_LowerLeft);
    
    if ( Ping != "" ) {
      C.DrawScreenText(Ping, XPos + AddFactor * 0.45, YPos - YL / C.ClipY, LowerOuter);
      C.DrawScreenText(PacketLoss $ "/" $ Netspeed, XPos + AddFactor * 0.45, YPos, LowerOuter);
    }
  }
  else {
    C.Font = GetFontSizeIndex(C, -3);
    C.TextSize("/", XL, YL);
    
    if ( Score != "" ) {
      if ( !bShowGoalCount || bShowGoalCount && bShowTouches && bShowReturns )
        C.DrawScreenText(Score $ "/" $ Kills $ "/" $ Deaths, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_LowerMiddle);
      else if ( bShowSuicideCount )
        C.DrawScreenText(Kills $ "/" $ Deaths $ "/" $ Suicides, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_LowerMiddle);
      else
        C.DrawScreenText(Kills $ "/" $ Deaths, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_LowerMiddle);
      if ( bShowGoalCount ) {
        if ( bShowTouches ) {
          if ( bShowReturns )
            C.DrawScreenText(Goals $ "/" $ Touches $ "/" $ Returns, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_UpperMiddle);
          else
            C.DrawScreenText(Score $ "/" $ Goals $ "/" $ Touches, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_UpperMiddle);
        }
        else {
          C.DrawScreenText(Score $ "/" $ Goals, XPos + AddFactor * 0.2, YPos - YL / C.ClipY, DP_UpperMiddle);
        }
      }
      else {
        C.DrawScreenText(Efficiency, XPos + AddFactor * 0.2 - 0.5 * XL / C.ClipX, YPos - YL / C.ClipY, DP_UpperRight);
        C.DrawScreenText(FPH, XPos + AddFactor * 0.2 + 0.5 * XL / C.ClipX, YPos - YL / C.ClipY, DP_UpperLeft);
      }
    }
    
    if ( PlayerLocation != "" )
      C.DrawScreenText(PlayerLocation, XPos + AddFactor * 0.45, YPos, LowerOuter);
    if ( PlayerOrders != "" ) {
      if ( Ping != "" )
        C.DrawScreenText(Ping $ "/" $ PlayerOrders, XPos + AddFactor * 0.45, YPos - YL / C.ClipY, LowerOuter);
      else
        C.DrawScreenText(PlayerOrders, XPos + AddFactor * 0.45, YPos - YL / C.ClipY, LowerOuter);
    }
    else if ( Ping != "" ) {
      if ( PlayerLocation != "" )
        C.DrawScreenText(Ping $ "/" $ PacketLoss $ "/" $ Netspeed, XPos + AddFactor * 0.45, YPos - YL / C.ClipY,
            LowerOuter);
      else
        C.DrawScreenText(Ping $ "/" $ PacketLoss $ "/" $ Netspeed, XPos + AddFactor * 0.45, YPos, LowerOuter);
    }
  }
}


//=============================================================================
// DrawStatsForWeapon
//
// Draws the stats for the specified weapon at the specified screen location.
//=============================================================================

function DrawStatsForWeapon(Canvas C, float XPos, out float YPos, float Width, optional TournamentWeaponStats.TWeaponStats Stats)
{
  local float XL, YL;
  local string WeaponName;
  local string Shots, Hits, Accuracy, TotalAccuracy, Damage, DamagePerHit;
  local float f;
  local int i;
  local string s[3];
  
  if ( Stats.bSlotUsed ) {
    if ( Stats.Weapon != None )
      WeaponName = Stats.Weapon.static.StaticItemName();
    else
      WeaponName = OtherString;
    Shots = Stats.Shots[0]$"/"$Stats.Shots[1]$"/"$Stats.Shots[2];
    Hits = Stats.ShotsHit[0]$"/"$Stats.ShotsHit[1]$"/"$Stats.ShotsHit[2];
    for (i = 0; i < 3; i++) {
      if ( Stats.Shots[i] == 0 )
        s[i] = "-";
      else {
        f = 100 * Stats.ShotsHit[i];
        f /= Stats.Shots[i];
        s[i] = int(f) $ "%";
      }
    }
    Accuracy = s[0]$"/"$s[1]$"/"$s[2];
    if ( Accuracy != "-/-/-" ) {
      f = 100 * (Stats.ShotsHit[0] + Stats.ShotsHit[1] + Stats.ShotsHit[2]);
      f /= Stats.Shots[0] + Stats.Shots[1] + Stats.Shots[2];
      TotalAccuracy = int(f) $ "%";
    }
    Damage = Stats.TotalDamage[0]$"/"$Stats.TotalDamage[1]$"/"$Stats.TotalDamage[2];
    for (i = 0; i < 3; i++) {
      if ( Stats.ShotsHit[i] == 0 )
        s[i] = "-";
      else {
        f = Stats.TotalDamage[i] / Stats.ShotsHit[i];
        s[i] = string(int(f));
      }
    }
    DamagePerHit = s[0]$"/"$s[1]$"/"$s[2];
  }
  else {
    WeaponName = WeaponCaption;
    Shots = ShotsCaption;
    Hits = HitsCaption;
    Accuracy = AccuracyCaption;
    Damage = DamageCaption;
    DamagePerHit = DamagePerHitCaption;
  }
  
  C.Font = GetFontSizeIndex(C, -3);
  C.TextSize("0", XL, YL);
  if ( Stats.bSlotUsed )
    C.DrawColor = StatsBG;
  else
    C.DrawColor = StatsHeaderBG;
  C.SetPos(XPos * C.ClipX, YPos * C.ClipY - 1);
  C.DrawRect(Texture'WhiteTexture', Width * C.ClipX, (YL + 1) * 2);
  C.DrawColor = WhiteColor;
  C.DrawScreenText(Shots, XPos + 0.4 * Width, YPos + (YL + 1) / C.ClipY, DP_LowerMiddle);
  C.DrawScreenText(Hits,  XPos + 0.4 * Width, YPos + (YL + 1) / C.ClipY, DP_UpperMiddle);
  if ( TotalAccuracy != "" ) {
    C.DrawScreenText(Accuracy,      XPos + 0.65 * Width, YPos + (YL + 1) / C.ClipY, DP_LowerMiddle);
    C.DrawScreenText(TotalAccuracy, XPos + 0.65 * Width, YPos + (YL + 1) / C.ClipY, DP_UpperMiddle);
  }
  else {
    C.DrawScreenText(Accuracy, XPos + 0.65 * Width, YPos + (YL + 1) / C.ClipY, DP_MiddleMiddle);
  }
  C.DrawScreenText(Damage,       XPos + 0.9 * Width, YPos + (YL + 1) / C.ClipY, DP_LowerMiddle);
  C.DrawScreenText(DamagePerHit, XPos + 0.9 * Width, YPos + (YL + 1) / C.ClipY, DP_UpperMiddle);
  C.Font = GetFontSizeIndex(C, -2);
  C.DrawScreenText(WeaponName, XPos + 0.01, YPos + YL / C.ClipY, DP_MiddleLeft);
  
  YPos += (2 * YL + 3) / C.ClipY;
}


//=============================================================================
// DrawStatsForPickup
//
// Draws the stats for the specified pickup at the specified screen location.
//=============================================================================

function DrawStatsForPickup(Canvas C, float XPos, out float YPos, float Width, optional TournamentPlayerReplicationInfo.TPickupStats Stats)
{
  local float XL, YL;
  local string PickupName;
  local string Amount, Count;
  
  if ( Stats.bSlotUsed ) {
    if ( Stats.Pickup != None )
      PickupName = class'TournamentUtil'.static.GetPickupName(Stats.Pickup);
    else
      PickupName = OtherString;
    Count = string(Stats.Count);
    if ( Stats.Amount > 0 )
      Amount = string(Stats.Amount);
  }
  else if ( Stats.Count > 0 ) {
    PickupName = TotalString;
    Count = string(Stats.Count);
    if ( Stats.Amount > 0 )
      Amount = string(Stats.Amount);
  }
  else {
    PickupName = PickupCaption;
    Count = PickupCountCaption;
    Amount = PickupAmountCaption;
  }
  
  C.Font = GetFontSizeIndex(C, -2);
  C.TextSize("0", XL, YL);
  if ( Stats.bSlotUsed )
    C.DrawColor = StatsBG;
  else
    C.DrawColor = StatsHeaderBG;
  C.SetPos(XPos * C.ClipX, YPos * C.ClipY - 1);
  C.DrawRect(Texture'WhiteTexture', Width * C.ClipX, YL);
  C.DrawColor = WhiteColor;
  C.DrawScreenText(PickupName, XPos + 0.01, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleLeft);
  C.Font = GetFontSizeIndex(C, -3);
  C.DrawScreenText(Count,  XPos + 0.7 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleRight);
  if ( Amount != "" )
    C.DrawScreenText(Amount, XPos + 0.99 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleRight);
  
  YPos += (YL + 1) / C.ClipY;
}


//=============================================================================
// DrawSortingOrder
//
// Draws the current sorting order centered above the specified position.
//=============================================================================

function DrawSortingOrder(Canvas C, float X, float Y)
{
  local string SortingString;
  
  SortingString = CurrentSortOrderString;
  ReplaceText(SortingString, "%s", SortOrderStrings[SortOrder]);
  ReplaceText(SortingString, "%k", FindKey("NextStats"));
  C.DrawColor = WhiteColor;
  C.Font = GetFontSizeIndex(C, -2);
  C.DrawScreenText(SortingString, X, Y, DP_UpperMiddle);
}


//=============================================================================
// DrawClock
//
// Draws the elapsed or remaining match time and the round clock.
//=============================================================================

function DrawClock(Canvas C)
{
  local int nSecondsMax;                // number of seconds displayable (time limit in seconds)
  local int nSecondsCurrent;            // elapsed game time in seconds
  local int nSecondsDisplayed;          // displayed number of seconds
  local int nSecondsInterval;           // interval between ticks
  local int nSecondsTick;
  local int ElapsedGameTime, GameStartTime;
    
  local TournamentGameReplicationInfo TGRI;
  local int i;
  local int GameEventCount;
  local int Seconds, Minutes;
  local float XL, YL;
  
  TGRI = TournamentGameReplicationInfo(GRI);
  if ( TGRI != None )
    GameStartTime = TGRI.GameStartTime;
  ElapsedGameTime = GRI.ElapsedTime - GameStartTime;
  
  nSecondsCurrent = ElapsedGameTime;
  
  if ( GRI.TimeLimit > 0 ) {
    nSecondsMax = 60 * GRI.TimeLimit;
  }
  else {
    nSecondsMax = 60 * 12;
    while (nSecondsMax <= ElapsedGameTime)
      nSecondsMax *= 2;
  }
  
  // draw background for clock
  ClockCircle.PosX = ClockPosition.X;
  ClockCircle.PosY = ClockPosition.Y;
  DrawSpriteWidget(C, ClockCircle);
  
  nSecondsInterval  = nSecondsMax / 15 + 59;
  nSecondsInterval -= nSecondsInterval % 60;
  
  nSecondsDisplayed = Min(ElapsedGameTime, nSecondsMax);
  
  // draw clock
  if ( nSecondsDisplayed > nSecondsMax / 2 ) {
    ClockFilledFirst.PosX = ClockPosition.X;
    ClockFilledFirst.PosY = ClockPosition.Y;
    DrawSpriteWidget(C, ClockFilledFirst);
    ClockFillSecond.Angle = (nSecondsMax - nSecondsDisplayed) * 65536 / nSecondsMax;
    ClockFillSecond.SpriteWidget.PosX = ClockPosition.X;
    ClockFillSecond.SpriteWidget.PosY = ClockPosition.Y;
    DrawRotatedWidget(C, ClockFillSecond);
  }
  else {
    ClockFillFirst.Angle = -nSecondsDisplayed * 65536 / nSecondsMax;
    ClockFillFirst.SpriteWidget.PosX = ClockPosition.X;
    ClockFillFirst.SpriteWidget.PosY = ClockPosition.Y;
    DrawRotatedWidget(C, ClockFillFirst);
  }
  
  ClockTick.SpriteWidget.PosX = ClockPosition.X;
  ClockTick.SpriteWidget.PosY = ClockPosition.Y;
  for (nSecondsTick = 0; nSecondsTick < nSecondsMax; nSecondsTick += nSecondsInterval) {
    ClockTick.Angle = -65536 * nSecondsTick / nSecondsMax;
    DrawRotatedWidget(C, ClockTick);
  }
  
  ClockMarker = default.ClockMarker;
  ClockMarker.SpriteWidget.PosX = ClockPosition.X;
  ClockMarker.SpriteWidget.PosY = ClockPosition.Y;
  ClockMarker.bFixedAngle = True;
  ClockMarker.OffsetRotatedY *= 0.8;
  
  if ( LocalHud.FlakMonkeyTime - GameStartTime > 0 ) {
    ClockMarker.SpriteWidget.TextureCoords = LocalHud.FlakMonkeyCoords;
    ClockMarker.Angle = -65536 * LocalHud.FlakMonkeyTime / nSecondsMax;
    ClockMarker.SpriteWidget.Tints[0] = LocalHud.EventColors[4];
    ClockMarker.SpriteWidget.Tints[1] = LocalHud.EventColors[4];
    
    DrawRotatedWidget(C, ClockMarker);
  }
  if ( LocalHud.ComboWhoreTime - GameStartTime > 0 ) {
    ClockMarker.SpriteWidget.TextureCoords = LocalHud.ComboWhoreCoords;
    ClockMarker.Angle = -65536 * LocalHud.ComboWhoreTime / nSecondsMax;
    ClockMarker.SpriteWidget.Tints[0] = LocalHud.EventColors[5];
    ClockMarker.SpriteWidget.Tints[1] = LocalHud.EventColors[5];
    
    DrawRotatedWidget(C, ClockMarker);
  }
  if ( LocalHud.HeadHunterTime - GameStartTime > 0 ) {
    ClockMarker.SpriteWidget.TextureCoords = LocalHud.HeadHunterCoords;
    ClockMarker.Angle = -65536 * LocalHud.HeadHunterTime / nSecondsMax;
    ClockMarker.SpriteWidget.Tints[0] = LocalHud.EventColors[6];
    ClockMarker.SpriteWidget.Tints[1] = LocalHud.EventColors[6];
    
    DrawRotatedWidget(C, ClockMarker);
  }
  
  // draw game events
  if ( TGRI != None ) {
    GameEventCount = TGRI.CountGameEvents();
    ClockMarker = default.ClockMarker;
    ClockMarker.SpriteWidget.PosX = ClockPosition.X;
    ClockMarker.SpriteWidget.PosY = ClockPosition.Y;
    for (i = 0; i < GameEventCount; i++) {
      ClockMarker.Angle = -65536 * TGRI.GetEventTime(i) / nSecondsMax;
      ClockMarker.SpriteWidget.Tints[0] = LocalHud.EventColors[TGRI.GetEventTeam(i)];
      ClockMarker.SpriteWidget.Tints[1] = ClockMarker.SpriteWidget.Tints[0];
      
      DrawRotatedWidget(C, ClockMarker);
    }
  }
  
  // draw elapsed time and (if applicable) remaining time or overtime
  if ( LocalHud != None )
    C.Font = LocalHud.GetFontSizeIndex(C, -2);
  C.DrawColor = class'TournamentHud'.static.LerpColor(LocalHud.ClockPulseTime, WhiteColor, HighlightColor);
  Seconds = nSecondsCurrent % 60;
  Minutes = nSecondsCurrent / 60;
  if ( GRI.TimeLimit == 0 ) {
    C.DrawScreenText(ClockElapsedTime, ClockPosition.X, ClockPosition.Y, DP_LowerMiddle);
    C.DrawScreenText(FormatTime(nSecondsCurrent),
        ClockPosition.X, ClockPosition.Y + 1.1 * YL / C.SizeY, DP_UpperMiddle);
  }
  else {
    C.TextSize("A", XL, YL);
    C.DrawScreenText(ClockElapsedTime, ClockPosition.X, ClockPosition.Y - 1.1 * YL / C.SizeY, DP_LowerMiddle);
    C.DrawScreenText(FormatTime(nSecondsCurrent), ClockPosition.X, ClockPosition.Y - 1.1 * YL / C.SizeY,
        DP_UpperMiddle);
    if ( nSecondsCurrent > nSecondsMax ) {
      C.DrawColor = class'TournamentHud'.static.LerpColor(LocalHud.ClockPulseTime, OverTimeColor, HighlightColor);
      C.DrawScreenText(ClockOverTime, ClockPosition.X, ClockPosition.Y + 1.1 * YL / C.SizeY, DP_LowerMiddle);
      C.DrawScreenText(FormatTime(nSecondsCurrent - nSecondsMax),
          ClockPosition.X, ClockPosition.Y + 1.1 * YL / C.SizeY, DP_UpperMiddle);
    }
    else {
      C.DrawScreenText(ClockRemainingTime, ClockPosition.X, ClockPosition.Y + 1.1 * YL / C.SizeY, DP_LowerMiddle);
      C.DrawScreenText(FormatTime(nSecondsMax - nSecondsCurrent),
          ClockPosition.X, ClockPosition.Y + 1.1 * YL / C.SizeY, DP_UpperMiddle);
    }
  }
}


//=============================================================================
// GetFontSizeIndex
//
// Draws a SpriteWidget.
//=============================================================================

function Font GetFontSizeIndex(Canvas C, int FontSize)
{
  if ( C.ClipX >= 512 )
    FontSize++;
  if ( C.ClipX >= 640 )
    FontSize++;
  if ( C.ClipX >= 800 )
    FontSize++;
  if ( C.ClipX >= 1024 )
    FontSize++;
  if ( C.ClipX >= 1280 )
    FontSize++;
  if ( C.ClipX >= 1600 )
    FontSize++;
  
  return HUDClass.static.LoadFontStatic(Clamp(8 - FontSize, 0, 8));
}


//=============================================================================
// FindKey
//
// Determines the key name that's bound to the given command. Partial commands
// match.
// (adapted from Mychaeel's code for the UTJB3 scoreboard)
//=============================================================================

function string FindKey(string TextCommand)
{
  local int IndexKey;
  local string TextKey;
  local string TextKeyLocalized;
  local string TextAlias;
  
  for (IndexKey = 0; IndexKey < 256; IndexKey++) {
    TextKey = Owner.ConsoleCommand("KEYNAME" @ IndexKey);
    if ( Len(TextKey) == 0 )
      continue;
    
    TextAlias = Owner.ConsoleCommand("KEYBINDING" @ TextKey);
    
    if ( InStr(Caps(TextAlias), Caps(TextCommand)) >= 0 ) {
      TextKeyLocalized = Owner.ConsoleCommand("LOCALIZEDKEYNAME" @ IndexKey);
      if ( TextKeyLocalized != "" )
        return TextKeyLocalized;
      return TextKey;
    }
  }
  
  return "";
}


//=============================================================================
// DrawSpriteWidget
//
// Draws a SpriteWidget.
//=============================================================================

simulated final function DrawSpriteWidget(Canvas C, out HudBase.SpriteWidget SpriteWidget)
{
  if ( LocalHud != None )
    LocalHud.DrawSpriteWidget(C, SpriteWidget);
}


//=============================================================================
// DrawNumericWidget
//
// Draws a NumericWidget with the specified DigitSet.
//=============================================================================

simulated final function DrawNumericWidget(Canvas C, out HudBase.NumericWidget NumericWidget, out HudBase.DigitSet DigitSet)
{
  if ( LocalHud != None )
    LocalHud.DrawNumericWidget(C, NumericWidget, DigitSet);
}


//=============================================================================
// DrawRotatedWidget
//
// Draws a RotatedWidget.
//=============================================================================

simulated final function DrawRotatedWidget(Canvas C, TournamentHUD.RotatedWidget RotatedWidget)
{
  if ( LocalHud != None )
    LocalHud.DrawRotatedWidget(C, RotatedWidget);
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  HeaderText = "%skill% %game% in %map%"
  OnlineString = "Online"
  
  GoalScoreString = "Frag Limit:"
  MaxLivesString = "Max. Lives:"
  
  ClockElapsedTime   = "Elapsed"
  ClockRemainingTime = "Remaining"
  ClockOverTime      = "Overtime"
  
  CurrentSortOrderString = "Sorted by %s (press [%k] to change)"
  SortOrderStrings       = ("Score","Efficiency","Kills","Deaths","FPH","Name","Ping")
  PlayerNotShown         = "One player not shown"
  PlayersNotShown        = "%i players not shown"
  
  ScoreCaption      = "Score"
  GoalsCaption      = "Captures"
  TouchesCaption    = "Steals"
  ReturnsCaption    = "Ret."
  KillsCaption      = "K"
  DeathsCaption     = "D"
  SuicidesCaption   = "S"
  EfficiencyCaption = "Eff."
  FPHCaption        = "FPH"
  PingCaption       = "Ping"
  PacketLossCaption = "PL"
  NetSpeedCaption   = "Speed"
  BotString         = "Bot"
  AdminString       = "Admin"
  OutString         = "Out"
  
  StatsForString = "Stats for %p"
  OtherString    = "Other"
  TotalString    = "Total"
  
  WeaponCaption       = "Weapon"
  ShotsCaption        = "Shots"
  HitsCaption         = "Hits"
  AccuracyCaption     = "Accuracy"
  DamageCaption       = "Damage"
  DamagePerHitCaption = "Dmg./Hit"
  
  PickupCaption       = "Pickup"
  PickupCountCaption  = "Count"
  PickupAmountCaption = "Amount"
  
  bShowGoalCount    = False
  bShowSuicideCount = True
  bShowRank         = True
  
  WhiteColor      = (R=255,G=255,B=255,A=255)
  OverTimeColor   = (R=255,G=000,B=000,A=255)
  HighlightColor  = (R=255,G=255,B=000,A=255)
  
  TeamColors(0) = (R=255,G=000,B=000,A=255)
  TeamColors(1) = (R=048,G=064,B=255,A=255)
  TeamColors(2) = (R=255,G=255,B=000,A=255)
  TeamColors(3) = (R=224,G=224,B=224,A=255)
  
  ScoresBG      = (R=000,G=000,B=000,A=128)
  StatsHeaderBG = (R=192,G=192,B=192,A=128)
  StatsBG       = (R=128,G=128,B=128,A=128)
  
  HUDClass = class'TournamentHUD'
  bDisplayMessages = True
  
  ClockScale       = 1.0
  ClockPosition    = (X=0.5,Y=0.1)
  ClockCircle      =               (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=256,Y1=000,X2=512,Y2=256),TextureScale=0.35,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.1,OffsetX=00,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=255,G=255,B=255,A=255),Tints[1]=(R=255,G=255,B=255,A=255))
  ClockFilledFirst =               (WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=128,Y1=038,X2=038,Y2=218),TextureScale=0.46,DrawPivot=DP_MiddleLeft,PosX=000.5,PosY=0.1,OffsetX=-1,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=064,G=064,B=056,A=192),Tints[1]=(R=064,G=064,B=064,A=192))
  ClockFillFirst   = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=128,Y1=038,X2=218,Y2=218),TextureScale=0.46,DrawPivot=DP_MiddleLeft,PosX=000.5,PosY=0.1,OffsetX=-1,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=064,G=064,B=064,A=192),Tints[1]=(R=056,G=056,B=056,A=192)),OffsetCenterX=00,OffsetCenterY=90)
  ClockFillSecond  = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=038,Y1=038,X2=128,Y2=218),TextureScale=0.46,DrawPivot=DP_MiddleRight,PosX=00.5,PosY=0.1,OffsetX=01,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=064,G=064,B=064,A=192),Tints[1]=(R=064,G=064,B=064,A=192)),OffsetCenterX=89,OffsetCenterY=90)
  ClockTick        = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=010,Y1=328,X2=028,Y2=350),TextureScale=0.30,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.1,OffsetX=00,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=96,G=96,B=96,A=192),Tints[1]=(R=96,G=96,B=96,A=192)),OffsetCenterX=09,OffsetCenterY=11,OffsetRotatedX=0,OffsetRotatedY=-140)
  ClockMarker      = (SpriteWidget=(WidgetTexture=Material'TournamentHUDElements',TextureCoords=(X1=007,Y1=357,X2=037,Y2=383),TextureScale=0.30,DrawPivot=DP_MiddleMiddle,PosX=0.5,PosY=0.1,OffsetX=00,OffsetY=0,ScaleMode=SM_Right,Scale=1.0,RenderStyle=STY_Alpha,Tints[0]=(R=192,G=192,B=192,A=255),Tints[1]=(R=192,G=192,B=192,A=255)),OffsetCenterX=15,OffsetCenterY=13,OffsetRotatedX=0,OffsetRotatedY=-140)
}