//=============================================================================
// TournamentScoreboardDM
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament DeathMatch score board.
//=============================================================================


class TournamentScoreboardDM extends TournamentScoreboard;


//=============================================================================
// DrawTitle
//
// Draw the scoreboard header with game and map info.
//=============================================================================

function DrawTitle(Canvas C)
{
  local string TitleString;
  local string FooterString;
  local float XL, YL;
  
  TitleString = HeaderText;
  if ( Level.Game != None && Level.NetMode == NM_Standalone )
    ReplaceText(TitleString, "%skill%",
        class'ScoreBoardDeathMatch'.default.SkillLevel[Clamp(Level.Game.GameDifficulty, 0, 7)]);
  else
    ReplaceText(TitleString, "%skill%", OnlineString);
  ReplaceText(TitleString, "%game%", GRI.GameName);
  ReplaceText(TitleString, "%map%", Level.Title);
  
  if ( UnrealPlayer(Owner).bDisplayLoser )
    FooterString = class'TournamentUtil'.static.Trim(class'HudBDeathMatch'.default.YouveLostTheMatch);
  else if ( UnrealPlayer(Owner).bDisplayWinner )
    FooterString = class'TournamentUtil'.static.Trim(class'HudBDeathMatch'.default.YouveWonTheMatch);
  else if ( PlayerController(Owner).IsDead() ) {
    if ( PlayerController(Owner).PlayerReplicationInfo.bOutOfLives )
      FooterString = class'TournamentUtil'.static.Trim(class'ScoreBoardDeathMatch'.default.OutFireText);
    else
      FooterString = class'TournamentUtil'.static.Trim(class'ScoreBoardDeathMatch'.default.Restart);
  }
  else {
    if ( GRI.MaxLives > 0 )
      FooterString = " " $ MaxLivesString @ GRI.MaxLives $ " ";
    if ( GRI.GoalScore > 0 )
      FooterString = " " $ GoalScoreString @ GRI.GoalScore @ FooterString;
  }
  
  C.Font = GetFontSizeIndex(C, 2);
  C.DrawColor = HighlightColor;
  C.TextSize("X", XL, YL);
  
  C.SetOrigin(0.5 * C.SizeX, 0.01);
  C.SetClip(0.35 * C.SizeX, C.SizeY);
  C.SetPos(0, 0);
  C.DrawText(TitleString);
  C.SetOrigin(0, 0);
  C.SetClip(C.SizeX, C.SizeY);
  
  C.DrawScreenText(FooterString, 0.5, 1.0, DP_LowerMiddle);
  
  C.Font = GetFontSizeIndex(C, -2);
  C.DrawColor = WhiteColor;
  C.DrawScreenText(class'TournamentUtil'.static.CurrentDate(Level), 1.0, 1.0 - YL / C.SizeX, DP_LowerRight);
}


//=============================================================================
// DrawScores
//
// Draw the actual scores.
//=============================================================================

function DrawScores(Canvas C)
{
  local int i;
  local int Rank;
  local array<PlayerReplicationInfo> Players, Spectators;
  local float YPos;
  local int nNotShown;
  local string NotShownText;
  
  // filter and sort PRIs
  for (i = 0; i < GRI.PRIArray.Length; i++) {
    if ( !GRI.PRIArray[i].bOnlySpectator && TournamentPlayerReplicationInfo(GRI.PRIArray[i]) != None ) {
      Players[Rank++] = GRI.PRIArray[i];
      TournamentPlayerReplicationInfo(GRI.PRIArray[i]).Rank = Rank;
    }
    else if ( GRI.PRIArray[i].bOnlySpectator && TournamentPlayerReplicationInfo(GRI.PRIArray[i]) != None )
      Spectators[Spectators.Length] = GRI.PRIArray[i];
  }
  SortPRIs(Players, 0, Players.Length - 1, SortOrder);
  
  YPos = 0.07;
  DrawPlayerScore(C, None, 0, YPos, False, Players.Length > MaxFullDetailPlayers);
  
  // draw player scores
  for (i = 0; i < Players.Length && i < MaxDisplayedPlayers; i++) {
    if ( GRI.bTeamGame ) {
      if ( Players.Length > MaxFullDetailPlayers )
        YPos += 0.045;
      else
        YPos += 0.06;
    }
    else
      YPos += FClamp(0.7 / Players.Length, 0.045, 0.06);
    DrawPlayerScore(C, Players[i], TournamentPlayerReplicationInfo(Players[i]).Rank, YPos, False,
        Players.Length > MaxFullDetailPlayers);
  }
  
  YPos += 0.04;
  DrawSortingOrder(C, 0.24, YPos);
  nNotShown = Players.Length - i;
  if ( nNotShown == 1 )
    C.DrawScreenText(PlayerNotShown, 0.24, YPos, DP_LowerMiddle);
  else if ( nNotShown > 1 ) {
    NotShownText = PlayersNotShown;
    ReplaceText(NotShownText, "%i", string(nNotShown));
    C.DrawScreenText(NotShownText, 0.24, YPos, DP_LowerMiddle);
  }
}


//=============================================================================
// DrawStats
//
// Draws the weapon and pickup stats.
//=============================================================================

function DrawStats(Canvas C)
{
  local float YPos;
  local PlayerController LocalPlayer;
  local TournamentPlayerReplicationInfo TPRI;
  local TournamentPlayerReplicationInfo.TPickupStats Stats, TotalStats;
  local int i, Count;
  local string StatsCaption;
  
  LocalPlayer = PlayerController(Owner);
  if ( !LocalPlayer.PlayerReplicationInfo.bOnlySpectator ) {
    if ( WeaponStats == None )
      WeaponStats = class'TournamentWeaponStats'.static.FindStatsFor(LocalPlayer.PlayerReplicationInfo);
    TPRI = TournamentPlayerReplicationInfo(LocalPlayer.PlayerReplicationInfo);
  }
  else if ( Pawn(LocalPlayer.ViewTarget) != None ) {
    WeaponStats = class'TournamentWeaponStats'.static.FindStatsFor(Pawn(LocalPlayer.ViewTarget).PlayerReplicationInfo);
    TPRI = TournamentPlayerReplicationInfo(Pawn(LocalPlayer.ViewTarget).PlayerReplicationInfo);
  }
  
  YPos = 0.25;
  C.DrawColor = WhiteColor;
  C.Font = GetFontSizeIndex(C, -1);
  StatsCaption = StatsForString;
  if ( WeaponStats != None && WeaponStats.PRI != None ) {
    ReplaceText(StatsCaption, "%p", WeaponStats.PRI.PlayerName);
    C.DrawScreenText(StatsCaption, 0.5, YPos, DP_LowerLeft);
    DrawStatsForWeapon(C, 0.5, YPos, 0.48);
    for (i = 0; i < WeaponStats.NumSlots(); i++)
      DrawStatsForWeapon(C, 0.5, YPos, 0.48, WeaponStats.GetStatsSlot(i));
    
    YPos += 0.03;
  }
  if ( TPRI != None && TPRI.NumSlots() > 0 ) {
    if ( WeaponStats == None || WeaponStats.PRI == None ) {
      ReplaceText(StatsCaption, "%p", TPRI.PlayerName);
      C.DrawScreenText(StatsCaption, 0.5, YPos, DP_LowerLeft);
    }
    DrawStatsForPickup(C, 0.5, YPos, 0.48);
    
    // health
    for (i = 0; i < TPRI.NumSlots() && class<TournamentHealth>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.5, YPos, 0.48, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.5, YPos, 0.48, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // shield
    for (Count = 0; i < TPRI.NumSlots() && class<ShieldPickup>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.5, YPos, 0.48, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.5, YPos, 0.48, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // adrenaline
    for (Count = 0; i < TPRI.NumSlots() && class<AdrenalinePickup>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.5, YPos, 0.48, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.5, YPos, 0.48, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // other pickups
    for (Count = 0; i < TPRI.NumSlots(); i++)
      DrawStatsForPickup(C, 0.5, YPos, 0.48, TPRI.GetStatsSlot(i));
  }
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  MaxDisplayedPlayers  = 14
  MaxFullDetailPlayers = 8
  ClockPosition = (X=0.925,Y=0.1)
}