//=============================================================================
// TournamentScoreboardTDM
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Team DeathMatch score board.
//=============================================================================


class TournamentScoreboardTDM extends TournamentScoreboard;


//=============================================================================
// DrawTitle
//
// Draw the scoreboard header with game and map info.
//=============================================================================

function DrawTitle(Canvas C)
{
  local string TitleString;
  local string FooterString;
  local float XL, YL;
  
  TitleString = HeaderText;
  if ( Level.Game != None && Level.NetMode == NM_Standalone )
    ReplaceText(TitleString, "%skill%",
        class'ScoreBoardDeathMatch'.default.SkillLevel[Clamp(Level.Game.GameDifficulty, 0, 7)]);
  else
    ReplaceText(TitleString, "%skill%", OnlineString);
  ReplaceText(TitleString, "%game%", GRI.GameName);
  ReplaceText(TitleString, "%map%", Level.Title);
  
  if ( UnrealPlayer(Owner).bDisplayLoser )
    FooterString = class'TournamentUtil'.static.Trim(class'HudBDeathMatch'.default.YouveLostTheMatch);
  else if ( UnrealPlayer(Owner).bDisplayWinner )
    FooterString = class'TournamentUtil'.static.Trim(class'HudBDeathMatch'.default.YouveWonTheMatch);
  else if ( PlayerController(Owner).IsDead() ) {
    if ( PlayerController(Owner).PlayerReplicationInfo.bOutOfLives )
      FooterString = class'TournamentUtil'.static.Trim(class'ScoreBoardDeathMatch'.default.OutFireText);
    else
      FooterString = class'TournamentUtil'.static.Trim(class'ScoreBoardDeathMatch'.default.Restart);
  }
  else {
    if ( GRI.MaxLives > 0 )
      FooterString = " " $ MaxLivesString @ GRI.MaxLives $ " ";
    if ( GRI.GoalScore > 0 )
      FooterString = " " $ GoalScoreString @ GRI.GoalScore @ FooterString;
  }
  
  C.Font = GetFontSizeIndex(C, 2);
  C.DrawColor = HighlightColor;
  C.TextSize("X", XL, YL);
  
  C.DrawScreenText(TitleString, 0.5, 0.01, DP_UpperMiddle);
  C.DrawScreenText(FooterString, 0.5, 1.0, DP_LowerMiddle);
  
  C.Font = GetFontSizeIndex(C, -2);
  C.DrawColor = WhiteColor;
  C.DrawScreenText(class'TournamentUtil'.static.CurrentDate(Level), 1.0, 1.0 - YL / C.SizeX, DP_LowerRight);
}


//=============================================================================
// DrawScores
//
// Draw the actual scores.
//=============================================================================

function DrawScores(Canvas C)
{
  local int i, iRed, iBlue;
  local int Rank;
  local array<PlayerReplicationInfo> RedPlayers, BluePlayers, Spectators;
  local float YPosBlue, YPosRed;
  local int nNotShown;
  local string NotShownText;
  local Decoration HasFlag;
  
  DrawTeamScores(C);
  
  for (i = 0; i < GRI.PRIArray.Length; i++) {
    if ( !GRI.PRIArray[i].bOnlySpectator && GRI.PRIArray[i].Team != None
        && TournamentPlayerReplicationInfo(GRI.PRIArray[i]) != None ) {
      if ( GRI.PRIArray[i].Team.TeamIndex == 1 )
        BluePlayers[BluePlayers.Length] = GRI.PRIArray[i];
      else
        RedPlayers[RedPlayers.Length] = GRI.PRIArray[i];
      TournamentPlayerReplicationInfo(GRI.PRIArray[i]).Rank = ++Rank;
    }
    else if ( GRI.PRIArray[i].bOnlySpectator && TournamentPlayerReplicationInfo(GRI.PRIArray[i]) != None )
      Spectators[Spectators.Length] = GRI.PRIArray[i];
  }
  SortPRIs(RedPlayers,  0, RedPlayers.Length - 1,  SortOrder);
  SortPRIs(BluePlayers, 0, BluePlayers.Length - 1, SortOrder);
  
  YPosRed  = 0.2;
  YPosBlue = 0.2;
  for (i = MaxDisplayedPlayers; i < RedPlayers.Length; i++) {
    if ( RedPlayers[i].HasFlag != None )
      HasFlag = RedPlayers[i].HasFlag;
      break;
    }
  DrawPlayerScore(C, None, 0, YPosRed, False, RedPlayers.Length > MaxFullDetailPlayers,, HasFlag);
  HasFlag = None;
  for (i = MaxDisplayedPlayers; i < BluePlayers.Length; i++) {
    if ( BluePlayers[i].HasFlag != None )
      HasFlag = BluePlayers[i].HasFlag;
      break;
    }
  DrawPlayerScore(C, None, 0, YPosBlue, True, BluePlayers.Length > MaxFullDetailPlayers,, HasFlag);
  
  // draw player scores
  for (i = 0; i < Max(BluePlayers.Length, RedPlayers.Length); i++) {
    if ( i < RedPlayers.Length && i < MaxDisplayedPlayers ) {
      iRed = i;
      if ( GRI.bTeamGame ) {
        if ( RedPlayers.Length > MaxFullDetailPlayers )
          YPosRed += 0.045;
        else
          YPosRed += 0.07;
      }
      else
        YPosRed += FClamp(0.65 / RedPlayers.Length, 0.045, 0.06);
      DrawPlayerScore(C, RedPlayers[i], TournamentPlayerReplicationInfo(RedPlayers[i]).Rank, YPosRed,
          False, RedPlayers.Length > MaxFullDetailPlayers);
    }
    if ( i < BluePlayers.Length && i < MaxDisplayedPlayers ) {
      iBlue = i;
      if ( GRI.bTeamGame ) {
        if ( BluePlayers.Length > MaxFullDetailPlayers )
          YPosBlue += 0.045;
        else
          YPosBlue += 0.07;
      }
      else
        YPosBlue += FClamp(0.65 / BluePlayers.Length, 0.045, 0.06);
      DrawPlayerScore(C, BluePlayers[i], TournamentPlayerReplicationInfo(BluePlayers[i]).Rank, YPosBlue,
          True, BluePlayers.Length > MaxFullDetailPlayers);
    }
  }
  
  YPosRed += 0.022;
  YPosBlue += 0.022;
  
  if ( Abs(FMax(YPosBlue, YPosRed) - 0.5) < 0.04 )
    DrawSortingOrder(C, 0.5, 0.54); // keep crosshair clearly visible
  else
    DrawSortingOrder(C, 0.5, FMax(YPosBlue, YPosRed));
  
  nNotShown = RedPlayers.Length - ++iRed;
  if ( nNotShown == 1 )
    C.DrawScreenText(PlayerNotShown, 0.17, YPosRed, DP_UpperMiddle);
  else if ( nNotShown > 1 ) {
    NotShownText = PlayersNotShown;
    ReplaceText(NotShownText, "%i", string(nNotShown));
    C.DrawScreenText(NotShownText, 0.17, YPosRed, DP_UpperMiddle);
  }
  
  nNotShown = BluePlayers.Length - ++iBlue;
  if ( nNotShown == 1 )
    C.DrawScreenText(PlayerNotShown, 0.82, YPosBlue, DP_UpperMiddle);
  else if ( nNotShown > 1 ) {
    NotShownText = PlayersNotShown;
    ReplaceText(NotShownText, "%i", string(nNotShown));
    C.DrawScreenText(NotShownText, 0.82, YPosBlue, DP_UpperMiddle);
  }
}


//=============================================================================
// DrawTeamScores
//
// Draw each team's score.
//=============================================================================

function DrawTeamScores(Canvas C)
{
  
}


//=============================================================================
// DrawStats
//
// Draws the weapon and pickup stats.
//=============================================================================

function DrawStats(Canvas C)
{
  local float YPos;
  local PlayerController LocalPlayer;
  local TournamentPlayerReplicationInfo TPRI;
  local TournamentPlayerReplicationInfo.TPickupStats Stats, TotalStats;
  local int i, Count;
  local string StatsCaption;
  
  LocalPlayer = PlayerController(Owner);
  if ( !LocalPlayer.PlayerReplicationInfo.bOnlySpectator ) {
    if ( WeaponStats == None )
      WeaponStats = class'TournamentWeaponStats'.static.FindStatsFor(LocalPlayer.PlayerReplicationInfo);
    TPRI = TournamentPlayerReplicationInfo(LocalPlayer.PlayerReplicationInfo);
  }
  else if ( Pawn(LocalPlayer.ViewTarget) != None ) {
    WeaponStats = class'TournamentWeaponStats'.static.FindStatsFor(Pawn(LocalPlayer.ViewTarget).PlayerReplicationInfo);
    TPRI = TournamentPlayerReplicationInfo(Pawn(LocalPlayer.ViewTarget).PlayerReplicationInfo);
  }
  
  C.DrawColor = WhiteColor;
  C.Font = GetFontSizeIndex(C, -1);
  StatsCaption = StatsForString;
  if ( WeaponStats != None && WeaponStats.PRI != None ) {
    YPos = 0.65;
    if ( TPRI == None || TPRI.NumSlots() == 0 ) {
      ReplaceText(StatsCaption, "%p", WeaponStats.PRI.PlayerName);
      C.DrawScreenText(StatsCaption, 0.25, YPos, DP_LowerLeft);
    }
    DrawStatsForWeapon(C, 0.24, YPos, 0.75);
    for (i = 0; i < WeaponStats.NumSlots(); i++)
      DrawStatsForWeapon(C, 0.24, YPos, 0.75, WeaponStats.GetStatsSlot(i));
  }
  if ( TPRI != None && TPRI.NumSlots() > 0 ) {
    YPos = 0.65;
    C.Font = GetFontSizeIndex(C, -1);
    ReplaceText(StatsCaption, "%p", TPRI.PlayerName);
    C.DrawScreenText(StatsCaption, 0.02, YPos, DP_LowerLeft);
    DrawStatsForPickup(C, 0.01, YPos, 0.22);
    
    // health
    for (i = 0; i < TPRI.NumSlots() && class<TournamentHealth>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.01, YPos, 0.22, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.01, YPos, 0.22, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // shield
    for (Count = 0; i < TPRI.NumSlots() && class<ShieldPickup>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.01, YPos, 0.22, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.01, YPos, 0.22, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // adrenaline
    for (Count = 0; i < TPRI.NumSlots() && class<AdrenalinePickup>(TPRI.GetStatsSlot(i).Pickup) != None; i++) {
      Stats = TPRI.GetStatsSlot(i);
      TotalStats.Count += Stats.Count;
      TotalStats.Amount += Stats.Amount;
      DrawStatsForPickup(C, 0.01, YPos, 0.22, Stats);
      Count++;
    }
    if ( Count > 1 ) {
      DrawStatsForPickup(C, 0.01, YPos, 0.22, TotalStats);
      TotalStats.Count = 0;
      TotalStats.Amount = 0;
    }
    
    // other pickups
    for (Count = 0; i < TPRI.NumSlots(); i++)
      DrawStatsForPickup(C, 0.01, YPos, 0.22, TPRI.GetStatsSlot(i));
  }
}


//=============================================================================
// DrawStatsForWeapon
//
// Draws the current sorting order centered above the specified position.
//=============================================================================

function DrawStatsForWeapon(Canvas C, float XPos, out float YPos, float Width, optional TournamentWeaponStats.TWeaponStats Stats)
{
  local float XL, YL;
  local string WeaponName;
  local string Shots, Hits, Accuracy, TotalAccuracy, Damage, DamagePerHit;
  local float f;
  local int i;
  local string s[3];
  
  if ( Stats.bSlotUsed ) {
    if ( Stats.Weapon != None )
      WeaponName = Stats.Weapon.static.StaticItemName();
    else
      WeaponName = OtherString;
    Shots = Stats.Shots[0]$"/"$Stats.Shots[1]$"/"$Stats.Shots[2];
    Hits = Stats.ShotsHit[0]$"/"$Stats.ShotsHit[1]$"/"$Stats.ShotsHit[2];
    for (i = 0; i < 3; i++) {
      if ( Stats.Shots[i] == 0 )
        s[i] = "-";
      else {
        f = 100 * Stats.ShotsHit[i];
        f /= Stats.Shots[i];
        s[i] = int(f) $ "%";
      }
    }
    Accuracy = s[0]$"/"$s[1]$"/"$s[2];
    if ( Accuracy != "-/-/-" ) {
      f = 100 * (Stats.ShotsHit[0] + Stats.ShotsHit[1] + Stats.ShotsHit[2]);
      f /= Stats.Shots[0] + Stats.Shots[1] + Stats.Shots[2];
      TotalAccuracy = int(f) $ "%";
    }
    Damage = Stats.TotalDamage[0]$"/"$Stats.TotalDamage[1]$"/"$Stats.TotalDamage[2];
    for (i = 0; i < 3; i++) {
      if ( Stats.ShotsHit[i] == 0 )
        s[i] = "-";
      else {
        f = Stats.TotalDamage[i] / Stats.ShotsHit[i];
        s[i] = string(int(f));
      }
    }
    DamagePerHit = s[0]$"/"$s[1]$"/"$s[2];
  }
  else {
    WeaponName = WeaponCaption;
    Shots = ShotsCaption;
    Hits = HitsCaption;
    Accuracy = AccuracyCaption;
    Damage = DamageCaption;
    DamagePerHit = DamagePerHitCaption;
  }
  
  C.Font = GetFontSizeIndex(C, -2);
  C.TextSize("0", XL, YL);
  if ( Stats.bSlotUsed )
    C.DrawColor = StatsBG;
  else
    C.DrawColor = StatsHeaderBG;
  C.SetPos(XPos * C.ClipX, YPos * C.ClipY - 1);
  C.DrawRect(Texture'WhiteTexture', Width * C.ClipX, YL);
  C.DrawColor = WhiteColor;
  C.DrawScreenText(WeaponName, XPos + 0.01, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleLeft);
  C.Font = GetFontSizeIndex(C, -3);
  C.DrawScreenText(Shots, XPos + 0.25 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  C.DrawScreenText(Hits,  XPos + 0.35 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  C.DrawScreenText(Accuracy, XPos + 0.5 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  if ( TotalAccuracy != "" )
    C.DrawScreenText("("$TotalAccuracy$")", XPos + 0.6 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  C.DrawScreenText(Damage,       XPos + 0.8 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  C.DrawScreenText(DamagePerHit, XPos + 0.95 * Width, YPos + (0.5 * YL + 1) / C.ClipY, DP_MiddleMiddle);
  
  YPos += (YL + 1) / C.ClipY;
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  ClockScale    = 0.8
  ClockPosition = (X=0.5,Y=0.13)
  MaxDisplayedPlayers  = 8
  MaxFullDetailPlayers = 5
}