//=============================================================================
// xBombingRun
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Bombing Run
//=============================================================================


class xBombingRun extends xGame.xBombingRun;


//=============================================================================
// Variables
//=============================================================================

var int WarmupTime;
var int WarmupCountdown;


//=============================================================================
// Reset
//
// Reset the game after warmup.
//=============================================================================

function Reset()
{
  Super.Reset();
  ResetCountDown = 0;
}


//=============================================================================
// InitGame
//
// Set the fake GameName for display purposes and to fool the master server.
//=============================================================================

event InitGame(string Options, out string Error)
{
  if ( class'xPawn'.default.ControllerClass != class'xBot' ) {
    class'xPawn'.default.ControllerClass = class'xBot';
    ConsoleCommand("OBJ GARBAGE");
  }
  GameName = class'xGame.xBombingRun'.default.GameName;
  Super.InitGame(Options, Error);
  SpawnProtectionTime = Clamp(GetIntOption(Options, "SpawnProt", SpawnProtectionTime), 0, 30);
  WarmupTime = 60 * Max(1, GetIntOption(Options, "WarmupTime", class'TournamentServerSettings'.default.WarmupTime));
  WarmupCountdown = WarmupTime;
}


//=============================================================================
// exec EndWarmup
//
// Ends the warmup time.
//=============================================================================

exec function EndWarmup()
{
  class'TournamentUtil'.static.EndWarmup(Self, WarmupCountdown);
}


//=============================================================================
// state PendingMatch
//
// Check for match or warmup start.
//=============================================================================

auto state PendingMatch
{
  function RestartPlayer(Controller aPlayer)
  {
    if ( WarmupCountDown > 3 )
      Global.RestartPlayer(aPlayer);
  }
  
  function Timer()
  {
    local Controller C;
    local TranslocatorBeacon Trans;
    
    Global.Timer();
    class'TournamentUtil'.static.PendingMatchTimer(Self, WarmupTime, WarmupCountDown);
    if ( StartupStage > 1 || ResetCountDown > CountDown )
      ResetCountDown = 0;
    
    if ( ResetCountDown > 0 ) {
      ResetCountDown--;
      if ( ResetCountDown < 3 ) {
        // blow up all redeemer guided warheads
        for (C = Level.ControllerList; C != None; C = C.NextController)
          if ( C.Pawn != None && C.Pawn.IsA('RedeemerWarhead') )
            C.Pawn.Fire(1);
      }
      if ( ResetCountDown == 8 ) {
        for (C = Level.ControllerList; C != None; C = C.NextController)
          if ( PlayerController(C) != None )
            PlayerController(C).PlayAnnouncement(NewRoundSound, 1, true);
      }
      else if ( ResetCountDown > 1 && ResetCountDown < 7 )
        BroadcastLocalizedMessage(class'TimerMessage', ResetCountDown-1);
      else if ( ResetCountDown == 1 ) {
        // reset all bot enemies
        Teams[0].AI.ClearEnemies();
        Teams[1].AI.ClearEnemies();
        
        // reset all players position and rotation on the field for the next round
        for (C = Level.ControllerList; C != None; C = C.NextController)
          if ( C.PlayerReplicationInfo != None && !C.PlayerReplicationInfo.bOnlySpectator ) {
            C.StartSpot = FindPlayerStart(C, C.PlayerReplicationInfo.Team.TeamIndex);
            if ( C.StartSpot != None ) {
              C.SetLocation(C.StartSpot.Location);
              C.SetRotation(C.StartSpot.Rotation);
            }
            if ( C.Pawn != None ) {
              if ( xPawn(C.Pawn) != None ) {
                if ( xPawn(C.Pawn).CurrentCombo != None ) {
                  C.Adrenaline = 0;
                  xPawn(C.Pawn).CurrentCombo.Destroy();
                }
                if ( xPawn(C.Pawn).UDamageTimer != None ) {
                  xPawn(C.Pawn).UDamageTimer.Destroy();
                  xPawn(C.Pawn).DisableUDamage();
                }
              }
              C.Pawn.Health = Max(C.Pawn.Health, C.Pawn.Default.Health);
              SetPlayerDefaults(C.Pawn);
              C.Pawn.SetLocation(C.StartSpot.Location);
              C.Pawn.SetRotation(C.StartSpot.Rotation);
              C.Pawn.Velocity = vect(0,0,0);
              C.Pawn.PlayTeleportEffect(false, true);
            }
            if ( C.StartSpot != None )
              C.ClientSetLocation(C.StartSpot.Location,C.StartSpot.Rotation);
          }
        
        // destroy translocator beacons
        foreach DynamicActors(class'TranslocatorBeacon', Trans)
          Trans.Destroy();
        
        Bomb.SendHome();
        ResetCountDown = 0;
      }
    }
    else {
      if ( Bomb == None )
        ForEach DynamicActors(class'xBombFlag', Bomb)
          break;
      if ( Bomb != None )
        GameReplicationInfo.FlagPos = Bomb.Position().Location;
    }
  }
}


//=============================================================================
// ParseChatPercVar
//
// The %L chat variable should also find shield and big health packs.
//=============================================================================

static function string ParseChatPercVar(Mutator BaseMutator, Controller Who, string Cmd)
{
  return class'TournamentUtil'.static.ParseChatPercVar(BaseMutator, Who,Cmd);
}


//=============================================================================
// ScoreEvent
//
// Count score events in the TPRIs.
//=============================================================================

function ScoreEvent(PlayerReplicationInfo Who, float Points, string Desc)
{
  if ( TournamentPlayerReplicationInfo(Who) != None )
    TournamentPlayerReplicationInfo(Who).AddScoreEvent(Desc);
  Super.ScoreEvent(Who, Points, Desc);
}


//=============================================================================
// IncrementGoalsScored
//
// Fix announcement for server demos.
//=============================================================================

function IncrementGoalsScored(PlayerReplicationInfo PRI)
{
  PRI.GoalsScored += 1;
  if ( PRI.GoalsScored == 3 ) {
    if ( TournamentPlayer(PRI.Owner) != None )
      TournamentPlayer(PRI.Owner).DelayedAnnouncement(HatTrickSound, 30);
    else if ( UnrealPlayer(PRI.Owner) != None )
      UnrealPlayer(PRI.Owner).ClientDelayedAnnouncement(HatTrickSound, 30);
  }
}


//=============================================================================
// TeamScoreEvent
//
// Add this game event to the events list.
//=============================================================================

function TeamScoreEvent(int Team, float Points, string Desc)
{
  TournamentGameReplicationInfo(GameReplicationInfo).AddGameEvent(GameReplicationInfo.ElapsedTime
      - TournamentGameReplicationInfo(GameReplicationInfo).GameStartTime, Team);
  Super.TeamScoreEvent(Team, Points, Desc);
}


//=============================================================================
// Login
//
// Make sure the player uses the correct pawn class.
//=============================================================================

event PlayerController Login(string Portal, string Options, out string Error)
{
  local PlayerController NewPlayer;
  
  NewPlayer = Super.Login(Portal, Options, Error);
  if ( NewPlayer != None && NewPlayer.PawnClass == None || NewPlayer.PawnClass == class'xPawn' )
    NewPlayer.PawnClass = class'TournamentPawn';
  
  return NewPlayer;
}


//=============================================================================
// ChangeLoadOut
//
// Loadout changes are not supported in Tournament.
//=============================================================================

function ChangeLoadOut(PlayerController P, string LoadoutName);


//=============================================================================
// SpawnBot
//
// Use the SuperBots bot controller class if desired.
//=============================================================================

function Bot SpawnBot(optional string BotName)
{
  return class'TournamentUtil'.static.SpawnBot(Self, BotName);
}


//=============================================================================
// Timer
//
// Global Timer event. Welcome new spectators.
//=============================================================================

event Timer()
{
  class'TournamentUtil'.static.CheckWelcome(Self);
  Super.Timer();
}


//=============================================================================
// ChangeName
//
// Sets a player's (new) name.
//=============================================================================

function ChangeName(Controller Other, string NewName, bool bNameChange)
{
  class'TournamentUtil'.static.ChangeName(Other, NewName, bNameChange);
}


//=============================================================================
// BroadcastDeathMessage
//
// Broadcast whether it was a kill with Double Damage if desired.
//=============================================================================

function BroadcastDeathMessage(Controller Killer, Controller Other, class<DamageType> DamageType)
{
  if ( class'TournamentServerSettings'.default.bBroadcastUDamageKills
      && Killer != None && Killer.Pawn != None && Killer.Pawn.HasUDamage() ) {
    if ( Killer == Other )
      BroadcastLocalized(Self, DeathMessageClass, 3, None, Other.PlayerReplicationInfo, DamageType);
    else
      BroadcastLocalized(Self, DeathMessageClass, 2, Killer.PlayerReplicationInfo,
          Other.PlayerReplicationInfo, DamageType);
  }
  else
    Super.BroadcastDeathMessage(Killer, Other, DamageType);
}


//=============================================================================
// FillPlayInfo
//
// Add Tournament-specific settings.
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  Super.FillPlayInfo(PlayInfo);
  PlayInfo.AddSetting("Tournament", "bWeaponShouldViewShake", "Weapon View Shake", 0, 0, "Check");
}


//=============================================================================
// GetServerDetails
//
// Add Tournament-specific settings.
//=============================================================================

function GetServerDetails(out ServerResponseLine ServerState)
{
  local int i;
  
  Super.GetServerDetails(ServerState);
  i = ServerState.ServerInfo.Length;
  ServerState.ServerInfo.Length = i+1;
  ServerState.ServerInfo[i].Key = "Mutator";
  ServerState.ServerInfo[i++].Value = "Tournament";
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  GameName="Tournament BR"
  GameMessageClass=class'TournamentGameMessage'
  HUDType="Tournament.TournamentHUDBombingRun"
  ScoreBoardType="Tournament.TournamentScoreboardBR"
  DefaultPlayerClassName="Tournament.TournamentPawn"
  PlayerControllerClassName="Tournament.TournamentPlayer"
  MutatorClass="Tournament.TournamentMutator"
  GameReplicationInfoClass=class'TournamentGameReplicationInfo'
  DeathMessageClass=class'TournamentDeathMessage'
  GoalScore=0
  TimeLimit=20
}