//=============================================================================
// xDoubleDom
// Copyright 2003-2004 by Wormbo <wormbo@onlinehome.de>
//
// Tournament Double Domination
//=============================================================================


class xDoubleDom extends xGame.xDoubleDom;


//=============================================================================
// Variables
//=============================================================================

var int WarmupTime;
var int WarmupCountdown;


//=============================================================================
// Reset
//
// Reset the game after warmup.
//=============================================================================

function Reset()
{
  ScoreCountDown = TimeToScore;
  DisabledCountDown = 0;
  xDomPoints[0].ResetPoint(True);
  xDomPoints[1].ResetPoint(True);
  Super.Reset();
}


//=============================================================================
// InitGame
//
// Set the fake GameName for display purposes and to fool the master server.
//=============================================================================

event InitGame(string Options, out string Error)
{
  if ( class'xPawn'.default.ControllerClass != class'xBot' ) {
    class'xPawn'.default.ControllerClass = class'xBot';
    ConsoleCommand("OBJ GARBAGE");
  }
  GameName = class'xGame.xDoubleDom'.default.GameName;
  Super.InitGame(Options, Error);
  SpawnProtectionTime = Clamp(GetIntOption(Options, "SpawnProt", SpawnProtectionTime), 0, 30);
  WarmupTime = 60 * Max(1, GetIntOption(Options, "WarmupTime", class'TournamentServerSettings'.default.WarmupTime));
  WarmupCountdown = WarmupTime;
}


//=============================================================================
// exec EndWarmup
//
// Ends the warmup time.
//=============================================================================

exec function EndWarmup()
{
  class'TournamentUtil'.static.EndWarmup(Self, WarmupCountdown);
}


//=============================================================================
// state PendingMatch
//
// Check for match or warmup start.
//=============================================================================

auto state PendingMatch
{
  function RestartPlayer(Controller aPlayer)
  {
    if ( WarmupCountDown > 3 )
      Global.RestartPlayer(aPlayer);
  }
  
  function Timer()
  {
    local Controller TeamMate;
    
    Global.Timer();
    class'TournamentUtil'.static.PendingMatchTimer(Self, WarmupTime, WarmupCountDown);
    
    if ( xDomPoints[0] == None || xDomPoints[1] == None )
      return;
    
    if ( CountDown == 0 ) {
      ScoreCountDown = TimeToScore;
      DisabledCountDown = 0;
      xDomPoints[0].ResetPoint(True);
      xDomPoints[1].ResetPoint(True);
      return;
    }
    else if ( StartupStage > 1 ) {
      ScoreCountDown = TimeToScore;
      DisabledCountDown = 0;
      xDomPoints[0].ResetPoint(False);
      xDomPoints[1].ResetPoint(False);
      return;
    }
    
    // are both points currently disabled?
    if ( DisabledCountDown > 0 ) {
      DisabledCountDown--;
      if ( DisabledCountDown == 7 ) {
        for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
          if ( PlayerController(TeamMate) != None )
            PlayerController(TeamMate).PlayAnnouncement(NewRoundSound, 1, true);
      }
      else if ( DisabledCountDown > 0 && DisabledCountDown < 6 )
        BroadcastLocalizedMessage(class'TimerMessage', DisabledCountDown);
      
      if ( DisabledCountDown == 0 ) {
        xDomPoints[0].ResetPoint(true);
        xDomPoints[1].ResetPoint(true);
      }
    }
    
    // nothing more to do unless the same team controls both points
    if ( xDomPoints[0].ControllingTeam != xDomPoints[1].ControllingTeam || xDomPoints[0].ControllingTeam == None )
      return;
    
    if ( ScoreCountDown == TimeToScore )
      for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
        if ( TeamMate.IsA('PlayerController') )
          PlayerController(TeamMate).PlayAnnouncement(ControlSounds[xDomPoints[0].ControllingTeam.TeamIndex], 1, true);
    
    if ( ScoreCountDown > 0 && ScoreCountDown < 9 )
      for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
        BroadcastLocalizedMessage(class'TimerMessage', ScoreCountDown);
    
    if ( ScoreCountDown > 0 ) {
      // decrement the time remaining until a point is scored
      ScoreCountDown--;
      return;
    }
    
    for (TeamMate = Level.ControllerList; TeamMate != None; TeamMate = TeamMate.NextController)
      if ( TeamMate.IsA('PlayerController') ) {
        PlayerController(TeamMate).PlayAnnouncement(ScoreSounds[xDomPoints[0].ControllingTeam.TeamIndex], 1, true);
        PlayerController(TeamMate).ClientPlaySound(class'CTFMessage'.Default.Riffs[Rand(3)]);
      }
    
    // award scoring players some Adrenaline and increase their score
    if ( xDomPoints[0].ControllingPawn != None && xDomPoints[0].ControllingPawn.Controller != None )
      xDomPoints[0].ControllingPawn.Controller.AwardAdrenaline(ADR_Goal);
    
    if ( xDomPoints[1].ControllingPawn != None && xDomPoints[1].ControllingPawn.Controller != None )
      xDomPoints[1].ControllingPawn.Controller.AwardAdrenaline(ADR_Goal);
    
    // reset count
    ScoreCountDown = TimeToScore;
    
    // make all control points be untouchable for a short period
    DisabledCountDown = TimeDisabled;
    
    // reset both domination points and disable them
    xDomPoints[0].ResetPoint(false);
    xDomPoints[1].ResetPoint(false);
  }
}


//=============================================================================
// ParseChatPercVar
//
// The %L chat variable should also find shield and big health packs.
//=============================================================================

static function string ParseChatPercVar(Mutator BaseMutator, Controller Who, string Cmd)
{
  return class'TournamentUtil'.static.ParseChatPercVar(BaseMutator, Who,Cmd);
}


//=============================================================================
// ScoreEvent
//
// Count score events in the TPRIs.
//=============================================================================

function ScoreEvent(PlayerReplicationInfo Who, float Points, string Desc)
{
  if ( TournamentPlayerReplicationInfo(Who) != None )
    TournamentPlayerReplicationInfo(Who).AddScoreEvent(Desc);
  Super.ScoreEvent(Who, Points, Desc);
}


//=============================================================================
// IncrementGoalsScored
//
// Fix announcement for server demos.
//=============================================================================

function IncrementGoalsScored(PlayerReplicationInfo PRI)
{
  PRI.GoalsScored += 1;
  if ( PRI.GoalsScored == 3 ) {
    if ( TournamentPlayer(PRI.Owner) != None )
      TournamentPlayer(PRI.Owner).DelayedAnnouncement(HatTrickSound, 30);
    else if ( UnrealPlayer(PRI.Owner) != None )
      UnrealPlayer(PRI.Owner).ClientDelayedAnnouncement(HatTrickSound, 30);
  }
}


//=============================================================================
// TeamScoreEvent
//
// Add this game event to the events list.
//=============================================================================

function TeamScoreEvent(int Team, float Points, string Desc)
{
  TournamentGameReplicationInfo(GameReplicationInfo).AddGameEvent(GameReplicationInfo.ElapsedTime
      - TournamentGameReplicationInfo(GameReplicationInfo).GameStartTime, Team);
  Super.TeamScoreEvent(Team, Points, Desc);
}


//=============================================================================
// Login
//
// Make sure the player uses the correct pawn class.
//=============================================================================

event PlayerController Login(string Portal, string Options, out string Error)
{
  local PlayerController NewPlayer;
  
  NewPlayer = Super.Login(Portal, Options, Error);
  if ( NewPlayer != None && NewPlayer.PawnClass == None || NewPlayer.PawnClass == class'xPawn' )
    NewPlayer.PawnClass = class'TournamentPawn';
  
  return NewPlayer;
}


//=============================================================================
// ChangeLoadOut
//
// Loadout changes are not supported in Tournament.
//=============================================================================

function ChangeLoadOut(PlayerController P, string LoadoutName);


//=============================================================================
// SpawnBot
//
// Use the SuperBots bot controller class if desired.
//=============================================================================

function Bot SpawnBot(optional string BotName)
{
  return class'TournamentUtil'.static.SpawnBot(Self, BotName);
}


//=============================================================================
// Timer
//
// Global Timer event. Welcome new spectators.
//=============================================================================

event Timer()
{
  class'TournamentUtil'.static.CheckWelcome(Self);
  Super.Timer();
}


//=============================================================================
// ChangeName
//
// Sets a player's (new) name.
//=============================================================================

function ChangeName(Controller Other, string NewName, bool bNameChange)
{
  class'TournamentUtil'.static.ChangeName(Other, NewName, bNameChange);
}


//=============================================================================
// BroadcastDeathMessage
//
// Broadcast whether it was a kill with Double Damage if desired.
//=============================================================================

function BroadcastDeathMessage(Controller Killer, Controller Other, class<DamageType> DamageType)
{
  if ( class'TournamentServerSettings'.default.bBroadcastUDamageKills
      && Killer != None && Killer.Pawn != None && Killer.Pawn.HasUDamage() ) {
    if ( Killer == Other )
      BroadcastLocalized(Self, DeathMessageClass, 3, None, Other.PlayerReplicationInfo, DamageType);
    else
      BroadcastLocalized(Self, DeathMessageClass, 2, Killer.PlayerReplicationInfo,
          Other.PlayerReplicationInfo, DamageType);
  }
  else
    Super.BroadcastDeathMessage(Killer, Other, DamageType);
}


//=============================================================================
// FillPlayInfo
//
// Add Tournament-specific settings.
//=============================================================================

static function FillPlayInfo(PlayInfo PlayInfo)
{
  Super.FillPlayInfo(PlayInfo);
  PlayInfo.AddSetting("Tournament", "bWeaponShouldViewShake", "Weapon View Shake", 0, 0, "Check");
}


//=============================================================================
// GetServerDetails
//
// Add Tournament-specific settings.
//=============================================================================

function GetServerDetails(out ServerResponseLine ServerState)
{
  local int i;
  
  Super.GetServerDetails(ServerState);
  i = ServerState.ServerInfo.Length;
  ServerState.ServerInfo.Length = i+1;
  ServerState.ServerInfo[i].Key = "Mutator";
  ServerState.ServerInfo[i++].Value = "Tournament";
}


//=============================================================================
// Default properties
//=============================================================================

defaultproperties
{
  GameName="Tournament DDom"
  GameMessageClass=class'TournamentGameMessage'
  HUDType="Tournament.TournamentHUDDoubleDom"
  ScoreBoardType="Tournament.TournamentScoreboardDD"
  DefaultPlayerClassName="Tournament.TournamentPawn"
  PlayerControllerClassName="Tournament.TournamentPlayer"
  MutatorClass="Tournament.TournamentMutator"
  GameReplicationInfoClass=class'TournamentGameReplicationInfo'
  DeathMessageClass=class'TournamentDeathMessage'
  GoalScore=0
  TimeLimit=20
}